/* -*- mode: c++; indent-tabs-mode: nil -*- */
/* @file QC_Datasource.qpp Datasource class definition

  Qore Programming Language

  Copyright (C) 2003 - 2017 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/QC_Datasource.h"
#include "qore/intern/QoreHashNodeIntern.h"

extern QoreClass* QC_ABSTRACTDATASOURCE;

static const char* DSC_ERR = "DATASOURCE-CONSTRUCTOR-ERROR";

static int ds_set_options(ManagedDatasource* ds, const QoreHashNode* opts, ExceptionSink* xsink) {
   ConstHashIterator hi(opts);
   while (hi.next()) {
      // skip "min" and "max" options
      if (!strcmp(hi.getKey(), "min") || !strcmp(hi.getKey(), "max"))
         continue;

      if (ds->setOptionInit(hi.getKey(), hi.getValue(), xsink))
         return -1;
   }
   return 0;
}

static void ds_constructor_hash(const QoreHashNode* opts, QoreObject* self, Queue* queue, const QoreValue& arg, ExceptionSink* xsink) {
   ReferenceHolder<Queue> q(queue, xsink);

   const char* str = check_hash_key(opts, "type", DSC_ERR, xsink);
   if (*xsink) return;

   if (!str) {
      xsink->raiseException(DSC_ERR, "expecting a string value with the 'type' key giving the driver name");
      return;
   }

   DBIDriver* db_driver = DBI.find(str, xsink);
   if (!db_driver) {
      if (!*xsink)
         xsink->raiseException("DATASOURCE-UNSUPPORTED-DATABASE", "DBI driver '%s' cannot be loaded", str);
      return;
   }

   ReferenceHolder<ManagedDatasource> ds(new ManagedDatasource(db_driver), xsink);

   str = check_hash_key(opts, "user", DSC_ERR, xsink);
   if (*xsink) return;
   if (str) ds->setPendingUsername(str);

   str = check_hash_key(opts, "pass", DSC_ERR, xsink);
   if (*xsink) return;
   if (str) ds->setPendingPassword(str);

   str = check_hash_key(opts, "db", DSC_ERR, xsink);
   if (*xsink) return;
   if (str) ds->setPendingDBName(str);

   str = check_hash_key(opts, "charset", DSC_ERR, xsink);
   if (*xsink) return;
   if (str) ds->setPendingDBEncoding(str);

   str = check_hash_key(opts, "host", DSC_ERR, xsink);
   if (*xsink) return;
   if (str) ds->setPendingHostName(str);

   int port = (int)opts->getValueKeyValue("port").getAsBigInt();
   if (port) {
      if (port < 0) {
         xsink->raiseException(DSC_ERR, "port value must be zero (meaning use the default port) or positive (value given: %d)", port);
         return;
      }
      ds->setPendingPort(port);
   }

   {
      QoreValue p = opts->getValueKeyValue("options", xsink);
      if (!p.isNothing()) {
         if (p.getType() != NT_HASH) {
            xsink->raiseException(DSC_ERR, "'options' key is not type 'hash' but is type '%s'", p.getTypeName());
            return;
         }
         const QoreHashNode* opt_hash = p.get<const QoreHashNode>();
         if (ds_set_options(*ds, opt_hash, xsink))
            return;
      }
   }

   if (queue)
      ds->setEventQueue(q.release(), arg.getReferencedValue(), xsink);

   self->setPrivate(CID_DATASOURCE, ds.release());
}
/* Qore class Qore::SQL::Datasource */

qore_classid_t CID_DATASOURCE;
QoreClass* QC_DATASOURCE;

// nothing Datasource::beginTransaction(){}
static QoreValue Datasource_beginTransaction(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 855 "QC_Datasource.qpp"
    ds->beginTransaction(xsink);
    return QoreValue();
}

// nothing Datasource::clearEventQueue(){}
static QoreValue Datasource_clearEventQueue(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1327 "QC_Datasource.qpp"
    ds->setEventQueue(0, 0, xsink);
    return QoreValue();
}

// nothing Datasource::close(){}
static QoreValue Datasource_close(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 491 "QC_Datasource.qpp"
    ds->close(xsink);
    return QoreValue();
}

// nothing Datasource::commit(){}
static QoreValue Datasource_commit(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 506 "QC_Datasource.qpp"
    ds->commit(xsink);
    return QoreValue();
}

// Datasource::constructor(string driver, *string user, *string pass, *string db, *string encoding, *string host, *softint port, *hash options, *Qore::Thread::Queue queue, auto arg) {}
static void Datasource_constructor_VsNsNsNsNsNsniNhC5QueueVA(QoreObject* self, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* driver = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* user = get_param_value(args, 1).get<const QoreStringNode>();
    const QoreStringNode* pass = get_param_value(args, 2).get<const QoreStringNode>();
    const QoreStringNode* db = get_param_value(args, 3).get<const QoreStringNode>();
    const QoreStringNode* encoding = get_param_value(args, 4).get<const QoreStringNode>();
    const QoreStringNode* host = get_param_value(args, 5).get<const QoreStringNode>();
    int64 port = HARD_QORE_VALUE_INT(args, 6);
    const QoreHashNode* options = get_param_value(args, 7).get<const QoreHashNode>();
    HARD_QORE_VALUE_OBJ_OR_NOTHING_DATA(queue, Queue, args, 8, CID_QUEUE, xsink);
    if (*xsink)
        return;
    QoreValue arg = get_param_value(args, 9);
# 314 "QC_Datasource.qpp"
ReferenceHolder<Queue> q(queue, xsink);

   if (!port && driver->find('@') != -1) {
      qore_offset_t colon = driver->find(':');
      qore_offset_t atsign = driver->find('@');
      if (colon < 0 || (atsign < colon)) {
         xsink->raiseException("DATASOURCE-PARSE-ERROR", "driver is missing from the datasource string");
         return;
      }

      ReferenceHolder<QoreHashNode> h(parseDatasource(driver->getBuffer(), xsink), xsink);
      if (!h) {
         assert(*xsink);
         return;
      }
      ds_constructor_hash(*h, self, q.release(), arg, xsink);
      return;
   }

   DBIDriver *db_driver = DBI.find(driver->getBuffer(), xsink);
   if (!db_driver) {
      if (!*xsink)
	 xsink->raiseException("DATASOURCE-UNSUPPORTED-DATABASE", "DBI driver '%s' cannot be loaded", driver->getBuffer());
      return;
   }

   ReferenceHolder<ManagedDatasource> ds(new ManagedDatasource(db_driver), xsink);

   if (user && !user->empty())
      ds->setPendingUsername(user->getBuffer());

   if (pass && !pass->empty())
      ds->setPendingPassword(pass->getBuffer());

   if (db && !db->empty())
      ds->setPendingDBName(db->getBuffer());

   if (encoding && !encoding->empty())
      ds->setPendingDBEncoding(encoding->getBuffer());

   if (host && !host->empty())
      ds->setPendingHostName(host->getBuffer());

   if (port) {
      if (port < 0) {
	 xsink->raiseException(DSC_ERR, "port value must be zero (meaning use the default port) or positive (value given: %d)", port);
	 return;
      }
      ds->setPendingPort((int)port);
   }

   if (options)
      ds_set_options(*ds, options, xsink);

   if (queue)
      ds->setEventQueue(q.release(), arg.getReferencedValue(), xsink);

   self->setPrivate(CID_DATASOURCE, ds.release());
}

// Datasource::constructor(string desc, *Qore::Thread::Queue queue, auto arg) {}
static void Datasource_constructor_VsC5QueueVA(QoreObject* self, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* desc = HARD_QORE_VALUE_STRING(args, 0);
    HARD_QORE_VALUE_OBJ_OR_NOTHING_DATA(queue, Queue, args, 1, CID_QUEUE, xsink);
    if (*xsink)
        return;
    QoreValue arg = get_param_value(args, 2);
# 391 "QC_Datasource.qpp"
ReferenceHolder<Queue> q(queue, xsink);

   TempEncodingHelper tstr(desc, QCS_UTF8, xsink);
   if (*xsink)
      return;
   qore_offset_t colon = tstr->find(':');
   qore_offset_t atsign = tstr->find('@');
   if (colon < 0 || (atsign < colon)) {
      xsink->raiseException("DATASOURCE-PARSE-ERROR", "driver is missing from the datasource string");
      return;
   }

   ReferenceHolder<QoreHashNode> h(parseDatasource(desc->getBuffer(), xsink), xsink);
   if (!h) {
      assert(*xsink);
      return;
   }
   ds_constructor_hash(*h, self, q.release(), arg, xsink);
   return;
}

// Datasource::constructor(hash opts, *Qore::Thread::Queue queue, auto arg) {}
static void Datasource_constructor_VhC5QueueVA(QoreObject* self, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreHashNode* opts = HARD_QORE_VALUE_HASH(args, 0);
    HARD_QORE_VALUE_OBJ_OR_NOTHING_DATA(queue, Queue, args, 1, CID_QUEUE, xsink);
    if (*xsink)
        return;
    QoreValue arg = get_param_value(args, 2);
# 435 "QC_Datasource.qpp"
ds_constructor_hash(opts, self, queue, arg, xsink);
}

// Datasource::copy() {}
static void Datasource_copy(QoreObject* self, QoreObject* old, ManagedDatasource* ds, ExceptionSink* xsink) {
# 460 "QC_Datasource.qpp"
self->setPrivate(CID_DATASOURCE, ds->copy());
}

// bool Datasource::currentThreadInTransaction(){}
static QoreValue Datasource_currentThreadInTransaction(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1209 "QC_Datasource.qpp"
    return ds->currentThreadInTransaction();
}

// *hash Datasource::describe(string sql, ...){}
static QoreValue Datasource_describe_VsVV(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* sql = HARD_QORE_VALUE_STRING(args, 0);
# 835 "QC_Datasource.qpp"
    ReferenceHolder<QoreListNode> argl((args->size() > 1 ? args->getOldList(1) : 0), xsink);
   return ds->describe(sql, *argl, xsink);
}

// Datasource::destructor() {}
static void Datasource_destructor(QoreObject* self, ManagedDatasource* ds, ExceptionSink* xsink) {
# 448 "QC_Datasource.qpp"
ds->destructor(xsink);
   ds->deref(xsink);
}

// auto Datasource::exec(string sql, ...){}
static QoreValue Datasource_exec_VsVV(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* sql = HARD_QORE_VALUE_STRING(args, 0);
# 565 "QC_Datasource.qpp"
    ReferenceHolder<QoreListNode> argl((args->size() > 1 ? args->getOldList(1) : 0), xsink);
   return ds->exec(sql, *argl, xsink);
}

// auto Datasource::execRaw(string sql){}
static QoreValue Datasource_execRaw_Vs(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* sql = HARD_QORE_VALUE_STRING(args, 0);
# 612 "QC_Datasource.qpp"
    return ds->execRaw(sql, xsink);
}

// bool Datasource::getAutoCommit(){}
static QoreValue Datasource_getAutoCommit(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 544 "QC_Datasource.qpp"
    return ds->getAutoCommit();
}

// int Datasource::getCapabilities(){}
static QoreValue Datasource_getCapabilities(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 884 "QC_Datasource.qpp"
    return ds->getCapabilities();
}

// list Datasource::getCapabilityList(){}
static QoreValue Datasource_getCapabilityList(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 898 "QC_Datasource.qpp"
    return ds->getCapabilityList();
}

// auto Datasource::getClientVersion(){}
static QoreValue Datasource_getClientVersion(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1181 "QC_Datasource.qpp"
    return ds->getClientVersion(xsink);
}

// hash Datasource::getConfigHash(){}
static QoreValue Datasource_getConfigHash(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1287 "QC_Datasource.qpp"
    return ds->getConfigHash(xsink);
}

// string Datasource::getConfigString(){}
static QoreValue Datasource_getConfigString(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1301 "QC_Datasource.qpp"
    return ds->getConfigString(xsink);
}

// string Datasource::getDBCharset(){}
static QoreValue Datasource_getDBCharset(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1048 "QC_Datasource.qpp"
    return ds->getPendingDBEncoding();
}

// string Datasource::getDBEncoding(){}
static QoreValue Datasource_getDBEncoding(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1062 "QC_Datasource.qpp"
    return ds->getPendingDBEncoding();
}

// *string Datasource::getDBName(){}
static QoreValue Datasource_getDBName(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1034 "QC_Datasource.qpp"
    return ds->getPendingDBName();
}

// string Datasource::getDriverName(){}
static QoreValue Datasource_getDriverName(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1151 "QC_Datasource.qpp"
    return new QoreStringNode(ds->getDriver()->getName());
}

// *string Datasource::getHostName(){}
static QoreValue Datasource_getHostName(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1102 "QC_Datasource.qpp"
    return ds->getPendingHostName();
}

// string Datasource::getOSCharset(){}
static QoreValue Datasource_getOSCharset(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1076 "QC_Datasource.qpp"
    const QoreEncoding *enc = ds->getQoreEncoding();
   return new QoreStringNode(enc ? enc->getCode() : "(unknown)");
}

// *string Datasource::getOSEncoding(){}
static QoreValue Datasource_getOSEncoding(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1089 "QC_Datasource.qpp"
    const QoreEncoding *enc = ds->getQoreEncoding();
   return enc ? new QoreStringNode(enc->getCode()) : 0;
}

// auto Datasource::getOption(string opt){}
static QoreValue Datasource_getOption_Vs(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* opt = HARD_QORE_VALUE_STRING(args, 0);
# 1273 "QC_Datasource.qpp"
    return ds->getOption(opt->getBuffer(), xsink);
}

// hash Datasource::getOptionHash(){}
static QoreValue Datasource_getOptionHash(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1244 "QC_Datasource.qpp"
    return ds->getOptionHash(xsink);
}

// *string Datasource::getPassword(){}
static QoreValue Datasource_getPassword(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1022 "QC_Datasource.qpp"
    return ds->getPendingPassword();
}

// *int Datasource::getPort(){}
static QoreValue Datasource_getPort(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1114 "QC_Datasource.qpp"
    int port = ds->getPort();
   return port ? new QoreBigIntNode(port) : 0;
}

// auto Datasource::getServerVersion(){}
static QoreValue Datasource_getServerVersion(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1167 "QC_Datasource.qpp"
    return ds->getServerVersion(xsink);
}

// int Datasource::getTransactionLockTimeout(){}
static QoreValue Datasource_getTransactionLockTimeout(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1139 "QC_Datasource.qpp"
    return ds->getTransactionLockTimeout();
}

// *string Datasource::getUserName(){}
static QoreValue Datasource_getUserName(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1010 "QC_Datasource.qpp"
    return ds->getPendingUsername();
}

// bool Datasource::inTransaction(){}
static QoreValue Datasource_inTransaction(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1193 "QC_Datasource.qpp"
    return ds->isInTransaction();
}

// nothing Datasource::open(){}
static QoreValue Datasource_open(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 476 "QC_Datasource.qpp"
    ds->open(xsink);
    return QoreValue();
}

// nothing Datasource::reset(){}
static QoreValue Datasource_reset(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 870 "QC_Datasource.qpp"
    ds->reset(xsink);
    return QoreValue();
}

// nothing Datasource::rollback(){}
static QoreValue Datasource_rollback(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 521 "QC_Datasource.qpp"
    ds->rollback(xsink);
    return QoreValue();
}

// auto Datasource::select(string sql, ...){}
static QoreValue Datasource_select_VsVV(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* sql = HARD_QORE_VALUE_STRING(args, 0);
# 650 "QC_Datasource.qpp"
    ReferenceHolder<QoreListNode> argl((args->size() > 1 ? args->getOldList(1) : 0), xsink);
   return ds->select(sql, *argl, xsink);
}

// *hash Datasource::selectRow(string sql, ...){}
static QoreValue Datasource_selectRow_VsVV(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* sql = HARD_QORE_VALUE_STRING(args, 0);
# 677 "QC_Datasource.qpp"
    ReferenceHolder<QoreListNode> argl((args->size() > 1 ? args->getOldList(1) : 0), xsink);
   return ds->selectRow(sql, *argl, xsink);
}

// auto Datasource::selectRows(string sql, ...){}
static QoreValue Datasource_selectRows_VsVV(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* sql = HARD_QORE_VALUE_STRING(args, 0);
# 709 "QC_Datasource.qpp"
    ReferenceHolder<QoreListNode> argl((args->size() > 1 ? args->getOldList(1) : 0), xsink);
   return ds->selectRows(sql, *argl, xsink);
}

// nothing Datasource::setAutoCommit(bool ac = True){}
static QoreValue Datasource_setAutoCommit_Vb(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    bool ac = HARD_QORE_VALUE_BOOL(args, 0);
# 532 "QC_Datasource.qpp"
    ds->setAutoCommit(ac, xsink);
    return QoreValue();
}

// nothing Datasource::setDBCharset(string encoding){}
static QoreValue Datasource_setDBCharset_Vs(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* encoding = HARD_QORE_VALUE_STRING(args, 0);
# 956 "QC_Datasource.qpp"
    ds->setPendingDBEncoding(encoding->getBuffer());
    return QoreValue();
}

// nothing Datasource::setDBEncoding(string encoding){}
static QoreValue Datasource_setDBEncoding_Vs(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* encoding = HARD_QORE_VALUE_STRING(args, 0);
# 970 "QC_Datasource.qpp"
    ds->setPendingDBEncoding(encoding->getBuffer());
    return QoreValue();
}

// nothing Datasource::setDBName(string db){}
static QoreValue Datasource_setDBName_Vs(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* db = HARD_QORE_VALUE_STRING(args, 0);
# 940 "QC_Datasource.qpp"
    ds->setPendingDBName(db->getBuffer());
    return QoreValue();
}

// nothing Datasource::setEventQueue(Qore::Thread::Queue queue, auto arg){}
static QoreValue Datasource_setEventQueue_C5QueueVA(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(queue, Queue, args, 0, CID_QUEUE, "Datasource::setEventQueue()", "Qore::Thread::Queue", xsink);
    if (*xsink)
        return 0;
    QoreValue arg = get_param_value(args, 1);
# 1314 "QC_Datasource.qpp"
    ReferenceHolder<Queue> q(queue, xsink);

   if (queue->getMax() != -1)
      return xsink->raiseException("QUEUE-ERROR", "the Queue object passed as a maximum size of %d entr%s, which could cause internal I/O to freeze in the Datasource object if used; use a Queue object with no maximum size for datasource event monitoring", queue->getMax(), queue->getMax() == 1 ? "y" : "ies");

   ds->setEventQueue(q.release(), arg.getReferencedValue(), xsink);
    return QoreValue();
}

// nothing Datasource::setHostName(string host){}
static QoreValue Datasource_setHostName_Vs(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* host = HARD_QORE_VALUE_STRING(args, 0);
# 984 "QC_Datasource.qpp"
    ds->setPendingHostName(host->getBuffer());
    return QoreValue();
}

// nothing Datasource::setOption(string opt, auto val){}
static QoreValue Datasource_setOption_VsVA(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* opt = HARD_QORE_VALUE_STRING(args, 0);
    QoreValue val = get_param_value(args, 1);
# 1259 "QC_Datasource.qpp"
    ds->setOption(opt->getBuffer(), val, xsink);
    return QoreValue();
}

// nothing Datasource::setPassword(string pass){}
static QoreValue Datasource_setPassword_Vs(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* pass = HARD_QORE_VALUE_STRING(args, 0);
# 926 "QC_Datasource.qpp"
    ds->setPendingPassword(pass->getBuffer());
    return QoreValue();
}

// nothing Datasource::setPort(softint port = 0){}
static QoreValue Datasource_setPort_vi(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 port = HARD_QORE_VALUE_INT(args, 0);
# 998 "QC_Datasource.qpp"
    ds->setPendingPort((int)port);
    return QoreValue();
}

// nothing Datasource::setTransactionLockTimeout(timeout timeout_ms = 0){}
static QoreValue Datasource_setTransactionLockTimeout_Vt(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 1127 "QC_Datasource.qpp"
    ds->setTransactionLockTimeout((int)timeout_ms);
    return QoreValue();
}

// nothing Datasource::setUserName(string user){}
static QoreValue Datasource_setUserName_Vs(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* user = HARD_QORE_VALUE_STRING(args, 0);
# 912 "QC_Datasource.qpp"
    ds->setPendingUsername(user->getBuffer());
    return QoreValue();
}

// int Datasource::transactionTid(){}
static QoreValue Datasource_transactionTid(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1228 "QC_Datasource.qpp"
    return ds->transactionTid();
}

// auto Datasource::vexec(string sql, *softlist vargs){}
static QoreValue Datasource_vexec_Vsnl(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* sql = HARD_QORE_VALUE_STRING(args, 0);
    const QoreListNode* vargs = get_param_value(args, 1).get<const QoreListNode>();
# 589 "QC_Datasource.qpp"
    return ds->exec(sql, vargs, xsink);
}

// auto Datasource::vselect(string sql, *softlist vargs){}
static QoreValue Datasource_vselect_Vsnl(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* sql = HARD_QORE_VALUE_STRING(args, 0);
    const QoreListNode* vargs = get_param_value(args, 1).get<const QoreListNode>();
# 745 "QC_Datasource.qpp"
    return ds->select(sql, vargs, xsink);
}

// *hash Datasource::vselectRow(string sql, *softlist vargs){}
static QoreValue Datasource_vselectRow_Vsnl(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* sql = HARD_QORE_VALUE_STRING(args, 0);
    const QoreListNode* vargs = get_param_value(args, 1).get<const QoreListNode>();
# 772 "QC_Datasource.qpp"
    return ds->selectRow(sql, vargs, xsink);
}

// auto Datasource::vselectRows(string sql, *softlist vargs){}
static QoreValue Datasource_vselectRows_Vsnl(QoreObject* self, ManagedDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* sql = HARD_QORE_VALUE_STRING(args, 0);
    const QoreListNode* vargs = get_param_value(args, 1).get<const QoreListNode>();
# 805 "QC_Datasource.qpp"
    return ds->selectRows(sql, vargs, xsink);
}

DLLLOCAL void preinitDatasourceClass() {
    QC_DATASOURCE = new QoreClass("Datasource", QDOM_DATABASE);
    CID_DATASOURCE = QC_DATASOURCE->getID();
    QC_DATASOURCE->setSystem();
}

DLLLOCAL QoreClass* initDatasourceClass(QoreNamespace& ns) {
    if (!QC_DATASOURCE)
        preinitDatasourceClass();

    // set parent class
    assert(QC_ABSTRACTDATASOURCE);
    QC_DATASOURCE->addBuiltinVirtualBaseClass(QC_ABSTRACTDATASOURCE);

    // nothing Datasource::beginTransaction(){}
    QC_DATASOURCE->addMethod("beginTransaction", (q_method_n_t)Datasource_beginTransaction, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing Datasource::clearEventQueue(){}
    QC_DATASOURCE->addMethod("clearEventQueue", (q_method_n_t)Datasource_clearEventQueue, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing Datasource::close(){}
    QC_DATASOURCE->addMethod("close", (q_method_n_t)Datasource_close, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing Datasource::commit(){}
    QC_DATASOURCE->addMethod("commit", (q_method_n_t)Datasource_commit, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // Datasource::constructor(string driver, *string user, *string pass, *string db, *string encoding, *string host, *softint port, *hash options, *Qore::Thread::Queue queue, auto arg) {}
    QC_DATASOURCE->addConstructor(Datasource_constructor_VsNsNsNsNsNsniNhC5QueueVA, false, QC_NO_FLAGS, QDOM_DEFAULT, 10, stringTypeInfo, NULL, "driver", stringOrNothingTypeInfo, NULL, "user", stringOrNothingTypeInfo, NULL, "pass", stringOrNothingTypeInfo, NULL, "db", stringOrNothingTypeInfo, NULL, "encoding", stringOrNothingTypeInfo, NULL, "host", softBigIntOrNothingTypeInfo, NULL, "port", hashOrNothingTypeInfo, NULL, "options", QC_QUEUE->getOrNothingTypeInfo(), NULL, "queue", autoTypeInfo, NULL, "arg");

    // Datasource::constructor(string desc, *Qore::Thread::Queue queue, auto arg) {}
    QC_DATASOURCE->addConstructor(Datasource_constructor_VsC5QueueVA, false, QC_NO_FLAGS, QDOM_DEFAULT, 3, stringTypeInfo, NULL, "desc", QC_QUEUE->getOrNothingTypeInfo(), NULL, "queue", autoTypeInfo, NULL, "arg");

    // Datasource::constructor(hash opts, *Qore::Thread::Queue queue, auto arg) {}
    QC_DATASOURCE->addConstructor(Datasource_constructor_VhC5QueueVA, false, QC_NO_FLAGS, QDOM_DEFAULT, 3, hashTypeInfo, NULL, "opts", QC_QUEUE->getOrNothingTypeInfo(), NULL, "queue", autoTypeInfo, NULL, "arg");

    // Datasource::copy() {}
    QC_DATASOURCE->setCopy((q_copy_t)Datasource_copy);

    // bool Datasource::currentThreadInTransaction(){}
    QC_DATASOURCE->addMethod("currentThreadInTransaction", (q_method_n_t)Datasource_currentThreadInTransaction, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // *hash Datasource::describe(string sql, ...){}
    QC_DATASOURCE->addMethod("describe", (q_method_n_t)Datasource_describe_VsVV, false, QC_USES_EXTRA_ARGS, QDOM_DEFAULT, hashOrNothingTypeInfo, 1, stringTypeInfo, NULL, "sql");

    // Datasource::destructor() {}
    QC_DATASOURCE->setDestructor((q_destructor_t)Datasource_destructor);

    // auto Datasource::exec(string sql, ...){}
    QC_DATASOURCE->addMethod("exec", (q_method_n_t)Datasource_exec_VsVV, false, QC_USES_EXTRA_ARGS, QDOM_DEFAULT, autoTypeInfo, 1, stringTypeInfo, NULL, "sql");

    // auto Datasource::execRaw(string sql){}
    QC_DATASOURCE->addMethod("execRaw", (q_method_n_t)Datasource_execRaw_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 1, stringTypeInfo, NULL, "sql");

    // bool Datasource::getAutoCommit(){}
    QC_DATASOURCE->addMethod("getAutoCommit", (q_method_n_t)Datasource_getAutoCommit, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // int Datasource::getCapabilities(){}
    QC_DATASOURCE->addMethod("getCapabilities", (q_method_n_t)Datasource_getCapabilities, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // list Datasource::getCapabilityList(){}
    QC_DATASOURCE->addMethod("getCapabilityList", (q_method_n_t)Datasource_getCapabilityList, false, QC_CONSTANT, QDOM_DEFAULT, listTypeInfo);

    // auto Datasource::getClientVersion(){}
    QC_DATASOURCE->addMethod("getClientVersion", (q_method_n_t)Datasource_getClientVersion, false, QC_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo);

    // hash Datasource::getConfigHash(){}
    QC_DATASOURCE->addMethod("getConfigHash", (q_method_n_t)Datasource_getConfigHash, false, QC_CONSTANT, QDOM_DEFAULT, hashTypeInfo);

    // string Datasource::getConfigString(){}
    QC_DATASOURCE->addMethod("getConfigString", (q_method_n_t)Datasource_getConfigString, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // string Datasource::getDBCharset(){}
    QC_DATASOURCE->addMethod("getDBCharset", (q_method_n_t)Datasource_getDBCharset, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // string Datasource::getDBEncoding(){}
    QC_DATASOURCE->addMethod("getDBEncoding", (q_method_n_t)Datasource_getDBEncoding, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // *string Datasource::getDBName(){}
    QC_DATASOURCE->addMethod("getDBName", (q_method_n_t)Datasource_getDBName, false, QC_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // string Datasource::getDriverName(){}
    QC_DATASOURCE->addMethod("getDriverName", (q_method_n_t)Datasource_getDriverName, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // *string Datasource::getHostName(){}
    QC_DATASOURCE->addMethod("getHostName", (q_method_n_t)Datasource_getHostName, false, QC_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // string Datasource::getOSCharset(){}
    QC_DATASOURCE->addMethod("getOSCharset", (q_method_n_t)Datasource_getOSCharset, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // *string Datasource::getOSEncoding(){}
    QC_DATASOURCE->addMethod("getOSEncoding", (q_method_n_t)Datasource_getOSEncoding, false, QC_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // auto Datasource::getOption(string opt){}
    QC_DATASOURCE->addMethod("getOption", (q_method_n_t)Datasource_getOption_Vs, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, autoTypeInfo, 1, stringTypeInfo, NULL, "opt");

    // hash Datasource::getOptionHash(){}
    QC_DATASOURCE->addMethod("getOptionHash", (q_method_n_t)Datasource_getOptionHash, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, hashTypeInfo);

    // *string Datasource::getPassword(){}
    QC_DATASOURCE->addMethod("getPassword", (q_method_n_t)Datasource_getPassword, false, QC_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // *int Datasource::getPort(){}
    QC_DATASOURCE->addMethod("getPort", (q_method_n_t)Datasource_getPort, false, QC_CONSTANT, QDOM_DEFAULT, bigIntOrNothingTypeInfo);

    // auto Datasource::getServerVersion(){}
    QC_DATASOURCE->addMethod("getServerVersion", (q_method_n_t)Datasource_getServerVersion, false, QC_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo);

    // int Datasource::getTransactionLockTimeout(){}
    QC_DATASOURCE->addMethod("getTransactionLockTimeout", (q_method_n_t)Datasource_getTransactionLockTimeout, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // *string Datasource::getUserName(){}
    QC_DATASOURCE->addMethod("getUserName", (q_method_n_t)Datasource_getUserName, false, QC_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // bool Datasource::inTransaction(){}
    QC_DATASOURCE->addMethod("inTransaction", (q_method_n_t)Datasource_inTransaction, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // nothing Datasource::open(){}
    QC_DATASOURCE->addMethod("open", (q_method_n_t)Datasource_open, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing Datasource::reset(){}
    QC_DATASOURCE->addMethod("reset", (q_method_n_t)Datasource_reset, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing Datasource::rollback(){}
    QC_DATASOURCE->addMethod("rollback", (q_method_n_t)Datasource_rollback, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // auto Datasource::select(string sql, ...){}
    QC_DATASOURCE->addMethod("select", (q_method_n_t)Datasource_select_VsVV, false, QC_USES_EXTRA_ARGS, QDOM_DEFAULT, autoTypeInfo, 1, stringTypeInfo, NULL, "sql");

    // *hash Datasource::selectRow(string sql, ...){}
    QC_DATASOURCE->addMethod("selectRow", (q_method_n_t)Datasource_selectRow_VsVV, false, QC_USES_EXTRA_ARGS, QDOM_DEFAULT, hashOrNothingTypeInfo, 1, stringTypeInfo, NULL, "sql");

    // auto Datasource::selectRows(string sql, ...){}
    QC_DATASOURCE->addMethod("selectRows", (q_method_n_t)Datasource_selectRows_VsVV, false, QC_USES_EXTRA_ARGS, QDOM_DEFAULT, autoTypeInfo, 1, stringTypeInfo, NULL, "sql");

    // nothing Datasource::setAutoCommit(bool ac = True){}
    QC_DATASOURCE->addMethod("setAutoCommit", (q_method_n_t)Datasource_setAutoCommit_Vb, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, boolTypeInfo, &True, "ac");

    // nothing Datasource::setDBCharset(string encoding){}
    QC_DATASOURCE->addMethod("setDBCharset", (q_method_n_t)Datasource_setDBCharset_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "encoding");

    // nothing Datasource::setDBEncoding(string encoding){}
    QC_DATASOURCE->addMethod("setDBEncoding", (q_method_n_t)Datasource_setDBEncoding_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "encoding");

    // nothing Datasource::setDBName(string db){}
    QC_DATASOURCE->addMethod("setDBName", (q_method_n_t)Datasource_setDBName_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "db");

    // nothing Datasource::setEventQueue(Qore::Thread::Queue queue, auto arg){}
    QC_DATASOURCE->addMethod("setEventQueue", (q_method_n_t)Datasource_setEventQueue_C5QueueVA, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, QC_QUEUE->getTypeInfo(), NULL, "queue", autoTypeInfo, NULL, "arg");

    // nothing Datasource::setHostName(string host){}
    QC_DATASOURCE->addMethod("setHostName", (q_method_n_t)Datasource_setHostName_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "host");

    // nothing Datasource::setOption(string opt, auto val){}
    QC_DATASOURCE->addMethod("setOption", (q_method_n_t)Datasource_setOption_VsVA, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, NULL, "opt", autoTypeInfo, NULL, "val");

    // nothing Datasource::setPassword(string pass){}
    QC_DATASOURCE->addMethod("setPassword", (q_method_n_t)Datasource_setPassword_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "pass");

    // nothing Datasource::setPort(softint port = 0){}
    QC_DATASOURCE->addMethod("setPort", (q_method_n_t)Datasource_setPort_vi, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, softBigIntTypeInfo, zero(), "port");

    // nothing Datasource::setTransactionLockTimeout(timeout timeout_ms = 0){}
    QC_DATASOURCE->addMethod("setTransactionLockTimeout", (q_method_n_t)Datasource_setTransactionLockTimeout_Vt, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, timeoutTypeInfo, zero(), "timeout_ms");

    // nothing Datasource::setUserName(string user){}
    QC_DATASOURCE->addMethod("setUserName", (q_method_n_t)Datasource_setUserName_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "user");

    // int Datasource::transactionTid(){}
    QC_DATASOURCE->addMethod("transactionTid", (q_method_n_t)Datasource_transactionTid, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // auto Datasource::vexec(string sql, *softlist vargs){}
    QC_DATASOURCE->addMethod("vexec", (q_method_n_t)Datasource_vexec_Vsnl, false, QC_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 2, stringTypeInfo, NULL, "sql", softListOrNothingTypeInfo, NULL, "vargs");

    // auto Datasource::vselect(string sql, *softlist vargs){}
    QC_DATASOURCE->addMethod("vselect", (q_method_n_t)Datasource_vselect_Vsnl, false, QC_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 2, stringTypeInfo, NULL, "sql", softListOrNothingTypeInfo, NULL, "vargs");

    // *hash Datasource::vselectRow(string sql, *softlist vargs){}
    QC_DATASOURCE->addMethod("vselectRow", (q_method_n_t)Datasource_vselectRow_Vsnl, false, QC_NO_FLAGS, QDOM_DEFAULT, hashOrNothingTypeInfo, 2, stringTypeInfo, NULL, "sql", softListOrNothingTypeInfo, NULL, "vargs");

    // auto Datasource::vselectRows(string sql, *softlist vargs){}
    QC_DATASOURCE->addMethod("vselectRows", (q_method_n_t)Datasource_vselectRows_Vsnl, false, QC_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 2, stringTypeInfo, NULL, "sql", softListOrNothingTypeInfo, NULL, "vargs");
# 121 "QC_Datasource.qpp"
# 128 "QC_Datasource.qpp"
# 131 "QC_Datasource.qpp"
# 134 "QC_Datasource.qpp"
# 137 "QC_Datasource.qpp"
# 140 "QC_Datasource.qpp"
# 143 "QC_Datasource.qpp"
# 146 "QC_Datasource.qpp"
// the following have no drivers yet
//const DSInformix = "informix";
//const DSTimesTen = "timesten";
    ns.addConstant("DSDB2", new QoreStringNode("db2"));
    ns.addConstant("DSFreeTDS", new QoreStringNode("freetds"));
    ns.addConstant("DSMSSQL", new QoreStringNode("freetds"));
    ns.addConstant("DSMySQL", new QoreStringNode("mysql"));
    ns.addConstant("DSOracle", new QoreStringNode("oracle"));
    ns.addConstant("DSPGSQL", new QoreStringNode("pgsql"));
    ns.addConstant("DSSQLite3", new QoreStringNode("sqlite3"));
    ns.addConstant("DSSybase", new QoreStringNode("sybase"));
# 154 "QC_Datasource.qpp"
# 161 "QC_Datasource.qpp"
# 164 "QC_Datasource.qpp"
# 167 "QC_Datasource.qpp"
# 170 "QC_Datasource.qpp"
# 173 "QC_Datasource.qpp"
# 176 "QC_Datasource.qpp"
# 179 "QC_Datasource.qpp"
# 182 "QC_Datasource.qpp"
# 185 "QC_Datasource.qpp"
# 188 "QC_Datasource.qpp"
# 191 "QC_Datasource.qpp"
# 194 "QC_Datasource.qpp"
# 197 "QC_Datasource.qpp"
# 200 "QC_Datasource.qpp"
# 203 "QC_Datasource.qpp"
# 206 "QC_Datasource.qpp"
    ns.addConstant("DBI_CAP_AUTORECONNECT", new QoreBigIntNode(DBI_CAP_AUTORECONNECT));
    ns.addConstant("DBI_CAP_BIND_BY_PLACEHOLDER", new QoreBigIntNode(DBI_CAP_BIND_BY_PLACEHOLDER));
    ns.addConstant("DBI_CAP_BIND_BY_VALUE", new QoreBigIntNode(DBI_CAP_BIND_BY_VALUE));
    ns.addConstant("DBI_CAP_CHARSET_SUPPORT", new QoreBigIntNode(DBI_CAP_CHARSET_SUPPORT));
    ns.addConstant("DBI_CAP_EVENTS", new QoreBigIntNode(DBI_CAP_EVENTS));
    ns.addConstant("DBI_CAP_HAS_ARRAY_BIND", new QoreBigIntNode(DBI_CAP_HAS_ARRAY_BIND));
    ns.addConstant("DBI_CAP_HAS_DESCRIBE", new QoreBigIntNode(DBI_CAP_HAS_DESCRIBE));
    ns.addConstant("DBI_CAP_HAS_EXECRAW", new QoreBigIntNode(DBI_CAP_HAS_EXECRAW));
    ns.addConstant("DBI_CAP_HAS_NUMBER_SUPPORT", new QoreBigIntNode(DBI_CAP_HAS_NUMBER_SUPPORT));
    ns.addConstant("DBI_CAP_HAS_OPTION_SUPPORT", new QoreBigIntNode(DBI_CAP_HAS_OPTION_SUPPORT));
    ns.addConstant("DBI_CAP_HAS_RESULTSET_OUTPUT", new QoreBigIntNode(DBI_CAP_HAS_RESULTSET_OUTPUT));
    ns.addConstant("DBI_CAP_HAS_SELECT_ROW", new QoreBigIntNode(DBI_CAP_HAS_SELECT_ROW));
    ns.addConstant("DBI_CAP_HAS_STATEMENT", new QoreBigIntNode(DBI_CAP_HAS_STATEMENT));
    ns.addConstant("DBI_CAP_LOB_SUPPORT", new QoreBigIntNode(DBI_CAP_LOB_SUPPORT));
    ns.addConstant("DBI_CAP_SERVER_TIME_ZONE", new QoreBigIntNode(DBI_CAP_SERVER_TIME_ZONE));
    ns.addConstant("DBI_CAP_STORED_PROCEDURES", new QoreBigIntNode(DBI_CAP_STORED_PROCEDURES));
    ns.addConstant("DBI_CAP_TIME_ZONE_SUPPORT", new QoreBigIntNode(DBI_CAP_TIME_ZONE_SUPPORT));
    ns.addConstant("DBI_CAP_TRANSACTION_MANAGEMENT", new QoreBigIntNode(DBI_CAP_TRANSACTION_MANAGEMENT));

    return QC_DATASOURCE;
}
