/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  QC_DatasourcePool.cpp

  Qore Programming Language

  Copyright (C) 2003 - 2017 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include <qore/DBI.h>

#include "qore/intern/QC_DatasourcePool.h"

extern QoreClass* QC_ABSTRACTDATASOURCE;

#define DP_MIN 3
#define DP_MAX 10

static const char *DSPC_ERR = "DATASOURCEPOOL-CONSTRUCTOR-ERROR";

static void dsp_constructor_hash(const QoreHashNode* opts, QoreObject* self, Queue* queue, const QoreValue& arg, ExceptionSink* xsink) {
   ReferenceHolder<Queue> q(queue, xsink);

   const char *str = check_hash_key(opts, "type", DSPC_ERR, xsink);
   if (*xsink)
      return;

   if (!str) {
      xsink->raiseException(DSPC_ERR, "expecting a string value with the 'type' key giving the driver name");
      return;
   }

   DBIDriver* db_driver = DBI.find(str);
   if (!db_driver) {
      if (!*xsink)
         xsink->raiseException("DATASOURCEPOOL-UNSUPPORTED-DATABASE", "no DBI driver can be found for database type '%s'", str);
      return;
   }

   const char* user = check_hash_key(opts, "user", DSPC_ERR, xsink);
   if (*xsink) return;

   const char* pass = check_hash_key(opts, "pass", DSPC_ERR, xsink);
   if (*xsink) return;

   const char* db = check_hash_key(opts, "db", DSPC_ERR, xsink);
   if (*xsink) return;

   const char* charset = check_hash_key(opts, "charset", DSPC_ERR, xsink);
   if (*xsink) return;

   const char* host = check_hash_key(opts, "host", DSPC_ERR, xsink);
   if (*xsink) return;

   bool found;
   int port = (int)opts->getValueKeyValue("port").getAsBigInt();
   if (port < 0) {
      xsink->raiseException(DSPC_ERR, "port value must be zero (meaning use the default port) or positive (value given: %d)", port);
      return;
   }

   // get options
   int min = 0, max = 0;

   QoreValue p = opts->getValueKeyValue("options");
   const QoreHashNode* opt_hash;
   if (!p.isNothing()) {
      if (p.getType() != NT_HASH) {
         xsink->raiseException(DSPC_ERR, "'options' key is not type 'hash', instead got type '%s'", p.getTypeName());
         return;
      }

      opt_hash = p.get<const QoreHashNode>();

      min = (int)opt_hash->getValueKeyValueExistence("min", found, xsink).getAsBigInt();
      if (*xsink)
         return;
      if (found) {
         if (min < 0) {
            xsink->raiseException(DSPC_ERR, "minimum connections must be > 0 (value given: %d)", min);
            return;
         }
      }

      max = (int)opt_hash->getValueKeyValueExistence("max", found, xsink).getAsBigInt();
      if (*xsink)
         return;
      if (found) {
         if (max < min) {
            xsink->raiseException(DSPC_ERR, "maximum connections must be >= min(%d) (value given: %d)", min, max);
            return;
         }
         if (!max) {
            xsink->raiseException(DSPC_ERR, "maximum connections must be > 0 (value given: %d)", min, max);
            return;
         }
      }
   }
   else
      opt_hash = nullptr;

   if (!min)
      min = DP_MIN;

   if (!max)
      max = DP_MAX;

   SimpleRefHolder<DatasourcePool> ds(new DatasourcePool(xsink, db_driver, user, pass, db, charset, host, min, max, port, opt_hash, q.release(), arg.getReferencedValue()));
   if (!*xsink)
      self->setPrivate(CID_DATASOURCEPOOL, ds.release());
   else
      ds->destructor(xsink);
}
/* Qore class Qore::SQL::DatasourcePool */

qore_classid_t CID_DATASOURCEPOOL;
QoreClass* QC_DATASOURCEPOOL;

// nothing DatasourcePool::beginTransaction(){}
static QoreValue DatasourcePool_beginTransaction(QoreObject* self, DatasourcePool* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 594 "QC_DatasourcePool.qpp"
    ds->beginTransaction(xsink);
    return QoreValue();
}

// nothing DatasourcePool::clearEventQueue(){}
static QoreValue DatasourcePool_clearEventQueue(QoreObject* self, DatasourcePool* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 977 "QC_DatasourcePool.qpp"
    ds->setEventQueue(0, 0, xsink);
    return QoreValue();
}

// nothing DatasourcePool::clearWarningCallback(){}
static QoreValue DatasourcePool_clearWarningCallback(QoreObject* self, DatasourcePool* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 880 "QC_DatasourcePool.qpp"
    ds->clearWarningCallback(xsink);
    return QoreValue();
}

// nothing DatasourcePool::commit(){}
static QoreValue DatasourcePool_commit(QoreObject* self, DatasourcePool* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 326 "QC_DatasourcePool.qpp"
    ds->commit(xsink);
    return QoreValue();
}

// DatasourcePool::constructor(string driver, *string user, *string pass, *string db, *string encoding, *string host, softint min = 3, softint max = 10, softint port = 0, *Qore::Thread::Queue queue, auto arg) {}
static void DatasourcePool_constructor_VsNsNsNsNsNsviviviC5QueueVA(QoreObject* self, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* driver = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* user = get_param_value(args, 1).get<const QoreStringNode>();
    const QoreStringNode* pass = get_param_value(args, 2).get<const QoreStringNode>();
    const QoreStringNode* db = get_param_value(args, 3).get<const QoreStringNode>();
    const QoreStringNode* encoding = get_param_value(args, 4).get<const QoreStringNode>();
    const QoreStringNode* host = get_param_value(args, 5).get<const QoreStringNode>();
    int64 min = HARD_QORE_VALUE_INT(args, 6);
    int64 max = HARD_QORE_VALUE_INT(args, 7);
    int64 port = HARD_QORE_VALUE_INT(args, 8);
    HARD_QORE_VALUE_OBJ_OR_NOTHING_DATA(queue, Queue, args, 9, CID_QUEUE, xsink);
    if (*xsink)
        return;
    QoreValue arg = get_param_value(args, 10);
# 198 "QC_DatasourcePool.qpp"
if (!port && min == 3 && max == 10 && driver->find(':') != -1) {
      ReferenceHolder<QoreHashNode> h(parseDatasource(driver->getBuffer(), xsink), xsink);
      if (!h) {
         assert(*xsink);
         return;
      }
      dsp_constructor_hash(*h, self, queue, arg, xsink);
      return;
   }

   ReferenceHolder<Queue> q(queue, xsink);

   DBIDriver *db_driver = DBI.find(driver->getBuffer());
   if (!db_driver) {
      xsink->raiseException("DATASOURCEPOOL-UNSUPPORTED-DATABASE", "no DBI driver can be found for database type '%s'", driver->getBuffer());
      return;
   }

   if (min <= 0) {
      xsink->raiseException(DSPC_ERR, "minimum connections must be > 0 (value given: " QLLD ")", min);
      return;
   }

   if (max < min) {
      xsink->raiseException(DSPC_ERR, "maximum connections must be >= min(%d) (value given: " QLLD ")", min, max);
      return;
   }

   if (port < 0) {
      xsink->raiseException(DSPC_ERR, "port value must be zero (meaning use the default port) or positive (value given: " QLLD ")", port);
      return;
   }

   SimpleRefHolder<DatasourcePool> ds(new DatasourcePool(xsink, db_driver, user ? user->getBuffer() : 0, pass ? pass->getBuffer() : 0,
							 db ? db->getBuffer() : 0, encoding ? encoding->getBuffer() : 0, host ? host->getBuffer() : 0,
							 min, max, port, 0, q.release(), arg.getReferencedValue()));
   if (!*xsink)
      self->setPrivate(CID_DATASOURCEPOOL, ds.release());
   else
      ds->destructor(xsink);
}

// DatasourcePool::constructor(string desc, *Qore::Thread::Queue queue, auto arg) {}
static void DatasourcePool_constructor_VsC5QueueVA(QoreObject* self, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* desc = HARD_QORE_VALUE_STRING(args, 0);
    HARD_QORE_VALUE_OBJ_OR_NOTHING_DATA(queue, Queue, args, 1, CID_QUEUE, xsink);
    if (*xsink)
        return;
    QoreValue arg = get_param_value(args, 2);
# 257 "QC_DatasourcePool.qpp"
ReferenceHolder<Queue> q(queue, xsink);
   ReferenceHolder<QoreHashNode> h(parseDatasource(desc->getBuffer(), xsink), xsink);
   if (!h) {
      assert(*xsink);
      return;
   }
   dsp_constructor_hash(*h, self, q.release(), arg, xsink);
}

// DatasourcePool::constructor(hash opts, *Qore::Thread::Queue queue, auto arg) {}
static void DatasourcePool_constructor_VhC5QueueVA(QoreObject* self, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreHashNode* opts = HARD_QORE_VALUE_HASH(args, 0);
    HARD_QORE_VALUE_OBJ_OR_NOTHING_DATA(queue, Queue, args, 1, CID_QUEUE, xsink);
    if (*xsink)
        return;
    QoreValue arg = get_param_value(args, 2);
# 289 "QC_DatasourcePool.qpp"
dsp_constructor_hash(opts, self, queue, arg, xsink);
}

// DatasourcePool::copy() {}
static void DatasourcePool_copy(QoreObject* self, QoreObject* old, DatasourcePool* ds, ExceptionSink* xsink) {
# 314 "QC_DatasourcePool.qpp"
self->setPrivate(CID_DATASOURCEPOOL, new DatasourcePool(*ds, xsink));
}

// bool DatasourcePool::currentThreadInTransaction(){}
static QoreValue DatasourcePool_currentThreadInTransaction(QoreObject* self, DatasourcePool* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 817 "QC_DatasourcePool.qpp"
    return ds->currentThreadInTransaction();
}

// DatasourcePool::destructor() {}
static void DatasourcePool_destructor(QoreObject* self, DatasourcePool* ds, ExceptionSink* xsink) {
# 302 "QC_DatasourcePool.qpp"
ds->destructor(xsink);
   ds->deref();
}

// auto DatasourcePool::exec(string sql, ...){}
static QoreValue DatasourcePool_exec_VsVV(QoreObject* self, DatasourcePool* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* sql = HARD_QORE_VALUE_STRING(args, 0);
# 357 "QC_DatasourcePool.qpp"
    ReferenceHolder<QoreListNode> vargs(args->size() > 1 ? args->getOldList(1) : 0, xsink);
   return ds->exec(sql, *vargs, xsink);
}

// auto DatasourcePool::execRaw(string sql){}
static QoreValue DatasourcePool_execRaw_Vs(QoreObject* self, DatasourcePool* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* sql = HARD_QORE_VALUE_STRING(args, 0);
# 400 "QC_DatasourcePool.qpp"
    return ds->execRaw(sql, xsink);
}

// int DatasourcePool::getCapabilities(){}
static QoreValue DatasourcePool_getCapabilities(QoreObject* self, DatasourcePool* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 993 "QC_DatasourcePool.qpp"
    return ds->getCapabilities();
}

// list DatasourcePool::getCapabilityList(){}
static QoreValue DatasourcePool_getCapabilityList(QoreObject* self, DatasourcePool* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1009 "QC_DatasourcePool.qpp"
    return ds->getCapabilityList();
}

// auto DatasourcePool::getClientVersion(){}
static QoreValue DatasourcePool_getClientVersion(QoreObject* self, DatasourcePool* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 791 "QC_DatasourcePool.qpp"
    return ds->getClientVersion(xsink);
}

// hash DatasourcePool::getConfigHash(){}
static QoreValue DatasourcePool_getConfigHash(QoreObject* self, DatasourcePool* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 854 "QC_DatasourcePool.qpp"
    return ds->getConfigHash(xsink);
}

// string DatasourcePool::getConfigString(){}
static QoreValue DatasourcePool_getConfigString(QoreObject* self, DatasourcePool* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 868 "QC_DatasourcePool.qpp"
    return ds->getConfigString(xsink);
}

// *string DatasourcePool::getDBCharset(){}
static QoreValue DatasourcePool_getDBCharset(QoreObject* self, DatasourcePool* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 644 "QC_DatasourcePool.qpp"
    return ds->getPendingDBEncoding();
}

// string DatasourcePool::getDBEncoding(){}
static QoreValue DatasourcePool_getDBEncoding(QoreObject* self, DatasourcePool* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 658 "QC_DatasourcePool.qpp"
    return ds->getPendingDBEncoding();
}

// *string DatasourcePool::getDBName(){}
static QoreValue DatasourcePool_getDBName(QoreObject* self, DatasourcePool* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 630 "QC_DatasourcePool.qpp"
    return ds->getPendingDBName();
}

// string DatasourcePool::getDriverName(){}
static QoreValue DatasourcePool_getDriverName(QoreObject* self, DatasourcePool* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 723 "QC_DatasourcePool.qpp"
    return new QoreStringNode(ds->getDriverName());
}

// int DatasourcePool::getErrorTimeout(){}
static QoreValue DatasourcePool_getErrorTimeout(QoreObject* self, DatasourcePool* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 951 "QC_DatasourcePool.qpp"
    return ds->getErrorTimeout();
}

// *string DatasourcePool::getHostName(){}
static QoreValue DatasourcePool_getHostName(QoreObject* self, DatasourcePool* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 698 "QC_DatasourcePool.qpp"
    return ds->getPendingHostName();
}

// int DatasourcePool::getMaximum(){}
static QoreValue DatasourcePool_getMaximum(QoreObject* self, DatasourcePool* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 750 "QC_DatasourcePool.qpp"
    return ds->getMax();
}

// int DatasourcePool::getMinimum(){}
static QoreValue DatasourcePool_getMinimum(QoreObject* self, DatasourcePool* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 737 "QC_DatasourcePool.qpp"
    return ds->getMin();
}

// string DatasourcePool::getOSCharset(){}
static QoreValue DatasourcePool_getOSCharset(QoreObject* self, DatasourcePool* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 672 "QC_DatasourcePool.qpp"
    const QoreEncoding *enc = ds->getQoreEncoding();
   return new QoreStringNode(enc ? enc->getCode() : "(unknown)");
}

// *string DatasourcePool::getOSEncoding(){}
static QoreValue DatasourcePool_getOSEncoding(QoreObject* self, DatasourcePool* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 685 "QC_DatasourcePool.qpp"
    const QoreEncoding *enc = ds->getQoreEncoding();
   return enc ? new QoreStringNode(enc->getCode()) : 0;
}

// auto DatasourcePool::getOption(string opt){}
static QoreValue DatasourcePool_getOption_Vs(QoreObject* self, DatasourcePool* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* opt = HARD_QORE_VALUE_STRING(args, 0);
# 840 "QC_DatasourcePool.qpp"
    return ds->getOption(opt->getBuffer(), xsink);
}

// hash DatasourcePool::getOptionHash(){}
static QoreValue DatasourcePool_getOptionHash(QoreObject* self, DatasourcePool* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 829 "QC_DatasourcePool.qpp"
    return ds->getOptionHash();
}

// *string DatasourcePool::getPassword(){}
static QoreValue DatasourcePool_getPassword(QoreObject* self, DatasourcePool* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 618 "QC_DatasourcePool.qpp"
    return ds->getPendingPassword();
}

// *int DatasourcePool::getPort(){}
static QoreValue DatasourcePool_getPort(QoreObject* self, DatasourcePool* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 710 "QC_DatasourcePool.qpp"
    int port = ds->getPendingPort();
   return port ? new QoreBigIntNode(port) : 0;
}

// auto DatasourcePool::getServerVersion(){}
static QoreValue DatasourcePool_getServerVersion(QoreObject* self, DatasourcePool* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 777 "QC_DatasourcePool.qpp"
    return ds->getServerVersion(xsink);
}

// *hash DatasourcePool::getUsageInfo(){}
static QoreValue DatasourcePool_getUsageInfo(QoreObject* self, DatasourcePool* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 923 "QC_DatasourcePool.qpp"
    return ds->getUsageInfo();
}

// *string DatasourcePool::getUserName(){}
static QoreValue DatasourcePool_getUserName(QoreObject* self, DatasourcePool* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 606 "QC_DatasourcePool.qpp"
    return ds->getPendingUsername();
}

// bool DatasourcePool::inTransaction(){}
static QoreValue DatasourcePool_inTransaction(QoreObject* self, DatasourcePool* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 803 "QC_DatasourcePool.qpp"
    return ds->inTransaction();
}

// nothing DatasourcePool::rollback(){}
static QoreValue DatasourcePool_rollback(QoreObject* self, DatasourcePool* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 338 "QC_DatasourcePool.qpp"
    ds->rollback(xsink);
    return QoreValue();
}

// auto DatasourcePool::select(string sql, ...){}
static QoreValue DatasourcePool_select_VsVV(QoreObject* self, DatasourcePool* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* sql = HARD_QORE_VALUE_STRING(args, 0);
# 436 "QC_DatasourcePool.qpp"
    ReferenceHolder<QoreListNode> vargs(args->size() > 1 ? args->getOldList(1) : 0, xsink);
   return ds->select(sql, *vargs, xsink);
}

// auto DatasourcePool::selectRow(string sql, ...){}
static QoreValue DatasourcePool_selectRow_VsVV(QoreObject* self, DatasourcePool* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* sql = HARD_QORE_VALUE_STRING(args, 0);
# 462 "QC_DatasourcePool.qpp"
    ReferenceHolder<QoreListNode> vargs(args->size() > 1 ? args->getOldList(1) : 0, xsink);
   return ds->selectRow(sql, *vargs, xsink);
}

// auto DatasourcePool::selectRows(string sql, ...){}
static QoreValue DatasourcePool_selectRows_VsVV(QoreObject* self, DatasourcePool* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* sql = HARD_QORE_VALUE_STRING(args, 0);
# 490 "QC_DatasourcePool.qpp"
    ReferenceHolder<QoreListNode> vargs(args->size() > 1 ? args->getOldList(1) : 0, xsink);
   return ds->selectRows(sql, *vargs, xsink);
}

// nothing DatasourcePool::setErrorTimeout(timeout ts){}
static QoreValue DatasourcePool_setErrorTimeout_Vt(QoreObject* self, DatasourcePool* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 ts = HARD_QORE_VALUE_INT(args, 0);
# 935 "QC_DatasourcePool.qpp"
    if (ts < 0)
      ts = 0;
   ds->setErrorTimeout(ts);
    return QoreValue();
}

// nothing DatasourcePool::setEventQueue(Qore::Thread::Queue queue, auto arg){}
static QoreValue DatasourcePool_setEventQueue_C5QueueVA(QoreObject* self, DatasourcePool* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(queue, Queue, args, 0, CID_QUEUE, "DatasourcePool::setEventQueue()", "Qore::Thread::Queue", xsink);
    if (*xsink)
        return 0;
    QoreValue arg = get_param_value(args, 1);
# 964 "QC_DatasourcePool.qpp"
    ReferenceHolder<Queue> q(queue, xsink);

   if (queue->getMax() != -1)
      return xsink->raiseException("QUEUE-ERROR", "the Queue object passed as a maximum size of %d entr%s, which could cause internal I/O to freeze in the Datasource object if used; use a Queue object with no maximum size for datasource event monitoring", queue->getMax(), queue->getMax() == 1 ? "y" : "ies");

   ds->setEventQueue(q.release(), arg.getReferencedValue(), xsink);
    return QoreValue();
}

// nothing DatasourcePool::setWarningCallback(timeout ms, code callback, auto arg){}
static QoreValue DatasourcePool_setWarningCallback_VtVcVA(QoreObject* self, DatasourcePool* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 ms = HARD_QORE_VALUE_INT(args, 0);
    const ResolvedCallReferenceNode* callback = get_param_value(args, 1).get<const ResolvedCallReferenceNode>();
    QoreValue arg = get_param_value(args, 2);
# 901 "QC_DatasourcePool.qpp"
    ds->setWarningCallback(ms, callback->refRefSelf(), arg.getReferencedValue(), xsink);
    return QoreValue();
}

// string DatasourcePool::toString(){}
static QoreValue DatasourcePool_toString(QoreObject* self, DatasourcePool* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 763 "QC_DatasourcePool.qpp"
    return ds->toString();
}

// auto DatasourcePool::vexec(string sql, *softlist vargs){}
static QoreValue DatasourcePool_vexec_Vsnl(QoreObject* self, DatasourcePool* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* sql = HARD_QORE_VALUE_STRING(args, 0);
    const QoreListNode* vargs = get_param_value(args, 1).get<const QoreListNode>();
# 379 "QC_DatasourcePool.qpp"
    return ds->exec(sql, vargs, xsink);
}

// auto DatasourcePool::vselect(string sql, *softlist vargs){}
static QoreValue DatasourcePool_vselect_Vsnl(QoreObject* self, DatasourcePool* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* sql = HARD_QORE_VALUE_STRING(args, 0);
    const QoreListNode* vargs = get_param_value(args, 1).get<const QoreListNode>();
# 524 "QC_DatasourcePool.qpp"
    return ds->select(sql, vargs, xsink);
}

// auto DatasourcePool::vselectRow(string sql, *softlist vargs){}
static QoreValue DatasourcePool_vselectRow_Vsnl(QoreObject* self, DatasourcePool* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* sql = HARD_QORE_VALUE_STRING(args, 0);
    const QoreListNode* vargs = get_param_value(args, 1).get<const QoreListNode>();
# 549 "QC_DatasourcePool.qpp"
    return ds->selectRow(sql, vargs, xsink);
}

// auto DatasourcePool::vselectRows(string sql, *softlist vargs){}
static QoreValue DatasourcePool_vselectRows_Vsnl(QoreObject* self, DatasourcePool* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* sql = HARD_QORE_VALUE_STRING(args, 0);
    const QoreListNode* vargs = get_param_value(args, 1).get<const QoreListNode>();
# 578 "QC_DatasourcePool.qpp"
    return ds->selectRows(sql, vargs, xsink);
}

DLLLOCAL void preinitDatasourcePoolClass() {
    QC_DATASOURCEPOOL = new QoreClass("DatasourcePool", QDOM_DATABASE);
    CID_DATASOURCEPOOL = QC_DATASOURCEPOOL->getID();
    QC_DATASOURCEPOOL->setSystem();
}

DLLLOCAL QoreClass* initDatasourcePoolClass(QoreNamespace& ns) {
    if (!QC_DATASOURCEPOOL)
        preinitDatasourcePoolClass();

    // set parent class
    assert(QC_ABSTRACTDATASOURCE);
    QC_DATASOURCEPOOL->addBuiltinVirtualBaseClass(QC_ABSTRACTDATASOURCE);

    // nothing DatasourcePool::beginTransaction(){}
    QC_DATASOURCEPOOL->addMethod("beginTransaction", (q_method_n_t)DatasourcePool_beginTransaction, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing DatasourcePool::clearEventQueue(){}
    QC_DATASOURCEPOOL->addMethod("clearEventQueue", (q_method_n_t)DatasourcePool_clearEventQueue, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing DatasourcePool::clearWarningCallback(){}
    QC_DATASOURCEPOOL->addMethod("clearWarningCallback", (q_method_n_t)DatasourcePool_clearWarningCallback, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing DatasourcePool::commit(){}
    QC_DATASOURCEPOOL->addMethod("commit", (q_method_n_t)DatasourcePool_commit, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // DatasourcePool::constructor(string driver, *string user, *string pass, *string db, *string encoding, *string host, softint min = 3, softint max = 10, softint port = 0, *Qore::Thread::Queue queue, auto arg) {}
    QC_DATASOURCEPOOL->addConstructor(DatasourcePool_constructor_VsNsNsNsNsNsviviviC5QueueVA, false, QC_NO_FLAGS, QDOM_DEFAULT, 11, stringTypeInfo, NULL, "driver", stringOrNothingTypeInfo, NULL, "user", stringOrNothingTypeInfo, NULL, "pass", stringOrNothingTypeInfo, NULL, "db", stringOrNothingTypeInfo, NULL, "encoding", stringOrNothingTypeInfo, NULL, "host", softBigIntTypeInfo, new QoreBigIntNode(3), "min", softBigIntTypeInfo, new QoreBigIntNode(10), "max", softBigIntTypeInfo, zero(), "port", QC_QUEUE->getOrNothingTypeInfo(), NULL, "queue", autoTypeInfo, NULL, "arg");

    // DatasourcePool::constructor(string desc, *Qore::Thread::Queue queue, auto arg) {}
    QC_DATASOURCEPOOL->addConstructor(DatasourcePool_constructor_VsC5QueueVA, false, QC_NO_FLAGS, QDOM_DEFAULT, 3, stringTypeInfo, NULL, "desc", QC_QUEUE->getOrNothingTypeInfo(), NULL, "queue", autoTypeInfo, NULL, "arg");

    // DatasourcePool::constructor(hash opts, *Qore::Thread::Queue queue, auto arg) {}
    QC_DATASOURCEPOOL->addConstructor(DatasourcePool_constructor_VhC5QueueVA, false, QC_NO_FLAGS, QDOM_DEFAULT, 3, hashTypeInfo, NULL, "opts", QC_QUEUE->getOrNothingTypeInfo(), NULL, "queue", autoTypeInfo, NULL, "arg");

    // DatasourcePool::copy() {}
    QC_DATASOURCEPOOL->setCopy((q_copy_t)DatasourcePool_copy);

    // bool DatasourcePool::currentThreadInTransaction(){}
    QC_DATASOURCEPOOL->addMethod("currentThreadInTransaction", (q_method_n_t)DatasourcePool_currentThreadInTransaction, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // DatasourcePool::destructor() {}
    QC_DATASOURCEPOOL->setDestructor((q_destructor_t)DatasourcePool_destructor);

    // auto DatasourcePool::exec(string sql, ...){}
    QC_DATASOURCEPOOL->addMethod("exec", (q_method_n_t)DatasourcePool_exec_VsVV, false, QC_USES_EXTRA_ARGS, QDOM_DEFAULT, autoTypeInfo, 1, stringTypeInfo, NULL, "sql");

    // auto DatasourcePool::execRaw(string sql){}
    QC_DATASOURCEPOOL->addMethod("execRaw", (q_method_n_t)DatasourcePool_execRaw_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 1, stringTypeInfo, NULL, "sql");

    // int DatasourcePool::getCapabilities(){}
    QC_DATASOURCEPOOL->addMethod("getCapabilities", (q_method_n_t)DatasourcePool_getCapabilities, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // list DatasourcePool::getCapabilityList(){}
    QC_DATASOURCEPOOL->addMethod("getCapabilityList", (q_method_n_t)DatasourcePool_getCapabilityList, false, QC_CONSTANT, QDOM_DEFAULT, listTypeInfo);

    // auto DatasourcePool::getClientVersion(){}
    QC_DATASOURCEPOOL->addMethod("getClientVersion", (q_method_n_t)DatasourcePool_getClientVersion, false, QC_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo);

    // hash DatasourcePool::getConfigHash(){}
    QC_DATASOURCEPOOL->addMethod("getConfigHash", (q_method_n_t)DatasourcePool_getConfigHash, false, QC_CONSTANT, QDOM_DEFAULT, hashTypeInfo);

    // string DatasourcePool::getConfigString(){}
    QC_DATASOURCEPOOL->addMethod("getConfigString", (q_method_n_t)DatasourcePool_getConfigString, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // *string DatasourcePool::getDBCharset(){}
    QC_DATASOURCEPOOL->addMethod("getDBCharset", (q_method_n_t)DatasourcePool_getDBCharset, false, QC_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // string DatasourcePool::getDBEncoding(){}
    QC_DATASOURCEPOOL->addMethod("getDBEncoding", (q_method_n_t)DatasourcePool_getDBEncoding, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // *string DatasourcePool::getDBName(){}
    QC_DATASOURCEPOOL->addMethod("getDBName", (q_method_n_t)DatasourcePool_getDBName, false, QC_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // string DatasourcePool::getDriverName(){}
    QC_DATASOURCEPOOL->addMethod("getDriverName", (q_method_n_t)DatasourcePool_getDriverName, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // int DatasourcePool::getErrorTimeout(){}
    QC_DATASOURCEPOOL->addMethod("getErrorTimeout", (q_method_n_t)DatasourcePool_getErrorTimeout, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // *string DatasourcePool::getHostName(){}
    QC_DATASOURCEPOOL->addMethod("getHostName", (q_method_n_t)DatasourcePool_getHostName, false, QC_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // int DatasourcePool::getMaximum(){}
    QC_DATASOURCEPOOL->addMethod("getMaximum", (q_method_n_t)DatasourcePool_getMaximum, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // int DatasourcePool::getMinimum(){}
    QC_DATASOURCEPOOL->addMethod("getMinimum", (q_method_n_t)DatasourcePool_getMinimum, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // string DatasourcePool::getOSCharset(){}
    QC_DATASOURCEPOOL->addMethod("getOSCharset", (q_method_n_t)DatasourcePool_getOSCharset, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // *string DatasourcePool::getOSEncoding(){}
    QC_DATASOURCEPOOL->addMethod("getOSEncoding", (q_method_n_t)DatasourcePool_getOSEncoding, false, QC_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // auto DatasourcePool::getOption(string opt){}
    QC_DATASOURCEPOOL->addMethod("getOption", (q_method_n_t)DatasourcePool_getOption_Vs, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, autoTypeInfo, 1, stringTypeInfo, NULL, "opt");

    // hash DatasourcePool::getOptionHash(){}
    QC_DATASOURCEPOOL->addMethod("getOptionHash", (q_method_n_t)DatasourcePool_getOptionHash, false, QC_CONSTANT, QDOM_DEFAULT, hashTypeInfo);

    // *string DatasourcePool::getPassword(){}
    QC_DATASOURCEPOOL->addMethod("getPassword", (q_method_n_t)DatasourcePool_getPassword, false, QC_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // *int DatasourcePool::getPort(){}
    QC_DATASOURCEPOOL->addMethod("getPort", (q_method_n_t)DatasourcePool_getPort, false, QC_CONSTANT, QDOM_DEFAULT, bigIntOrNothingTypeInfo);

    // auto DatasourcePool::getServerVersion(){}
    QC_DATASOURCEPOOL->addMethod("getServerVersion", (q_method_n_t)DatasourcePool_getServerVersion, false, QC_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo);

    // *hash DatasourcePool::getUsageInfo(){}
    QC_DATASOURCEPOOL->addMethod("getUsageInfo", (q_method_n_t)DatasourcePool_getUsageInfo, false, QC_CONSTANT, QDOM_DEFAULT, hashOrNothingTypeInfo);

    // *string DatasourcePool::getUserName(){}
    QC_DATASOURCEPOOL->addMethod("getUserName", (q_method_n_t)DatasourcePool_getUserName, false, QC_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // bool DatasourcePool::inTransaction(){}
    QC_DATASOURCEPOOL->addMethod("inTransaction", (q_method_n_t)DatasourcePool_inTransaction, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // nothing DatasourcePool::rollback(){}
    QC_DATASOURCEPOOL->addMethod("rollback", (q_method_n_t)DatasourcePool_rollback, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // auto DatasourcePool::select(string sql, ...){}
    QC_DATASOURCEPOOL->addMethod("select", (q_method_n_t)DatasourcePool_select_VsVV, false, QC_USES_EXTRA_ARGS, QDOM_DEFAULT, autoTypeInfo, 1, stringTypeInfo, NULL, "sql");

    // auto DatasourcePool::selectRow(string sql, ...){}
    QC_DATASOURCEPOOL->addMethod("selectRow", (q_method_n_t)DatasourcePool_selectRow_VsVV, false, QC_USES_EXTRA_ARGS, QDOM_DEFAULT, autoTypeInfo, 1, stringTypeInfo, NULL, "sql");

    // auto DatasourcePool::selectRows(string sql, ...){}
    QC_DATASOURCEPOOL->addMethod("selectRows", (q_method_n_t)DatasourcePool_selectRows_VsVV, false, QC_USES_EXTRA_ARGS, QDOM_DEFAULT, autoTypeInfo, 1, stringTypeInfo, NULL, "sql");

    // nothing DatasourcePool::setErrorTimeout(timeout ts){}
    QC_DATASOURCEPOOL->addMethod("setErrorTimeout", (q_method_n_t)DatasourcePool_setErrorTimeout_Vt, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, timeoutTypeInfo, NULL, "ts");

    // nothing DatasourcePool::setEventQueue(Qore::Thread::Queue queue, auto arg){}
    QC_DATASOURCEPOOL->addMethod("setEventQueue", (q_method_n_t)DatasourcePool_setEventQueue_C5QueueVA, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, QC_QUEUE->getTypeInfo(), NULL, "queue", autoTypeInfo, NULL, "arg");

    // nothing DatasourcePool::setWarningCallback(timeout ms, code callback, auto arg){}
    QC_DATASOURCEPOOL->addMethod("setWarningCallback", (q_method_n_t)DatasourcePool_setWarningCallback_VtVcVA, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 3, timeoutTypeInfo, NULL, "ms", codeTypeInfo, NULL, "callback", autoTypeInfo, NULL, "arg");

    // string DatasourcePool::toString(){}
    QC_DATASOURCEPOOL->addMethod("toString", (q_method_n_t)DatasourcePool_toString, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // auto DatasourcePool::vexec(string sql, *softlist vargs){}
    QC_DATASOURCEPOOL->addMethod("vexec", (q_method_n_t)DatasourcePool_vexec_Vsnl, false, QC_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 2, stringTypeInfo, NULL, "sql", softListOrNothingTypeInfo, NULL, "vargs");

    // auto DatasourcePool::vselect(string sql, *softlist vargs){}
    QC_DATASOURCEPOOL->addMethod("vselect", (q_method_n_t)DatasourcePool_vselect_Vsnl, false, QC_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 2, stringTypeInfo, NULL, "sql", softListOrNothingTypeInfo, NULL, "vargs");

    // auto DatasourcePool::vselectRow(string sql, *softlist vargs){}
    QC_DATASOURCEPOOL->addMethod("vselectRow", (q_method_n_t)DatasourcePool_vselectRow_Vsnl, false, QC_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 2, stringTypeInfo, NULL, "sql", softListOrNothingTypeInfo, NULL, "vargs");

    // auto DatasourcePool::vselectRows(string sql, *softlist vargs){}
    QC_DATASOURCEPOOL->addMethod("vselectRows", (q_method_n_t)DatasourcePool_vselectRows_Vsnl, false, QC_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 2, stringTypeInfo, NULL, "sql", softListOrNothingTypeInfo, NULL, "vargs");

    return QC_DATASOURCEPOOL;
}
