/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  QC_Dir.qpp

  Qore Programming Language

  Copyright (C) 2003 - 2017 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/QC_Dir.h"
#include "qore/intern/QC_File.h"
#include "qore/intern/QoreHashNodeIntern.h"
#include "qore/intern/qore_qd_private.h"

#include <sys/types.h>
#include <errno.h>

QoreListNode* qore_qd_private::list(ExceptionSink *xsink, int stat_filter, const QoreString *regex, int regex_options, bool full) const {
    AutoLocker al(m);

    if (dirname.empty()) {
        xsink->raiseException("DIR-READ-ERROR", "cannot list directory; no directory is set");
        return nullptr;
    }

    SimpleRefHolder<QoreRegex> re(0);

    if (regex) {
        re = new QoreRegex(*regex, regex_options, xsink);
        if (*xsink)
            return nullptr;
    }

    // avoid memory leaks...
    ReferenceHolder<QoreListNode> lst(new QoreListNode, xsink);

    DIR *dptr = opendir(dirname.c_str());
    if (!dptr) {
        xsink->raiseErrnoException("DIR-READ-FAILURE", errno, "error opening directory '%s' for reading", dirname.c_str());
        return nullptr;
    }
    ON_BLOCK_EXIT(closedir, dptr);

    struct dirent *de;
    while ((de = readdir(dptr))) {
        if (!strcmp(de->d_name, ".") || !strcmp(de->d_name, ".."))
            continue;

        // if there is a regular expression, see if the name matches
        if (regex) {
            QoreString targ(de->d_name, enc);
            bool b = re->exec(&targ, xsink);
            if (*xsink)
                return nullptr;
            if (!b)
                continue;
        }

        // if we are filtering out directories, then we have to stat the file
        if (full || stat_filter != -1) {
            QoreString fname(dirname);
            fname.concat(QORE_DIR_SEP);
            fname.concat(de->d_name);
            struct stat buf;
            if (!full) {
                if (stat(fname.getBuffer(), buf, xsink))
                  return 0;

                if (!(buf.st_mode & stat_filter))
                  continue;

                lst->push(new QoreStringNode(de->d_name, enc));
                continue;
            }
#ifdef HAVE_LSTAT
            if (lstat(fname.getBuffer(), buf, xsink))
                return 0;

            SimpleRefHolder<QoreStringNode> lpath;

            if (S_ISLNK(buf.st_mode)) {
                {
                    char lbuf[QORE_PATH_MAX + 1];
                    qore_offset_t len = readlink(fname.getBuffer(), lbuf, QORE_PATH_MAX);
                    if (len < 0) {
                        xsink->raiseErrnoException("DIR-READ-FAILURE", errno, "readlink('%s') failed", fname.getBuffer());
                        return 0;
                    }
                    assert(len <= QORE_PATH_MAX);
                    lbuf[len] = '\0';
                    lpath = new QoreStringNode(lbuf);
                }

                if (stat(fname.getBuffer(), buf, xsink))
                    return 0;
            }
#else
            if (stat(fname.getBuffer(), buf, xsink))
                return 0;
#endif
            if (stat_filter != -1 && !(buf.st_mode & stat_filter))
                continue;
            QoreHashNode* h = stat_to_hash(buf, hashdeclDirStatInfo);
            qore_hash_private::get(*h)->setKeyValueIntern("name", new QoreStringNode(de->d_name, enc));
#ifdef HAVE_LSTAT
            if (*lpath)
                qore_hash_private::get(*h)->setKeyValueIntern("link", lpath.release());
#endif
            lst->push(h);
            continue;
        }

        // not full, no filter
        lst->push(new QoreStringNode(de->d_name, enc));
        continue;
    }

    return lst.release();
}

QoreHashNode* qore_qd_private::hstat(ExceptionSink *xsink) const {
    AutoLocker al(m);

    if (dirname.empty()) {
        xsink->raiseException("DIR-HSTAT-ERROR", "cannot stat; no directory is set");
        return nullptr;
    }

    struct stat sbuf;
    if (::stat(dirname.c_str(), &sbuf)) {
        xsink->raiseErrnoException("DIR-HSTAT-FAILURE", errno, "stat() call failed on '%s'", dirname.c_str());
        return nullptr;
    }

    return stat_to_hash(sbuf);
}
TypedHashDecl* init_hashdecl_DirStatInfo(QoreNamespace& ns) {
    TypedHashDecl* hd = new TypedHashDecl("DirStatInfo");
    hd->addMember("atime", dateTypeInfo, QoreValue());
    hd->addMember("blksize", bigIntTypeInfo, QoreValue());
    hd->addMember("blocks", bigIntTypeInfo, QoreValue());
    hd->addMember("ctime", dateTypeInfo, QoreValue());
    hd->addMember("dev", bigIntTypeInfo, QoreValue());
    hd->addMember("gid", bigIntTypeInfo, QoreValue());
    hd->addMember("inode", bigIntTypeInfo, QoreValue());
    hd->addMember("link", stringOrNothingTypeInfo, QoreValue());
    hd->addMember("mode", bigIntTypeInfo, QoreValue());
    hd->addMember("mtime", dateTypeInfo, QoreValue());
    hd->addMember("name", stringTypeInfo, QoreValue());
    hd->addMember("nlink", bigIntTypeInfo, QoreValue());
    hd->addMember("perm", stringTypeInfo, QoreValue());
    hd->addMember("rdev", bigIntTypeInfo, QoreValue());
    hd->addMember("size", bigIntTypeInfo, QoreValue());
    hd->addMember("type", stringTypeInfo, QoreValue());
    hd->addMember("uid", bigIntTypeInfo, QoreValue());
    ns.addSystemHashDecl(hd);
    return hd;
}

/* Qore class Qore::Dir */

qore_classid_t CID_DIR;
QoreClass* QC_DIR;

// bool Dir::chdir(string path){}
static QoreValue Dir_chdir_Vs(QoreObject* self, Dir* d, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 262 "QC_Dir.qpp"
    return !d->chdir(path->getBuffer(), xsink);
}

// nothing Dir::chgrp(int gid){}
static QoreValue Dir_chgrp_Vi(QoreObject* self, Dir* d, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 gid = HARD_QORE_VALUE_INT(args, 0);
# 435 "QC_Dir.qpp"
    #ifdef HAVE_CHOWN
   d->chown((uid_t)-1, (gid_t)gid, xsink);
   return QoreValue();
#else
   return missing_method_error("Dir::chgrp", "UNIX_FILEMGT", xsink);
#endif
    return QoreValue();
}

// nothing Dir::chgrp(string groupname){}
static QoreValue Dir_chgrp_Vs(QoreObject* self, Dir* d, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* groupname = HARD_QORE_VALUE_STRING(args, 0);
# 467 "QC_Dir.qpp"
    #ifdef HAVE_CHOWN
   gid_t gid;

   // Try getting GID for name
   int rc = q_gname2gid(groupname->getBuffer(), gid);
   if (rc)
      xsink->raiseException("DIR-CHGRP-PARAMETER-ERROR", "no groupid found for group '%s'", groupname->getBuffer());
   else
      d->chown((uid_t)-1, gid, xsink);
   return QoreValue();
#else
   return missing_method_error("Dir::chgrp", "UNIX_FILEMGT", xsink);
#endif
    return QoreValue();
}

// nothing Dir::chmod(softint mode){}
static QoreValue Dir_chmod_vi(QoreObject* self, Dir* d, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 mode = HARD_QORE_VALUE_INT(args, 0);
# 339 "QC_Dir.qpp"
    d->chmod((int)mode, xsink);
    return QoreValue();
}

// nothing Dir::chown(int uid){}
static QoreValue Dir_chown_Vi(QoreObject* self, Dir* d, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 uid = HARD_QORE_VALUE_INT(args, 0);
# 365 "QC_Dir.qpp"
    #ifdef HAVE_CHOWN
    d->chown((uid_t)uid, (gid_t)-1, xsink);
    return QoreValue();
#else
   return missing_method_error("Dir::chown", "UNIX_FILEMGT", xsink);
#endif
    return QoreValue();
}

// nothing Dir::chown(string username){}
static QoreValue Dir_chown_Vs(QoreObject* self, Dir* d, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* username = HARD_QORE_VALUE_STRING(args, 0);
# 397 "QC_Dir.qpp"
    #ifdef HAVE_CHOWN
   uid_t uid;

   // Try getting UID for username
   int rc = q_uname2uid(username->getBuffer(), uid);
   if (rc)
      xsink->raiseException("DIR-CHOWN-PARAMETER-ERROR", "no userid found for user '%s'", username->getBuffer());
   else
      d->chown(uid, (gid_t)-1, xsink);
   return QoreValue();
#else
   return missing_method_error("Dir::chown", "UNIX_FILEMGT", xsink);
#endif
    return QoreValue();
}

// Dir::constructor(*string encoding) {}
static void Dir_constructor_Ns(QoreObject* self, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* encoding = get_param_value(args, 0).get<const QoreStringNode>();
# 221 "QC_Dir.qpp"
const QoreEncoding *qe = encoding ? QEM.findCreate(encoding) : QCS_DEFAULT;
   SimpleRefHolder<Dir> d(new Dir(xsink, qe));
   if (*xsink)
      return;
   self->setPrivate(CID_DIR, d.release());
}

// Dir::copy() {}
static void Dir_copy(QoreObject* self, QoreObject* old, Dir* d, ExceptionSink* xsink) {
# 236 "QC_Dir.qpp"
SimpleRefHolder<Dir> nd(new Dir(xsink, *d));
   if (*xsink)
      return;

   self->setPrivate(CID_DIR, nd.release());
}

// int Dir::create(softint mode = 0777){}
static QoreValue Dir_create_vi(QoreObject* self, Dir* d, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 mode = HARD_QORE_VALUE_INT(args, 0);
# 316 "QC_Dir.qpp"
    return d->create((int)mode, xsink); // throws exception
}

// bool Dir::exists(){}
static QoreValue Dir_exists(QoreObject* self, Dir* d, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 289 "QC_Dir.qpp"
    return !d->checkPath();
}

// hash<StatInfo> Dir::hstat(){}
static QoreValue Dir_hstat(QoreObject* self, Dir* d, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 870 "QC_Dir.qpp"
    return d->hstat(xsink);
}

// list Dir::list(bool full = False){}
static QoreValue Dir_list_Vb(QoreObject* self, Dir* d, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    bool full = HARD_QORE_VALUE_BOOL(args, 0);
# 563 "QC_Dir.qpp"
    return d->list(xsink, -1, 0, 0, full);
}

// list Dir::list(string regex, softint regex_options = 0, softbool full = False){}
static QoreValue Dir_list_Vsvivb(QoreObject* self, Dir* d, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* regex = HARD_QORE_VALUE_STRING(args, 0);
    int64 regex_options = HARD_QORE_VALUE_INT(args, 1);
    bool full = HARD_QORE_VALUE_BOOL(args, 2);
# 596 "QC_Dir.qpp"
    return d->list(xsink, -1, regex, (int)regex_options, full);
}

// list Dir::listDirs(bool full = False){}
static QoreValue Dir_listDirs_Vb(QoreObject* self, Dir* d, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    bool full = HARD_QORE_VALUE_BOOL(args, 0);
# 688 "QC_Dir.qpp"
    return d->list(xsink, S_IFDIR, 0, 0, full);
}

// list Dir::listDirs(string regex, softint regex_options = 0, softbool full = False){}
static QoreValue Dir_listDirs_Vsvivb(QoreObject* self, Dir* d, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* regex = HARD_QORE_VALUE_STRING(args, 0);
    int64 regex_options = HARD_QORE_VALUE_INT(args, 1);
    bool full = HARD_QORE_VALUE_BOOL(args, 2);
# 721 "QC_Dir.qpp"
    return d->list(xsink, S_IFDIR, regex, (int)regex_options, full);
}

// list Dir::listFiles(bool full = False){}
static QoreValue Dir_listFiles_Vb(QoreObject* self, Dir* d, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    bool full = HARD_QORE_VALUE_BOOL(args, 0);
# 626 "QC_Dir.qpp"
    return d->list(xsink, S_IFMT^S_IFDIR, 0, 0, full);
}

// list Dir::listFiles(string regex, softint regex_options = 0, softbool full = False){}
static QoreValue Dir_listFiles_Vsvivb(QoreObject* self, Dir* d, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* regex = HARD_QORE_VALUE_STRING(args, 0);
    int64 regex_options = HARD_QORE_VALUE_INT(args, 1);
    bool full = HARD_QORE_VALUE_BOOL(args, 2);
# 659 "QC_Dir.qpp"
    return d->list(xsink, S_IFMT^S_IFDIR, regex, (int)regex_options, full);
}

// nothing Dir::mkdir(string subdir, softint mode = 0777){}
static QoreValue Dir_mkdir_Vsvi(QoreObject* self, Dir* d, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* subdir = HARD_QORE_VALUE_STRING(args, 0);
    int64 mode = HARD_QORE_VALUE_INT(args, 1);
# 503 "QC_Dir.qpp"
    // check if there is a path delimiter
   if (strchr(subdir->getBuffer(), QORE_DIR_SEP))
      xsink->raiseException("DIR-MKDIR-PARAMETER-ERROR", "only single, direct subdirectories are allowed");
   else
      d->mkdir(xsink, subdir->getBuffer(), (int)mode);
    return QoreValue();
}

// Dir Dir::openDir(string subdir, *string encoding){}
static QoreValue Dir_openDir_VsNs(QoreObject* self, Dir* d, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* subdir = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* encoding = get_param_value(args, 1).get<const QoreStringNode>();
# 775 "QC_Dir.qpp"
    // check if there is a path delimiter in
   if (strchr(subdir->getBuffer(), QORE_DIR_SEP))
      return xsink->raiseException("DIR-OPENDIR-PARAMETER-ERROR", "only direct directory names without path (i.e. without '%c' characters) are allowed", QORE_DIR_SEP);

   const QoreEncoding *qe = encoding ? QEM.findCreate(encoding) : d->getEncoding();
   ReferenceHolder<Dir> dc(new Dir(xsink, qe, d->getPath(subdir->getBuffer()).c_str()), xsink);
   // create the qoreObject and set the Dir object as private data of the class tagged with the CID_DIR class ID
   return new QoreObject(QC_DIR, getProgram(), dc.release());
}

// File Dir::openFile(string filename, int flags = O_RDONLY, int mode = 0666, *string encoding){}
static QoreValue Dir_openFile_VsViViNs(QoreObject* self, Dir* d, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* filename = HARD_QORE_VALUE_STRING(args, 0);
    int64 flags = HARD_QORE_VALUE_INT(args, 1);
    int64 mode = HARD_QORE_VALUE_INT(args, 2);
    const QoreStringNode* encoding = get_param_value(args, 3).get<const QoreStringNode>();
# 743 "QC_Dir.qpp"
    if (strchr(filename->getBuffer(), QORE_DIR_SEP))
      return xsink->raiseException("DIR-OPENFILE-PARAMETER-ERROR", "only filenames without path (i.e. without '%c' characters) are allowed", QORE_DIR_SEP);

   const QoreEncoding *qe = encoding ? QEM.findCreate(encoding) : QCS_DEFAULT;

   // open the file with exception
   ReferenceHolder<File> f(new File(qe), xsink);
   std::string path = d->getPath(filename->getBuffer());

   if (f->open2(xsink, path.c_str(), flags, mode, qe)) {
      assert(*xsink);
      return QoreValue();
   }

   // create the QoreObject and set the File object as private data of the class tagged with the CID_FILE class ID
   return new QoreObject(QC_FILE, getProgram(), f.release());
}

// *string Dir::path(){}
static QoreValue Dir_path(QoreObject* self, Dir* d, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 276 "QC_Dir.qpp"
    return d->dirname();
}

// bool Dir::removeFile(string file){}
static QoreValue Dir_removeFile_Vs(QoreObject* self, Dir* d, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* file = HARD_QORE_VALUE_STRING(args, 0);
# 809 "QC_Dir.qpp"
    // check if there is a path delimiter in
   if (strchr(file->getBuffer(), QORE_DIR_SEP))
      return xsink->raiseException("DIR-REMOVEFILE-PARAMETER-ERROR", "only filenames without path (i.e. without '%c' characters) are allowed", QORE_DIR_SEP);

   std::string path = d->getPath(file->getBuffer());
   errno = 0; // clear errno flag
   if (unlink(path.c_str()) && errno != ENOENT)
      return xsink->raiseErrnoException("DIR-REMOVEFILE-FAILURE", errno, "error removing file '%s'", file->getBuffer());

   // if an errno was set it must be ENOENT at this point.
   // so we return that no file is removed
   return errno ? false : true;
}

// nothing Dir::rmdir(string subdir){}
static QoreValue Dir_rmdir_Vs(QoreObject* self, Dir* d, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* subdir = HARD_QORE_VALUE_STRING(args, 0);
# 532 "QC_Dir.qpp"
    // check if there is a path delimiter
   if (strchr(subdir->getBuffer(), QORE_DIR_SEP))
      xsink->raiseException("DIR-RMDIR-PARAMETER-ERROR", "only direct subdirectories are allowed");
   else
      d->rmdir(subdir->getBuffer(), xsink);
    return QoreValue();
}

// list Dir::stat(){}
static QoreValue Dir_stat(QoreObject* self, Dir* d, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 845 "QC_Dir.qpp"
    return d->stat(xsink);
}

// hash<FilesystemInfo> Dir::statvfs(){}
static QoreValue Dir_statvfs(QoreObject* self, Dir* d, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 896 "QC_Dir.qpp"
    #ifdef Q_HAVE_STATVFS
   return d->statvfs(xsink);
#else
   return missing_method_error("Dir::statvfs", "STATVFS", xsink);
#endif
    return QoreValue();
}

DLLLOCAL void preinitDirClass() {
    QC_DIR = new QoreClass("Dir", QDOM_FILESYSTEM);
    CID_DIR = QC_DIR->getID();
    QC_DIR->setSystem();
}

DLLLOCAL QoreClass* initDirClass(QoreNamespace& ns) {
    if (!QC_DIR)
        preinitDirClass();

    // bool Dir::chdir(string path){}
    QC_DIR->addMethod("chdir", (q_method_n_t)Dir_chdir_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo, 1, stringTypeInfo, NULL, "path");

    // nothing Dir::chgrp(int gid){}
    QC_DIR->addMethod("chgrp", (q_method_n_t)Dir_chgrp_Vi, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, bigIntTypeInfo, NULL, "gid");

    // nothing Dir::chgrp(string groupname){}
    QC_DIR->addMethod("chgrp", (q_method_n_t)Dir_chgrp_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "groupname");

    // nothing Dir::chmod(softint mode){}
    QC_DIR->addMethod("chmod", (q_method_n_t)Dir_chmod_vi, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, softBigIntTypeInfo, NULL, "mode");

    // nothing Dir::chown(int uid){}
    QC_DIR->addMethod("chown", (q_method_n_t)Dir_chown_Vi, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, bigIntTypeInfo, NULL, "uid");

    // nothing Dir::chown(string username){}
    QC_DIR->addMethod("chown", (q_method_n_t)Dir_chown_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "username");

    // Dir::constructor(*string encoding) {}
    QC_DIR->addConstructor(Dir_constructor_Ns, false, QC_NO_FLAGS, QDOM_DEFAULT, 1, stringOrNothingTypeInfo, NULL, "encoding");

    // Dir::copy() {}
    QC_DIR->setCopy((q_copy_t)Dir_copy);

    // int Dir::create(softint mode = 0777){}
    QC_DIR->addMethod("create", (q_method_n_t)Dir_create_vi, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, softBigIntTypeInfo, new QoreBigIntNode(0777), "mode");

    // bool Dir::exists(){}
    QC_DIR->addMethod("exists", (q_method_n_t)Dir_exists, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

    // hash<StatInfo> Dir::hstat(){}
    QC_DIR->addMethod("hstat", (q_method_n_t)Dir_hstat, false, QC_NO_FLAGS, QDOM_DEFAULT, hashdeclStatInfo->getTypeInfo(false));

    // list Dir::list(bool full = False){}
    QC_DIR->addMethod("list", (q_method_n_t)Dir_list_Vb, false, QC_NO_FLAGS, QDOM_DEFAULT, listTypeInfo, 1, boolTypeInfo, &False, "full");

    // list Dir::list(string regex, softint regex_options = 0, softbool full = False){}
    QC_DIR->addMethod("list", (q_method_n_t)Dir_list_Vsvivb, false, QC_NO_FLAGS, QDOM_DEFAULT, listTypeInfo, 3, stringTypeInfo, NULL, "regex", softBigIntTypeInfo, zero(), "regex_options", softBoolTypeInfo, &False, "full");

    // list Dir::listDirs(bool full = False){}
    QC_DIR->addMethod("listDirs", (q_method_n_t)Dir_listDirs_Vb, false, QC_NO_FLAGS, QDOM_DEFAULT, listTypeInfo, 1, boolTypeInfo, &False, "full");

    // list Dir::listDirs(string regex, softint regex_options = 0, softbool full = False){}
    QC_DIR->addMethod("listDirs", (q_method_n_t)Dir_listDirs_Vsvivb, false, QC_NO_FLAGS, QDOM_DEFAULT, listTypeInfo, 3, stringTypeInfo, NULL, "regex", softBigIntTypeInfo, zero(), "regex_options", softBoolTypeInfo, &False, "full");

    // list Dir::listFiles(bool full = False){}
    QC_DIR->addMethod("listFiles", (q_method_n_t)Dir_listFiles_Vb, false, QC_NO_FLAGS, QDOM_DEFAULT, listTypeInfo, 1, boolTypeInfo, &False, "full");

    // list Dir::listFiles(string regex, softint regex_options = 0, softbool full = False){}
    QC_DIR->addMethod("listFiles", (q_method_n_t)Dir_listFiles_Vsvivb, false, QC_NO_FLAGS, QDOM_DEFAULT, listTypeInfo, 3, stringTypeInfo, NULL, "regex", softBigIntTypeInfo, zero(), "regex_options", softBoolTypeInfo, &False, "full");

    // nothing Dir::mkdir(string subdir, softint mode = 0777){}
    QC_DIR->addMethod("mkdir", (q_method_n_t)Dir_mkdir_Vsvi, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, NULL, "subdir", softBigIntTypeInfo, new QoreBigIntNode(0777), "mode");

    // Dir Dir::openDir(string subdir, *string encoding){}
    QC_DIR->addMethod("openDir", (q_method_n_t)Dir_openDir_VsNs, false, QC_NO_FLAGS, QDOM_DEFAULT, QC_DIR->getTypeInfo(), 2, stringTypeInfo, NULL, "subdir", stringOrNothingTypeInfo, NULL, "encoding");

    // File Dir::openFile(string filename, int flags = O_RDONLY, int mode = 0666, *string encoding){}
    QC_DIR->addMethod("openFile", (q_method_n_t)Dir_openFile_VsViViNs, false, QC_NO_FLAGS, QDOM_DEFAULT, QC_FILE->getTypeInfo(), 4, stringTypeInfo, NULL, "filename", bigIntTypeInfo, new QoreBigIntNode(O_RDONLY), "flags", bigIntTypeInfo, new QoreBigIntNode(0666), "mode", stringOrNothingTypeInfo, NULL, "encoding");

    // *string Dir::path(){}
    QC_DIR->addMethod("path", (q_method_n_t)Dir_path, false, QC_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // bool Dir::removeFile(string file){}
    QC_DIR->addMethod("removeFile", (q_method_n_t)Dir_removeFile_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo, 1, stringTypeInfo, NULL, "file");

    // nothing Dir::rmdir(string subdir){}
    QC_DIR->addMethod("rmdir", (q_method_n_t)Dir_rmdir_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "subdir");

    // list Dir::stat(){}
    QC_DIR->addMethod("stat", (q_method_n_t)Dir_stat, false, QC_NO_FLAGS, QDOM_DEFAULT, listTypeInfo);

    // hash<FilesystemInfo> Dir::statvfs(){}
    QC_DIR->addMethod("statvfs", (q_method_n_t)Dir_statvfs, false, QC_NO_FLAGS, QDOM_DEFAULT, hashdeclFilesystemInfo->getTypeInfo(false));

    return QC_DIR;
}
