/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  QC_GetOpt.qpp

  Qore Programming Language

  Copyright (C) 2003 - 2017 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/QC_GetOpt.h"

#include <string.h>

static inline int process_type(const char* key, int& attributes, char* opt, qore_type_t& at, ExceptionSink* xsink) {
   assert(at == -1);
   const char* type_name = 0;
   // get type
   switch (*opt) {
      case 's':
	 at = NT_STRING;
	 type_name = QoreStringNode::getStaticTypeName();
	 break;
      case 'i':
	 at = NT_INT;
	 type_name = QoreBigIntNode::getStaticTypeName();
	 break;
      case 'f':
	 at = NT_FLOAT;
	 type_name = QoreFloatNode::getStaticTypeName();
	 break;
      case 'b':
	 at = NT_BOOLEAN;
	 type_name = QoreBoolNode::getStaticTypeName();
	 break;
      case 'd':
	 at = NT_DATE;
	 type_name = DateTimeNode::getStaticTypeName();
	 break;
      case 'h':
	 at = NT_HASH;
	 type_name = QoreHashNode::getStaticTypeName();
	 break;
      case '@':
	 at = NT_STRING;
	 attributes |= QGO_OPT_LIST;
	 type_name = QoreStringNode::getStaticTypeName();
	 break;
      case '+':
	 at = NT_INT;
	 attributes |= QGO_OPT_ADDITIVE;
	 type_name = QoreBigIntNode::getStaticTypeName();
	 break;
   }
   if (at == -1) {
      xsink->raiseException("GETOPT-OPTION-ERROR", "type '%c' for key '%s' is unknown", *opt, key);
      return -1;
   }
   if (!opt[1])
      return 0;

   if (opt[2]) {
      xsink->raiseException("GETOPT-OPTION-ERROR", "invalid attributes in option '%s'", key);
      return -1;
   }

   // process modifiers
   if (opt[1] == '@') {
      if (attributes & QGO_OPT_LIST) {
	 xsink->raiseException("GETOPT-OPTION-ERROR", "list attribute doubled in option key '%s'", key);
	 return -1;
      }
      if (attributes & QGO_OPT_ADDITIVE) {
	 xsink->raiseException("GETOPT-OPTION-ERROR", "option '%s' cannot have both additive and list attributes turned on", key);
	 return -1;
      }
      attributes |= QGO_OPT_LIST;
      return 0;
   }
   if (opt[1] == '+') {
      if (attributes & QGO_OPT_ADDITIVE) {
	 xsink->raiseException("GETOPT-OPTION-ERROR", "additive attribute doubled in option key '%s'", key);
	 return -1;
      }
      if (attributes & QGO_OPT_LIST) {
	 xsink->raiseException("GETOPT-OPTION-ERROR", "option '%s' cannot have both additive and list attributes turned on", key);
	 return -1;
      }
      if (at != NT_INT && at != NT_FLOAT) {
	 xsink->raiseException("GETOPT-OPTION-ERROR", "additive attributes for type '%s' are not supported (option '%s')", type_name, key);
	 return -1;
      }
      attributes |= QGO_OPT_ADDITIVE;
      return 0;
   }

   xsink->raiseException("GETOPT-OPTION-ERROR", "unknown modifier '%c' in option '%s'", opt[1], key);
   return -1;
}

static AbstractQoreNode* GETOPT_parse2_intern(ReferenceHolder<QoreHashNode>& rv, ExceptionSink* xsink) {
   assert(rv);

   // check for _ERRORS_ key
   const QoreListNode* l = reinterpret_cast<const QoreListNode*>(rv->getKeyValue("_ERRORS_"));
   if (!l)
      return rv.release();

   const QoreStringNode* err = reinterpret_cast<const QoreStringNode*>(l->retrieve_entry(0));
   return xsink->raiseException("GETOPT-ERROR", err->stringRefSelf());
}

static AbstractQoreNode* GETOPT_parse3_intern(ReferenceHolder<QoreHashNode>& rv, ExceptionSink* xsink) {
   assert(rv);

   // check for _ERRORS_ key
   const QoreListNode* l = reinterpret_cast<const QoreListNode*>(rv->getKeyValue("_ERRORS_"));
   if (!l)
      return rv.release();

   const QoreStringNode* err = reinterpret_cast<const QoreStringNode*>(l->retrieve_entry(0));
   const QoreStringNode* sn = getProgram()->getScriptName();
   if (sn)
      fprintf(stderr, "%s: ", sn->getBuffer());
   fprintf(stderr, "%s\n", err->getBuffer());
   qore_exit_process(1);

   return 0;
}
/* Qore class Qore::GetOpt */

qore_classid_t CID_GETOPT;
QoreClass* QC_GETOPT;

// GetOpt::constructor(hash options) {}
static void GetOpt_constructor_Vh(QoreObject* self, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreHashNode* options = HARD_QORE_VALUE_HASH(args, 0);
# 188 "QC_GetOpt.qpp"
SimpleRefHolder<GetOpt> g(new GetOpt);

   ConstHashIterator hi(options);
   QoreString vstr;
   while (hi.next()) {
      const char *k = hi.getKey();
      if (!strcmp(k, "_ERRORS_")) {
	 xsink->raiseException("GETOPT-PARAMETER-ERROR", "option key '%s' is reserved for errors in the output hash", k);
	 break;
      }

      const AbstractQoreNode* v = hi.getValue();
      if (get_node_type(v) != NT_STRING) {
	 xsink->raiseException("GETOPT-PARAMETER-ERROR", "value of option key '%s' is not a string (%s)", k, get_type_name(v));
	 break;
      }

      const QoreStringNode* str = reinterpret_cast<const QoreStringNode*>(v);

      qore_type_t at = -1;
      const char *long_opt = 0;
      char short_opt = '\0';
      int attributes = QGO_OPT_NONE;

      // reset buffer
      vstr.clear();
      vstr.concat(str->getBuffer());
      const char* val = vstr.getBuffer();

      // get data type, if any
      char *tok = strchrs(val, "=:");
      if (tok) {
	 if (tok[1] && process_type(k, attributes, tok + 1, at, xsink))
	    break;
	 if ((*tok) == '=')
	    attributes |= QGO_OPT_MANDATORY;
	 else if (attributes & QGO_OPT_LIST) {
	    xsink->raiseException("GETOPT-OPTION-ERROR", "option '%s' takes a list and therefore must have mandatory arguments", k);
	    break;
	 }

	 (*tok) = '\0';
      }
      // get option names
      if (!val[0]) {
	 //printd(5, "making exception key='%s' tok=%p val=%p val='%s'\n", k, tok, val, val);
	 xsink->raiseException("GETOPT-PARAMETER-ERROR", "value of option key '%s' has no option specifiers", k);
	 break;
      }
      tok = (char*)strchr(val, ',');
      if (tok) {
	 if (tok == (val + 1)) {
	    short_opt = val[0];
	    long_opt = val + 2;
	 }
	 else if (tok - val == (signed)(strlen(val) - 2)) {
	    (*tok) = 0;
	    short_opt = tok[1];
	    long_opt = val;
	 }
	 else { // if the comma is not in the second or second-to-last position, then it's an error
	    xsink->raiseException("GETOPT-OPTION-ERROR", "user options can only be specified with one short option and one long option, however two long options were given for key '%s' (%s)", k, val);
	    break;
	 }
      }
      else if (val[1])
	 long_opt = val;
      else
	 short_opt = val[0];
      int rc = g->add(k, short_opt, long_opt, at, attributes);
      if (rc == QGO_ERR_DUP_SHORT_OPT) {
	 xsink->raiseException("GETOPT-OPTION-ERROR", "short option '%c' was duplicated in key '%s'", short_opt, k);
	 break;
      }
      if (rc == QGO_ERR_DUP_LONG_OPT) {
	 xsink->raiseException("GETOPT-OPTION-ERROR", "long option '%s' was duplicated in key '%s'", long_opt, k);
	 break;
      }
      if (rc == QGO_ERR_DUP_NAME) {
	 xsink->raiseException("GETOPT-OPTION-ERROR", "option '%s' was duplicated", k);
	 break;
      }
   }
   if (*xsink)
      return;

   self->setPrivate(CID_GETOPT, g.release());
}

// GetOpt::copy() {}
static void GetOpt_copy(QoreObject* self, QoreObject* old, GetOpt* g, ExceptionSink* xsink) {
# 282 "QC_GetOpt.qpp"
xsink->raiseException("GETOPT-COPY-ERROR", "copying GetOpt objects is not supported");
}

// hash GetOpt::parse(reference<list<string>> pgm_args){}
static QoreValue GetOpt_parse_C23reference_list_string__(QoreObject* self, GetOpt* g, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const ReferenceNode* pgm_args = HARD_QORE_VALUE_REF(args, 0);
# 307 "QC_GetOpt.qpp"
    QoreTypeSafeReferenceHelper ref(pgm_args, xsink);
   if (!ref)
      return QoreValue();

   if (ref.getType() != NT_LIST)
      return new QoreHashNode;

   QoreListNode* l = reinterpret_cast<QoreListNode*>(ref.getUnique(xsink));
   if (*xsink)
      return QoreValue();

   return g->parse(l, true, xsink);
}

// hash GetOpt::parse(softlist pgm_args){}
static QoreValue GetOpt_parse_vl(QoreObject* self, GetOpt* g, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* pgm_args = HARD_QORE_VALUE_LIST(args, 0);
# 341 "QC_GetOpt.qpp"
    return g->parse(const_cast<QoreListNode*>(pgm_args), false, xsink);
}

// hash GetOpt::parse2(reference<list<string>> pgm_args){}
static QoreValue GetOpt_parse2_C23reference_list_string__(QoreObject* self, GetOpt* g, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const ReferenceNode* pgm_args = HARD_QORE_VALUE_REF(args, 0);
# 367 "QC_GetOpt.qpp"
    QoreTypeSafeReferenceHelper ref(pgm_args, xsink);
   if (!ref)
      return QoreValue();

   if (ref.getType() != NT_LIST)
      return new QoreHashNode;

   QoreListNode* l = reinterpret_cast<QoreListNode*>(ref.getUnique(xsink));
   if (*xsink)
      return QoreValue();

   ReferenceHolder<QoreHashNode> rv(g->parse(l, true, xsink), xsink);
   return *xsink ? QoreValue() : GETOPT_parse2_intern(rv, xsink);
}

// hash GetOpt::parse2(softlist pgm_args){}
static QoreValue GetOpt_parse2_vl(QoreObject* self, GetOpt* g, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* pgm_args = HARD_QORE_VALUE_LIST(args, 0);
# 405 "QC_GetOpt.qpp"
    ReferenceHolder<QoreHashNode> rv(g->parse(const_cast<QoreListNode*>(pgm_args), false, xsink), xsink);
   return *xsink ? 0 : GETOPT_parse2_intern(rv, xsink);
}

// hash GetOpt::parse3(reference<list<string>> pgm_args){}
static QoreValue GetOpt_parse3_C23reference_list_string__(QoreObject* self, GetOpt* g, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const ReferenceNode* pgm_args = HARD_QORE_VALUE_REF(args, 0);
# 424 "QC_GetOpt.qpp"
    QoreTypeSafeReferenceHelper ref(pgm_args, xsink);
   if (!ref)
      return QoreValue();

   if (ref.getType() != NT_LIST)
      return new QoreHashNode;

   QoreListNode* l = reinterpret_cast<QoreListNode*>(ref.getUnique(xsink));
   if (*xsink)
      return QoreValue();

   ReferenceHolder<QoreHashNode> rv(g->parse(l, true, xsink), xsink);
   return *xsink ? 0 : GETOPT_parse3_intern(rv, xsink);
}

// hash GetOpt::parse3(softlist pgm_args){}
static QoreValue GetOpt_parse3_vl(QoreObject* self, GetOpt* g, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* pgm_args = HARD_QORE_VALUE_LIST(args, 0);
# 456 "QC_GetOpt.qpp"
    ReferenceHolder<QoreHashNode> rv(g->parse(const_cast<QoreListNode*>(pgm_args), false, xsink), xsink);
   return *xsink ? 0 : GETOPT_parse3_intern(rv, xsink);
}

DLLLOCAL void preinitGetOptClass() {
    QC_GETOPT = new QoreClass("GetOpt", QDOM_DEFAULT);
    CID_GETOPT = QC_GETOPT->getID();
    QC_GETOPT->setSystem();
}

DLLLOCAL QoreClass* initGetOptClass(QoreNamespace& ns) {
    if (!QC_GETOPT)
        preinitGetOptClass();

    // GetOpt::constructor(hash options) {}
    QC_GETOPT->addConstructor(GetOpt_constructor_Vh, false, QC_NO_FLAGS, QDOM_DEFAULT, 1, hashTypeInfo, NULL, "options");

    // GetOpt::copy() {}
    QC_GETOPT->setCopy((q_copy_t)GetOpt_copy);

    // hash GetOpt::parse(reference<list<string>> pgm_args){}
    QC_GETOPT->addMethod("parse", (q_method_n_t)GetOpt_parse_C23reference_list_string__, false, QC_NO_FLAGS, QDOM_DEFAULT, hashTypeInfo, 1, qore_get_complex_reference_type(qore_get_complex_list_type(stringTypeInfo)), NULL, "pgm_args");

    // hash GetOpt::parse(softlist pgm_args){}
    QC_GETOPT->addMethod("parse", (q_method_n_t)GetOpt_parse_vl, false, QC_NO_FLAGS, QDOM_DEFAULT, hashTypeInfo, 1, softListTypeInfo, NULL, "pgm_args");

    // hash GetOpt::parse2(reference<list<string>> pgm_args){}
    QC_GETOPT->addMethod("parse2", (q_method_n_t)GetOpt_parse2_C23reference_list_string__, false, QC_NO_FLAGS, QDOM_DEFAULT, hashTypeInfo, 1, qore_get_complex_reference_type(qore_get_complex_list_type(stringTypeInfo)), NULL, "pgm_args");

    // hash GetOpt::parse2(softlist pgm_args){}
    QC_GETOPT->addMethod("parse2", (q_method_n_t)GetOpt_parse2_vl, false, QC_NO_FLAGS, QDOM_DEFAULT, hashTypeInfo, 1, softListTypeInfo, NULL, "pgm_args");

    // hash GetOpt::parse3(reference<list<string>> pgm_args){}
    QC_GETOPT->addMethod("parse3", (q_method_n_t)GetOpt_parse3_C23reference_list_string__, false, QC_NO_FLAGS, QDOM_PROCESS, hashTypeInfo, 1, qore_get_complex_reference_type(qore_get_complex_list_type(stringTypeInfo)), NULL, "pgm_args");

    // hash GetOpt::parse3(softlist pgm_args){}
    QC_GETOPT->addMethod("parse3", (q_method_n_t)GetOpt_parse3_vl, false, QC_NO_FLAGS, QDOM_PROCESS, hashTypeInfo, 1, softListTypeInfo, NULL, "pgm_args");

    return QC_GETOPT;
}
