/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  QC_HTTPClient.qpp

  Qore Programming Language

  Copyright (C) 2006 - 2017 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include <qore/QoreHttpClientObject.h>
#include "qore/intern/QC_HTTPClient.h"
#include "qore/intern/QC_Queue.h"
#include "qore/intern/ssl_constants.h"
#include "qore/minitest.hpp"
#include "qore/QoreSSLCertificate.h"
#include "qore/QoreSSLPrivateKey.h"

extern QoreClass* QC_SOCKET;

#ifdef DEBUG_TESTS
#  include "tests/QC_HTTPClient_tests.cpp"
#endif
/* Qore class Qore::HTTPClient */

qore_classid_t CID_HTTPCLIENT;
QoreClass* QC_HTTPCLIENT;

// nothing HTTPClient::clearProxyURL(){}
static QoreValue HTTPClient_clearProxyURL(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1107 "QC_HTTPClient.qpp"
    client->clearProxyURL();
    return QoreValue();
}

// nothing HTTPClient::clearProxyUserPassword(){}
static QoreValue HTTPClient_clearProxyUserPassword(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1346 "QC_HTTPClient.qpp"
    client->clearProxyUserPassword();
    return QoreValue();
}

// nothing HTTPClient::clearStats(){}
static QoreValue HTTPClient_clearStats(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1476 "QC_HTTPClient.qpp"
    client->clearStats();
    return QoreValue();
}

// nothing HTTPClient::clearUserPassword(){}
static QoreValue HTTPClient_clearUserPassword(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1303 "QC_HTTPClient.qpp"
    client->clearUserPassword();
    return QoreValue();
}

// nothing HTTPClient::clearWarningQueue(){}
static QoreValue HTTPClient_clearWarningQueue(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1398 "QC_HTTPClient.qpp"
    client->clearWarningQueue(xsink);
    return QoreValue();
}

// nothing HTTPClient::connect(){}
static QoreValue HTTPClient_connect(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 279 "QC_HTTPClient.qpp"
    client->connect(xsink);
    return QoreValue();
}

// HTTPClient::constructor(hash opts) {}
static void HTTPClient_constructor_Vh(QoreObject* self, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreHashNode* opts = HARD_QORE_VALUE_HASH(args, 0);
# 169 "QC_HTTPClient.qpp"
ReferenceHolder<QoreHttpClientObject> client(new QoreHttpClientObject, xsink);
   if (client->setOptions(opts, xsink))
      return;

   self->setPrivate(CID_HTTPCLIENT, client.release());
}

// HTTPClient::constructor() {}
static void HTTPClient_constructor(QoreObject* self, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 184 "QC_HTTPClient.qpp"
self->setPrivate(CID_HTTPCLIENT, new QoreHttpClientObject);
}

// HTTPClient::copy() {}
static void HTTPClient_copy(QoreObject* self, QoreObject* old, QoreHttpClientObject* client, ExceptionSink* xsink) {
# 192 "QC_HTTPClient.qpp"
xsink->raiseException("HTTPCLIENT-COPY-ERROR", "copying HTTPClient objects is not yet supported");
}

// HTTPClient::destructor() {}
static void HTTPClient_destructor(QoreObject* self, QoreHttpClientObject* client, ExceptionSink* xsink) {
# 203 "QC_HTTPClient.qpp"
// have to clear callbacks before destroying
   client->cleanup(xsink);
   client->deref(xsink);
}

// nothing HTTPClient::disconnect(){}
static QoreValue HTTPClient_disconnect(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 290 "QC_HTTPClient.qpp"
    client->disconnect();
    return QoreValue();
}

// *string HTTPClient::get(string path, *hash headers, *reference<hash> info){}
static QoreValue HTTPClient_get_VsNhC16_reference_hash_(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
    const QoreHashNode* headers = get_param_value(args, 1).get<const QoreHashNode>();
    const ReferenceNode* info = get_param_value(args, 2).get<const ReferenceNode>();
# 814 "QC_HTTPClient.qpp"
    OptHashRefHelper ohrh(info, xsink);
   ReferenceHolder<AbstractQoreNode> rv(client->get(path->getBuffer(), headers, *ohrh, xsink), xsink);
   return *xsink ? 0 : rv.release();
}

// int HTTPClient::getConnectTimeout(){}
static QoreValue HTTPClient_getConnectTimeout(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1213 "QC_HTTPClient.qpp"
    return client->getConnectTimeout();
}

// *string HTTPClient::getConnectionPath(){}
static QoreValue HTTPClient_getConnectionPath(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1383 "QC_HTTPClient.qpp"
    const char* p = client->getConnectionPath();
   return p ? new QoreStringNode(p) : 0;
}

// string HTTPClient::getDefaultPath(){}
static QoreValue HTTPClient_getDefaultPath(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1370 "QC_HTTPClient.qpp"
    const char* dp = client->getDefaultPath();
   return dp ? new QoreStringNode(dp) : 0;
}

// string HTTPClient::getEncoding(){}
static QoreValue HTTPClient_getEncoding(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1022 "QC_HTTPClient.qpp"
    return new QoreStringNode(client->getEncoding()->getCode());
}

// string HTTPClient::getHTTPVersion(){}
static QoreValue HTTPClient_getHTTPVersion(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 231 "QC_HTTPClient.qpp"
    return new QoreStringNode(client->getHTTPVersion());
}

// int HTTPClient::getMaxRedirects(){}
static QoreValue HTTPClient_getMaxRedirects(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1158 "QC_HTTPClient.qpp"
    return client->getMaxRedirects();
}

// bool HTTPClient::getNoDelay(){}
static QoreValue HTTPClient_getNoDelay(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1248 "QC_HTTPClient.qpp"
    return client->getNoDelay();
}

// *string HTTPClient::getProxyURL(){}
static QoreValue HTTPClient_getProxyURL(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1096 "QC_HTTPClient.qpp"
    return client->getProxyURL();
}

// int HTTPClient::getTimeout(){}
static QoreValue HTTPClient_getTimeout(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 998 "QC_HTTPClient.qpp"
    return client->getTimeout();
}

// *string HTTPClient::getURL(){}
static QoreValue HTTPClient_getURL(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1055 "QC_HTTPClient.qpp"
    return client->getURL();
}

// hash HTTPClient::getUsageInfo(){}
static QoreValue HTTPClient_getUsageInfo(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1462 "QC_HTTPClient.qpp"
    return client->getUsageInfo();
}

// hash HTTPClient::head(string path, *hash headers, *reference<hash> info){}
static QoreValue HTTPClient_head_VsNhC16_reference_hash_(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
    const QoreHashNode* headers = get_param_value(args, 1).get<const QoreHashNode>();
    const ReferenceNode* info = get_param_value(args, 2).get<const ReferenceNode>();
# 861 "QC_HTTPClient.qpp"
    OptHashRefHelper ohrh(info, xsink);
   ReferenceHolder<QoreHashNode> rv(client->head(path->getBuffer(), headers, *ohrh, xsink), xsink);
   return *xsink ? 0 : rv.release();
}

// bool HTTPClient::isConnected(){}
static QoreValue HTTPClient_isConnected(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1260 "QC_HTTPClient.qpp"
    return client->isConnected();
}

// bool HTTPClient::isProxySecure(){}
static QoreValue HTTPClient_isProxySecure(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1132 "QC_HTTPClient.qpp"
    return client->isProxySecure();
}

// bool HTTPClient::isSecure(){}
static QoreValue HTTPClient_isSecure(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 260 "QC_HTTPClient.qpp"
    return client->isSecure();
}

// *string HTTPClient::post(string path, string body, *hash headers, *reference<hash> info){}
static QoreValue HTTPClient_post_VsVsNhC16_reference_hash_(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* body = HARD_QORE_VALUE_STRING(args, 1);
    const QoreHashNode* headers = get_param_value(args, 2).get<const QoreHashNode>();
    const ReferenceNode* info = get_param_value(args, 3).get<const ReferenceNode>();
# 912 "QC_HTTPClient.qpp"
    OptHashRefHelper ohrh(info, xsink);
   // issue 1813: convert string to socket's character encoding as documented
   TempEncodingHelper btmp(body, client->getEncoding(), xsink);
   if (*xsink)
      return QoreValue();
   ReferenceHolder<AbstractQoreNode> rv(client->post(path->getBuffer(), headers, btmp->c_str(), btmp->size(), *ohrh, xsink), xsink);
#ifdef DEBUG
   assert(!rv || rv->getType() == NT_STRING);
#endif
   return *xsink ? 0 : rv.release();
}

// *string HTTPClient::post(string path, *binary body, *hash headers, *reference<hash> info){}
static QoreValue HTTPClient_post_VsNyNhC16_reference_hash_(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
    const BinaryNode* body = get_param_value(args, 1).get<const BinaryNode>();
    const QoreHashNode* headers = get_param_value(args, 2).get<const QoreHashNode>();
    const ReferenceNode* info = get_param_value(args, 3).get<const ReferenceNode>();
# 969 "QC_HTTPClient.qpp"
    OptHashRefHelper ohrh(info, xsink);
   ReferenceHolder<AbstractQoreNode> rv(client->post(path->getBuffer(), headers, body ? body->getPtr() : 0, body ? body->size() : 0, *ohrh, xsink), xsink);
#ifdef DEBUG
   assert(!rv || rv->getType() == NT_STRING);
#endif
   return *xsink ? 0 : rv.release();
}

// hash HTTPClient::send(string body, string method, *string path, *hash headers, softbool getbody = False, *reference<hash> info){}
static QoreValue HTTPClient_send_VsVsNsNhvbC16_reference_hash_(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* body = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* method = HARD_QORE_VALUE_STRING(args, 1);
    const QoreStringNode* path = get_param_value(args, 2).get<const QoreStringNode>();
    const QoreHashNode* headers = get_param_value(args, 3).get<const QoreHashNode>();
    bool getbody = HARD_QORE_VALUE_BOOL(args, 4);
    const ReferenceNode* info = get_param_value(args, 5).get<const ReferenceNode>();
# 339 "QC_HTTPClient.qpp"
    OptHashRefHelper ohrh(info, xsink);
   // issue 1813: convert string to socket's character encoding as documented
   TempEncodingHelper btmp(body, client->getEncoding(), xsink);
   if (*xsink)
      return QoreValue();

   ReferenceHolder<QoreHashNode> rv(client->send(method->getBuffer(), path && !path->empty() ? path->getBuffer() : 0, headers, btmp->c_str(), btmp->size(), getbody, *ohrh, xsink), xsink);
   return *xsink ? 0 : rv.release();
}

// hash HTTPClient::send(*binary body, string method, *string path, *hash headers, softbool getbody = False, *reference<hash> info){}
static QoreValue HTTPClient_send_NyVsNsNhvbC16_reference_hash_(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* body = get_param_value(args, 0).get<const BinaryNode>();
    const QoreStringNode* method = HARD_QORE_VALUE_STRING(args, 1);
    const QoreStringNode* path = get_param_value(args, 2).get<const QoreStringNode>();
    const QoreHashNode* headers = get_param_value(args, 3).get<const QoreHashNode>();
    bool getbody = HARD_QORE_VALUE_BOOL(args, 4);
    const ReferenceNode* info = get_param_value(args, 5).get<const ReferenceNode>();
# 395 "QC_HTTPClient.qpp"
    OptHashRefHelper ohrh(info, xsink);
   ReferenceHolder<QoreHashNode> rv(client->send(method->getBuffer(), path && !path->empty() ? path->getBuffer() : 0, headers, body ? body->getPtr() : 0, body ? body->size() : 0, getbody, *ohrh, xsink), xsink);
   return *xsink ? 0 : rv.release();
}

// nothing HTTPClient::send(Qore::OutputStream os, *data body, string method, *string path, *hash headers, timeout timeout_ms = 0, softbool getbody = False, *reference<hash> info, *code rcb){}
static QoreValue HTTPClient_send_C12OutputStreamNDVsNsNhVtvbC16_reference_hash_Nc(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(os, OutputStream, args, 0, CID_OUTPUTSTREAM, "HTTPClient::send()", "Qore::OutputStream", xsink);
    if (*xsink)
        return 0;
    const AbstractQoreNode* body = get_param_value(args, 1).get<const AbstractQoreNode>();
    const QoreStringNode* method = HARD_QORE_VALUE_STRING(args, 2);
    const QoreStringNode* path = get_param_value(args, 3).get<const QoreStringNode>();
    const QoreHashNode* headers = get_param_value(args, 4).get<const QoreHashNode>();
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 5);
    bool getbody = HARD_QORE_VALUE_BOOL(args, 6);
    const ReferenceNode* info = get_param_value(args, 7).get<const ReferenceNode>();
    const ResolvedCallReferenceNode* rcb = get_param_value(args, 8).get<const ResolvedCallReferenceNode>();
# 625 "QC_HTTPClient.qpp"
    SimpleRefHolder<OutputStream> osHolder(os);
   OptHashRefHelper ohrh(info, xsink);
   const char* pstr = path && !path->empty() ? path->c_str() : nullptr;
   if (get_node_type(body) == NT_STRING) {
      const QoreStringNode* str = reinterpret_cast<const QoreStringNode*>(body);
      TempEncodingHelper tmp(str, client->getEncoding(), xsink);
      client->sendWithOutputStream(method->getBuffer(), pstr, headers, str->c_str(), str->size(), getbody, *ohrh, timeout_ms, rcb, self, os, xsink);
   }
   else {
      const BinaryNode* b = reinterpret_cast<const BinaryNode*>(body);
      client->sendWithOutputStream(method->getBuffer(), pstr, headers, b ? b->getPtr() : nullptr, b ? b->size() : 0, getbody, *ohrh, timeout_ms, rcb, self, os, xsink);
   }
    return QoreValue();
}

// nothing HTTPClient::sendChunked(Qore::OutputStream os, Qore::InputStream is, string method, int max_chunk_size = 4096, *string path, *hash headers, timeout timeout_ms = 0, softbool getbody = False, *reference<hash> info, *code rcb, *code tcb){}
static QoreValue HTTPClient_sendChunked_C12OutputStreamC11InputStreamVsViNsNhVtvbC16_reference_hash_NcNc(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(os, OutputStream, args, 0, CID_OUTPUTSTREAM, "HTTPClient::sendChunked()", "Qore::OutputStream", xsink);
    if (*xsink)
        return 0;
    HARD_QORE_VALUE_OBJ_DATA(is, InputStream, args, 1, CID_INPUTSTREAM, "HTTPClient::sendChunked()", "Qore::InputStream", xsink);
    if (*xsink)
        return 0;
    const QoreStringNode* method = HARD_QORE_VALUE_STRING(args, 2);
    int64 max_chunk_size = HARD_QORE_VALUE_INT(args, 3);
    const QoreStringNode* path = get_param_value(args, 4).get<const QoreStringNode>();
    const QoreHashNode* headers = get_param_value(args, 5).get<const QoreHashNode>();
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 6);
    bool getbody = HARD_QORE_VALUE_BOOL(args, 7);
    const ReferenceNode* info = get_param_value(args, 8).get<const ReferenceNode>();
    const ResolvedCallReferenceNode* rcb = get_param_value(args, 9).get<const ResolvedCallReferenceNode>();
    const ResolvedCallReferenceNode* tcb = get_param_value(args, 10).get<const ResolvedCallReferenceNode>();
# 693 "QC_HTTPClient.qpp"
    SimpleRefHolder<OutputStream> osHolder(os);
   SimpleRefHolder<InputStream> isHolder(is);
   if (max_chunk_size < 1) {
      xsink->raiseException("HTTP-CHUNK-ERROR", "the max chunk size must be greater than 0; value given is: " QLLD, max_chunk_size);
      return QoreValue();
   }
   OptHashRefHelper ohrh(info, xsink);
   client->sendChunked(method->getBuffer(), path && !path->empty() ? path->getBuffer() : nullptr, headers, getbody, *ohrh, timeout_ms, rcb, self, os, is, (size_t)max_chunk_size, tcb, xsink);
    return QoreValue();
}

// nothing HTTPClient::sendWithCallbacks(code scb, code rcb, string method, *string path, *hash headers, timeout timeout_ms = 0, softbool getbody = False, *reference<hash> info){}
static QoreValue HTTPClient_sendWithCallbacks_VcVcVsNsNhVtvbC16_reference_hash_(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const ResolvedCallReferenceNode* scb = get_param_value(args, 0).get<const ResolvedCallReferenceNode>();
    const ResolvedCallReferenceNode* rcb = get_param_value(args, 1).get<const ResolvedCallReferenceNode>();
    const QoreStringNode* method = HARD_QORE_VALUE_STRING(args, 2);
    const QoreStringNode* path = get_param_value(args, 3).get<const QoreStringNode>();
    const QoreHashNode* headers = get_param_value(args, 4).get<const QoreHashNode>();
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 5);
    bool getbody = HARD_QORE_VALUE_BOOL(args, 6);
    const ReferenceNode* info = get_param_value(args, 7).get<const ReferenceNode>();
# 762 "QC_HTTPClient.qpp"
    OptHashRefHelper ohrh(info, xsink);
   client->sendWithCallbacks(method->getBuffer(), path && !path->empty() ? path->getBuffer() : 0, headers, scb, getbody, *ohrh, timeout_ms, rcb, self, xsink);
    return QoreValue();
}

// nothing HTTPClient::sendWithRecvCallback(code rcb, string body, string method, *string path, *hash headers, timeout timeout_ms = 0, softbool getbody = False, *reference<hash> info){}
static QoreValue HTTPClient_sendWithRecvCallback_VcVsVsNsNhVtvbC16_reference_hash_(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const ResolvedCallReferenceNode* rcb = get_param_value(args, 0).get<const ResolvedCallReferenceNode>();
    const QoreStringNode* body = HARD_QORE_VALUE_STRING(args, 1);
    const QoreStringNode* method = HARD_QORE_VALUE_STRING(args, 2);
    const QoreStringNode* path = get_param_value(args, 3).get<const QoreStringNode>();
    const QoreHashNode* headers = get_param_value(args, 4).get<const QoreHashNode>();
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 5);
    bool getbody = HARD_QORE_VALUE_BOOL(args, 6);
    const ReferenceNode* info = get_param_value(args, 7).get<const ReferenceNode>();
# 513 "QC_HTTPClient.qpp"
    OptHashRefHelper ohrh(info, xsink);
   // issue 1813: convert string to socket's character encoding as documented
   TempEncodingHelper btmp(body, client->getEncoding(), xsink);
   if (*xsink)
      return QoreValue();
   client->sendWithRecvCallback(method->getBuffer(), path && !path->empty() ? path->getBuffer() : 0, headers, btmp->c_str(), btmp->size(), getbody, *ohrh, timeout_ms, rcb, self, xsink);
    return QoreValue();
}

// nothing HTTPClient::sendWithRecvCallback(code rcb, *binary body, string method, *string path, *hash headers, timeout timeout_ms = 0, softbool getbody = False, *reference<hash> info){}
static QoreValue HTTPClient_sendWithRecvCallback_VcNyVsNsNhVtvbC16_reference_hash_(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const ResolvedCallReferenceNode* rcb = get_param_value(args, 0).get<const ResolvedCallReferenceNode>();
    const BinaryNode* body = get_param_value(args, 1).get<const BinaryNode>();
    const QoreStringNode* method = HARD_QORE_VALUE_STRING(args, 2);
    const QoreStringNode* path = get_param_value(args, 3).get<const QoreStringNode>();
    const QoreHashNode* headers = get_param_value(args, 4).get<const QoreHashNode>();
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 5);
    bool getbody = HARD_QORE_VALUE_BOOL(args, 6);
    const ReferenceNode* info = get_param_value(args, 7).get<const ReferenceNode>();
# 575 "QC_HTTPClient.qpp"
    OptHashRefHelper ohrh(info, xsink);
   client->sendWithRecvCallback(method->getBuffer(), path && !path->empty() ? path->getBuffer() : nullptr, headers, body ? body->getPtr() : nullptr, body ? body->size() : 0, getbody, *ohrh, timeout_ms, rcb, self, xsink);
    return QoreValue();
}

// hash HTTPClient::sendWithSendCallback(code scb, string method, *string path, *hash headers, timeout timeout_ms = 0, softbool getbody = False, *reference<hash> info){}
static QoreValue HTTPClient_sendWithSendCallback_VcVsNsNhVtvbC16_reference_hash_(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const ResolvedCallReferenceNode* scb = get_param_value(args, 0).get<const ResolvedCallReferenceNode>();
    const QoreStringNode* method = HARD_QORE_VALUE_STRING(args, 1);
    const QoreStringNode* path = get_param_value(args, 2).get<const QoreStringNode>();
    const QoreHashNode* headers = get_param_value(args, 3).get<const QoreHashNode>();
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 4);
    bool getbody = HARD_QORE_VALUE_BOOL(args, 5);
    const ReferenceNode* info = get_param_value(args, 6).get<const ReferenceNode>();
# 452 "QC_HTTPClient.qpp"
    OptHashRefHelper ohrh(info, xsink);
   ReferenceHolder<QoreHashNode> rv(client->sendWithSendCallback(method->getBuffer(), path && !path->empty() ? path->getBuffer() : 0, headers, scb, getbody, *ohrh, timeout_ms, xsink), xsink);
   return *xsink ? 0 : rv.release();
}

// nothing HTTPClient::setConnectTimeout(timeout timeout_ms = -1){}
static QoreValue HTTPClient_setConnectTimeout_Vt(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 1201 "QC_HTTPClient.qpp"
    client->setConnectTimeout((int)timeout_ms);
    return QoreValue();
}

// nothing HTTPClient::setDefaultPath(*string path){}
static QoreValue HTTPClient_setDefaultPath_Ns(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = get_param_value(args, 0).get<const QoreStringNode>();
# 1358 "QC_HTTPClient.qpp"
    client->setDefaultPath(path ? path->getBuffer() : 0);
    return QoreValue();
}

// nothing HTTPClient::setEncoding(string encoding){}
static QoreValue HTTPClient_setEncoding_Vs(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* encoding = HARD_QORE_VALUE_STRING(args, 0);
# 1010 "QC_HTTPClient.qpp"
    client->setEncoding(QEM.findCreate(encoding));
    return QoreValue();
}

// nothing HTTPClient::setEventQueue(){}
static QoreValue HTTPClient_setEventQueue(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1169 "QC_HTTPClient.qpp"
    client->setEventQueue(0, xsink);
    return QoreValue();
}

// nothing HTTPClient::setEventQueue(Qore::Thread::Queue queue){}
static QoreValue HTTPClient_setEventQueue_C5Queue(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(queue, Queue, args, 0, CID_QUEUE, "HTTPClient::setEventQueue()", "Qore::Thread::Queue", xsink);
    if (*xsink)
        return 0;
# 1185 "QC_HTTPClient.qpp"
    ReferenceHolder<Queue> q(queue, xsink);
   if (queue->getMax() != -1)
      return xsink->raiseException("QUEUE-ERROR", "the Queue object passed as a maximum size of %d entr%s, which could cause internal I/O to freeze in the HttpClient object if used; use a Queue object with no maximum size for I/O monitoring", queue->getMax(), queue->getMax() == 1 ? "y" : "ies");
   // pass reference from QoreObject::getReferencedPrivateData() to function
   client->setEventQueue(q.release(), xsink);
    return QoreValue();
}

// nothing HTTPClient::setHTTPVersion(string ver){}
static QoreValue HTTPClient_setHTTPVersion_Vs(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* ver = HARD_QORE_VALUE_STRING(args, 0);
# 219 "QC_HTTPClient.qpp"
    client->setHTTPVersion(ver->getBuffer(), xsink);
    return QoreValue();
}

// nothing HTTPClient::setMaxRedirects(softint mr = 0){}
static QoreValue HTTPClient_setMaxRedirects_vi(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 mr = HARD_QORE_VALUE_INT(args, 0);
# 1146 "QC_HTTPClient.qpp"
    client->setMaxRedirects((int)mr);
    return QoreValue();
}

// int HTTPClient::setNoDelay(softbool b = True){}
static QoreValue HTTPClient_setNoDelay_vb(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    bool b = HARD_QORE_VALUE_BOOL(args, 0);
# 1235 "QC_HTTPClient.qpp"
    return client->setNoDelay(b);
}

// nothing HTTPClient::setPersistent(){}
static QoreValue HTTPClient_setPersistent(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1493 "QC_HTTPClient.qpp"
    client->setPersistent(xsink);
    return QoreValue();
}

// nothing HTTPClient::setProxySecure(softbool b = True){}
static QoreValue HTTPClient_setProxySecure_vb(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    bool b = HARD_QORE_VALUE_BOOL(args, 0);
# 1120 "QC_HTTPClient.qpp"
    client->setProxySecure(b);
    return QoreValue();
}

// nothing HTTPClient::setProxyURL(){}
static QoreValue HTTPClient_setProxyURL(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1067 "QC_HTTPClient.qpp"
    client->clearProxyURL();
    return QoreValue();
}

// nothing HTTPClient::setProxyURL(string url){}
static QoreValue HTTPClient_setProxyURL_Vs(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* url = HARD_QORE_VALUE_STRING(args, 0);
# 1084 "QC_HTTPClient.qpp"
    client->setProxyURL(url->getBuffer(), xsink);
    return QoreValue();
}

// nothing HTTPClient::setProxyUserPassword(string user, string pass){}
static QoreValue HTTPClient_setProxyUserPassword_VsVs(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* user = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* pass = HARD_QORE_VALUE_STRING(args, 1);
# 1318 "QC_HTTPClient.qpp"
    client->setProxyUserPassword(user->getBuffer(), pass->getBuffer());
    return QoreValue();
}

// nothing HTTPClient::setProxyUserPassword(){}
static QoreValue HTTPClient_setProxyUserPassword(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1334 "QC_HTTPClient.qpp"
    client->clearProxyUserPassword();
    return QoreValue();
}

// nothing HTTPClient::setSecure(softbool secure = True){}
static QoreValue HTTPClient_setSecure_vb(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    bool secure = HARD_QORE_VALUE_BOOL(args, 0);
# 247 "QC_HTTPClient.qpp"
    client->setSecure(secure);
    return QoreValue();
}

// nothing HTTPClient::setTimeout(timeout timeout_ms = 0){}
static QoreValue HTTPClient_setTimeout_Vt(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 986 "QC_HTTPClient.qpp"
    client->setTimeout((int)timeout_ms);
    return QoreValue();
}

// nothing HTTPClient::setURL(string url){}
static QoreValue HTTPClient_setURL_Vs(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* url = HARD_QORE_VALUE_STRING(args, 0);
# 1043 "QC_HTTPClient.qpp"
    client->setURL(url->getBuffer(), xsink);
    return QoreValue();
}

// nothing HTTPClient::setUserPassword(string user, string pass){}
static QoreValue HTTPClient_setUserPassword_VsVs(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* user = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* pass = HARD_QORE_VALUE_STRING(args, 1);
# 1275 "QC_HTTPClient.qpp"
    client->setUserPassword(user->getBuffer(), pass->getBuffer());
    return QoreValue();
}

// nothing HTTPClient::setUserPassword(){}
static QoreValue HTTPClient_setUserPassword(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1291 "QC_HTTPClient.qpp"
    client->clearUserPassword();
    return QoreValue();
}

// nothing HTTPClient::setWarningQueue(int warning_ms, int warning_bs, Queue queue, auto arg, timeout min_ms = 1s){}
static QoreValue HTTPClient_setWarningQueue_ViViC5QueueVAVt(QoreObject* self, QoreHttpClientObject* client, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 warning_ms = HARD_QORE_VALUE_INT(args, 0);
    int64 warning_bs = HARD_QORE_VALUE_INT(args, 1);
    HARD_QORE_VALUE_OBJ_DATA(queue, Queue, args, 2, CID_QUEUE, "HTTPClient::setWarningQueue()", "Queue", xsink);
    if (*xsink)
        return 0;
    QoreValue arg = get_param_value(args, 3);
    int64 min_ms = HARD_QORE_VALUE_INT(args, 4);
# 1435 "QC_HTTPClient.qpp"
    ReferenceHolder<Queue> q(queue, xsink);
   if (queue->getMax() != -1)
      return xsink->raiseException("QUEUE-ERROR", "the Queue object passed as a maximum size of %d entr%s, which could cause internal I/O to freeze in the Socket object if used; use a Queue object with no maximum size for I/O monitoring", queue->getMax(), queue->getMax() == 1 ? "y" : "ies");
   // pass reference from QoreObject::getReferencedPrivateData() to function
   client->setWarningQueue(xsink, warning_ms, warning_bs, q.release(), arg.getReferencedValue(), min_ms);
    return QoreValue();
}

DLLLOCAL void preinitHTTPClientClass() {
    QC_HTTPCLIENT = new QoreClass("HTTPClient", QDOM_NETWORK);
    CID_HTTPCLIENT = QC_HTTPCLIENT->getID();
    QC_HTTPCLIENT->setSystem();
}

DLLLOCAL QoreClass* initHTTPClientClass(QoreNamespace& ns) {
    if (!QC_HTTPCLIENT)
        preinitHTTPClientClass();

    // set parent class
    assert(QC_SOCKET);
    QC_HTTPCLIENT->addBuiltinVirtualBaseClass(QC_SOCKET);

    // nothing HTTPClient::clearProxyURL(){}
    QC_HTTPCLIENT->addMethod("clearProxyURL", (q_method_n_t)HTTPClient_clearProxyURL, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing HTTPClient::clearProxyUserPassword(){}
    QC_HTTPCLIENT->addMethod("clearProxyUserPassword", (q_method_n_t)HTTPClient_clearProxyUserPassword, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing HTTPClient::clearStats(){}
    QC_HTTPCLIENT->addMethod("clearStats", (q_method_n_t)HTTPClient_clearStats, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing HTTPClient::clearUserPassword(){}
    QC_HTTPCLIENT->addMethod("clearUserPassword", (q_method_n_t)HTTPClient_clearUserPassword, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing HTTPClient::clearWarningQueue(){}
    QC_HTTPCLIENT->addMethod("clearWarningQueue", (q_method_n_t)HTTPClient_clearWarningQueue, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing HTTPClient::connect(){}
    QC_HTTPCLIENT->addMethod("connect", (q_method_n_t)HTTPClient_connect, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // HTTPClient::constructor(hash opts) {}
    QC_HTTPCLIENT->addConstructor(HTTPClient_constructor_Vh, false, QC_NO_FLAGS, QDOM_DEFAULT, 1, hashTypeInfo, NULL, "opts");

    // HTTPClient::constructor() {}
    QC_HTTPCLIENT->addConstructor(HTTPClient_constructor, false, QC_NO_FLAGS, QDOM_DEFAULT);

    // HTTPClient::copy() {}
    QC_HTTPCLIENT->setCopy((q_copy_t)HTTPClient_copy);

    // HTTPClient::destructor() {}
    QC_HTTPCLIENT->setDestructor((q_destructor_t)HTTPClient_destructor);

    // nothing HTTPClient::disconnect(){}
    QC_HTTPCLIENT->addMethod("disconnect", (q_method_n_t)HTTPClient_disconnect, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // *string HTTPClient::get(string path, *hash headers, *reference<hash> info){}
    QC_HTTPCLIENT->addMethod("get", (q_method_n_t)HTTPClient_get_VsNhC16_reference_hash_, false, QC_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo, 3, stringTypeInfo, NULL, "path", hashOrNothingTypeInfo, NULL, "headers", qore_get_complex_reference_or_nothing_type(hashTypeInfo), NULL, "info");

    // int HTTPClient::getConnectTimeout(){}
    QC_HTTPCLIENT->addMethod("getConnectTimeout", (q_method_n_t)HTTPClient_getConnectTimeout, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // *string HTTPClient::getConnectionPath(){}
    QC_HTTPCLIENT->addMethod("getConnectionPath", (q_method_n_t)HTTPClient_getConnectionPath, false, QC_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // string HTTPClient::getDefaultPath(){}
    QC_HTTPCLIENT->addMethod("getDefaultPath", (q_method_n_t)HTTPClient_getDefaultPath, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // string HTTPClient::getEncoding(){}
    QC_HTTPCLIENT->addMethod("getEncoding", (q_method_n_t)HTTPClient_getEncoding, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // string HTTPClient::getHTTPVersion(){}
    QC_HTTPCLIENT->addMethod("getHTTPVersion", (q_method_n_t)HTTPClient_getHTTPVersion, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // int HTTPClient::getMaxRedirects(){}
    QC_HTTPCLIENT->addMethod("getMaxRedirects", (q_method_n_t)HTTPClient_getMaxRedirects, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // bool HTTPClient::getNoDelay(){}
    QC_HTTPCLIENT->addMethod("getNoDelay", (q_method_n_t)HTTPClient_getNoDelay, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // *string HTTPClient::getProxyURL(){}
    QC_HTTPCLIENT->addMethod("getProxyURL", (q_method_n_t)HTTPClient_getProxyURL, false, QC_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // int HTTPClient::getTimeout(){}
    QC_HTTPCLIENT->addMethod("getTimeout", (q_method_n_t)HTTPClient_getTimeout, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // *string HTTPClient::getURL(){}
    QC_HTTPCLIENT->addMethod("getURL", (q_method_n_t)HTTPClient_getURL, false, QC_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // hash HTTPClient::getUsageInfo(){}
    QC_HTTPCLIENT->addMethod("getUsageInfo", (q_method_n_t)HTTPClient_getUsageInfo, false, QC_CONSTANT, QDOM_DEFAULT, hashTypeInfo);

    // hash HTTPClient::head(string path, *hash headers, *reference<hash> info){}
    QC_HTTPCLIENT->addMethod("head", (q_method_n_t)HTTPClient_head_VsNhC16_reference_hash_, false, QC_NO_FLAGS, QDOM_DEFAULT, hashTypeInfo, 3, stringTypeInfo, NULL, "path", hashOrNothingTypeInfo, NULL, "headers", qore_get_complex_reference_or_nothing_type(hashTypeInfo), NULL, "info");

    // bool HTTPClient::isConnected(){}
    QC_HTTPCLIENT->addMethod("isConnected", (q_method_n_t)HTTPClient_isConnected, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // bool HTTPClient::isProxySecure(){}
    QC_HTTPCLIENT->addMethod("isProxySecure", (q_method_n_t)HTTPClient_isProxySecure, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // bool HTTPClient::isSecure(){}
    QC_HTTPCLIENT->addMethod("isSecure", (q_method_n_t)HTTPClient_isSecure, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // *string HTTPClient::post(string path, string body, *hash headers, *reference<hash> info){}
    QC_HTTPCLIENT->addMethod("post", (q_method_n_t)HTTPClient_post_VsVsNhC16_reference_hash_, false, QC_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo, 4, stringTypeInfo, NULL, "path", stringTypeInfo, NULL, "body", hashOrNothingTypeInfo, NULL, "headers", qore_get_complex_reference_or_nothing_type(hashTypeInfo), NULL, "info");

    // *string HTTPClient::post(string path, *binary body, *hash headers, *reference<hash> info){}
    QC_HTTPCLIENT->addMethod("post", (q_method_n_t)HTTPClient_post_VsNyNhC16_reference_hash_, false, QC_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo, 4, stringTypeInfo, NULL, "path", binaryOrNothingTypeInfo, NULL, "body", hashOrNothingTypeInfo, NULL, "headers", qore_get_complex_reference_or_nothing_type(hashTypeInfo), NULL, "info");

    // hash HTTPClient::send(string body, string method, *string path, *hash headers, softbool getbody = False, *reference<hash> info){}
    QC_HTTPCLIENT->addMethod("send", (q_method_n_t)HTTPClient_send_VsVsNsNhvbC16_reference_hash_, false, QC_NO_FLAGS, QDOM_DEFAULT, hashTypeInfo, 6, stringTypeInfo, NULL, "body", stringTypeInfo, NULL, "method", stringOrNothingTypeInfo, NULL, "path", hashOrNothingTypeInfo, NULL, "headers", softBoolTypeInfo, &False, "getbody", qore_get_complex_reference_or_nothing_type(hashTypeInfo), NULL, "info");

    // hash HTTPClient::send(*binary body, string method, *string path, *hash headers, softbool getbody = False, *reference<hash> info){}
    QC_HTTPCLIENT->addMethod("send", (q_method_n_t)HTTPClient_send_NyVsNsNhvbC16_reference_hash_, false, QC_NO_FLAGS, QDOM_DEFAULT, hashTypeInfo, 6, binaryOrNothingTypeInfo, NULL, "body", stringTypeInfo, NULL, "method", stringOrNothingTypeInfo, NULL, "path", hashOrNothingTypeInfo, NULL, "headers", softBoolTypeInfo, &False, "getbody", qore_get_complex_reference_or_nothing_type(hashTypeInfo), NULL, "info");

    // nothing HTTPClient::send(Qore::OutputStream os, *data body, string method, *string path, *hash headers, timeout timeout_ms = 0, softbool getbody = False, *reference<hash> info, *code rcb){}
    QC_HTTPCLIENT->addMethod("send", (q_method_n_t)HTTPClient_send_C12OutputStreamNDVsNsNhVtvbC16_reference_hash_Nc, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 9, QC_OUTPUTSTREAM->getTypeInfo(), NULL, "os", dataOrNothingTypeInfo, NULL, "body", stringTypeInfo, NULL, "method", stringOrNothingTypeInfo, NULL, "path", hashOrNothingTypeInfo, NULL, "headers", timeoutTypeInfo, zero(), "timeout_ms", softBoolTypeInfo, &False, "getbody", qore_get_complex_reference_or_nothing_type(hashTypeInfo), NULL, "info", codeOrNothingTypeInfo, NULL, "rcb");

    // nothing HTTPClient::sendChunked(Qore::OutputStream os, Qore::InputStream is, string method, int max_chunk_size = 4096, *string path, *hash headers, timeout timeout_ms = 0, softbool getbody = False, *reference<hash> info, *code rcb, *code tcb){}
    QC_HTTPCLIENT->addMethod("sendChunked", (q_method_n_t)HTTPClient_sendChunked_C12OutputStreamC11InputStreamVsViNsNhVtvbC16_reference_hash_NcNc, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 11, QC_OUTPUTSTREAM->getTypeInfo(), NULL, "os", QC_INPUTSTREAM->getTypeInfo(), NULL, "is", stringTypeInfo, NULL, "method", bigIntTypeInfo, new QoreBigIntNode(4096), "max_chunk_size", stringOrNothingTypeInfo, NULL, "path", hashOrNothingTypeInfo, NULL, "headers", timeoutTypeInfo, zero(), "timeout_ms", softBoolTypeInfo, &False, "getbody", qore_get_complex_reference_or_nothing_type(hashTypeInfo), NULL, "info", codeOrNothingTypeInfo, NULL, "rcb", codeOrNothingTypeInfo, NULL, "tcb");

    // nothing HTTPClient::sendWithCallbacks(code scb, code rcb, string method, *string path, *hash headers, timeout timeout_ms = 0, softbool getbody = False, *reference<hash> info){}
    QC_HTTPCLIENT->addMethod("sendWithCallbacks", (q_method_n_t)HTTPClient_sendWithCallbacks_VcVcVsNsNhVtvbC16_reference_hash_, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 8, codeTypeInfo, NULL, "scb", codeTypeInfo, NULL, "rcb", stringTypeInfo, NULL, "method", stringOrNothingTypeInfo, NULL, "path", hashOrNothingTypeInfo, NULL, "headers", timeoutTypeInfo, zero(), "timeout_ms", softBoolTypeInfo, &False, "getbody", qore_get_complex_reference_or_nothing_type(hashTypeInfo), NULL, "info");

    // nothing HTTPClient::sendWithRecvCallback(code rcb, string body, string method, *string path, *hash headers, timeout timeout_ms = 0, softbool getbody = False, *reference<hash> info){}
    QC_HTTPCLIENT->addMethod("sendWithRecvCallback", (q_method_n_t)HTTPClient_sendWithRecvCallback_VcVsVsNsNhVtvbC16_reference_hash_, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 8, codeTypeInfo, NULL, "rcb", stringTypeInfo, NULL, "body", stringTypeInfo, NULL, "method", stringOrNothingTypeInfo, NULL, "path", hashOrNothingTypeInfo, NULL, "headers", timeoutTypeInfo, zero(), "timeout_ms", softBoolTypeInfo, &False, "getbody", qore_get_complex_reference_or_nothing_type(hashTypeInfo), NULL, "info");

    // nothing HTTPClient::sendWithRecvCallback(code rcb, *binary body, string method, *string path, *hash headers, timeout timeout_ms = 0, softbool getbody = False, *reference<hash> info){}
    QC_HTTPCLIENT->addMethod("sendWithRecvCallback", (q_method_n_t)HTTPClient_sendWithRecvCallback_VcNyVsNsNhVtvbC16_reference_hash_, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 8, codeTypeInfo, NULL, "rcb", binaryOrNothingTypeInfo, NULL, "body", stringTypeInfo, NULL, "method", stringOrNothingTypeInfo, NULL, "path", hashOrNothingTypeInfo, NULL, "headers", timeoutTypeInfo, zero(), "timeout_ms", softBoolTypeInfo, &False, "getbody", qore_get_complex_reference_or_nothing_type(hashTypeInfo), NULL, "info");

    // hash HTTPClient::sendWithSendCallback(code scb, string method, *string path, *hash headers, timeout timeout_ms = 0, softbool getbody = False, *reference<hash> info){}
    QC_HTTPCLIENT->addMethod("sendWithSendCallback", (q_method_n_t)HTTPClient_sendWithSendCallback_VcVsNsNhVtvbC16_reference_hash_, false, QC_NO_FLAGS, QDOM_DEFAULT, hashTypeInfo, 7, codeTypeInfo, NULL, "scb", stringTypeInfo, NULL, "method", stringOrNothingTypeInfo, NULL, "path", hashOrNothingTypeInfo, NULL, "headers", timeoutTypeInfo, zero(), "timeout_ms", softBoolTypeInfo, &False, "getbody", qore_get_complex_reference_or_nothing_type(hashTypeInfo), NULL, "info");

    // nothing HTTPClient::setConnectTimeout(timeout timeout_ms = -1){}
    QC_HTTPCLIENT->addMethod("setConnectTimeout", (q_method_n_t)HTTPClient_setConnectTimeout_Vt, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // nothing HTTPClient::setDefaultPath(*string path){}
    QC_HTTPCLIENT->addMethod("setDefaultPath", (q_method_n_t)HTTPClient_setDefaultPath_Ns, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringOrNothingTypeInfo, NULL, "path");

    // nothing HTTPClient::setEncoding(string encoding){}
    QC_HTTPCLIENT->addMethod("setEncoding", (q_method_n_t)HTTPClient_setEncoding_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "encoding");

    // nothing HTTPClient::setEventQueue(){}
    QC_HTTPCLIENT->addMethod("setEventQueue", (q_method_n_t)HTTPClient_setEventQueue, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing HTTPClient::setEventQueue(Qore::Thread::Queue queue){}
    QC_HTTPCLIENT->addMethod("setEventQueue", (q_method_n_t)HTTPClient_setEventQueue_C5Queue, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, QC_QUEUE->getTypeInfo(), NULL, "queue");

    // nothing HTTPClient::setHTTPVersion(string ver){}
    QC_HTTPCLIENT->addMethod("setHTTPVersion", (q_method_n_t)HTTPClient_setHTTPVersion_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "ver");

    // nothing HTTPClient::setMaxRedirects(softint mr = 0){}
    QC_HTTPCLIENT->addMethod("setMaxRedirects", (q_method_n_t)HTTPClient_setMaxRedirects_vi, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, softBigIntTypeInfo, zero(), "mr");

    // int HTTPClient::setNoDelay(softbool b = True){}
    QC_HTTPCLIENT->addMethod("setNoDelay", (q_method_n_t)HTTPClient_setNoDelay_vb, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, softBoolTypeInfo, &True, "b");

    // nothing HTTPClient::setPersistent(){}
    QC_HTTPCLIENT->addMethod("setPersistent", (q_method_n_t)HTTPClient_setPersistent, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing HTTPClient::setProxySecure(softbool b = True){}
    QC_HTTPCLIENT->addMethod("setProxySecure", (q_method_n_t)HTTPClient_setProxySecure_vb, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, softBoolTypeInfo, &True, "b");

    // nothing HTTPClient::setProxyURL(){}
    QC_HTTPCLIENT->addMethod("setProxyURL", (q_method_n_t)HTTPClient_setProxyURL, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing HTTPClient::setProxyURL(string url){}
    QC_HTTPCLIENT->addMethod("setProxyURL", (q_method_n_t)HTTPClient_setProxyURL_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "url");

    // nothing HTTPClient::setProxyUserPassword(string user, string pass){}
    QC_HTTPCLIENT->addMethod("setProxyUserPassword", (q_method_n_t)HTTPClient_setProxyUserPassword_VsVs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, NULL, "user", stringTypeInfo, NULL, "pass");

    // nothing HTTPClient::setProxyUserPassword(){}
    QC_HTTPCLIENT->addMethod("setProxyUserPassword", (q_method_n_t)HTTPClient_setProxyUserPassword, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing HTTPClient::setSecure(softbool secure = True){}
    QC_HTTPCLIENT->addMethod("setSecure", (q_method_n_t)HTTPClient_setSecure_vb, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, softBoolTypeInfo, &True, "secure");

    // nothing HTTPClient::setTimeout(timeout timeout_ms = 0){}
    QC_HTTPCLIENT->addMethod("setTimeout", (q_method_n_t)HTTPClient_setTimeout_Vt, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, timeoutTypeInfo, zero(), "timeout_ms");

    // nothing HTTPClient::setURL(string url){}
    QC_HTTPCLIENT->addMethod("setURL", (q_method_n_t)HTTPClient_setURL_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "url");

    // nothing HTTPClient::setUserPassword(string user, string pass){}
    QC_HTTPCLIENT->addMethod("setUserPassword", (q_method_n_t)HTTPClient_setUserPassword_VsVs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, NULL, "user", stringTypeInfo, NULL, "pass");

    // nothing HTTPClient::setUserPassword(){}
    QC_HTTPCLIENT->addMethod("setUserPassword", (q_method_n_t)HTTPClient_setUserPassword, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing HTTPClient::setWarningQueue(int warning_ms, int warning_bs, Queue queue, auto arg, timeout min_ms = 1s){}
    QC_HTTPCLIENT->addMethod("setWarningQueue", (q_method_n_t)HTTPClient_setWarningQueue_ViViC5QueueVAVt, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 5, bigIntTypeInfo, NULL, "warning_ms", bigIntTypeInfo, NULL, "warning_bs", QC_QUEUE->getTypeInfo(), NULL, "queue", autoTypeInfo, NULL, "arg", timeoutTypeInfo, DateTimeNode::makeRelative(0, 0, 0, 0, 0, 1, 0), "min_ms");

    return QC_HTTPCLIENT;
}
