/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  QC_Program.qpp

  Qore Programming Language

  Copyright (C) 2003 - 2017 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/QC_Program.h"
#include "qore/intern/QC_ProgramControl.h"
#include "qore/intern/QC_TimeZone.h"
#include "qore/intern/QC_Breakpoint.h"
#include "qore/intern/qore_program_private.h"
#include "qore/intern/QoreObjectIntern.h"
#include "qore/intern/ModuleInfo.h"
#include "qore/ParseOptionMap.h"

#include "qore/intern/ModuleInfo.h"
extern QoreModuleManager QMM;
/* Qore class Qore::Program */

qore_classid_t CID_PROGRAM;
QoreClass* QC_PROGRAM;

// auto Program::callFunction(string name, ...){}
static QoreValue Program_callFunction_VsVV(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 930 "QC_Program.qpp"
    TempEncodingHelper tmp(name, QCS_DEFAULT, xsink);
   if (!tmp)
      return QoreValue();

   ReferenceHolder<QoreListNode> vargs(xsink);
   if (args->size() > 1)
      vargs = args->getOldList(1);

   return p->callFunction(tmp->getBuffer(), *vargs, xsink);
}

// auto Program::callFunctionArgs(string name, *softlist vargs){}
static QoreValue Program_callFunctionArgs_Vsnl(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
    const QoreListNode* vargs = get_param_value(args, 1).get<const QoreListNode>();
# 961 "QC_Program.qpp"
    TempEncodingHelper tmp(name, QCS_DEFAULT, xsink);
   if (!tmp)
      return QoreValue();
   return p->callFunction(tmp->getBuffer(), vargs, xsink);
}

// Program::constructor(softint po = PO_DEFAULT) {}
static void Program_constructor_vi(QoreObject* self, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 po = HARD_QORE_VALUE_INT(args, 0);
# 692 "QC_Program.qpp"
if (po & PO_SYSTEM_OPS) {
      xsink->raiseException("PROGRAM-OPTION-ERROR", "parse options (0x" QLLX ") contain restricted options that can only be set by the system", po);
      return;
   }

   QoreProgram* pgm = new QoreProgram(getProgram(), po);
   //pgm->registerQoreObject(self, xsink);
   assert(!xsink->isEvent());

   self->setPrivate(CID_PROGRAM, pgm);
   printd(5, "Program::constructor() pgm: %p, pgmid: %d, self: %p\n", pgm, pgm->getProgramId(), self);
}

// Program::copy() {}
static void Program_copy(QoreObject* self, QoreObject* old, QoreProgram* p, ExceptionSink* xsink) {
# 710 "QC_Program.qpp"
xsink->raiseException("PROGRAM-COPY-ERROR", "copying Program objects is currently unsupported");
}

// nothing Program::define(string def, auto val){}
static QoreValue Program_define_VsVA(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* def = HARD_QORE_VALUE_STRING(args, 0);
    QoreValue val = get_param_value(args, 1);
# 1418 "QC_Program.qpp"
    TempEncodingHelper t(def, QCS_DEFAULT, xsink);
   if (t)
      qore_program_private::runTimeDefine(p, t->getBuffer(), val.getReferencedValue(), xsink);
    return QoreValue();
}

// Program::destructor() {}
static void Program_destructor(QoreObject* self, QoreProgram* p, ExceptionSink* xsink) {
# 717 "QC_Program.qpp"
printd(5, "Program::destructor() pgm: %p, pgmid: %d, self: %p\n", p, p->getProgramId(), self);
   // inherited system object destructor is not executed
   //p->unregisterQoreObject(self, xsink);

   p->waitForTerminationAndDeref(xsink);
   /*
   if (p->is_unique()) {
      p->waitForTerminationAndDeref(xsink);
   } else {
      // if the underlying object is created from C++ then it has reference and cannot be terminated
      p->deref(xsink);
   }
   */
}

// nothing Program::disableParseOptions(softint opt){}
static QoreValue Program_disableParseOptions_vi(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 opt = HARD_QORE_VALUE_INT(args, 0);
# 1217 "QC_Program.qpp"
    if (opt & PO_SYSTEM_OPS)
      return xsink->raiseException("PROGRAM-OPTION-ERROR", "parse options (0x" QLLX ") contain restricted options that can only be set by the system", opt);

   p->disableParseOptions(opt, xsink);
    return QoreValue();
}

// bool Program::existsFunction(string name){}
static QoreValue Program_existsFunction_Vs(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 981 "QC_Program.qpp"
    TempEncodingHelper tmp(name, QCS_DEFAULT, xsink);
   if (!tmp)
      return false;
   return p->existsFunction(tmp->getBuffer());
}

// list Program::findFunctionVariants(string function){}
static QoreValue Program_findFunctionVariants_Vs(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* function = HARD_QORE_VALUE_STRING(args, 0);
# 1774 "QC_Program.qpp"
    ReferenceHolder<QoreValueList> vl(p->runtimeFindCallVariants(function->c_str(), xsink), xsink);
   if (!vl)
      return xsink->raiseException("PROGRAM-ERROR", "cannot find any function \"%s\"", function->c_str());
   return vl->getOldList();
}

// *hash Program::getAllDefines(){}
static QoreValue Program_getAllDefines(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1480 "QC_Program.qpp"
    return qore_program_private::runTimeGetAllDefines(p);
}

// auto Program::getDefine(string def){}
static QoreValue Program_getDefine_Vs(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* def = HARD_QORE_VALUE_STRING(args, 0);
# 1463 "QC_Program.qpp"
    TempEncodingHelper t(def, QCS_DEFAULT, xsink);
   return t ? qore_program_private::runTimeGetDefine(p, t->getBuffer()) : 0;
}

// auto Program::getGlobalVariable(string varname, *reference<bool> rexists){}
static QoreValue Program_getGlobalVariable_VsC16_reference_bool_(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* varname = HARD_QORE_VALUE_STRING(args, 0);
    const ReferenceNode* rexists = get_param_value(args, 1).get<const ReferenceNode>();
# 1321 "QC_Program.qpp"
    TempEncodingHelper t(varname, QCS_DEFAULT, xsink);
   if (!t)
      return QoreValue();

    bool found;
    ReferenceHolder<AbstractQoreNode> rv(p->getGlobalVariableValue(t->getBuffer(), found), xsink);

    if (rexists) {
	QoreTypeSafeReferenceHelper r(rexists, xsink);
	if (!r)
	    return QoreValue();

	if (r.assign(get_bool_node(found), xsink))
	    return QoreValue();
    }

    return rv.release();
}

// hash Program::getGlobalVars(){}
static QoreValue Program_getGlobalVars(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1738 "QC_Program.qpp"
    return p->getGlobalVars();
}

// int Program::getParseOptions(){}
static QoreValue Program_getParseOptions(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1174 "QC_Program.qpp"
    return p->getParseOptions64();
}

// ProgramControl Program::getProgram(){}
static QoreValue Program_getProgram(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1798 "QC_Program.qpp"
    return QoreProgram::getQoreObject(p);
}

// int Program::getProgramId(){}
static QoreValue Program_getProgramId(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1491 "QC_Program.qpp"
    return p->getProgramId();
}

// *string Program::getScriptDir(){}
static QoreValue Program_getScriptDir(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1265 "QC_Program.qpp"
    return p->getScriptDir();
}

// *string Program::getScriptName(){}
static QoreValue Program_getScriptName(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1279 "QC_Program.qpp"
    return p->getScriptName();
}

// *string Program::getScriptPath(){}
static QoreValue Program_getScriptPath(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1293 "QC_Program.qpp"
    return p->getScriptPath();
}

// list<int> Program::getThreadList(){}
static QoreValue Program_getThreadList(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1791 "QC_Program.qpp"
    return p->getThreadList();
}

// TimeZone Program::getTimeZone(){}
static QoreValue Program_getTimeZone(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1405 "QC_Program.qpp"
    return new QoreObject(QC_TIMEZONE, 0, new TimeZoneData(p->currentTZ()));
}

// softlist Program::getUserFunctionList(){}
static QoreValue Program_getUserFunctionList(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1162 "QC_Program.qpp"
    return p->getUserFunctionList();
}

// nothing Program::importClass(string cls, *string new_name, *softbool inject){}
static QoreValue Program_importClass_VsNsnb(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* cls = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* new_name = get_param_value(args, 1).get<const QoreStringNode>();
    bool inject = HARD_QORE_VALUE_BOOL(args, 2);
# 1096 "QC_Program.qpp"
    TempEncodingHelper tmp(cls, QCS_DEFAULT, xsink);
   if (!tmp) {
      assert(*xsink);
      return QoreValue();
   }

   TempEncodingHelper nn;
   if (new_name) {
      nn.set(new_name, QCS_DEFAULT, xsink);
      if (!nn) {
         assert(*xsink);
         return QoreValue();
      }

      if (nn->empty())
         return xsink->raiseException("CLASS-IMPORT-ERROR", "the new class name while importing class '%s' may not be empty", tmp->c_str());
   }
   qore_program_private::get(*p)->importClass(xsink, *qore_program_private::get(*getProgram()), tmp->c_str(), new_name ? nn->c_str() : nullptr, inject);
    return QoreValue();
}

// nothing Program::importFunction(string func){}
static QoreValue Program_importFunction_Vs(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* func = HARD_QORE_VALUE_STRING(args, 0);
# 1016 "QC_Program.qpp"
    TempEncodingHelper tmp(func, QCS_DEFAULT, xsink);
   if (!tmp)
      return QoreValue();
   qore_program_private::exportFunction(getProgram(), xsink, p, tmp->getBuffer());
    return QoreValue();
}

// nothing Program::importFunction(string func, string new_name, *softbool inject){}
static QoreValue Program_importFunction_VsVsnb(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* func = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* new_name = HARD_QORE_VALUE_STRING(args, 1);
    bool inject = HARD_QORE_VALUE_BOOL(args, 2);
# 1043 "QC_Program.qpp"
    TempEncodingHelper tfunc(func, QCS_DEFAULT, xsink);
   if (!tfunc)
      return QoreValue();
   TempEncodingHelper tnew_name(new_name, QCS_DEFAULT, xsink);
   if (!tnew_name)
      return QoreValue();
   if (tnew_name->empty())
      xsink->raiseException("FUNCTION-IMPORT-ERROR", "the new function name while importing function '%s' may not be empty", func);

   qore_program_private::exportFunction(getProgram(), xsink, p, tfunc->getBuffer(), tnew_name->getBuffer(), inject);
    return QoreValue();
}

// nothing Program::importGlobalVariable(string varname, bool readonly = False){}
static QoreValue Program_importGlobalVariable_VsVb(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* varname = HARD_QORE_VALUE_STRING(args, 0);
    bool readonly = HARD_QORE_VALUE_BOOL(args, 1);
# 1070 "QC_Program.qpp"
    TempEncodingHelper tmp(varname, QCS_DEFAULT, xsink);
   if (!tmp)
      return QoreValue();
   qore_program_private::exportGlobalVariable(getProgram(), tmp->getBuffer(), readonly, p, xsink);
    return QoreValue();
}

// nothing Program::importHashDecl(string name, *string new_name){}
static QoreValue Program_importHashDecl_VsNs(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* new_name = get_param_value(args, 1).get<const QoreStringNode>();
# 1133 "QC_Program.qpp"
    TempEncodingHelper tmp(name, QCS_DEFAULT, xsink);
    if (!tmp) {
       assert(*xsink);
       return QoreValue();
    }

    TempEncodingHelper nn;
    if (new_name) {
       nn.set(new_name, QCS_DEFAULT, xsink);
       if (!nn) {
          assert(*xsink);
          return QoreValue();
       }

       if (nn->empty())
          return xsink->raiseException("HASHDECL-IMPORT-ERROR", "the new hashdecl name while importing hashdecl '%s' may not be empty", tmp->c_str());
    }
    qore_program_private::get(*p)->importHashDecl(xsink, *qore_program_private::get(*getProgram()), tmp->c_str(), new_name ? nn->c_str() : nullptr);
    return QoreValue();
}

// nothing Program::importSystemApi(){}
static QoreValue Program_importSystemApi(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1700 "QC_Program.qpp"
    qore_program_private::runtimeImportSystemApi(*p, xsink);
    return QoreValue();
}

// nothing Program::importSystemClasses(){}
static QoreValue Program_importSystemClasses(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1636 "QC_Program.qpp"
    qore_program_private::runtimeImportSystemClasses(*p, xsink);
    return QoreValue();
}

// nothing Program::importSystemConstants(){}
static QoreValue Program_importSystemConstants(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1668 "QC_Program.qpp"
    qore_program_private::runtimeImportSystemConstants(*p, xsink);
    return QoreValue();
}

// nothing Program::importSystemFunctions(){}
static QoreValue Program_importSystemFunctions(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1684 "QC_Program.qpp"
    qore_program_private::runtimeImportSystemFunctions(*p, xsink);
    return QoreValue();
}

// nothing Program::importSystemHashDecls(){}
static QoreValue Program_importSystemHashDecls(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1652 "QC_Program.qpp"
    qore_program_private::runtimeImportSystemHashDecls(*p, xsink);
    return QoreValue();
}

// bool Program::isDefined(string def){}
static QoreValue Program_isDefined_Vs(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* def = HARD_QORE_VALUE_STRING(args, 0);
# 1448 "QC_Program.qpp"
    TempEncodingHelper t(def, QCS_DEFAULT, xsink);
   return t ? qore_program_private::runTimeIsDefined(p, t->getBuffer()) : false;
}

// nothing Program::loadApplyToPrivateUserModule(string name){}
static QoreValue Program_loadApplyToPrivateUserModule_Vs(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 1606 "QC_Program.qpp"
    // remove the QoreProgram private data from the object
  printd(5, "Program::loadApplyToPrivateUserModule pgm: %p, pgmid: %d, self: %p\n", p, p->getProgramId(), self);
#ifdef DEBUG
   QoreProgram* pgm = (QoreProgram*)
#endif
   qore_object_private::get(*self)->getAndRemovePrivateData(CID_PROGRAM, xsink);
#ifdef DEBUG
   assert(pgm == p);
#endif
   if (!*xsink) {
      QMM.runTimeLoadModule(*xsink, name->getBuffer(), 0, p, QMLO_INJECT | QMLO_PRIVATE);
      // getAndRemovePrivateData removed private data, so destructor is not executed. We must remove reference to this object
      //p->unregisterQoreObject(self, xsink);
      self->doDelete(xsink);
   }
    return QoreValue();
}

// nothing Program::loadApplyToUserModule(string name, *softbool reinject){}
static QoreValue Program_loadApplyToUserModule_Vsnb(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
    bool reinject = HARD_QORE_VALUE_BOOL(args, 1);
# 1570 "QC_Program.qpp"
    // remove the QoreProgram private data from the object
  printd(5, "Program::loadApplyToUserModule pgm: %p, pgmid: %d, self: %p\n", p, p->getProgramId(), self);
#ifdef DEBUG
   QoreProgram* pgm = (QoreProgram*)
#endif
   qore_object_private::get(*self)->getAndRemovePrivateData(CID_PROGRAM, xsink);
#ifdef DEBUG
   assert(pgm == p);
#endif
   if (!*xsink) {
      QMM.runTimeLoadModule(*xsink, name->getBuffer(), 0, p, QMLO_INJECT | (reinject ? QMLO_REINJECT : 0));
      // getAndRemovePrivateData removed private data, so destructor is not executed. We must remove reference to this object
      //p->unregisterQoreObject(self, xsink);
      self->doDelete(xsink);
   }
    return QoreValue();
}

// nothing Program::loadModule(string name){}
static QoreValue Program_loadModule_Vs(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 1518 "QC_Program.qpp"
    QMM.runTimeLoadModule(*xsink, name->getBuffer(), p);
    return QoreValue();
}

// nothing Program::loadUserModuleWithProgram(string name, Qore::Program pgm){}
static QoreValue Program_loadUserModuleWithProgram_VsC7Program(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
    TAKE_HARD_QORE_VALUE_OBJ_DATA(pgm, QoreProgram, args, 1, CID_PROGRAM, "Program::loadUserModuleWithProgram()", "Qore::Program", xsink);
    if (*xsink)
        return 0;
# 1547 "QC_Program.qpp"
    QMM.runTimeLoadModule(*xsink, name->getBuffer(), p, pgm, QMLO_INJECT);
    return QoreValue();
}

// nothing Program::lockOptions(){}
static QoreValue Program_lockOptions(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1304 "QC_Program.qpp"
    p->lockOptions();
    return QoreValue();
}

// *hash Program::parse(string code, string label, *softint warning_mask, *string source, *softint offset, softbool format_label = True){}
static QoreValue Program_parse_VsVsniNsnivb(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* code = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* label = HARD_QORE_VALUE_STRING(args, 1);
    int64 warning_mask = HARD_QORE_VALUE_INT(args, 2);
    const QoreStringNode* source = get_param_value(args, 3).get<const QoreStringNode>();
    int64 offset = HARD_QORE_VALUE_INT(args, 4);
    bool format_label = HARD_QORE_VALUE_BOOL(args, 5);
# 768 "QC_Program.qpp"
    QoreStringMaker nlabel(label->getEncoding(), format_label ? "<run-time-loaded: %s>" : "%s", label->getBuffer());

   if (warning_mask) {
      ExceptionSink wsink;
      p->parse(code, &nlabel, xsink, &wsink, warning_mask, source, offset);
      if (!wsink.isException())
         return QoreValue();

      QoreException *e = wsink.catchException();
      return e->makeExceptionObjectAndDelete(xsink);
   }

   p->parse(code, &nlabel, xsink, 0, 0, source, offset);
    return QoreValue();
}

// nothing Program::parseCommit(){}
static QoreValue Program_parseCommit(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 856 "QC_Program.qpp"
    p->parseCommit(xsink);
    return QoreValue();
}

// *hash Program::parseCommit(int warning_mask){}
static QoreValue Program_parseCommit_Vi(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 warning_mask = HARD_QORE_VALUE_INT(args, 0);
# 882 "QC_Program.qpp"
    ExceptionSink wsink;
   p->parseCommit(xsink, &wsink, warning_mask);
   if (!wsink.isException())
      return QoreValue();

   QoreException *e = wsink.catchException();
   return e->makeExceptionObjectAndDelete(xsink);
}

// *hash Program::parsePending(string code, string label, *softint warning_mask, *string source, *softint offset, softbool format_label = True){}
static QoreValue Program_parsePending_VsVsniNsnivb(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* code = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* label = HARD_QORE_VALUE_STRING(args, 1);
    int64 warning_mask = HARD_QORE_VALUE_INT(args, 2);
    const QoreStringNode* source = get_param_value(args, 3).get<const QoreStringNode>();
    int64 offset = HARD_QORE_VALUE_INT(args, 4);
    bool format_label = HARD_QORE_VALUE_BOOL(args, 5);
# 822 "QC_Program.qpp"
    QoreStringMaker nlabel(label->getEncoding(), format_label ? "<run-time-loaded: %s>" : "%s", label->getBuffer());

   if (warning_mask) {
      ExceptionSink wsink;
      p->parsePending(code, &nlabel, xsink, &wsink, warning_mask, source, offset);
      if (!wsink.isException())
         return QoreValue();

      QoreException *e = wsink.catchException();
      return e->makeExceptionObjectAndDelete(xsink);
   }

   p->parsePending(code, &nlabel, xsink, 0, 0, source, offset);
    return QoreValue();
}

// nothing Program::parseRollback(){}
static QoreValue Program_parseRollback(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 907 "QC_Program.qpp"
    p->parseRollback(xsink);
    return QoreValue();
}

// nothing Program::replaceParseOptions(softint opt){}
static QoreValue Program_replaceParseOptions_vi(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 opt = HARD_QORE_VALUE_INT(args, 0);
# 1239 "QC_Program.qpp"
    p->replaceParseOptions(opt, xsink);
    return QoreValue();
}

// auto Program::run(){}
static QoreValue Program_run(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 996 "QC_Program.qpp"
    return p->run(xsink);
}

// nothing Program::setGlobalVarValue(string name, auto value){}
static QoreValue Program_setGlobalVarValue_VsVA(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
    QoreValue value = get_param_value(args, 1);
# 1759 "QC_Program.qpp"
    p->setGlobalVarValue(name->c_str(), value.refSelf(), xsink);
    return QoreValue();
}

// nothing Program::setParseOptions(softint opt = PO_DEFAULT){}
static QoreValue Program_setParseOptions_vi(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 opt = HARD_QORE_VALUE_INT(args, 0);
# 1194 "QC_Program.qpp"
    if (opt & PO_SYSTEM_OPS)
      return xsink->raiseException("PROGRAM-OPTION-ERROR", "parse options (0x" QLLX ") contain restricted options that can only be set by the system", opt);

   p->setParseOptions(opt, xsink);
    return QoreValue();
}

// nothing Program::setScriptPath(*string path){}
static QoreValue Program_setScriptPath_Ns(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = get_param_value(args, 0).get<const QoreStringNode>();
# 1251 "QC_Program.qpp"
    p->setScriptPath(path ? path->getBuffer() : 0);
    return QoreValue();
}

// bool Program::setThreadInit(*code init){}
static QoreValue Program_setThreadInit_Nc(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const ResolvedCallReferenceNode* init = get_param_value(args, 0).get<const ResolvedCallReferenceNode>();
# 1722 "QC_Program.qpp"
    return qore_program_private::setThreadInit(*p, init, xsink);
}

// nothing Program::setTimeZone(TimeZone zone){}
static QoreValue Program_setTimeZone_C8TimeZone(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(zone, TimeZoneData, args, 0, CID_TIMEZONE, "Program::setTimeZone()", "TimeZone", xsink);
    if (*xsink)
        return 0;
# 1393 "QC_Program.qpp"
    p->setTZ(zone->get());
    return QoreValue();
}

// nothing Program::setTimeZoneRegion(string region){}
static QoreValue Program_setTimeZoneRegion_Vs(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* region = HARD_QORE_VALUE_STRING(args, 0);
# 1353 "QC_Program.qpp"
    const AbstractQoreZoneInfo *zone = QTZM.findLoadRegion(region->getBuffer(), xsink);
   if (!*xsink)
      p->setTZ(zone);
    return QoreValue();
}

// nothing Program::setTimeZoneUTCOffset(softint seconds_east){}
static QoreValue Program_setTimeZoneUTCOffset_vi(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 seconds_east = HARD_QORE_VALUE_INT(args, 0);
# 1378 "QC_Program.qpp"
    const AbstractQoreZoneInfo *zone = QTZM.findCreateOffsetZone(seconds_east);
   p->setTZ(zone);
    return QoreValue();
}

// nothing Program::undefine(string def){}
static QoreValue Program_undefine_Vs(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* def = HARD_QORE_VALUE_STRING(args, 0);
# 1432 "QC_Program.qpp"
    TempEncodingHelper t(def, QCS_DEFAULT, xsink);
   if (t)
      qore_program_private::runTimeUnDefine(p, t->getBuffer(), xsink);
    return QoreValue();
}

DLLLOCAL void preinitProgramClass() {
    QC_PROGRAM = new QoreClass("Program", QDOM_DEFAULT);
    CID_PROGRAM = QC_PROGRAM->getID();
    QC_PROGRAM->setSystem();
}

DLLLOCAL QoreClass* initProgramClass(QoreNamespace& ns) {
    if (!QC_PROGRAM)
        preinitProgramClass();

    // auto Program::callFunction(string name, ...){}
    QC_PROGRAM->addMethod("callFunction", (q_method_n_t)Program_callFunction_VsVV, false, QC_USES_EXTRA_ARGS, QDOM_DEFAULT, autoTypeInfo, 1, stringTypeInfo, NULL, "name");

    // auto Program::callFunctionArgs(string name, *softlist vargs){}
    QC_PROGRAM->addMethod("callFunctionArgs", (q_method_n_t)Program_callFunctionArgs_Vsnl, false, QC_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 2, stringTypeInfo, NULL, "name", softListOrNothingTypeInfo, NULL, "vargs");

    // Program::constructor(softint po = PO_DEFAULT) {}
    QC_PROGRAM->addConstructor(Program_constructor_vi, false, QC_NO_FLAGS, QDOM_EMBEDDED_LOGIC, 1, softBigIntTypeInfo, new QoreBigIntNode(PO_DEFAULT), "po");

    // Program::copy() {}
    QC_PROGRAM->setCopy((q_copy_t)Program_copy);

    // nothing Program::define(string def, auto val){}
    QC_PROGRAM->addMethod("define", (q_method_n_t)Program_define_VsVA, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, NULL, "def", autoTypeInfo, NULL, "val");

    // Program::destructor() {}
    QC_PROGRAM->setDestructor((q_destructor_t)Program_destructor);

    // nothing Program::disableParseOptions(softint opt){}
    QC_PROGRAM->addMethod("disableParseOptions", (q_method_n_t)Program_disableParseOptions_vi, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, softBigIntTypeInfo, NULL, "opt");

    // bool Program::existsFunction(string name){}
    QC_PROGRAM->addMethod("existsFunction", (q_method_n_t)Program_existsFunction_Vs, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo, 1, stringTypeInfo, NULL, "name");

    // list Program::findFunctionVariants(string function){}
    QC_PROGRAM->addMethod("findFunctionVariants", (q_method_n_t)Program_findFunctionVariants_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, listTypeInfo, 1, stringTypeInfo, NULL, "function");

    // *hash Program::getAllDefines(){}
    QC_PROGRAM->addMethod("getAllDefines", (q_method_n_t)Program_getAllDefines, false, QC_NO_FLAGS, QDOM_DEFAULT, hashOrNothingTypeInfo);

    // auto Program::getDefine(string def){}
    QC_PROGRAM->addMethod("getDefine", (q_method_n_t)Program_getDefine_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 1, stringTypeInfo, NULL, "def");

    // auto Program::getGlobalVariable(string varname, *reference<bool> rexists){}
    QC_PROGRAM->addMethod("getGlobalVariable", (q_method_n_t)Program_getGlobalVariable_VsC16_reference_bool_, false, QC_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 2, stringTypeInfo, NULL, "varname", qore_get_complex_reference_or_nothing_type(boolTypeInfo), NULL, "rexists");

    // hash Program::getGlobalVars(){}
    QC_PROGRAM->addMethod("getGlobalVars", (q_method_n_t)Program_getGlobalVars, false, QC_CONSTANT, QDOM_DEFAULT, hashTypeInfo);

    // int Program::getParseOptions(){}
    QC_PROGRAM->addMethod("getParseOptions", (q_method_n_t)Program_getParseOptions, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // ProgramControl Program::getProgram(){}
    QC_PROGRAM->addMethod("getProgram", (q_method_n_t)Program_getProgram, false, QC_RET_VALUE_ONLY, QDOM_DEBUGGER, QC_PROGRAMCONTROL->getTypeInfo());

    // int Program::getProgramId(){}
    QC_PROGRAM->addMethod("getProgramId", (q_method_n_t)Program_getProgramId, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo);

    // *string Program::getScriptDir(){}
    QC_PROGRAM->addMethod("getScriptDir", (q_method_n_t)Program_getScriptDir, false, QC_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // *string Program::getScriptName(){}
    QC_PROGRAM->addMethod("getScriptName", (q_method_n_t)Program_getScriptName, false, QC_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // *string Program::getScriptPath(){}
    QC_PROGRAM->addMethod("getScriptPath", (q_method_n_t)Program_getScriptPath, false, QC_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // list<int> Program::getThreadList(){}
    QC_PROGRAM->addMethod("getThreadList", (q_method_n_t)Program_getThreadList, false, QC_CONSTANT, QDOM_DEFAULT, qore_get_complex_list_type(bigIntTypeInfo));

    // TimeZone Program::getTimeZone(){}
    QC_PROGRAM->addMethod("getTimeZone", (q_method_n_t)Program_getTimeZone, false, QC_NO_FLAGS, QDOM_DEFAULT, QC_TIMEZONE->getTypeInfo());

    // softlist Program::getUserFunctionList(){}
    QC_PROGRAM->addMethod("getUserFunctionList", (q_method_n_t)Program_getUserFunctionList, false, QC_NO_FLAGS, QDOM_DEFAULT, softListTypeInfo);

    // nothing Program::importClass(string cls, *string new_name, *softbool inject){}
    QC_PROGRAM->addMethod("importClass", (q_method_n_t)Program_importClass_VsNsnb, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 3, stringTypeInfo, NULL, "cls", stringOrNothingTypeInfo, NULL, "new_name", softBoolOrNothingTypeInfo, NULL, "inject");

    // nothing Program::importFunction(string func){}
    QC_PROGRAM->addMethod("importFunction", (q_method_n_t)Program_importFunction_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "func");

    // nothing Program::importFunction(string func, string new_name, *softbool inject){}
    QC_PROGRAM->addMethod("importFunction", (q_method_n_t)Program_importFunction_VsVsnb, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 3, stringTypeInfo, NULL, "func", stringTypeInfo, NULL, "new_name", softBoolOrNothingTypeInfo, NULL, "inject");

    // nothing Program::importGlobalVariable(string varname, bool readonly = False){}
    QC_PROGRAM->addMethod("importGlobalVariable", (q_method_n_t)Program_importGlobalVariable_VsVb, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, NULL, "varname", boolTypeInfo, &False, "readonly");

    // nothing Program::importHashDecl(string name, *string new_name){}
    QC_PROGRAM->addMethod("importHashDecl", (q_method_n_t)Program_importHashDecl_VsNs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, NULL, "name", stringOrNothingTypeInfo, NULL, "new_name");

    // nothing Program::importSystemApi(){}
    QC_PROGRAM->addMethod("importSystemApi", (q_method_n_t)Program_importSystemApi, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing Program::importSystemClasses(){}
    QC_PROGRAM->addMethod("importSystemClasses", (q_method_n_t)Program_importSystemClasses, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing Program::importSystemConstants(){}
    QC_PROGRAM->addMethod("importSystemConstants", (q_method_n_t)Program_importSystemConstants, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing Program::importSystemFunctions(){}
    QC_PROGRAM->addMethod("importSystemFunctions", (q_method_n_t)Program_importSystemFunctions, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing Program::importSystemHashDecls(){}
    QC_PROGRAM->addMethod("importSystemHashDecls", (q_method_n_t)Program_importSystemHashDecls, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // bool Program::isDefined(string def){}
    QC_PROGRAM->addMethod("isDefined", (q_method_n_t)Program_isDefined_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo, 1, stringTypeInfo, NULL, "def");

    // nothing Program::loadApplyToPrivateUserModule(string name){}
    QC_PROGRAM->addMethod("loadApplyToPrivateUserModule", (q_method_n_t)Program_loadApplyToPrivateUserModule_Vs, false, QC_NO_FLAGS, QDOM_MODULES, nothingTypeInfo, 1, stringTypeInfo, NULL, "name");

    // nothing Program::loadApplyToUserModule(string name, *softbool reinject){}
    QC_PROGRAM->addMethod("loadApplyToUserModule", (q_method_n_t)Program_loadApplyToUserModule_Vsnb, false, QC_NO_FLAGS, QDOM_MODULES|QDOM_INJECTION, nothingTypeInfo, 2, stringTypeInfo, NULL, "name", softBoolOrNothingTypeInfo, NULL, "reinject");

    // nothing Program::loadModule(string name){}
    QC_PROGRAM->addMethod("loadModule", (q_method_n_t)Program_loadModule_Vs, false, QC_NO_FLAGS, QDOM_MODULES, nothingTypeInfo, 1, stringTypeInfo, NULL, "name");

    // nothing Program::loadUserModuleWithProgram(string name, Qore::Program pgm){}
    QC_PROGRAM->addMethod("loadUserModuleWithProgram", (q_method_n_t)Program_loadUserModuleWithProgram_VsC7Program, false, QC_NO_FLAGS, QDOM_MODULES|QDOM_INJECTION, nothingTypeInfo, 2, stringTypeInfo, NULL, "name", QC_PROGRAM->getTypeInfo(), NULL, "pgm");

    // nothing Program::lockOptions(){}
    QC_PROGRAM->addMethod("lockOptions", (q_method_n_t)Program_lockOptions, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // *hash Program::parse(string code, string label, *softint warning_mask, *string source, *softint offset, softbool format_label = True){}
    QC_PROGRAM->addMethod("parse", (q_method_n_t)Program_parse_VsVsniNsnivb, false, QC_NO_FLAGS, QDOM_EMBEDDED_LOGIC, hashOrNothingTypeInfo, 6, stringTypeInfo, NULL, "code", stringTypeInfo, NULL, "label", softBigIntOrNothingTypeInfo, NULL, "warning_mask", stringOrNothingTypeInfo, NULL, "source", softBigIntOrNothingTypeInfo, NULL, "offset", softBoolTypeInfo, &True, "format_label");

    // nothing Program::parseCommit(){}
    QC_PROGRAM->addMethod("parseCommit", (q_method_n_t)Program_parseCommit, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // *hash Program::parseCommit(int warning_mask){}
    QC_PROGRAM->addMethod("parseCommit", (q_method_n_t)Program_parseCommit_Vi, false, QC_NO_FLAGS, QDOM_DEFAULT, hashOrNothingTypeInfo, 1, bigIntTypeInfo, NULL, "warning_mask");

    // *hash Program::parsePending(string code, string label, *softint warning_mask, *string source, *softint offset, softbool format_label = True){}
    QC_PROGRAM->addMethod("parsePending", (q_method_n_t)Program_parsePending_VsVsniNsnivb, false, QC_NO_FLAGS, QDOM_EMBEDDED_LOGIC, hashOrNothingTypeInfo, 6, stringTypeInfo, NULL, "code", stringTypeInfo, NULL, "label", softBigIntOrNothingTypeInfo, NULL, "warning_mask", stringOrNothingTypeInfo, NULL, "source", softBigIntOrNothingTypeInfo, NULL, "offset", softBoolTypeInfo, &True, "format_label");

    // nothing Program::parseRollback(){}
    QC_PROGRAM->addMethod("parseRollback", (q_method_n_t)Program_parseRollback, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing Program::replaceParseOptions(softint opt){}
    QC_PROGRAM->addMethod("replaceParseOptions", (q_method_n_t)Program_replaceParseOptions_vi, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, softBigIntTypeInfo, NULL, "opt");

    // auto Program::run(){}
    QC_PROGRAM->addMethod("run", (q_method_n_t)Program_run, false, QC_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo);

    // nothing Program::setGlobalVarValue(string name, auto value){}
    QC_PROGRAM->addMethod("setGlobalVarValue", (q_method_n_t)Program_setGlobalVarValue_VsVA, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, NULL, "name", autoTypeInfo, NULL, "value");

    // nothing Program::setParseOptions(softint opt = PO_DEFAULT){}
    QC_PROGRAM->addMethod("setParseOptions", (q_method_n_t)Program_setParseOptions_vi, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, softBigIntTypeInfo, new QoreBigIntNode(PO_DEFAULT), "opt");

    // nothing Program::setScriptPath(*string path){}
    QC_PROGRAM->addMethod("setScriptPath", (q_method_n_t)Program_setScriptPath_Ns, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringOrNothingTypeInfo, NULL, "path");

    // bool Program::setThreadInit(*code init){}
    QC_PROGRAM->addMethod("setThreadInit", (q_method_n_t)Program_setThreadInit_Nc, false, QC_NO_FLAGS, QDOM_THREAD_CONTROL, boolTypeInfo, 1, codeOrNothingTypeInfo, NULL, "init");

    // nothing Program::setTimeZone(TimeZone zone){}
    QC_PROGRAM->addMethod("setTimeZone", (q_method_n_t)Program_setTimeZone_C8TimeZone, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, QC_TIMEZONE->getTypeInfo(), NULL, "zone");

    // nothing Program::setTimeZoneRegion(string region){}
    QC_PROGRAM->addMethod("setTimeZoneRegion", (q_method_n_t)Program_setTimeZoneRegion_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "region");

    // nothing Program::setTimeZoneUTCOffset(softint seconds_east){}
    QC_PROGRAM->addMethod("setTimeZoneUTCOffset", (q_method_n_t)Program_setTimeZoneUTCOffset_vi, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, softBigIntTypeInfo, NULL, "seconds_east");

    // nothing Program::undefine(string def){}
    QC_PROGRAM->addMethod("undefine", (q_method_n_t)Program_undefine_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "def");
# 47 "QC_Program.qpp"
# 58 "QC_Program.qpp"
# 65 "QC_Program.qpp"
# 72 "QC_Program.qpp"
# 79 "QC_Program.qpp"
# 86 "QC_Program.qpp"
# 93 "QC_Program.qpp"
# 96 "QC_Program.qpp"
# 101 "QC_Program.qpp"
# 106 "QC_Program.qpp"
# 109 "QC_Program.qpp"
# 112 "QC_Program.qpp"
# 118 "QC_Program.qpp"
# 125 "QC_Program.qpp"
# 132 "QC_Program.qpp"
# 138 "QC_Program.qpp"
# 144 "QC_Program.qpp"
# 150 "QC_Program.qpp"
# 156 "QC_Program.qpp"
# 162 "QC_Program.qpp"
# 165 "QC_Program.qpp"
# 172 "QC_Program.qpp"
# 178 "QC_Program.qpp"
# 184 "QC_Program.qpp"
# 190 "QC_Program.qpp"
# 196 "QC_Program.qpp"
# 202 "QC_Program.qpp"
# 209 "QC_Program.qpp"
# 215 "QC_Program.qpp"
# 221 "QC_Program.qpp"
# 227 "QC_Program.qpp"
# 237 "QC_Program.qpp"
# 243 "QC_Program.qpp"
# 249 "QC_Program.qpp"
# 256 "QC_Program.qpp"
# 262 "QC_Program.qpp"
# 269 "QC_Program.qpp"
# 275 "QC_Program.qpp"
# 281 "QC_Program.qpp"
# 287 "QC_Program.qpp"
# 293 "QC_Program.qpp"
# 299 "QC_Program.qpp"
# 305 "QC_Program.qpp"
# 312 "QC_Program.qpp"
# 318 "QC_Program.qpp"
# 325 "QC_Program.qpp"
# 332 "QC_Program.qpp"
# 338 "QC_Program.qpp"
# 347 "QC_Program.qpp"
# 355 "QC_Program.qpp"
# 362 "QC_Program.qpp"
# 369 "QC_Program.qpp"
# 378 "QC_Program.qpp"
# 385 "QC_Program.qpp"
# 392 "QC_Program.qpp"
# 403 "QC_Program.qpp"
# 414 "QC_Program.qpp"
# 430 "QC_Program.qpp"
# 437 "QC_Program.qpp"
# 444 "QC_Program.qpp"
# 451 "QC_Program.qpp"
# 458 "QC_Program.qpp"
# 465 "QC_Program.qpp"
# 472 "QC_Program.qpp"
# 479 "QC_Program.qpp"
# 486 "QC_Program.qpp"
# 493 "QC_Program.qpp"
# 498 "QC_Program.qpp"
# 503 "QC_Program.qpp"
    ns.addConstant("PO_ALLOW_BARE_REFS", new QoreBigIntNode(PO_ALLOW_BARE_REFS));
    ns.addConstant("PO_ALLOW_DEBUGGER", new QoreBigIntNode(PO_ALLOW_DEBUGGER));
    ns.addConstant("PO_ALLOW_INJECTION", new QoreBigIntNode(PO_ALLOW_INJECTION));
    ns.addConstant("PO_ALLOW_WEAK_REFERENCES", new QoreBigIntNode(PO_ALLOW_WEAK_REFERENCES));
    ns.addConstant("PO_ASSUME_LOCAL", new QoreBigIntNode(PO_ASSUME_LOCAL));
    ns.addConstant("PO_BROKEN_INT_ASSIGNMENTS", new QoreBigIntNode(PO_BROKEN_INT_ASSIGNMENTS));
    ns.addConstant("PO_BROKEN_LIST_PARSING", new QoreBigIntNode(PO_BROKEN_LIST_PARSING));
    ns.addConstant("PO_BROKEN_LOGIC_PRECEDENCE", new QoreBigIntNode(PO_BROKEN_LOGIC_PRECEDENCE));
    ns.addConstant("PO_BROKEN_LOOP_STATEMENT", new QoreBigIntNode(PO_BROKEN_LOOP_STATEMENT));
    ns.addConstant("PO_BROKEN_OPERATORS", new QoreBigIntNode(PO_BROKEN_OPERATORS));
    ns.addConstant("PO_BROKEN_REFERENCES", new QoreBigIntNode(PO_BROKEN_REFERENCES));
    ns.addConstant("PO_DEFAULT", new QoreBigIntNode(PO_DEFAULT));
    ns.addConstant("PO_FREE_OPTIONS", new QoreBigIntNode(PO_FREE_OPTIONS));
    ns.addConstant("PO_IN_MODULE", new QoreBigIntNode(PO_IN_MODULE));
    ns.addConstant("PO_LOCKDOWN", new QoreBigIntNode(PO_LOCKDOWN));
    ns.addConstant("PO_LOCK_WARNINGS", new QoreBigIntNode(PO_LOCK_WARNINGS));
    ns.addConstant("PO_NEW_STYLE", new QoreBigIntNode(PO_NEW_STYLE));
    ns.addConstant("PO_NO_API", new QoreBigIntNode(PO_NO_API));
    ns.addConstant("PO_NO_CHILD_PO_RESTRICTIONS", new QoreBigIntNode(PO_NO_CHILD_PO_RESTRICTIONS));
    ns.addConstant("PO_NO_CLASS_DEFS", new QoreBigIntNode(PO_NO_CLASS_DEFS));
    ns.addConstant("PO_NO_CONSTANT_DEFS", new QoreBigIntNode(PO_NO_CONSTANT_DEFS));
    ns.addConstant("PO_NO_DATABASE", new QoreBigIntNode(PO_NO_DATABASE));
    ns.addConstant("PO_NO_DEBUGGING", new QoreBigIntNode(PO_NO_DEBUGGING));
    ns.addConstant("PO_NO_EMBEDDED_LOGIC", new QoreBigIntNode(PO_NO_EMBEDDED_LOGIC));
    ns.addConstant("PO_NO_EXTERNAL_ACCESS", new QoreBigIntNode(PO_NO_EXTERNAL_ACCESS));
    ns.addConstant("PO_NO_EXTERNAL_INFO", new QoreBigIntNode(PO_NO_EXTERNAL_INFO));
    ns.addConstant("PO_NO_EXTERNAL_PROCESS", new QoreBigIntNode(PO_NO_EXTERNAL_PROCESS));
    ns.addConstant("PO_NO_FILESYSTEM", new QoreBigIntNode(PO_NO_FILESYSTEM));
    ns.addConstant("PO_NO_GLOBAL_VARS", new QoreBigIntNode(PO_NO_GLOBAL_VARS));
    ns.addConstant("PO_NO_GUI", new QoreBigIntNode(PO_NO_GUI));
    ns.addConstant("PO_NO_INHERIT_GLOBAL_VARS", new QoreBigIntNode(PO_NO_INHERIT_GLOBAL_VARS));
    ns.addConstant("PO_NO_INHERIT_SYSTEM_CLASSES", new QoreBigIntNode(PO_NO_INHERIT_SYSTEM_CLASSES));
    ns.addConstant("PO_NO_INHERIT_SYSTEM_CONSTANTS", new QoreBigIntNode(PO_NO_INHERIT_SYSTEM_CONSTANTS));
    ns.addConstant("PO_NO_INHERIT_SYSTEM_FUNC_VARIANTS", new QoreBigIntNode(PO_NO_INHERIT_SYSTEM_FUNC_VARIANTS));
    ns.addConstant("PO_NO_INHERIT_SYSTEM_HASHDECLS", new QoreBigIntNode(PO_NO_INHERIT_SYSTEM_HASHDECLS));
    ns.addConstant("PO_NO_INHERIT_USER_CLASSES", new QoreBigIntNode(PO_NO_INHERIT_USER_CLASSES));
    ns.addConstant("PO_NO_INHERIT_USER_CONSTANTS", new QoreBigIntNode(PO_NO_INHERIT_USER_CONSTANTS));
    ns.addConstant("PO_NO_INHERIT_USER_FUNC_VARIANTS", new QoreBigIntNode(PO_NO_INHERIT_USER_FUNC_VARIANTS));
    ns.addConstant("PO_NO_INHERIT_USER_HASHDECLS", new QoreBigIntNode(PO_NO_INHERIT_USER_HASHDECLS));
    ns.addConstant("PO_NO_IO", new QoreBigIntNode(PO_NO_IO));
    ns.addConstant("PO_NO_LOCALE_CONTROL", new QoreBigIntNode(PO_NO_LOCALE_CONTROL));
    ns.addConstant("PO_NO_MODULES", new QoreBigIntNode(PO_NO_MODULES));
    ns.addConstant("PO_NO_NAMESPACE_DEFS", new QoreBigIntNode(PO_NO_NAMESPACE_DEFS));
    ns.addConstant("PO_NO_NETWORK", new QoreBigIntNode(PO_NO_NETWORK));
    ns.addConstant("PO_NO_NEW", new QoreBigIntNode(PO_NO_NEW));
    ns.addConstant("PO_NO_PROCESS_CONTROL", new QoreBigIntNode(PO_NO_PROCESS_CONTROL));
    ns.addConstant("PO_NO_SUBROUTINE_DEFS", new QoreBigIntNode(PO_NO_SUBROUTINE_DEFS));
    ns.addConstant("PO_NO_SYSTEM_API", new QoreBigIntNode(PO_NO_SYSTEM_API));
    ns.addConstant("PO_NO_SYSTEM_CLASSES", new QoreBigIntNode(PO_NO_INHERIT_SYSTEM_CLASSES));
    ns.addConstant("PO_NO_SYSTEM_FUNC_VARIANTS", new QoreBigIntNode(PO_NO_INHERIT_SYSTEM_FUNC_VARIANTS));
    ns.addConstant("PO_NO_TERMINAL_IO", new QoreBigIntNode(PO_NO_TERMINAL_IO));
    ns.addConstant("PO_NO_THREADS", new QoreBigIntNode(PO_NO_THREADS));
    ns.addConstant("PO_NO_THREAD_CLASSES", new QoreBigIntNode(PO_NO_THREAD_CLASSES));
    ns.addConstant("PO_NO_THREAD_CONTROL", new QoreBigIntNode(PO_NO_THREAD_CONTROL));
    ns.addConstant("PO_NO_THREAD_INFO", new QoreBigIntNode(PO_NO_THREAD_INFO));
    ns.addConstant("PO_NO_TOP_LEVEL_STATEMENTS", new QoreBigIntNode(PO_NO_TOP_LEVEL_STATEMENTS));
    ns.addConstant("PO_NO_UNCONTROLLED_APIS", new QoreBigIntNode(PO_NO_UNCONTROLLED_APIS));
    ns.addConstant("PO_NO_USER_API", new QoreBigIntNode(PO_NO_USER_API));
    ns.addConstant("PO_NO_USER_CLASSES", new QoreBigIntNode(PO_NO_INHERIT_USER_CLASSES));
    ns.addConstant("PO_POSITIVE_OPTIONS", new QoreBigIntNode(PO_POSITIVE_OPTIONS));
    ns.addConstant("PO_REQUIRE_OUR", new QoreBigIntNode(PO_REQUIRE_OUR));
    ns.addConstant("PO_REQUIRE_PROTOTYPES", new QoreBigIntNode(PO_REQUIRE_PROTOTYPES));
    ns.addConstant("PO_REQUIRE_TYPES", new QoreBigIntNode(PO_REQUIRE_TYPES));
    ns.addConstant("PO_STRICT_ARGS", new QoreBigIntNode(PO_STRICT_ARGS));
    ns.addConstant("PO_STRICT_BOOLEAN_EVAL", new QoreBigIntNode(PO_STRICT_BOOLEAN_EVAL));
    ns.addConstant("PO_STRONG_ENCAPSULATION", new QoreBigIntNode(PO_STRONG_ENCAPSULATION));
    ns.addConstant("ParseOptionCmdCodeMap", (ParseOptionMap::getCodeToStringMap()));
    ns.addConstant("ParseOptionCmdStringMap", (ParseOptionMap::getStringToCodeMap()));
    ns.addConstant("ParseOptionCodeMap", (pomaps.getCodeToStringMap()));
    ns.addConstant("ParseOptionStringMap", (pomaps.getStringToCodeMap()));
# 517 "QC_Program.qpp"
# 524 "QC_Program.qpp"
# 529 "QC_Program.qpp"
# 544 "QC_Program.qpp"
# 549 "QC_Program.qpp"
# 554 "QC_Program.qpp"
# 559 "QC_Program.qpp"
# 564 "QC_Program.qpp"
# 569 "QC_Program.qpp"
# 574 "QC_Program.qpp"
# 579 "QC_Program.qpp"
# 592 "QC_Program.qpp"
# 595 "QC_Program.qpp"
# 602 "QC_Program.qpp"
# 607 "QC_Program.qpp"
# 612 "QC_Program.qpp"
# 617 "QC_Program.qpp"
# 622 "QC_Program.qpp"
# 627 "QC_Program.qpp"
# 632 "QC_Program.qpp"
    ns.addConstant("WARN_ALL", new QoreBigIntNode(QP_WARN_ALL));
    ns.addConstant("WARN_BROKEN_LOGIC_PRECEDENCE", new QoreBigIntNode(QP_WARN_BROKEN_LOGIC_PRECEDENCE));
    ns.addConstant("WARN_CALL_WITH_TYPE_ERRORS", new QoreBigIntNode(QP_WARN_CALL_WITH_TYPE_ERRORS));
    ns.addConstant("WARN_DEFAULT", new QoreBigIntNode(QP_WARN_DEFAULT));
    ns.addConstant("WARN_DEPRECATED", new QoreBigIntNode(QP_WARN_DEPRECATED));
    ns.addConstant("WARN_DUPLICATE_BLOCK_VARS", new QoreBigIntNode(QP_WARN_DUPLICATE_BLOCK_VARS));
    ns.addConstant("WARN_DUPLICATE_GLOBAL_VARS", new QoreBigIntNode(QP_WARN_DUPLICATE_GLOBAL_VARS));
    ns.addConstant("WARN_DUPLICATE_HASH_KEY", new QoreBigIntNode(QP_WARN_DUPLICATE_HASH_KEY));
    ns.addConstant("WARN_DUPLICATE_LOCAL_VARS", new QoreBigIntNode(QP_WARN_DUPLICATE_LOCAL_VARS));
    ns.addConstant("WARN_EXCESS_ARGS", new QoreBigIntNode(QP_WARN_EXCESS_ARGS));
    ns.addConstant("WARN_INVALID_OPERATION", new QoreBigIntNode(QP_WARN_INVALID_OPERATION));
    ns.addConstant("WARN_MODULES", new QoreBigIntNode(QP_WARN_MODULES));
    ns.addConstant("WARN_NONE", new QoreBigIntNode(QP_WARN_NONE));
    ns.addConstant("WARN_NONEXISTENT_METHOD_CALL", new QoreBigIntNode(QP_WARN_NONEXISTENT_METHOD_CALL));
    ns.addConstant("WARN_RETURN_VALUE_IGNORED", new QoreBigIntNode(QP_WARN_RETURN_VALUE_IGNORED));
    ns.addConstant("WARN_UNDECLARED_VAR", new QoreBigIntNode(QP_WARN_UNDECLARED_VAR));
    ns.addConstant("WARN_UNKNOWN_WARNING", new QoreBigIntNode(QP_WARN_UNKNOWN_WARNING));
    ns.addConstant("WARN_UNREACHABLE_CODE", new QoreBigIntNode(QP_WARN_UNREACHABLE_CODE));
    ns.addConstant("WARN_UNREFERENCED_VARIABLE", new QoreBigIntNode(QP_WARN_UNREFERENCED_VARIABLE));
    ns.addConstant("WARN_WARNING_MASK_UNCHANGED", new QoreBigIntNode(QP_WARN_WARNING_MASK_UNCHANGED));

    return QC_PROGRAM;
}
