/* -*- mode: c++; indent-tabs-mode: nil -*- */
/* @file QC_Queue.qpp Queue class definition

  Qore Programming Language

  Copyright (C) 2003 - 2017 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/QC_Queue.h"
#include "qore/intern/QoreQueueIntern.h"
/* Qore class Qore::Thread::Queue */

qore_classid_t CID_QUEUE;
QoreClass* QC_QUEUE;

// nothing Queue::clear(){}
static QoreValue Queue_clear(QoreObject* self, Queue *q, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 183 "QC_Queue.qpp"
    q->clear(xsink);
    return QoreValue();
}

// nothing Queue::clearError(){}
static QoreValue Queue_clearError(QoreObject* self, Queue *q, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 298 "QC_Queue.qpp"
    q->clearError();
    return QoreValue();
}

// Queue::constructor(int max = -1) {}
static void Queue_constructor_Vi(QoreObject* self, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 max = HARD_QORE_VALUE_INT(args, 0);
# 59 "QC_Queue.qpp"
if (!max || (max < 0 && max != -1) || max > 0x7fffffff)
      xsink->raiseException("QUEUE-SIZE-ERROR", QLLD" is an invalid size for a Queue", max);
   else
      self->setPrivate(CID_QUEUE, new Queue(max));
}

// Queue::copy() {}
static void Queue_copy(QoreObject* self, QoreObject* old, Queue *q, ExceptionSink* xsink) {
# 79 "QC_Queue.qpp"
self->setPrivate(CID_QUEUE, new Queue(*q));
}

// Queue::destructor() {}
static void Queue_destructor(QoreObject* self, Queue *q, ExceptionSink* xsink) {
# 71 "QC_Queue.qpp"
qore_queue_private::destructor(*q, xsink);
   q->deref(xsink);
}

// bool Queue::empty(){}
static QoreValue Queue_empty(QoreObject* self, Queue *q, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 209 "QC_Queue.qpp"
    return q->empty();
}

// auto Queue::get(timeout timeout_ms = 0){}
static QoreValue Queue_get_Vt(QoreObject* self, Queue *q, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 134 "QC_Queue.qpp"
    AbstractQoreNode *rv;

   if (timeout_ms) {
      bool to;
      rv = q->shift(xsink, timeout_ms, &to);
      if (to)
	 xsink->raiseException("QUEUE-TIMEOUT", "timed out after %d ms", timeout_ms);
   }
   else
      rv = q->shift(xsink);

   return rv;
}

// int Queue::getReadWaiting(){}
static QoreValue Queue_getReadWaiting(QoreObject* self, Queue *q, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 251 "QC_Queue.qpp"
    return q->getReadWaiting();
}

// int Queue::getWaiting(){}
static QoreValue Queue_getWaiting(QoreObject* self, Queue *q, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 235 "QC_Queue.qpp"
    return q->getReadWaiting();
}

// int Queue::getWriteWaiting(){}
static QoreValue Queue_getWriteWaiting(QoreObject* self, Queue *q, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 265 "QC_Queue.qpp"
    return q->getWriteWaiting();
}

// nothing Queue::insert(auto arg, timeout timeout_ms = 0){}
static QoreValue Queue_insert_VAVt(QoreObject* self, Queue *q, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue arg = get_param_value(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 114 "QC_Queue.qpp"
    bool to;
   q->insert(xsink, arg.getReferencedValue(), timeout_ms, &to);
   if (to)
      xsink->raiseException("QUEUE-TIMEOUT", "timed out after %d ms", timeout_ms);
    return QoreValue();
}

// int Queue::max(){}
static QoreValue Queue_max(QoreObject* self, Queue *q, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 221 "QC_Queue.qpp"
    return q->getMax();
}

// auto Queue::pop(timeout timeout_ms = 0){}
static QoreValue Queue_pop_Vt(QoreObject* self, Queue *q, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 162 "QC_Queue.qpp"
    AbstractQoreNode *rv;

   if (timeout_ms) {
      bool to;
      rv = q->pop(xsink, timeout_ms, &to);
      if (to)
	 xsink->raiseException("QUEUE-TIMEOUT", "timed out after %d ms", timeout_ms);
   }
   else
      rv = q->pop(xsink);

   return rv;
}

// nothing Queue::push(auto arg, timeout timeout_ms = 0){}
static QoreValue Queue_push_VAVt(QoreObject* self, Queue *q, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue arg = get_param_value(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 95 "QC_Queue.qpp"
    bool to;
   q->push(xsink, arg.getReferencedValue(), timeout_ms, &to);
   if (to)
      xsink->raiseException("QUEUE-TIMEOUT", "timed out after %d ms", timeout_ms);
    return QoreValue();
}

// nothing Queue::setError(string err, string desc){}
static QoreValue Queue_setError_VsVs(QoreObject* self, Queue *q, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* err = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* desc = HARD_QORE_VALUE_STRING(args, 1);
# 285 "QC_Queue.qpp"
    q->setError(err->c_str(), desc, xsink);
    return QoreValue();
}

// int Queue::size(){}
static QoreValue Queue_size(QoreObject* self, Queue *q, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 195 "QC_Queue.qpp"
    return q->size();
}

DLLLOCAL void preinitQueueClass() {
    QC_QUEUE = new QoreClass("Queue", QDOM_THREAD_CLASS);
    CID_QUEUE = QC_QUEUE->getID();
    QC_QUEUE->setSystem();
}

DLLLOCAL QoreClass* initQueueClass(QoreNamespace& ns) {
    if (!QC_QUEUE)
        preinitQueueClass();

    // nothing Queue::clear(){}
    QC_QUEUE->addMethod("clear", (q_method_n_t)Queue_clear, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing Queue::clearError(){}
    QC_QUEUE->addMethod("clearError", (q_method_n_t)Queue_clearError, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // Queue::constructor(int max = -1) {}
    QC_QUEUE->addConstructor(Queue_constructor_Vi, false, QC_NO_FLAGS, QDOM_DEFAULT, 1, bigIntTypeInfo, new QoreBigIntNode(-1), "max");

    // Queue::copy() {}
    QC_QUEUE->setCopy((q_copy_t)Queue_copy);

    // Queue::destructor() {}
    QC_QUEUE->setDestructor((q_destructor_t)Queue_destructor);

    // bool Queue::empty(){}
    QC_QUEUE->addMethod("empty", (q_method_n_t)Queue_empty, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // auto Queue::get(timeout timeout_ms = 0){}
    QC_QUEUE->addMethod("get", (q_method_n_t)Queue_get_Vt, false, QC_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 1, timeoutTypeInfo, zero(), "timeout_ms");

    // int Queue::getReadWaiting(){}
    QC_QUEUE->addMethod("getReadWaiting", (q_method_n_t)Queue_getReadWaiting, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // int Queue::getWaiting(){}
    QC_QUEUE->addMethod("getWaiting", (q_method_n_t)Queue_getWaiting, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // int Queue::getWriteWaiting(){}
    QC_QUEUE->addMethod("getWriteWaiting", (q_method_n_t)Queue_getWriteWaiting, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // nothing Queue::insert(auto arg, timeout timeout_ms = 0){}
    QC_QUEUE->addMethod("insert", (q_method_n_t)Queue_insert_VAVt, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, autoTypeInfo, NULL, "arg", timeoutTypeInfo, zero(), "timeout_ms");

    // int Queue::max(){}
    QC_QUEUE->addMethod("max", (q_method_n_t)Queue_max, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // auto Queue::pop(timeout timeout_ms = 0){}
    QC_QUEUE->addMethod("pop", (q_method_n_t)Queue_pop_Vt, false, QC_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 1, timeoutTypeInfo, zero(), "timeout_ms");

    // nothing Queue::push(auto arg, timeout timeout_ms = 0){}
    QC_QUEUE->addMethod("push", (q_method_n_t)Queue_push_VAVt, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, autoTypeInfo, NULL, "arg", timeoutTypeInfo, zero(), "timeout_ms");

    // nothing Queue::setError(string err, string desc){}
    QC_QUEUE->addMethod("setError", (q_method_n_t)Queue_setError_VsVs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, NULL, "err", stringTypeInfo, NULL, "desc");

    // int Queue::size(){}
    QC_QUEUE->addMethod("size", (q_method_n_t)Queue_size, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    return QC_QUEUE;
}
