/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  QC_Socket.qpp

  Socket class definition

  Qore Programming Language

  Copyright (C) 2003 - 2017 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include "qore/Qore.h"
#include "qore/intern/QC_Socket.h"
#include "qore/intern/ssl_constants.h"
#include "qore/intern/QC_Queue.h"
#include "qore/QoreSSLCertificate.h"
#include "qore/QoreSSLPrivateKey.h"

#include <errno.h>
#include <string.h>

static void hash_set_int_key(QoreHashNode& h, int k, const char* str) {
   char buf[15];
   sprintf(buf, "%d", k);
   h.setKeyValue(buf, new QoreStringNode(str), 0);
}

static QoreHashNode* get_network_address_family_map() {
   QoreHashNode* rv = new QoreHashNode;

   hash_set_int_key(*rv, AF_INET, "AF_INET");
   hash_set_int_key(*rv, AF_INET6, "AF_INET6");
   hash_set_int_key(*rv, AF_UNIX, "AF_UNIX");
   hash_set_int_key(*rv, AF_UNSPEC, "AF_UNSPEC");

   return rv;
}

static int64 checkOpenResult(int rc, const char *method_name, ExceptionSink *xsink) {
   if (*xsink)
      return 0;
   if (rc == -2) {
      xsink->raiseException("SOCKET-NOT-OPEN", "socket must be open before Socket::%s() call", method_name);
      return 0;
   }
   return rc;
}
/* Qore class Qore::Socket */

qore_classid_t CID_SOCKET;
QoreClass* QC_SOCKET;

// Socket Socket::accept(){}
static QoreValue Socket_accept(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 752 "QC_Socket.qpp"
    QoreSocketObject *n = s->accept(-1, xsink);
   if (!n) {
      assert(*xsink);
      return QoreValue();
   }

   assert(n);

   // ensure that a socket object is returned (and not a subclass)
   QoreObject *ns = new QoreObject(QC_SOCKET, getProgram(), n);

   // save backwards-compatible peer parameters as members in new object (deprecated: Socket::getPeerInfo() should be used in the future)
   my_socket_priv::setAccept(*n, ns);

   return ns;
}

// *Socket Socket::accept(timeout timeout_ms){}
static QoreValue Socket_accept_Vt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 787 "QC_Socket.qpp"
    QoreSocketObject *n = s->accept(timeout_ms, xsink);
   if (!n)
      return QoreValue();

   // ensure that a socket object is returned (and not a subclass)
   QoreObject *ns = new QoreObject(QC_SOCKET, getProgram(), n);

   // save backwards-compatible peer parameters as members in new object (deprecated: Socket::getPeerInfo() should be used in the future)
   my_socket_priv::setAccept(*n, ns);

   return ns;
}

// nothing Socket::acceptAllCertificates(bool accept_all = True){}
static QoreValue Socket_acceptAllCertificates_Vb(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    bool accept_all = HARD_QORE_VALUE_BOOL(args, 0);
# 2940 "QC_Socket.qpp"
    s->acceptAllCertificates(accept_all);
    return QoreValue();
}

// Socket Socket::acceptSSL(){}
static QoreValue Socket_acceptSSL(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 816 "QC_Socket.qpp"
    QoreSocketObject *n = s->acceptSSL(-1, xsink);
   if (!n) {
      assert(*xsink);
      return QoreValue();
   }

   // ensure that a socket object is returned (and not a subclass)
   QoreObject *ns = new QoreObject(QC_SOCKET, getProgram(), n);

   // save backwards-compatible peer parameters as members in new object (deprecated: Socket::getPeerInfo() should be used in the future)
   my_socket_priv::setAccept(*n, ns);

   return ns;
}

// *Socket Socket::acceptSSL(timeout timeout_ms){}
static QoreValue Socket_acceptSSL_Vt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 849 "QC_Socket.qpp"
    QoreSocketObject* n = s->acceptSSL(timeout_ms, xsink);
   if (!n)
      return QoreValue();

   // ensure that a socket object is returned (and not a subclass)
   QoreObject* ns = new QoreObject(QC_SOCKET, getProgram(), n);

   // save backwards-compatible peer parameters as members in new object (deprecated: Socket::getPeerInfo() should be used in the future)
   my_socket_priv::setAccept(*n, ns);

   return ns;
}

// int Socket::bind(string str, softbool reuseaddr = False){}
static QoreValue Socket_bind_Vsvb(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    bool reuseaddr = HARD_QORE_VALUE_BOOL(args, 1);
# 656 "QC_Socket.qpp"
    return s->bind(str->getBuffer(), reuseaddr);
}

// int Socket::bind(int port, softbool reuseaddr = False){}
static QoreValue Socket_bind_Vivb(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 port = HARD_QORE_VALUE_INT(args, 0);
    bool reuseaddr = HARD_QORE_VALUE_BOOL(args, 1);
# 675 "QC_Socket.qpp"
    return s->bind(port, reuseaddr);
}

// nothing Socket::bindINET(*string iface, *softstring service, softbool reuseaddr = False, softint family = AF_UNSPEC, softint socktype = SOCK_STREAM, softint protocol = 0){}
static QoreValue Socket_bindINET_Nsnsvbvivivi(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* iface = get_param_value(args, 0).get<const QoreStringNode>();
    const QoreStringNode* service = get_param_value(args, 1).get<const QoreStringNode>();
    bool reuseaddr = HARD_QORE_VALUE_BOOL(args, 2);
    int64 family = HARD_QORE_VALUE_INT(args, 3);
    int64 socktype = HARD_QORE_VALUE_INT(args, 4);
    int64 protocol = HARD_QORE_VALUE_INT(args, 5);
# 729 "QC_Socket.qpp"
    if ((!iface || iface->empty())
       && (!service || service->empty()))
      return xsink->raiseException("SOCKET-BIND-ERROR", "both interace (first parameter) and service (second parameter) were either not present or empty strings; at least one of the first 2 parameters must be present for a successful call to Socket::bindINET()");

   s->bindINET(iface ? iface->getBuffer() : 0, service ? service->getBuffer() : 0, reuseaddr, family, socktype, protocol, xsink);
    return QoreValue();
}

// nothing Socket::bindUNIX(string path, softint socktype = SOCK_STREAM, softint protocol = 0){}
static QoreValue Socket_bindUNIX_Vsvivi(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
    int64 socktype = HARD_QORE_VALUE_INT(args, 1);
    int64 protocol = HARD_QORE_VALUE_INT(args, 2);
# 696 "QC_Socket.qpp"
    s->bindUNIX(path->getBuffer(), socktype, protocol, xsink);
    return QoreValue();
}

// nothing Socket::clearStats(){}
static QoreValue Socket_clearStats(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2866 "QC_Socket.qpp"
    s->clearStats();
    return QoreValue();
}

// nothing Socket::clearWarningQueue(){}
static QoreValue Socket_clearWarningQueue(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2787 "QC_Socket.qpp"
    s->clearWarningQueue(xsink);
    return QoreValue();
}

// int Socket::close(){}
static QoreValue Socket_close(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2289 "QC_Socket.qpp"
    return s->close();
}

// nothing Socket::connect(string target, timeout timeout_ms = -1){}
static QoreValue Socket_connect_VsVt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* target = HARD_QORE_VALUE_STRING(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 490 "QC_Socket.qpp"
    s->connect(target->getBuffer(), timeout_ms, xsink);
    return QoreValue();
}

// nothing Socket::connectINET(string host, softstring service, timeout timeout_ms = -1, softint family = AF_UNSPEC, softint socktype = SOCK_STREAM, softint protocol = 0){}
static QoreValue Socket_connectINET_VsvsVtvivivi(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* host = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* service = HARD_QORE_VALUE_STRING(args, 1);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 2);
    int64 family = HARD_QORE_VALUE_INT(args, 3);
    int64 socktype = HARD_QORE_VALUE_INT(args, 4);
    int64 protocol = HARD_QORE_VALUE_INT(args, 5);
# 519 "QC_Socket.qpp"
    s->connectINET2(host->getBuffer(), service->getBuffer(), family, socktype, protocol, timeout_ms, xsink);
    return QoreValue();
}

// nothing Socket::connectINETSSL(string host, softstring service, timeout timeout_ms = -1, softint family = AF_UNSPEC, softint socktype = SOCK_STREAM, softint protocol = 0){}
static QoreValue Socket_connectINETSSL_VsvsVtvivivi(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* host = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* service = HARD_QORE_VALUE_STRING(args, 1);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 2);
    int64 family = HARD_QORE_VALUE_INT(args, 3);
    int64 socktype = HARD_QORE_VALUE_INT(args, 4);
    int64 protocol = HARD_QORE_VALUE_INT(args, 5);
# 606 "QC_Socket.qpp"
    s->connectINET2SSL(host->getBuffer(), service->getBuffer(), family, socktype, protocol, timeout_ms, xsink);
    return QoreValue();
}

// nothing Socket::connectSSL(string target, timeout timeout_ms = -1){}
static QoreValue Socket_connectSSL_VsVt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* target = HARD_QORE_VALUE_STRING(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 576 "QC_Socket.qpp"
    s->connectSSL(target->getBuffer(), timeout_ms, xsink);
    return QoreValue();
}

// nothing Socket::connectUNIX(string path, softint socktype = SOCK_STREAM, softint protocol = 0){}
static QoreValue Socket_connectUNIX_Vsvivi(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
    int64 socktype = HARD_QORE_VALUE_INT(args, 1);
    int64 protocol = HARD_QORE_VALUE_INT(args, 2);
# 543 "QC_Socket.qpp"
    s->connectUNIX(path->getBuffer(), socktype, protocol, xsink);
    return QoreValue();
}

// nothing Socket::connectUNIXSSL(string path, softint socktype = SOCK_STREAM, softint protocol = 0){}
static QoreValue Socket_connectUNIXSSL_Vsvivi(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
    int64 socktype = HARD_QORE_VALUE_INT(args, 1);
    int64 protocol = HARD_QORE_VALUE_INT(args, 2);
# 631 "QC_Socket.qpp"
    s->connectUNIXSSL(path->getBuffer(), socktype, protocol, xsink);
    return QoreValue();
}

// Socket::constructor() {}
static void Socket_constructor(QoreObject* self, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 452 "QC_Socket.qpp"
self->setPrivate(CID_SOCKET, new QoreSocketObject);
}

// Socket::copy() {}
static void Socket_copy(QoreObject* self, QoreObject* old, QoreSocketObject* s, ExceptionSink* xsink) {
# 460 "QC_Socket.qpp"
self->setPrivate(CID_SOCKET, new QoreSocketObject);
}

// bool Socket::getAcceptAllCertificates(){}
static QoreValue Socket_getAcceptAllCertificates(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2959 "QC_Socket.qpp"
    return s->getAcceptAllCertificates();
}

// string Socket::getCharset(){}
static QoreValue Socket_getCharset(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2422 "QC_Socket.qpp"
    return new QoreStringNode(s->getEncoding()->getCode());
}

// string Socket::getEncoding(){}
static QoreValue Socket_getEncoding(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2407 "QC_Socket.qpp"
    return new QoreStringNode(s->getEncoding()->getCode());
}

// bool Socket::getNoDelay(){}
static QoreValue Socket_getNoDelay(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2693 "QC_Socket.qpp"
    return s->getNoDelay();
}

// hash Socket::getPeerInfo(bool host_lookup = True){}
static QoreValue Socket_getPeerInfo_Vb(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    bool host_lookup = HARD_QORE_VALUE_BOOL(args, 0);
# 2714 "QC_Socket.qpp"
    return s->getPeerInfo(xsink, host_lookup);
}

// int Socket::getPort(){}
static QoreValue Socket_getPort(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2330 "QC_Socket.qpp"
    return s->getPort();
}

// int Socket::getRecvTimeout(){}
static QoreValue Socket_getRecvTimeout(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2400 "QC_Socket.qpp"
    return s->getRecvTimeout();
}

// *string Socket::getSSLCipherName(){}
static QoreValue Socket_getSSLCipherName(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2491 "QC_Socket.qpp"
    const char *str = s->getSSLCipherName();
   return str ? new QoreStringNode(str) : 0;
}

// *string Socket::getSSLCipherVersion(){}
static QoreValue Socket_getSSLCipherVersion(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2504 "QC_Socket.qpp"
    const char *str = s->getSSLCipherVersion();
   return str ? new QoreStringNode(str) : 0;
}

// int Socket::getSendTimeout(){}
static QoreValue Socket_getSendTimeout(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2388 "QC_Socket.qpp"
    return s->getSendTimeout();
}

// int Socket::getSocket(){}
static QoreValue Socket_getSocket(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2342 "QC_Socket.qpp"
    return s->getSocket();
}

// hash Socket::getSocketInfo(bool host_lookup = True){}
static QoreValue Socket_getSocketInfo_Vb(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    bool host_lookup = HARD_QORE_VALUE_BOOL(args, 0);
# 2735 "QC_Socket.qpp"
    return s->getSocketInfo(xsink, host_lookup);
}

// int Socket::getSslVerifyMode(){}
static QoreValue Socket_getSslVerifyMode(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2919 "QC_Socket.qpp"
    return s->getSslVerifyMode();
}

// hash Socket::getUsageInfo(){}
static QoreValue Socket_getUsageInfo(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2852 "QC_Socket.qpp"
    return s->getUsageInfo();
}

// bool Socket::isDataAvailable(timeout timeout_ms = 0){}
static QoreValue Socket_isDataAvailable_Vt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 2448 "QC_Socket.qpp"
    return s->isDataAvailable(xsink, (int)timeout_ms);
}

// bool Socket::isOpen(){}
static QoreValue Socket_isOpen(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2479 "QC_Socket.qpp"
    return s->isOpen();
}

// bool Socket::isSecure(){}
static QoreValue Socket_isSecure(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2517 "QC_Socket.qpp"
    return s->isSecure();
}

// bool Socket::isWriteFinished(timeout timeout_ms = 0){}
static QoreValue Socket_isWriteFinished_Vt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 2467 "QC_Socket.qpp"
    return s->isWriteFinished(xsink, (int)timeout_ms);
}

// int Socket::listen(int backlog = 20){}
static QoreValue Socket_listen_Vi(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 backlog = HARD_QORE_VALUE_INT(args, 0);
# 877 "QC_Socket.qpp"
    return checkOpenResult(s->listen((int)backlog), "listen", xsink);
}

// bool Socket::pendingHttpChunkedBody(){}
static QoreValue Socket_pendingHttpChunkedBody(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2878 "QC_Socket.qpp"
    return s->pendingHttpChunkedBody();
}

// hash Socket::readHTTPChunkedBody(timeout timeout_ms = -1){}
static QoreValue Socket_readHTTPChunkedBody_Vt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 2149 "QC_Socket.qpp"
    // when rc = -3 it's a timeout, but rv will be NULL anyway, so we do nothing
   return s->readHTTPChunkedBody(timeout_ms, xsink);
}

// hash Socket::readHTTPChunkedBodyBinary(timeout timeout_ms = -1){}
static QoreValue Socket_readHTTPChunkedBodyBinary_Vt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 2176 "QC_Socket.qpp"
    // when rc = -3 it's a timeout, but rv will be NULL anyway, so we do nothing
   return s->readHTTPChunkedBodyBinary(timeout_ms, xsink);
}

// nothing Socket::readHTTPChunkedBodyBinaryWithCallback(code rcb, timeout timeout_ms = -1){}
static QoreValue Socket_readHTTPChunkedBodyBinaryWithCallback_VcVt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const ResolvedCallReferenceNode* rcb = get_param_value(args, 0).get<const ResolvedCallReferenceNode>();
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 2271 "QC_Socket.qpp"
    s->readHTTPChunkedBodyBinaryWithCallback(*rcb, self, timeout_ms, xsink);
    return QoreValue();
}

// hash Socket::readHTTPChunkedBodyToOutputStream(Qore::OutputStream os, timeout timeout_ms = -1){}
static QoreValue Socket_readHTTPChunkedBodyToOutputStream_C12OutputStreamVt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(os, OutputStream, args, 0, CID_OUTPUTSTREAM, "Socket::readHTTPChunkedBodyToOutputStream()", "Qore::OutputStream", xsink);
    if (*xsink)
        return 0;
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 2204 "QC_Socket.qpp"
    SimpleRefHolder<OutputStream> osHolder(os);
   return s->readHTTPChunkedBodyToOutputStream(os, timeout_ms, xsink);
}

// nothing Socket::readHTTPChunkedBodyWithCallback(code rcb, timeout timeout_ms = -1){}
static QoreValue Socket_readHTTPChunkedBodyWithCallback_VcVt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const ResolvedCallReferenceNode* rcb = get_param_value(args, 0).get<const ResolvedCallReferenceNode>();
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 2238 "QC_Socket.qpp"
    s->readHTTPChunkedBodyWithCallback(*rcb, self, timeout_ms, xsink);
    return QoreValue();
}

// hash Socket::readHTTPHeader(timeout timeout_ms = -1, *reference<hash> info){}
static QoreValue Socket_readHTTPHeader_VtC16_reference_hash_(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
    const ReferenceNode* info = get_param_value(args, 1).get<const ReferenceNode>();
# 2094 "QC_Socket.qpp"
    OptHashRefHelper ohrh(info, xsink);
   return s->readHTTPHeader(xsink, *ohrh, timeout_ms);
}

// string Socket::readHTTPHeaderString(timeout timeout_ms = -1){}
static QoreValue Socket_readHTTPHeaderString_Vt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 2123 "QC_Socket.qpp"
    return s->readHTTPHeaderString(xsink, timeout_ms);
}

// string Socket::recv(softint size = 0, timeout timeout_ms = -1){}
static QoreValue Socket_recv_viVt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 size = HARD_QORE_VALUE_INT(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 1372 "QC_Socket.qpp"
    if (size > 0)
      return s->recv(size, timeout_ms, xsink);

   QoreStringNode* str = s->recv(timeout_ms, xsink);
   return !*xsink && !str ? new QoreStringNode : str;
}

// binary Socket::recvBinary(softint size = 0, timeout timeout_ms = -1){}
static QoreValue Socket_recvBinary_viVt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 size = HARD_QORE_VALUE_INT(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 1404 "QC_Socket.qpp"
    if (size > 0)
      return s->recvBinary(size, timeout_ms, xsink);

   BinaryNode* str = s->recvBinary(timeout_ms, xsink);
   return !*xsink && !str ? new BinaryNode : str;
}

// nothing Socket::recvToOutputStream(Qore::OutputStream os, softint size = -1, timeout timeout_ms = -1){}
static QoreValue Socket_recvToOutputStream_C12OutputStreamviVt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(os, OutputStream, args, 0, CID_OUTPUTSTREAM, "Socket::recvToOutputStream()", "Qore::OutputStream", xsink);
    if (*xsink)
        return 0;
    int64 size = HARD_QORE_VALUE_INT(args, 1);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 2);
# 1436 "QC_Socket.qpp"
    SimpleRefHolder<OutputStream> osHolder(os);
   s->recvToOutputStream(os, size, timeout_ms, xsink);
    return QoreValue();
}

// int Socket::recvi1(timeout timeout_ms = -1){}
static QoreValue Socket_recvi1_Vt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 1464 "QC_Socket.qpp"
    char b;
   s->recvi1(timeout_ms, &b, xsink);
   return b;
}

// int Socket::recvi2(timeout timeout_ms = -1){}
static QoreValue Socket_recvi2_Vt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 1493 "QC_Socket.qpp"
    short b;
   s->recvi2(timeout_ms, &b, xsink);
   return b;
}

// int Socket::recvi2LSB(timeout timeout_ms = -1){}
static QoreValue Socket_recvi2LSB_Vt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 1580 "QC_Socket.qpp"
    short b;
   s->recvi2LSB(timeout_ms, &b, xsink);
   return b;
}

// int Socket::recvi4(timeout timeout_ms = -1){}
static QoreValue Socket_recvi4_Vt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 1522 "QC_Socket.qpp"
    int b;
   s->recvi4(timeout_ms, &b, xsink);
   return b;
}

// int Socket::recvi4LSB(timeout timeout_ms = -1){}
static QoreValue Socket_recvi4LSB_Vt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 1609 "QC_Socket.qpp"
    int b;
   s->recvi4LSB(timeout_ms, &b, xsink);
   return b;
}

// int Socket::recvi8(timeout timeout_ms = -1){}
static QoreValue Socket_recvi8_Vt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 1551 "QC_Socket.qpp"
    int64 b;
   s->recvi8(timeout_ms, &b, xsink);
   return b;
}

// int Socket::recvi8LSB(timeout timeout_ms = -1){}
static QoreValue Socket_recvi8LSB_Vt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 1638 "QC_Socket.qpp"
    int64 b;
   s->recvi8LSB(timeout_ms, &b, xsink);
   return b;
}

// int Socket::recvu1(timeout timeout_ms = -1){}
static QoreValue Socket_recvu1_Vt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 1667 "QC_Socket.qpp"
    unsigned char b;
   s->recvu1(timeout_ms, &b, xsink);
   return b;
}

// int Socket::recvu2(timeout timeout_ms = -1){}
static QoreValue Socket_recvu2_Vt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 1696 "QC_Socket.qpp"
    unsigned short b;
   s->recvu2(timeout_ms, &b, xsink);
   return b;
}

// int Socket::recvu2LSB(timeout timeout_ms = -1){}
static QoreValue Socket_recvu2LSB_Vt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 1754 "QC_Socket.qpp"
    unsigned short b;
   s->recvu2LSB(timeout_ms, &b, xsink);
   return b;
}

// int Socket::recvu4(timeout timeout_ms = -1){}
static QoreValue Socket_recvu4_Vt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 1725 "QC_Socket.qpp"
    unsigned int b;
   s->recvu4(timeout_ms, &b, xsink);
   return b;
}

// int Socket::recvu4LSB(timeout timeout_ms = -1){}
static QoreValue Socket_recvu4LSB_Vt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 1783 "QC_Socket.qpp"
    unsigned b;
   s->recvu4LSB(timeout_ms, &b, xsink);
   return b;
}

// int Socket::send(binary bin, timeout timeout_ms = -1){}
static QoreValue Socket_send_VyVt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* bin = HARD_QORE_VALUE_BINARY(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 909 "QC_Socket.qpp"
    return s->send(bin, timeout_ms, xsink);
}

// int Socket::send(string str, timeout timeout_ms = -1){}
static QoreValue Socket_send_VsVt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 942 "QC_Socket.qpp"
    return s->send(str, timeout_ms, xsink);
}

// nothing Socket::send2(binary bin, timeout timeout_ms = -1){}
static QoreValue Socket_send2_VyVt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* bin = HARD_QORE_VALUE_BINARY(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 1033 "QC_Socket.qpp"
    s->send(bin, timeout_ms, xsink);
    return QoreValue();
}

// nothing Socket::send2(string str, timeout timeout_ms = -1){}
static QoreValue Socket_send2_VsVt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 1063 "QC_Socket.qpp"
    s->send(str, timeout_ms, xsink);
    return QoreValue();
}

// int Socket::sendBinary(string str, timeout timeout_ms = -1){}
static QoreValue Socket_sendBinary_VsVt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 973 "QC_Socket.qpp"
    return s->send(str->getBuffer(), str->strlen(), timeout_ms, xsink);
}

// int Socket::sendBinary(binary bin, timeout timeout_ms = -1){}
static QoreValue Socket_sendBinary_VyVt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* bin = HARD_QORE_VALUE_BINARY(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 1004 "QC_Socket.qpp"
    return s->send(bin, timeout_ms, xsink);
}

// nothing Socket::sendBinary2(string str, timeout timeout_ms = -1){}
static QoreValue Socket_sendBinary2_VsVt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 1091 "QC_Socket.qpp"
    s->send(str->getBuffer(), str->strlen(), timeout_ms, xsink);
    return QoreValue();
}

// nothing Socket::sendBinary2(binary bin, timeout timeout_ms = -1){}
static QoreValue Socket_sendBinary2_VyVt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* bin = HARD_QORE_VALUE_BINARY(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 1119 "QC_Socket.qpp"
    s->send(bin, timeout_ms, xsink);
    return QoreValue();
}

// nothing Socket::sendFromInputStream(Qore::InputStream is, softint size = -1, timeout timeout_ms = -1){}
static QoreValue Socket_sendFromInputStream_C11InputStreamviVt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(is, InputStream, args, 0, CID_INPUTSTREAM, "Socket::sendFromInputStream()", "Qore::InputStream", xsink);
    if (*xsink)
        return 0;
    int64 size = HARD_QORE_VALUE_INT(args, 1);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 2);
# 1145 "QC_Socket.qpp"
    SimpleRefHolder<InputStream> isHolder(is);
   s->sendFromInputStream(is, size, timeout_ms, xsink);
    return QoreValue();
}

// nothing Socket::sendHTTPChunkedBodyFromInputStream(Qore::InputStream is, int max_chunk_size = 4096, timeout timeout_ms = -1, *code tcb){}
static QoreValue Socket_sendHTTPChunkedBodyFromInputStream_C11InputStreamViVtNc(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(is, InputStream, args, 0, CID_INPUTSTREAM, "Socket::sendHTTPChunkedBodyFromInputStream()", "Qore::InputStream", xsink);
    if (*xsink)
        return 0;
    int64 max_chunk_size = HARD_QORE_VALUE_INT(args, 1);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 2);
    const ResolvedCallReferenceNode* tcb = get_param_value(args, 3).get<const ResolvedCallReferenceNode>();
# 2015 "QC_Socket.qpp"
    SimpleRefHolder<InputStream> isHolder(is);
   if (max_chunk_size < 1) {
      xsink->raiseException("HTTP-CHUNK-ERROR", "the max chunk size must be greater than 0; value given is: " QLLD, max_chunk_size);
      return QoreValue();
   }
   s->sendHTTPChunkedBodyFromInputStream(is, max_chunk_size, timeout_ms, tcb, xsink);
    return QoreValue();
}

// nothing Socket::sendHTTPChunkedBodyTrailer(*hash trailer, timeout timeout_ms = -1){}
static QoreValue Socket_sendHTTPChunkedBodyTrailer_NhVt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreHashNode* trailer = get_param_value(args, 0).get<const QoreHashNode>();
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 2044 "QC_Socket.qpp"
    s->sendHTTPChunkedBodyTrailer(trailer, timeout_ms, xsink);
    return QoreValue();
}

// nothing Socket::sendHTTPMessage(string method, string path, string http_version, hash headers, *string body, *reference<hash> info, timeout timeout_ms = -1){}
static QoreValue Socket_sendHTTPMessage_VsVsVsVhNsC16_reference_hash_Vt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* method = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 1);
    const QoreStringNode* http_version = HARD_QORE_VALUE_STRING(args, 2);
    const QoreHashNode* headers = HARD_QORE_VALUE_HASH(args, 3);
    const QoreStringNode* body = get_param_value(args, 4).get<const QoreStringNode>();
    const ReferenceNode* info = get_param_value(args, 5).get<const ReferenceNode>();
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 6);
# 1818 "QC_Socket.qpp"
    TempEncodingHelper tmp_data;
   if (body && !tmp_data.set(body, s->getEncoding(), xsink))
      return QoreValue();

   OptHashRefHelper ohrh(info, xsink);
   s->sendHTTPMessage(xsink, *ohrh, method->getBuffer(), path->getBuffer(), http_version->getBuffer(), headers, body ? tmp_data->getBuffer() : 0, body ? tmp_data->strlen() : 0, QORE_SOURCE_SOCKET, timeout_ms);
    return QoreValue();
}

// nothing Socket::sendHTTPMessage(string method, string path, string http_version, hash headers, binary body, *reference<hash> info, timeout timeout_ms = -1){}
static QoreValue Socket_sendHTTPMessage_VsVsVsVhVyC16_reference_hash_Vt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* method = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 1);
    const QoreStringNode* http_version = HARD_QORE_VALUE_STRING(args, 2);
    const QoreHashNode* headers = HARD_QORE_VALUE_HASH(args, 3);
    const BinaryNode* body = HARD_QORE_VALUE_BINARY(args, 4);
    const ReferenceNode* info = get_param_value(args, 5).get<const ReferenceNode>();
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 6);
# 1854 "QC_Socket.qpp"
    OptHashRefHelper ohrh(info, xsink);
   s->sendHTTPMessage(xsink, *ohrh, method->getBuffer(), path->getBuffer(), http_version->getBuffer(), headers, body->getPtr(), body->size(), QORE_SOURCE_SOCKET, timeout_ms);
    return QoreValue();
}

// nothing Socket::sendHTTPMessageWithCallback(code scb, string method, string path, string http_version, hash headers, *reference<hash> info, timeout timeout_ms = -1){}
static QoreValue Socket_sendHTTPMessageWithCallback_VcVsVsVsVhC16_reference_hash_Vt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const ResolvedCallReferenceNode* scb = get_param_value(args, 0).get<const ResolvedCallReferenceNode>();
    const QoreStringNode* method = HARD_QORE_VALUE_STRING(args, 1);
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 2);
    const QoreStringNode* http_version = HARD_QORE_VALUE_STRING(args, 3);
    const QoreHashNode* headers = HARD_QORE_VALUE_HASH(args, 4);
    const ReferenceNode* info = get_param_value(args, 5).get<const ReferenceNode>();
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 6);
# 1888 "QC_Socket.qpp"
    OptHashRefHelper ohrh(info, xsink);
   s->sendHTTPMessageWithCallback(xsink, *ohrh, method->getBuffer(), path->getBuffer(), http_version->getBuffer(), headers, *scb, QORE_SOURCE_SOCKET, timeout_ms);
    return QoreValue();
}

// nothing Socket::sendHTTPResponse(softint status_code, string status_desc, string http_version, hash headers, *string body, timeout timeout_ms = -1){}
static QoreValue Socket_sendHTTPResponse_viVsVsVhNsVt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 status_code = HARD_QORE_VALUE_INT(args, 0);
    const QoreStringNode* status_desc = HARD_QORE_VALUE_STRING(args, 1);
    const QoreStringNode* http_version = HARD_QORE_VALUE_STRING(args, 2);
    const QoreHashNode* headers = HARD_QORE_VALUE_HASH(args, 3);
    const QoreStringNode* body = get_param_value(args, 4).get<const QoreStringNode>();
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 5);
# 1918 "QC_Socket.qpp"
    if (status_code < 100 || status_code >= 600)
      return xsink->raiseException("SOCKET-SENDHTTPRESPONSE-STATUS-ERROR", "expecting valid HTTP status code between 100 and 599 as first parameter of Socket::sendHTTPResponse() call, got value %d instead", status_code);

   TempEncodingHelper tmp_data;
   if (body && !tmp_data.set(body, s->getEncoding(), xsink))
      return QoreValue();

   s->sendHTTPResponse(xsink, status_code, status_desc->getBuffer(), http_version->getBuffer(), headers, body ? tmp_data->getBuffer() : 0, body ? tmp_data->strlen() : 0, QORE_SOURCE_SOCKET, timeout_ms);
    return QoreValue();
}

// nothing Socket::sendHTTPResponse(softint status_code, string status_desc, string http_version, hash headers, binary body, timeout timeout_ms = -1){}
static QoreValue Socket_sendHTTPResponse_viVsVsVhVyVt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 status_code = HARD_QORE_VALUE_INT(args, 0);
    const QoreStringNode* status_desc = HARD_QORE_VALUE_STRING(args, 1);
    const QoreStringNode* http_version = HARD_QORE_VALUE_STRING(args, 2);
    const QoreHashNode* headers = HARD_QORE_VALUE_HASH(args, 3);
    const BinaryNode* body = HARD_QORE_VALUE_BINARY(args, 4);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 5);
# 1952 "QC_Socket.qpp"
    if (status_code < 100 || status_code >= 600)
      return xsink->raiseException("SOCKET-SENDHTTPRESPONSE-STATUS-ERROR", "expecting valid HTTP status code between 100 and 599 as first parameter of Socket::sendHTTPResponse() call, got value %d instead", status_code);

   s->sendHTTPResponse(xsink, status_code, status_desc->getBuffer(), http_version->getBuffer(), headers, body ? body->getPtr() : 0, body ? body->size() : 0, QORE_SOURCE_SOCKET, timeout_ms);
    return QoreValue();
}

// nothing Socket::sendHTTPResponseWithCallback(code scb, softint status_code, string status_desc, string http_version, hash headers, timeout timeout_ms = -1){}
static QoreValue Socket_sendHTTPResponseWithCallback_VcviVsVsVhVt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const ResolvedCallReferenceNode* scb = get_param_value(args, 0).get<const ResolvedCallReferenceNode>();
    int64 status_code = HARD_QORE_VALUE_INT(args, 1);
    const QoreStringNode* status_desc = HARD_QORE_VALUE_STRING(args, 2);
    const QoreStringNode* http_version = HARD_QORE_VALUE_STRING(args, 3);
    const QoreHashNode* headers = HARD_QORE_VALUE_HASH(args, 4);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 5);
# 1984 "QC_Socket.qpp"
    if (status_code < 100 || status_code >= 600)
      return xsink->raiseException("SOCKET-SENDHTTPRESPONSE-STATUS-ERROR", "expecting valid HTTP status code between 100 and 599 as second parameter of Socket::sendHTTPResponseWithCallback() call, got value %d instead", status_code);

   s->sendHTTPResponseWithCallback(xsink, status_code, status_desc->getBuffer(), http_version->getBuffer(), headers, *scb, QORE_SOURCE_SOCKET, timeout_ms);
    return QoreValue();
}

// int Socket::sendi1(softint i = 0, timeout timeout_ms = -1){}
static QoreValue Socket_sendi1_viVt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 i = HARD_QORE_VALUE_INT(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 1174 "QC_Socket.qpp"
    return s->sendi1((char)i, timeout_ms, xsink);
}

// int Socket::sendi2(softint i = 0, timeout timeout_ms = -1){}
static QoreValue Socket_sendi2_viVt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 i = HARD_QORE_VALUE_INT(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 1202 "QC_Socket.qpp"
    return s->sendi2((short)i, timeout_ms, xsink);
}

// int Socket::sendi2LSB(softint i = 0, timeout timeout_ms = -1){}
static QoreValue Socket_sendi2LSB_viVt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 i = HARD_QORE_VALUE_INT(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 1286 "QC_Socket.qpp"
    return s->sendi2LSB((short)i, timeout_ms, xsink);
}

// int Socket::sendi4(softint i = 0, timeout timeout_ms = -1){}
static QoreValue Socket_sendi4_viVt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 i = HARD_QORE_VALUE_INT(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 1230 "QC_Socket.qpp"
    return s->sendi4((int)i, timeout_ms, xsink);
}

// int Socket::sendi4LSB(softint i = 0, timeout timeout_ms = -1){}
static QoreValue Socket_sendi4LSB_viVt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 i = HARD_QORE_VALUE_INT(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 1314 "QC_Socket.qpp"
    return s->sendi4LSB((int)i, timeout_ms, xsink);
}

// int Socket::sendi8(softint i = 0, timeout timeout_ms = -1){}
static QoreValue Socket_sendi8_viVt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 i = HARD_QORE_VALUE_INT(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 1258 "QC_Socket.qpp"
    return s->sendi8(i, timeout_ms, xsink);
}

// int Socket::sendi8LSB(softint i = 0, timeout timeout_ms = -1){}
static QoreValue Socket_sendi8LSB_viVt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 i = HARD_QORE_VALUE_INT(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 1342 "QC_Socket.qpp"
    return s->sendi8LSB(i, timeout_ms, xsink);
}

// nothing Socket::setCertificate(SSLCertificate cert){}
static QoreValue Socket_setCertificate_C14SSLCertificate(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(cert, QoreSSLCertificate, args, 0, CID_SSLCERTIFICATE, "Socket::setCertificate()", "SSLCertificate", xsink);
    if (*xsink)
        return 0;
# 2542 "QC_Socket.qpp"
    // pass reference from QoreObject::getReferencedPrivateData() to QoreSocketObject::setCertificate()
   s->setCertificate(cert);
    return QoreValue();
}

// nothing Socket::setCertificate(string cert_pem){}
static QoreValue Socket_setCertificate_Vs(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* cert_pem = HARD_QORE_VALUE_STRING(args, 0);
# 2555 "QC_Socket.qpp"
    SimpleRefHolder<QoreSSLCertificate> cert(new QoreSSLCertificate(cert_pem, xsink));
   if (*xsink)
      return QoreValue();

   // QoreSocketObject::setCertificate() takes over ownership of certificate reference
   s->setCertificate(cert.release());
    return QoreValue();
}

// nothing Socket::setCertificate(binary cert_der){}
static QoreValue Socket_setCertificate_Vy(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* cert_der = HARD_QORE_VALUE_BINARY(args, 0);
# 2572 "QC_Socket.qpp"
    SimpleRefHolder<QoreSSLCertificate> cert(new QoreSSLCertificate(cert_der, xsink));
   if (*xsink)
      return QoreValue();

   // QoreSocketObject::setCertificate() takes over ownership of certificate reference
   s->setCertificate(cert.release());
    return QoreValue();
}

// nothing Socket::setCharset(string encoding){}
static QoreValue Socket_setCharset_Vs(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* encoding = HARD_QORE_VALUE_STRING(args, 0);
# 2429 "QC_Socket.qpp"
    s->setEncoding(QEM.findCreate(encoding));
    return QoreValue();
}

// nothing Socket::setEncoding(string encoding){}
static QoreValue Socket_setEncoding_Vs(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* encoding = HARD_QORE_VALUE_STRING(args, 0);
# 2414 "QC_Socket.qpp"
    s->setEncoding(QEM.findCreate(encoding));
    return QoreValue();
}

// nothing Socket::setEventQueue(){}
static QoreValue Socket_setEventQueue(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2635 "QC_Socket.qpp"
    s->setEventQueue(0, xsink);
    return QoreValue();
}

// nothing Socket::setEventQueue(Queue queue){}
static QoreValue Socket_setEventQueue_C5Queue(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(queue, Queue, args, 0, CID_QUEUE, "Socket::setEventQueue()", "Queue", xsink);
    if (*xsink)
        return 0;
# 2651 "QC_Socket.qpp"
    ReferenceHolder<Queue> q(queue, xsink);
   if (queue->getMax() != -1)
      return xsink->raiseException("QUEUE-ERROR", "the Queue object passed as a maximum size of %d entr%s, which could cause internal I/O to freeze in the Socket object if used; use a Queue object with no maximum size for I/O monitoring", queue->getMax(), queue->getMax() == 1 ? "y" : "ies");

   // pass reference from QoreObject::getReferencedPrivateData() to function
   s->setEventQueue(q.release(), xsink);
    return QoreValue();
}

// int Socket::setNoDelay(bool nd = True){}
static QoreValue Socket_setNoDelay_Vb(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    bool nd = HARD_QORE_VALUE_BOOL(args, 0);
# 2679 "QC_Socket.qpp"
    return s->setNoDelay(nd);
}

// nothing Socket::setPrivateKey(SSLPrivateKey key){}
static QoreValue Socket_setPrivateKey_C13SSLPrivateKey(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(key, QoreSSLPrivateKey, args, 0, CID_SSLPRIVATEKEY, "Socket::setPrivateKey()", "SSLPrivateKey", xsink);
    if (*xsink)
        return 0;
# 2589 "QC_Socket.qpp"
    // pass reference from QoreObject::getReferencedPrivateData() to QoreSocketObject::setPrivateKey()
   s->setPrivateKey(key);
    return QoreValue();
}

// nothing Socket::setPrivateKey(string key_pem, *string pass){}
static QoreValue Socket_setPrivateKey_VsNs(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* key_pem = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* pass = get_param_value(args, 1).get<const QoreStringNode>();
# 2603 "QC_Socket.qpp"
    SimpleRefHolder<QoreSSLPrivateKey> key(new QoreSSLPrivateKey(key_pem, pass ? pass->getBuffer() : 0, xsink));
   if (*xsink)
      return QoreValue();

   s->setPrivateKey(key.release());
    return QoreValue();
}

// nothing Socket::setPrivateKey(binary key_der){}
static QoreValue Socket_setPrivateKey_Vy(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* key_der = HARD_QORE_VALUE_BINARY(args, 0);
# 2619 "QC_Socket.qpp"
    SimpleRefHolder<QoreSSLPrivateKey> key(new QoreSSLPrivateKey(key_der, xsink));
   if (*xsink)
      return QoreValue();

   s->setPrivateKey(key.release());
    return QoreValue();
}

// int Socket::setRecvTimeout(timeout timeout_ms){}
static QoreValue Socket_setRecvTimeout_Vt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 2376 "QC_Socket.qpp"
    return s->setRecvTimeout(timeout_ms);
}

// int Socket::setSendTimeout(timeout timeout_ms){}
static QoreValue Socket_setSendTimeout_Vt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 2359 "QC_Socket.qpp"
    return s->setSendTimeout(timeout_ms);
}

// nothing Socket::setSslVerifyMode(int mode){}
static QoreValue Socket_setSslVerifyMode_Vi(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 mode = HARD_QORE_VALUE_INT(args, 0);
# 2900 "QC_Socket.qpp"
    s->setSslVerifyMode((int)mode);
    return QoreValue();
}

// nothing Socket::setWarningQueue(int warning_ms, int warning_bs, Queue queue, auto arg, timeout min_ms = 1s){}
static QoreValue Socket_setWarningQueue_ViViC5QueueVAVt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 warning_ms = HARD_QORE_VALUE_INT(args, 0);
    int64 warning_bs = HARD_QORE_VALUE_INT(args, 1);
    HARD_QORE_VALUE_OBJ_DATA(queue, Queue, args, 2, CID_QUEUE, "Socket::setWarningQueue()", "Queue", xsink);
    if (*xsink)
        return 0;
    QoreValue arg = get_param_value(args, 3);
    int64 min_ms = HARD_QORE_VALUE_INT(args, 4);
# 2824 "QC_Socket.qpp"
    ReferenceHolder<Queue> q(queue, xsink);
   if (queue->getMax() != -1)
      return xsink->raiseException("QUEUE-ERROR", "the Queue object passed as a maximum size of %d entr%s, which could cause internal I/O to freeze in the Socket object if used; use a Queue object with no maximum size for I/O monitoring", queue->getMax(), queue->getMax() == 1 ? "y" : "ies");

   // pass reference from QoreObject::getReferencedPrivateData() to function
   s->setWarningQueue(xsink, warning_ms, warning_bs, q.release(), arg.getReferencedValue(), min_ms);
    return QoreValue();
}

// int Socket::shutdown(){}
static QoreValue Socket_shutdown(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2304 "QC_Socket.qpp"
    return s->shutdown();
}

// nothing Socket::shutdownSSL(){}
static QoreValue Socket_shutdownSSL(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2318 "QC_Socket.qpp"
    s->shutdownSSL(xsink);
    return QoreValue();
}

// nothing Socket::upgradeClientToSSL(timeout timeout_ms = -1){}
static QoreValue Socket_upgradeClientToSSL_Vt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 2754 "QC_Socket.qpp"
    s->upgradeClientToSSL(timeout_ms, xsink);
    return QoreValue();
}

// nothing Socket::upgradeServerToSSL(timeout timeout_ms = -1){}
static QoreValue Socket_upgradeServerToSSL_Vt(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 2773 "QC_Socket.qpp"
    s->upgradeServerToSSL(timeout_ms, xsink);
    return QoreValue();
}

// *string Socket::verifyPeerCertificate(){}
static QoreValue Socket_verifyPeerCertificate(QoreObject* self, QoreSocketObject* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2529 "QC_Socket.qpp"
    const char *c = getSSLCVCode(s->verifyPeerCertificate());
   return c ? new QoreStringNode(c) : 0;
}

DLLLOCAL void preinitSocketClass() {
    QC_SOCKET = new QoreClass("Socket", QDOM_NETWORK);
    CID_SOCKET = QC_SOCKET->getID();
    QC_SOCKET->setSystem();
}

DLLLOCAL QoreClass* initSocketClass(QoreNamespace& ns) {
    if (!QC_SOCKET)
        preinitSocketClass();

    // public members
    QC_SOCKET->addMember("source", Public, stringOrNothingTypeInfo);
    QC_SOCKET->addMember("source_host", Public, stringOrNothingTypeInfo);

    QC_SOCKET->unsetPublicMemberFlag();

    // Socket Socket::accept(){}
    QC_SOCKET->addMethod("accept", (q_method_n_t)Socket_accept, false, QC_NO_FLAGS, QDOM_DEFAULT, QC_SOCKET->getTypeInfo());

    // *Socket Socket::accept(timeout timeout_ms){}
    QC_SOCKET->addMethod("accept", (q_method_n_t)Socket_accept_Vt, false, QC_NO_FLAGS, QDOM_DEFAULT, QC_SOCKET->getOrNothingTypeInfo(), 1, timeoutTypeInfo, NULL, "timeout_ms");

    // nothing Socket::acceptAllCertificates(bool accept_all = True){}
    QC_SOCKET->addMethod("acceptAllCertificates", (q_method_n_t)Socket_acceptAllCertificates_Vb, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, boolTypeInfo, &True, "accept_all");

    // Socket Socket::acceptSSL(){}
    QC_SOCKET->addMethod("acceptSSL", (q_method_n_t)Socket_acceptSSL, false, QC_NO_FLAGS, QDOM_DEFAULT, QC_SOCKET->getTypeInfo());

    // *Socket Socket::acceptSSL(timeout timeout_ms){}
    QC_SOCKET->addMethod("acceptSSL", (q_method_n_t)Socket_acceptSSL_Vt, false, QC_NO_FLAGS, QDOM_DEFAULT, QC_SOCKET->getOrNothingTypeInfo(), 1, timeoutTypeInfo, NULL, "timeout_ms");

    // int Socket::bind(string str, softbool reuseaddr = False){}
    QC_SOCKET->addMethod("bind", (q_method_n_t)Socket_bind_Vsvb, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 2, stringTypeInfo, NULL, "str", softBoolTypeInfo, &False, "reuseaddr");

    // int Socket::bind(int port, softbool reuseaddr = False){}
    QC_SOCKET->addMethod("bind", (q_method_n_t)Socket_bind_Vivb, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 2, bigIntTypeInfo, NULL, "port", softBoolTypeInfo, &False, "reuseaddr");

    // nothing Socket::bindINET(*string iface, *softstring service, softbool reuseaddr = False, softint family = AF_UNSPEC, softint socktype = SOCK_STREAM, softint protocol = 0){}
    QC_SOCKET->addMethod("bindINET", (q_method_n_t)Socket_bindINET_Nsnsvbvivivi, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 6, stringOrNothingTypeInfo, NULL, "iface", softStringOrNothingTypeInfo, NULL, "service", softBoolTypeInfo, &False, "reuseaddr", softBigIntTypeInfo, new QoreBigIntNode(AF_UNSPEC), "family", softBigIntTypeInfo, new QoreBigIntNode(SOCK_STREAM), "socktype", softBigIntTypeInfo, zero(), "protocol");

    // nothing Socket::bindUNIX(string path, softint socktype = SOCK_STREAM, softint protocol = 0){}
    QC_SOCKET->addMethod("bindUNIX", (q_method_n_t)Socket_bindUNIX_Vsvivi, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 3, stringTypeInfo, NULL, "path", softBigIntTypeInfo, new QoreBigIntNode(SOCK_STREAM), "socktype", softBigIntTypeInfo, zero(), "protocol");

    // nothing Socket::clearStats(){}
    QC_SOCKET->addMethod("clearStats", (q_method_n_t)Socket_clearStats, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing Socket::clearWarningQueue(){}
    QC_SOCKET->addMethod("clearWarningQueue", (q_method_n_t)Socket_clearWarningQueue, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // int Socket::close(){}
    QC_SOCKET->addMethod("close", (q_method_n_t)Socket_close, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // nothing Socket::connect(string target, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("connect", (q_method_n_t)Socket_connect_VsVt, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, NULL, "target", timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // nothing Socket::connectINET(string host, softstring service, timeout timeout_ms = -1, softint family = AF_UNSPEC, softint socktype = SOCK_STREAM, softint protocol = 0){}
    QC_SOCKET->addMethod("connectINET", (q_method_n_t)Socket_connectINET_VsvsVtvivivi, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 6, stringTypeInfo, NULL, "host", softStringTypeInfo, NULL, "service", timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms", softBigIntTypeInfo, new QoreBigIntNode(AF_UNSPEC), "family", softBigIntTypeInfo, new QoreBigIntNode(SOCK_STREAM), "socktype", softBigIntTypeInfo, zero(), "protocol");

    // nothing Socket::connectINETSSL(string host, softstring service, timeout timeout_ms = -1, softint family = AF_UNSPEC, softint socktype = SOCK_STREAM, softint protocol = 0){}
    QC_SOCKET->addMethod("connectINETSSL", (q_method_n_t)Socket_connectINETSSL_VsvsVtvivivi, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 6, stringTypeInfo, NULL, "host", softStringTypeInfo, NULL, "service", timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms", softBigIntTypeInfo, new QoreBigIntNode(AF_UNSPEC), "family", softBigIntTypeInfo, new QoreBigIntNode(SOCK_STREAM), "socktype", softBigIntTypeInfo, zero(), "protocol");

    // nothing Socket::connectSSL(string target, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("connectSSL", (q_method_n_t)Socket_connectSSL_VsVt, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, NULL, "target", timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // nothing Socket::connectUNIX(string path, softint socktype = SOCK_STREAM, softint protocol = 0){}
    QC_SOCKET->addMethod("connectUNIX", (q_method_n_t)Socket_connectUNIX_Vsvivi, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 3, stringTypeInfo, NULL, "path", softBigIntTypeInfo, new QoreBigIntNode(SOCK_STREAM), "socktype", softBigIntTypeInfo, zero(), "protocol");

    // nothing Socket::connectUNIXSSL(string path, softint socktype = SOCK_STREAM, softint protocol = 0){}
    QC_SOCKET->addMethod("connectUNIXSSL", (q_method_n_t)Socket_connectUNIXSSL_Vsvivi, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 3, stringTypeInfo, NULL, "path", softBigIntTypeInfo, new QoreBigIntNode(SOCK_STREAM), "socktype", softBigIntTypeInfo, zero(), "protocol");

    // Socket::constructor() {}
    QC_SOCKET->addConstructor(Socket_constructor, false, QC_NO_FLAGS, QDOM_DEFAULT);

    // Socket::copy() {}
    QC_SOCKET->setCopy((q_copy_t)Socket_copy);

    // bool Socket::getAcceptAllCertificates(){}
    QC_SOCKET->addMethod("getAcceptAllCertificates", (q_method_n_t)Socket_getAcceptAllCertificates, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // string Socket::getCharset(){}
    QC_SOCKET->addMethod("getCharset", (q_method_n_t)Socket_getCharset, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // string Socket::getEncoding(){}
    QC_SOCKET->addMethod("getEncoding", (q_method_n_t)Socket_getEncoding, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // bool Socket::getNoDelay(){}
    QC_SOCKET->addMethod("getNoDelay", (q_method_n_t)Socket_getNoDelay, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // hash Socket::getPeerInfo(bool host_lookup = True){}
    QC_SOCKET->addMethod("getPeerInfo", (q_method_n_t)Socket_getPeerInfo_Vb, false, QC_NO_FLAGS, QDOM_DEFAULT, hashTypeInfo, 1, boolTypeInfo, &True, "host_lookup");

    // int Socket::getPort(){}
    QC_SOCKET->addMethod("getPort", (q_method_n_t)Socket_getPort, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // int Socket::getRecvTimeout(){}
    QC_SOCKET->addMethod("getRecvTimeout", (q_method_n_t)Socket_getRecvTimeout, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // *string Socket::getSSLCipherName(){}
    QC_SOCKET->addMethod("getSSLCipherName", (q_method_n_t)Socket_getSSLCipherName, false, QC_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // *string Socket::getSSLCipherVersion(){}
    QC_SOCKET->addMethod("getSSLCipherVersion", (q_method_n_t)Socket_getSSLCipherVersion, false, QC_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // int Socket::getSendTimeout(){}
    QC_SOCKET->addMethod("getSendTimeout", (q_method_n_t)Socket_getSendTimeout, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // int Socket::getSocket(){}
    QC_SOCKET->addMethod("getSocket", (q_method_n_t)Socket_getSocket, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // hash Socket::getSocketInfo(bool host_lookup = True){}
    QC_SOCKET->addMethod("getSocketInfo", (q_method_n_t)Socket_getSocketInfo_Vb, false, QC_NO_FLAGS, QDOM_DEFAULT, hashTypeInfo, 1, boolTypeInfo, &True, "host_lookup");

    // int Socket::getSslVerifyMode(){}
    QC_SOCKET->addMethod("getSslVerifyMode", (q_method_n_t)Socket_getSslVerifyMode, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // hash Socket::getUsageInfo(){}
    QC_SOCKET->addMethod("getUsageInfo", (q_method_n_t)Socket_getUsageInfo, false, QC_CONSTANT, QDOM_DEFAULT, hashTypeInfo);

    // bool Socket::isDataAvailable(timeout timeout_ms = 0){}
    QC_SOCKET->addMethod("isDataAvailable", (q_method_n_t)Socket_isDataAvailable_Vt, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo, 1, timeoutTypeInfo, zero(), "timeout_ms");

    // bool Socket::isOpen(){}
    QC_SOCKET->addMethod("isOpen", (q_method_n_t)Socket_isOpen, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // bool Socket::isSecure(){}
    QC_SOCKET->addMethod("isSecure", (q_method_n_t)Socket_isSecure, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // bool Socket::isWriteFinished(timeout timeout_ms = 0){}
    QC_SOCKET->addMethod("isWriteFinished", (q_method_n_t)Socket_isWriteFinished_Vt, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo, 1, timeoutTypeInfo, zero(), "timeout_ms");

    // int Socket::listen(int backlog = 20){}
    QC_SOCKET->addMethod("listen", (q_method_n_t)Socket_listen_Vi, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, bigIntTypeInfo, new QoreBigIntNode(20), "backlog");

    // bool Socket::pendingHttpChunkedBody(){}
    QC_SOCKET->addMethod("pendingHttpChunkedBody", (q_method_n_t)Socket_pendingHttpChunkedBody, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

    // hash Socket::readHTTPChunkedBody(timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("readHTTPChunkedBody", (q_method_n_t)Socket_readHTTPChunkedBody_Vt, false, QC_NO_FLAGS, QDOM_DEFAULT, hashTypeInfo, 1, timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // hash Socket::readHTTPChunkedBodyBinary(timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("readHTTPChunkedBodyBinary", (q_method_n_t)Socket_readHTTPChunkedBodyBinary_Vt, false, QC_NO_FLAGS, QDOM_DEFAULT, hashTypeInfo, 1, timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // nothing Socket::readHTTPChunkedBodyBinaryWithCallback(code rcb, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("readHTTPChunkedBodyBinaryWithCallback", (q_method_n_t)Socket_readHTTPChunkedBodyBinaryWithCallback_VcVt, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, codeTypeInfo, NULL, "rcb", timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // hash Socket::readHTTPChunkedBodyToOutputStream(Qore::OutputStream os, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("readHTTPChunkedBodyToOutputStream", (q_method_n_t)Socket_readHTTPChunkedBodyToOutputStream_C12OutputStreamVt, false, QC_NO_FLAGS, QDOM_DEFAULT, hashTypeInfo, 2, QC_OUTPUTSTREAM->getTypeInfo(), NULL, "os", timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // nothing Socket::readHTTPChunkedBodyWithCallback(code rcb, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("readHTTPChunkedBodyWithCallback", (q_method_n_t)Socket_readHTTPChunkedBodyWithCallback_VcVt, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, codeTypeInfo, NULL, "rcb", timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // hash Socket::readHTTPHeader(timeout timeout_ms = -1, *reference<hash> info){}
    QC_SOCKET->addMethod("readHTTPHeader", (q_method_n_t)Socket_readHTTPHeader_VtC16_reference_hash_, false, QC_NO_FLAGS, QDOM_DEFAULT, hashTypeInfo, 2, timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms", qore_get_complex_reference_or_nothing_type(hashTypeInfo), NULL, "info");

    // string Socket::readHTTPHeaderString(timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("readHTTPHeaderString", (q_method_n_t)Socket_readHTTPHeaderString_Vt, false, QC_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 1, timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // string Socket::recv(softint size = 0, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("recv", (q_method_n_t)Socket_recv_viVt, false, QC_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 2, softBigIntTypeInfo, zero(), "size", timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // binary Socket::recvBinary(softint size = 0, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("recvBinary", (q_method_n_t)Socket_recvBinary_viVt, false, QC_NO_FLAGS, QDOM_DEFAULT, binaryTypeInfo, 2, softBigIntTypeInfo, zero(), "size", timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // nothing Socket::recvToOutputStream(Qore::OutputStream os, softint size = -1, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("recvToOutputStream", (q_method_n_t)Socket_recvToOutputStream_C12OutputStreamviVt, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 3, QC_OUTPUTSTREAM->getTypeInfo(), NULL, "os", softBigIntTypeInfo, new QoreBigIntNode(-1), "size", timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // int Socket::recvi1(timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("recvi1", (q_method_n_t)Socket_recvi1_Vt, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // int Socket::recvi2(timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("recvi2", (q_method_n_t)Socket_recvi2_Vt, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // int Socket::recvi2LSB(timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("recvi2LSB", (q_method_n_t)Socket_recvi2LSB_Vt, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // int Socket::recvi4(timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("recvi4", (q_method_n_t)Socket_recvi4_Vt, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // int Socket::recvi4LSB(timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("recvi4LSB", (q_method_n_t)Socket_recvi4LSB_Vt, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // int Socket::recvi8(timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("recvi8", (q_method_n_t)Socket_recvi8_Vt, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // int Socket::recvi8LSB(timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("recvi8LSB", (q_method_n_t)Socket_recvi8LSB_Vt, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // int Socket::recvu1(timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("recvu1", (q_method_n_t)Socket_recvu1_Vt, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // int Socket::recvu2(timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("recvu2", (q_method_n_t)Socket_recvu2_Vt, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // int Socket::recvu2LSB(timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("recvu2LSB", (q_method_n_t)Socket_recvu2LSB_Vt, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // int Socket::recvu4(timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("recvu4", (q_method_n_t)Socket_recvu4_Vt, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // int Socket::recvu4LSB(timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("recvu4LSB", (q_method_n_t)Socket_recvu4LSB_Vt, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // int Socket::send(binary bin, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("send", (q_method_n_t)Socket_send_VyVt, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 2, binaryTypeInfo, NULL, "bin", timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // int Socket::send(string str, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("send", (q_method_n_t)Socket_send_VsVt, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 2, stringTypeInfo, NULL, "str", timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // nothing Socket::send2(binary bin, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("send2", (q_method_n_t)Socket_send2_VyVt, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, binaryTypeInfo, NULL, "bin", timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // nothing Socket::send2(string str, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("send2", (q_method_n_t)Socket_send2_VsVt, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, NULL, "str", timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // int Socket::sendBinary(string str, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("sendBinary", (q_method_n_t)Socket_sendBinary_VsVt, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 2, stringTypeInfo, NULL, "str", timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // int Socket::sendBinary(binary bin, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("sendBinary", (q_method_n_t)Socket_sendBinary_VyVt, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 2, binaryTypeInfo, NULL, "bin", timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // nothing Socket::sendBinary2(string str, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("sendBinary2", (q_method_n_t)Socket_sendBinary2_VsVt, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, NULL, "str", timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // nothing Socket::sendBinary2(binary bin, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("sendBinary2", (q_method_n_t)Socket_sendBinary2_VyVt, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, binaryTypeInfo, NULL, "bin", timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // nothing Socket::sendFromInputStream(Qore::InputStream is, softint size = -1, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("sendFromInputStream", (q_method_n_t)Socket_sendFromInputStream_C11InputStreamviVt, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 3, QC_INPUTSTREAM->getTypeInfo(), NULL, "is", softBigIntTypeInfo, new QoreBigIntNode(-1), "size", timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // nothing Socket::sendHTTPChunkedBodyFromInputStream(Qore::InputStream is, int max_chunk_size = 4096, timeout timeout_ms = -1, *code tcb){}
    QC_SOCKET->addMethod("sendHTTPChunkedBodyFromInputStream", (q_method_n_t)Socket_sendHTTPChunkedBodyFromInputStream_C11InputStreamViVtNc, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 4, QC_INPUTSTREAM->getTypeInfo(), NULL, "is", bigIntTypeInfo, new QoreBigIntNode(4096), "max_chunk_size", timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms", codeOrNothingTypeInfo, NULL, "tcb");

    // nothing Socket::sendHTTPChunkedBodyTrailer(*hash trailer, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("sendHTTPChunkedBodyTrailer", (q_method_n_t)Socket_sendHTTPChunkedBodyTrailer_NhVt, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, hashOrNothingTypeInfo, NULL, "trailer", timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // nothing Socket::sendHTTPMessage(string method, string path, string http_version, hash headers, *string body, *reference<hash> info, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("sendHTTPMessage", (q_method_n_t)Socket_sendHTTPMessage_VsVsVsVhNsC16_reference_hash_Vt, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 7, stringTypeInfo, NULL, "method", stringTypeInfo, NULL, "path", stringTypeInfo, NULL, "http_version", hashTypeInfo, NULL, "headers", stringOrNothingTypeInfo, NULL, "body", qore_get_complex_reference_or_nothing_type(hashTypeInfo), NULL, "info", timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // nothing Socket::sendHTTPMessage(string method, string path, string http_version, hash headers, binary body, *reference<hash> info, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("sendHTTPMessage", (q_method_n_t)Socket_sendHTTPMessage_VsVsVsVhVyC16_reference_hash_Vt, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 7, stringTypeInfo, NULL, "method", stringTypeInfo, NULL, "path", stringTypeInfo, NULL, "http_version", hashTypeInfo, NULL, "headers", binaryTypeInfo, NULL, "body", qore_get_complex_reference_or_nothing_type(hashTypeInfo), NULL, "info", timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // nothing Socket::sendHTTPMessageWithCallback(code scb, string method, string path, string http_version, hash headers, *reference<hash> info, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("sendHTTPMessageWithCallback", (q_method_n_t)Socket_sendHTTPMessageWithCallback_VcVsVsVsVhC16_reference_hash_Vt, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 7, codeTypeInfo, NULL, "scb", stringTypeInfo, NULL, "method", stringTypeInfo, NULL, "path", stringTypeInfo, NULL, "http_version", hashTypeInfo, NULL, "headers", qore_get_complex_reference_or_nothing_type(hashTypeInfo), NULL, "info", timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // nothing Socket::sendHTTPResponse(softint status_code, string status_desc, string http_version, hash headers, *string body, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("sendHTTPResponse", (q_method_n_t)Socket_sendHTTPResponse_viVsVsVhNsVt, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 6, softBigIntTypeInfo, NULL, "status_code", stringTypeInfo, NULL, "status_desc", stringTypeInfo, NULL, "http_version", hashTypeInfo, NULL, "headers", stringOrNothingTypeInfo, NULL, "body", timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // nothing Socket::sendHTTPResponse(softint status_code, string status_desc, string http_version, hash headers, binary body, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("sendHTTPResponse", (q_method_n_t)Socket_sendHTTPResponse_viVsVsVhVyVt, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 6, softBigIntTypeInfo, NULL, "status_code", stringTypeInfo, NULL, "status_desc", stringTypeInfo, NULL, "http_version", hashTypeInfo, NULL, "headers", binaryTypeInfo, NULL, "body", timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // nothing Socket::sendHTTPResponseWithCallback(code scb, softint status_code, string status_desc, string http_version, hash headers, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("sendHTTPResponseWithCallback", (q_method_n_t)Socket_sendHTTPResponseWithCallback_VcviVsVsVhVt, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 6, codeTypeInfo, NULL, "scb", softBigIntTypeInfo, NULL, "status_code", stringTypeInfo, NULL, "status_desc", stringTypeInfo, NULL, "http_version", hashTypeInfo, NULL, "headers", timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // int Socket::sendi1(softint i = 0, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("sendi1", (q_method_n_t)Socket_sendi1_viVt, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 2, softBigIntTypeInfo, zero(), "i", timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // int Socket::sendi2(softint i = 0, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("sendi2", (q_method_n_t)Socket_sendi2_viVt, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 2, softBigIntTypeInfo, zero(), "i", timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // int Socket::sendi2LSB(softint i = 0, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("sendi2LSB", (q_method_n_t)Socket_sendi2LSB_viVt, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 2, softBigIntTypeInfo, zero(), "i", timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // int Socket::sendi4(softint i = 0, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("sendi4", (q_method_n_t)Socket_sendi4_viVt, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 2, softBigIntTypeInfo, zero(), "i", timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // int Socket::sendi4LSB(softint i = 0, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("sendi4LSB", (q_method_n_t)Socket_sendi4LSB_viVt, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 2, softBigIntTypeInfo, zero(), "i", timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // int Socket::sendi8(softint i = 0, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("sendi8", (q_method_n_t)Socket_sendi8_viVt, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 2, softBigIntTypeInfo, zero(), "i", timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // int Socket::sendi8LSB(softint i = 0, timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("sendi8LSB", (q_method_n_t)Socket_sendi8LSB_viVt, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 2, softBigIntTypeInfo, zero(), "i", timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // nothing Socket::setCertificate(SSLCertificate cert){}
    QC_SOCKET->addMethod("setCertificate", (q_method_n_t)Socket_setCertificate_C14SSLCertificate, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, QC_SSLCERTIFICATE->getTypeInfo(), NULL, "cert");

    // nothing Socket::setCertificate(string cert_pem){}
    QC_SOCKET->addMethod("setCertificate", (q_method_n_t)Socket_setCertificate_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "cert_pem");

    // nothing Socket::setCertificate(binary cert_der){}
    QC_SOCKET->addMethod("setCertificate", (q_method_n_t)Socket_setCertificate_Vy, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, binaryTypeInfo, NULL, "cert_der");

    // nothing Socket::setCharset(string encoding){}
    QC_SOCKET->addMethod("setCharset", (q_method_n_t)Socket_setCharset_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "encoding");

    // nothing Socket::setEncoding(string encoding){}
    QC_SOCKET->addMethod("setEncoding", (q_method_n_t)Socket_setEncoding_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "encoding");

    // nothing Socket::setEventQueue(){}
    QC_SOCKET->addMethod("setEventQueue", (q_method_n_t)Socket_setEventQueue, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing Socket::setEventQueue(Queue queue){}
    QC_SOCKET->addMethod("setEventQueue", (q_method_n_t)Socket_setEventQueue_C5Queue, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, QC_QUEUE->getTypeInfo(), NULL, "queue");

    // int Socket::setNoDelay(bool nd = True){}
    QC_SOCKET->addMethod("setNoDelay", (q_method_n_t)Socket_setNoDelay_Vb, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, boolTypeInfo, &True, "nd");

    // nothing Socket::setPrivateKey(SSLPrivateKey key){}
    QC_SOCKET->addMethod("setPrivateKey", (q_method_n_t)Socket_setPrivateKey_C13SSLPrivateKey, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, QC_SSLPRIVATEKEY->getTypeInfo(), NULL, "key");

    // nothing Socket::setPrivateKey(string key_pem, *string pass){}
    QC_SOCKET->addMethod("setPrivateKey", (q_method_n_t)Socket_setPrivateKey_VsNs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, NULL, "key_pem", stringOrNothingTypeInfo, NULL, "pass");

    // nothing Socket::setPrivateKey(binary key_der){}
    QC_SOCKET->addMethod("setPrivateKey", (q_method_n_t)Socket_setPrivateKey_Vy, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, binaryTypeInfo, NULL, "key_der");

    // int Socket::setRecvTimeout(timeout timeout_ms){}
    QC_SOCKET->addMethod("setRecvTimeout", (q_method_n_t)Socket_setRecvTimeout_Vt, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, timeoutTypeInfo, NULL, "timeout_ms");

    // int Socket::setSendTimeout(timeout timeout_ms){}
    QC_SOCKET->addMethod("setSendTimeout", (q_method_n_t)Socket_setSendTimeout_Vt, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, timeoutTypeInfo, NULL, "timeout_ms");

    // nothing Socket::setSslVerifyMode(int mode){}
    QC_SOCKET->addMethod("setSslVerifyMode", (q_method_n_t)Socket_setSslVerifyMode_Vi, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, bigIntTypeInfo, NULL, "mode");

    // nothing Socket::setWarningQueue(int warning_ms, int warning_bs, Queue queue, auto arg, timeout min_ms = 1s){}
    QC_SOCKET->addMethod("setWarningQueue", (q_method_n_t)Socket_setWarningQueue_ViViC5QueueVAVt, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 5, bigIntTypeInfo, NULL, "warning_ms", bigIntTypeInfo, NULL, "warning_bs", QC_QUEUE->getTypeInfo(), NULL, "queue", autoTypeInfo, NULL, "arg", timeoutTypeInfo, DateTimeNode::makeRelative(0, 0, 0, 0, 0, 1, 0), "min_ms");

    // int Socket::shutdown(){}
    QC_SOCKET->addMethod("shutdown", (q_method_n_t)Socket_shutdown, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // nothing Socket::shutdownSSL(){}
    QC_SOCKET->addMethod("shutdownSSL", (q_method_n_t)Socket_shutdownSSL, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing Socket::upgradeClientToSSL(timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("upgradeClientToSSL", (q_method_n_t)Socket_upgradeClientToSSL_Vt, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // nothing Socket::upgradeServerToSSL(timeout timeout_ms = -1){}
    QC_SOCKET->addMethod("upgradeServerToSSL", (q_method_n_t)Socket_upgradeServerToSSL_Vt, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // *string Socket::verifyPeerCertificate(){}
    QC_SOCKET->addMethod("verifyPeerCertificate", (q_method_n_t)Socket_verifyPeerCertificate, false, QC_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);
# 72 "QC_Socket.qpp"
# 87 "QC_Socket.qpp"
# 105 "QC_Socket.qpp"
    ns.addConstant("SSL_VERIFY_CLIENT_ONCE", new QoreBigIntNode(SSL_VERIFY_CLIENT_ONCE));
    ns.addConstant("SSL_VERIFY_FAIL_IF_NO_PEER_CERT", new QoreBigIntNode(SSL_VERIFY_FAIL_IF_NO_PEER_CERT));
    ns.addConstant("SSL_VERIFY_NONE", new QoreBigIntNode(SSL_VERIFY_NONE));
    ns.addConstant("SSL_VERIFY_PEER", new QoreBigIntNode(SSL_VERIFY_PEER));
# 132 "QC_Socket.qpp"
# 142 "QC_Socket.qpp"
# 145 "QC_Socket.qpp"
# 148 "QC_Socket.qpp"
# 151 "QC_Socket.qpp"
# 154 "QC_Socket.qpp"
# 157 "QC_Socket.qpp"
# 160 "QC_Socket.qpp"
# 163 "QC_Socket.qpp"
# 166 "QC_Socket.qpp"
# 169 "QC_Socket.qpp"
# 172 "QC_Socket.qpp"
# 175 "QC_Socket.qpp"
# 178 "QC_Socket.qpp"
# 181 "QC_Socket.qpp"
# 184 "QC_Socket.qpp"
# 187 "QC_Socket.qpp"
# 190 "QC_Socket.qpp"
# 193 "QC_Socket.qpp"
# 196 "QC_Socket.qpp"
# 199 "QC_Socket.qpp"
# 202 "QC_Socket.qpp"
# 205 "QC_Socket.qpp"
# 208 "QC_Socket.qpp"
# 211 "QC_Socket.qpp"
# 214 "QC_Socket.qpp"
# 217 "QC_Socket.qpp"
# 220 "QC_Socket.qpp"
# 223 "QC_Socket.qpp"
# 226 "QC_Socket.qpp"
# 229 "QC_Socket.qpp"
# 232 "QC_Socket.qpp"
# 235 "QC_Socket.qpp"
# 238 "QC_Socket.qpp"
    ns.addConstant("X509_V_ERR_AKID_ISSUER_SERIAL_MISMATCH", new QoreStringNode("X509_V_ERR_AKID_ISSUER_SERIAL_MISMATCH"));
    ns.addConstant("X509_V_ERR_AKID_SKID_MISMATCH", new QoreStringNode("X509_V_ERR_AKID_SKID_MISMATCH"));
    ns.addConstant("X509_V_ERR_APPLICATION_VERIFICATION", new QoreStringNode("X509_V_ERR_APPLICATION_VERIFICATION"));
    ns.addConstant("X509_V_ERR_CERT_CHAIN_TOO_LONG", new QoreStringNode("X509_V_ERR_CERT_CHAIN_TOO_LONG"));
    ns.addConstant("X509_V_ERR_CERT_HAS_EXPIRED", new QoreStringNode("X509_V_ERR_CERT_HAS_EXPIRED"));
    ns.addConstant("X509_V_ERR_CERT_NOT_YET_VALID", new QoreStringNode("X509_V_ERR_CERT_NOT_YET_VALID"));
    ns.addConstant("X509_V_ERR_CERT_REJECTED", new QoreStringNode("X509_V_ERR_CERT_REJECTED"));
    ns.addConstant("X509_V_ERR_CERT_REVOKED", new QoreStringNode("X509_V_ERR_CERT_REVOKED"));
    ns.addConstant("X509_V_ERR_CERT_SIGNATURE_FAILURE", new QoreStringNode("X509_V_ERR_CERT_SIGNATURE_FAILURE"));
    ns.addConstant("X509_V_ERR_CERT_UNTRUSTED", new QoreStringNode("X509_V_ERR_CERT_UNTRUSTED"));
    ns.addConstant("X509_V_ERR_CRL_HAS_EXPIRED", new QoreStringNode("X509_V_ERR_CRL_HAS_EXPIRED"));
    ns.addConstant("X509_V_ERR_CRL_NOT_YET_VALID", new QoreStringNode("X509_V_ERR_CRL_NOT_YET_VALID"));
    ns.addConstant("X509_V_ERR_CRL_SIGNATURE_FAILURE", new QoreStringNode("X509_V_ERR_CRL_SIGNATURE_FAILURE"));
    ns.addConstant("X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT", new QoreStringNode("X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT"));
    ns.addConstant("X509_V_ERR_ERROR_IN_CERT_NOT_AFTER_FIELD", new QoreStringNode("X509_V_ERR_ERROR_IN_CERT_NOT_AFTER_FIELD"));
    ns.addConstant("X509_V_ERR_ERROR_IN_CERT_NOT_BEFORE_FIELD", new QoreStringNode("X509_V_ERR_ERROR_IN_CERT_NOT_BEFORE_FIELD"));
    ns.addConstant("X509_V_ERR_ERROR_IN_CRL_LAST_UPDATE_FIELD", new QoreStringNode("X509_V_ERR_ERROR_IN_CRL_LAST_UPDATE_FIELD"));
    ns.addConstant("X509_V_ERR_ERROR_IN_CRL_NEXT_UPDATE_FIELD", new QoreStringNode("X509_V_ERR_ERROR_IN_CRL_NEXT_UPDATE_FIELD"));
    ns.addConstant("X509_V_ERR_INVALID_CA", new QoreStringNode("X509_V_ERR_INVALID_CA"));
    ns.addConstant("X509_V_ERR_INVALID_PURPOSE", new QoreStringNode("X509_V_ERR_INVALID_PURPOSE"));
    ns.addConstant("X509_V_ERR_KEYUSAGE_NO_CERTSIGN", new QoreStringNode("X509_V_ERR_KEYUSAGE_NO_CERTSIGN"));
    ns.addConstant("X509_V_ERR_OUT_OF_MEM", new QoreStringNode("X509_V_ERR_OUT_OF_MEM"));
    ns.addConstant("X509_V_ERR_PATH_LENGTH_EXCEEDED", new QoreStringNode("X509_V_ERR_PATH_LENGTH_EXCEEDED"));
    ns.addConstant("X509_V_ERR_SELF_SIGNED_CERT_IN_CHAIN", new QoreStringNode("X509_V_ERR_SELF_SIGNED_CERT_IN_CHAIN"));
    ns.addConstant("X509_V_ERR_SUBJECT_ISSUER_MISMATCH", new QoreStringNode("X509_V_ERR_SUBJECT_ISSUER_MISMATCH"));
    ns.addConstant("X509_V_ERR_UNABLE_TO_DECODE_ISSUER_PUBLIC_KEY", new QoreStringNode("X509_V_ERR_UNABLE_TO_DECODE_ISSUER_PUBLIC_KEY"));
    ns.addConstant("X509_V_ERR_UNABLE_TO_DECRYPT_CERT_SIGNATURE", new QoreStringNode("X509_V_ERR_UNABLE_TO_DECRYPT_CERT_SIGNATURE"));
    ns.addConstant("X509_V_ERR_UNABLE_TO_DECRYPT_CRL_SIGNATURE", new QoreStringNode("X509_V_ERR_UNABLE_TO_DECRYPT_CRL_SIGNATURE"));
    ns.addConstant("X509_V_ERR_UNABLE_TO_GET_CRL", new QoreStringNode("X509_V_ERR_UNABLE_TO_GET_CRL"));
    ns.addConstant("X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT", new QoreStringNode("X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT"));
    ns.addConstant("X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT_LOCALLY", new QoreStringNode("X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT_LOCALLY"));
    ns.addConstant("X509_V_ERR_UNABLE_TO_VERIFY_LEAF_SIGNATURE", new QoreStringNode("X509_V_ERR_UNABLE_TO_VERIFY_LEAF_SIGNATURE"));
    ns.addConstant("X509_V_OK", new QoreStringNode("X509_V_OK"));
    QoreHashNode* hash_X509_VerificationReasons = new QoreHashNode;
    hash_X509_VerificationReasons->setKeyValue("X509_V_OK", new QoreStringNode("OK"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT", new QoreStringNode("Unable to get issuer certificate"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_UNABLE_TO_GET_CRL", new QoreStringNode("Unable to get certificate CRL"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_UNABLE_TO_DECRYPT_CERT_SIGNATURE", new QoreStringNode("Unable to decrypt certificate's signature. This means that the actual signature value could not be determined rather than it not matching the expected value; this is only meaningful for RSA"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_UNABLE_TO_DECRYPT_CRL_SIGNATURE", new QoreStringNode("Unable to decrypt CRL's signature"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_UNABLE_TO_DECODE_ISSUER_PUBLIC_KEY", new QoreStringNode("Unable to decode issuer public key (SubjectPublicKeyInfo)"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_CERT_SIGNATURE_FAILURE", new QoreStringNode("Certificate signature failure; the signature of the certificate is invalid"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_CRL_SIGNATURE_FAILURE", new QoreStringNode("CRL signature failure; the signature of the certificate is invalid"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_CERT_NOT_YET_VALID", new QoreStringNode("Certificate is not yet valid"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_CERT_HAS_EXPIRED", new QoreStringNode("Certificate has expired"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_CRL_NOT_YET_VALID", new QoreStringNode("CRL is not yet valid"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_CRL_HAS_EXPIRED", new QoreStringNode("CRL has expired"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_ERROR_IN_CERT_NOT_BEFORE_FIELD", new QoreStringNode("Format error in certificate's notBefore field (invalid time)"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_ERROR_IN_CERT_NOT_AFTER_FIELD", new QoreStringNode("Format error in certificate's notAfter field (invalid time)"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_ERROR_IN_CRL_LAST_UPDATE_FIELD", new QoreStringNode("Format error in CRL's lastUpdate field (invalid time)"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_ERROR_IN_CRL_NEXT_UPDATE_FIELD", new QoreStringNode("Format error in CRL's nextUpdate field (invalid time)"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_OUT_OF_MEM", new QoreStringNode("Out of memory error"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT", new QoreStringNode("Certificate is self-signed and cannot be found in the trusted list"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_SELF_SIGNED_CERT_IN_CHAIN", new QoreStringNode("Self signed certificate in certificate chain"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT_LOCALLY", new QoreStringNode("Unable to get local issuer certificate. This normally means the list of trusted certificates is not complete"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_UNABLE_TO_VERIFY_LEAF_SIGNATURE", new QoreStringNode("Unable to verify the first certificate"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_CERT_CHAIN_TOO_LONG", new QoreStringNode("Certificate chain too long"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_CERT_REVOKED", new QoreStringNode("Certificate has been revoked"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_INVALID_CA", new QoreStringNode("Invalid CA certificate"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_PATH_LENGTH_EXCEEDED", new QoreStringNode("The basicConstraints pathlength parameter has been exceeded"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_INVALID_PURPOSE", new QoreStringNode("The certificate cannot be used for the specified purpose"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_CERT_UNTRUSTED", new QoreStringNode("Root CA is not marked as trusted for the specified purpose"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_CERT_REJECTED", new QoreStringNode("Root CA is marked to reject the specified purpose"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_SUBJECT_ISSUER_MISMATCH", new QoreStringNode("The current candidate issuer certificate was rejected because its subject name did not match the issuer name of the current certificate"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_AKID_SKID_MISMATCH", new QoreStringNode("The current candidate issuer certificate was rejected because its subject key identifier was present and did not match the authority key identifier of the current certificate"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_AKID_ISSUER_SERIAL_MISMATCH", new QoreStringNode("Issuer name and serial number of candidate certificate do not match the authority key identifier of the current certificate"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_KEYUSAGE_NO_CERTSIGN", new QoreStringNode("The keyUsage extension does not permit certificate signing"), 0);
    hash_X509_VerificationReasons->setKeyValue("X509_V_ERR_APPLICATION_VERIFICATION", new QoreStringNode("Verification failure"), 0);

    ns.addConstant("X509_VerificationReasons", hash_X509_VerificationReasons);
# 276 "QC_Socket.qpp"
# 283 "QC_Socket.qpp"
# 286 "QC_Socket.qpp"
#ifndef AF_LOCAL
#define AF_LOCAL AF_UNIX
#endif
# 293 "QC_Socket.qpp"
# 296 "QC_Socket.qpp"
# 299 "QC_Socket.qpp"
# 302 "QC_Socket.qpp"
    ns.addConstant("AFMap", (get_network_address_family_map()));
    QoreHashNode* hash_AFStrMap = new QoreHashNode;
    hash_AFStrMap->setKeyValue("AF_INET", new QoreBigIntNode(AF_INET), 0);
    hash_AFStrMap->setKeyValue("AF_INET6", new QoreBigIntNode(AF_INET6), 0);
    hash_AFStrMap->setKeyValue("AF_UNIX", new QoreBigIntNode(AF_UNIX), 0);
    hash_AFStrMap->setKeyValue("AF_LOCAL", new QoreBigIntNode(AF_UNIX), 0);
    hash_AFStrMap->setKeyValue("AF_UNSPEC", new QoreBigIntNode(AF_UNSPEC), 0);

    ns.addConstant("AFStrMap", hash_AFStrMap);
    ns.addConstant("AF_INET", new QoreBigIntNode(AF_INET));
    ns.addConstant("AF_INET6", new QoreBigIntNode(AF_INET6));
    ns.addConstant("AF_LOCAL", new QoreBigIntNode(AF_LOCAL));
    ns.addConstant("AF_UNIX", new QoreBigIntNode(AF_UNIX));
    ns.addConstant("AF_UNSPEC", new QoreBigIntNode(AF_UNSPEC));
# 313 "QC_Socket.qpp"
#ifndef AI_ADDRCONFIG
#define AI_ADDRCONFIG 0
#endif
#ifndef AI_ALL
#define AI_ALL 0
#endif
#ifndef AI_CANONNAME
#define AI_CANONNAME 0
#endif
#ifndef AI_NUMERICHOST
#define AI_NUMERICHOST 0
#endif
#ifndef AI_NUMERICSERV
#define AI_NUMERICSERV 0
#endif
#ifndef AI_PASSIVE
#define AI_PASSIVE 0
#endif
#ifndef AI_V4MAPPED
#define AI_V4MAPPED 0
#endif
# 340 "QC_Socket.qpp"
# 343 "QC_Socket.qpp"
# 346 "QC_Socket.qpp"
# 349 "QC_Socket.qpp"
# 352 "QC_Socket.qpp"
# 358 "QC_Socket.qpp"
    ns.addConstant("AI_ADDRCONFIG", new QoreBigIntNode(AI_ADDRCONFIG));
    ns.addConstant("AI_ALL", new QoreBigIntNode(AI_ALL));
    ns.addConstant("AI_CANONNAME", new QoreBigIntNode(AI_CANONNAME));
    ns.addConstant("AI_NUMERICHOST", new QoreBigIntNode(AI_NUMERICHOST));
    ns.addConstant("AI_NUMERICSERV", new QoreBigIntNode(AI_NUMERICSERV));
    ns.addConstant("AI_PASSIVE", new QoreBigIntNode(AI_PASSIVE));
    ns.addConstant("AI_V4MAPPED", new QoreBigIntNode(AI_V4MAPPED));
# 373 "QC_Socket.qpp"
    ns.addConstant("IPPROTO_TCP", new QoreBigIntNode(IPPROTO_TCP));
    ns.addConstant("IPPROTO_UDP", new QoreBigIntNode(IPPROTO_UDP));
# 384 "QC_Socket.qpp"
# 391 "QC_Socket.qpp"
    ns.addConstant("SOCK_DGRAM", new QoreBigIntNode(SOCK_DGRAM));
    ns.addConstant("SOCK_RAW", new QoreBigIntNode(SOCK_RAW));
    ns.addConstant("SOCK_STREAM", new QoreBigIntNode(SOCK_STREAM));

    return QC_SOCKET;
}
