/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  QC_TimeZone.qpp

  Qore Programming Language

  Copyright (C) 2003 - 2016 David Nichols

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/QC_TimeZone.h"
/* Qore class Qore::TimeZone */

qore_classid_t CID_TIMEZONE;
QoreClass* QC_TIMEZONE;

// int TimeZone::UTCOffset(){}
static QoreValue TimeZone_UTCOffset(QoreObject* self, TimeZoneData* z, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 101 "QC_TimeZone.qpp"
    return AbstractQoreZoneInfo::getUTCOffset(z->get());
}

// TimeZone::constructor(string region) {}
static void TimeZone_constructor_Vs(QoreObject* self, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* region = HARD_QORE_VALUE_STRING(args, 0);
# 56 "QC_TimeZone.qpp"
bool is_path = (!region->empty() && region->getBuffer()[0] == '.') || q_absolute_path(region->getBuffer());
   if (is_path && runtime_check_parse_option(PO_NO_FILESYSTEM))
      xsink->raiseException("ILLEGAL-FILESYSTEM-ACCESS", "cannot create a TimeZone object from absolute path '%s' when sandboxing restriction PO_NO_FILESYSTEM is set", region->getBuffer());
   const AbstractQoreZoneInfo *zone = is_path
      ? QTZM.findLoadRegionFromPath(region->getBuffer(), xsink)
      : QTZM.findLoadRegion(region->getBuffer(), xsink);
   if (*xsink)
      return;

   self->setPrivate(CID_TIMEZONE, new TimeZoneData(zone));
}

// TimeZone::constructor(softint seconds_east) {}
static void TimeZone_constructor_vi(QoreObject* self, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 seconds_east = HARD_QORE_VALUE_INT(args, 0);
# 77 "QC_TimeZone.qpp"
const AbstractQoreZoneInfo *zone = QTZM.findCreateOffsetZone(seconds_east);
   self->setPrivate(CID_TIMEZONE, new TimeZoneData(zone));
}

// TimeZone::copy() {}
static void TimeZone_copy(QoreObject* self, QoreObject* old, TimeZoneData* z, ExceptionSink* xsink) {
# 89 "QC_TimeZone.qpp"
self->setPrivate(CID_TIMEZONE, new TimeZoneData(*z));
}

// date TimeZone::date(softint secs, softint us = 0){}
static QoreValue TimeZone_date_vivi(QoreObject* self, TimeZoneData* z, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 secs = HARD_QORE_VALUE_INT(args, 0);
    int64 us = HARD_QORE_VALUE_INT(args, 1);
# 140 "QC_TimeZone.qpp"
    return DateTimeNode::makeAbsolute(z->get(), secs, (int)us);
}

// date TimeZone::date(date d){}
static QoreValue TimeZone_date_Vd(QoreObject* self, TimeZoneData* z, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* d = HARD_QORE_VALUE_DATE(args, 0);
# 152 "QC_TimeZone.qpp"
    return DateTimeNode::makeAbsolute(z->get(), d->getEpochSecondsUTC(), d->getMicrosecond());
}

// date TimeZone::date(string dtstr){}
static QoreValue TimeZone_date_Vs(QoreObject* self, TimeZoneData* z, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* dtstr = HARD_QORE_VALUE_STRING(args, 0);
# 166 "QC_TimeZone.qpp"
    return new DateTimeNode(z->get(), dtstr->getBuffer());
}

// date TimeZone::date(string dtstr, string mask){}
static QoreValue TimeZone_date_VsVs(QoreObject* self, TimeZoneData* z, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* dtstr = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* mask = HARD_QORE_VALUE_STRING(args, 1);
# 189 "QC_TimeZone.qpp"
    return make_date_with_mask(z->get(), *dtstr, *mask, xsink);
}

// date TimeZone::dateMs(softint ms){}
static QoreValue TimeZone_dateMs_vi(QoreObject* self, TimeZoneData* z, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 ms = HARD_QORE_VALUE_INT(args, 0);
# 203 "QC_TimeZone.qpp"
    return DateTimeNode::makeAbsolute(z->get(), ms / 1000, (int)((ms % 1000) * 1000));
}

// date TimeZone::dateUs(softint us){}
static QoreValue TimeZone_dateUs_vi(QoreObject* self, TimeZoneData* z, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 us = HARD_QORE_VALUE_INT(args, 0);
# 217 "QC_TimeZone.qpp"
    return DateTimeNode::makeAbsolute(z->get(), us / 1000000, (int)(us % 1000000));
}

// bool TimeZone::hasDST(){}
static QoreValue TimeZone_hasDST(QoreObject* self, TimeZoneData* z, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 115 "QC_TimeZone.qpp"
    return AbstractQoreZoneInfo::hasDST(z->get());
}

// string TimeZone::region(){}
static QoreValue TimeZone_region(QoreObject* self, TimeZoneData* z, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 127 "QC_TimeZone.qpp"
    return new QoreStringNode(AbstractQoreZoneInfo::getRegionName(z->get()));
}

// static TimeZone TimeZone::get(){}
static QoreValue static_TimeZone_get(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 230 "QC_TimeZone.qpp"
return new QoreObject(QC_TIMEZONE, 0, new TimeZoneData(currentTZ()));
}

// static nothing TimeZone::set(TimeZone zone){}
static QoreValue static_TimeZone_set_C8TimeZone(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(zone, TimeZoneData, args, 0, CID_TIMEZONE, "TimeZone::set()", "TimeZone", xsink);
    if (*xsink)
        return 0;
# 244 "QC_TimeZone.qpp"
ReferenceHolder<TimeZoneData> holder(zone, xsink);
   getProgram()->setTZ(zone->get());
    return QoreValue();
}

// static nothing TimeZone::setRegion(string region){}
static QoreValue static_TimeZone_setRegion_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* region = HARD_QORE_VALUE_STRING(args, 0);
# 274 "QC_TimeZone.qpp"
const AbstractQoreZoneInfo *zone = QTZM.findLoadRegion(region->getBuffer(), xsink);
   if (!*xsink)
      getProgram()->setTZ(zone);
    return QoreValue();
}

// static nothing TimeZone::setUTCOffset(softint seconds_east){}
static QoreValue static_TimeZone_setUTCOffset_vi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 seconds_east = HARD_QORE_VALUE_INT(args, 0);
# 258 "QC_TimeZone.qpp"
const AbstractQoreZoneInfo *zone = QTZM.findCreateOffsetZone(seconds_east);
   getProgram()->setTZ(zone);
    return QoreValue();
}

DLLLOCAL void preinitTimeZoneClass() {
    QC_TIMEZONE = new QoreClass("TimeZone", QDOM_DEFAULT);
    CID_TIMEZONE = QC_TIMEZONE->getID();
    QC_TIMEZONE->setSystem();
}

DLLLOCAL QoreClass* initTimeZoneClass(QoreNamespace& ns) {
    if (!QC_TIMEZONE)
        preinitTimeZoneClass();

    // int TimeZone::UTCOffset(){}
    QC_TIMEZONE->addMethod("UTCOffset", (q_method_n_t)TimeZone_UTCOffset, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // TimeZone::constructor(string region) {}
    QC_TIMEZONE->addConstructor(TimeZone_constructor_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, 1, stringTypeInfo, NULL, "region");

    // TimeZone::constructor(softint seconds_east) {}
    QC_TIMEZONE->addConstructor(TimeZone_constructor_vi, false, QC_NO_FLAGS, QDOM_DEFAULT, 1, softBigIntTypeInfo, NULL, "seconds_east");

    // TimeZone::copy() {}
    QC_TIMEZONE->setCopy((q_copy_t)TimeZone_copy);

    // date TimeZone::date(softint secs, softint us = 0){}
    QC_TIMEZONE->addMethod("date", (q_method_n_t)TimeZone_date_vivi, false, QC_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 2, softBigIntTypeInfo, NULL, "secs", softBigIntTypeInfo, zero(), "us");

    // date TimeZone::date(date d){}
    QC_TIMEZONE->addMethod("date", (q_method_n_t)TimeZone_date_Vd, false, QC_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 1, dateTypeInfo, NULL, "d");

    // date TimeZone::date(string dtstr){}
    QC_TIMEZONE->addMethod("date", (q_method_n_t)TimeZone_date_Vs, false, QC_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 1, stringTypeInfo, NULL, "dtstr");

    // date TimeZone::date(string dtstr, string mask){}
    QC_TIMEZONE->addMethod("date", (q_method_n_t)TimeZone_date_VsVs, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, dateTypeInfo, 2, stringTypeInfo, NULL, "dtstr", stringTypeInfo, NULL, "mask");

    // date TimeZone::dateMs(softint ms){}
    QC_TIMEZONE->addMethod("dateMs", (q_method_n_t)TimeZone_dateMs_vi, false, QC_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 1, softBigIntTypeInfo, NULL, "ms");

    // date TimeZone::dateUs(softint us){}
    QC_TIMEZONE->addMethod("dateUs", (q_method_n_t)TimeZone_dateUs_vi, false, QC_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 1, softBigIntTypeInfo, NULL, "us");

    // bool TimeZone::hasDST(){}
    QC_TIMEZONE->addMethod("hasDST", (q_method_n_t)TimeZone_hasDST, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // string TimeZone::region(){}
    QC_TIMEZONE->addMethod("region", (q_method_n_t)TimeZone_region, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // static TimeZone TimeZone::get(){}
    QC_TIMEZONE->addStaticMethod("get", (q_func_n_t)static_TimeZone_get, false, QC_CONSTANT, QDOM_DEFAULT, QC_TIMEZONE->getTypeInfo());

    // static nothing TimeZone::set(TimeZone zone){}
    QC_TIMEZONE->addStaticMethod("set", (q_func_n_t)static_TimeZone_set_C8TimeZone, false, QC_NO_FLAGS, QDOM_LOCALE_CONTROL, nothingTypeInfo, 1, QC_TIMEZONE->getTypeInfo(), NULL, "zone");

    // static nothing TimeZone::setRegion(string region){}
    QC_TIMEZONE->addStaticMethod("setRegion", (q_func_n_t)static_TimeZone_setRegion_Vs, false, QC_NO_FLAGS, QDOM_LOCALE_CONTROL, nothingTypeInfo, 1, stringTypeInfo, NULL, "region");

    // static nothing TimeZone::setUTCOffset(softint seconds_east){}
    QC_TIMEZONE->addStaticMethod("setUTCOffset", (q_func_n_t)static_TimeZone_setUTCOffset_vi, false, QC_NO_FLAGS, QDOM_LOCALE_CONTROL, nothingTypeInfo, 1, softBigIntTypeInfo, NULL, "seconds_east");

    return QC_TIMEZONE;
}
