/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  ql_crypto.qpp

  Qore Programming Language

  Copyright (C) 2003 - 2017 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/ql_crypto.h"
#include "qore/intern/EncryptionTransforms.h"

#include <stdio.h>
#include <stdlib.h>

#include <openssl/err.h>
#include <openssl/rand.h>

#define QCRYPTO_DECRYPT 0
#define QCRYPTO_ENCRYPT 1

extern QoreClass* QC_TRANSFORM;

// maximum length of a MAC value in bytes
#define Q_TAG_MAX 4096

// size of buffer when en/decrypting monolithic data
#define Q_CRYPTO_BUFSIZE 16384

// NOTE: the trailing null ('\0') is not included when encrypting strings or calculating digests

// default initialization vector
static unsigned char def_iv[] = { 0, 0, 0, 0, 0, 0, 0, 0 };

/*
*/
class CryptoHelper : public BaseHelper {
private:
   unsigned char *iv, *output;
   int output_len;
   const QoreEncoding* qe;

   DLLLOCAL const char *getOrdinal(int n) {
      assert(n > 0 && n <= 3);
      return n == 1 ? "first" : (n == 2 ? "second" : "third");
   }

   DLLLOCAL void setKey(const QoreValueList *args, int n) {
      assert(n > 0 && n <= 3);
      const QoreValue pt = get_param_value(args, n);
      if (pt.getType() == NT_STRING) {
         const QoreStringNode* str = pt.get<const QoreStringNode>();
         key[n - 1] = (unsigned char*)str->getBuffer();
         keylen[n - 1] = str->strlen();
         return;
      }

      const BinaryNode* b = pt.get<const BinaryNode>();
      key[n - 1] = (unsigned char*)b->getPtr();
      keylen[n - 1] = b->size();
   }

   DLLLOCAL void setEncoding(const QoreValueList* args, int n) {
      QoreValue qv = get_param_value(args, n);
      qe = qv.getType() == NT_STRING ? QEM.findCreate(qv.get<const QoreStringNode>()) : QCS_DEFAULT;
   }

   // set the initialization vector (used with hard typing)
   DLLLOCAL int setIV(const char *err, const QoreValue pt, ExceptionSink *xsink) {
      assert(!pt.isNothing());
      if (pt.getType() == NT_STRING) {
         const QoreStringNode* str = pt.get<const QoreStringNode>();
         if (str->strlen() < 8) {
            xsink->raiseException(err, "the input vector must be at least 8 bytes long (%d bytes passed)", str->strlen());
            return -1;
         }
         iv = (unsigned char*)str->getBuffer();
         return 0;
      }

      assert(pt.getType() == NT_BINARY);
      const BinaryNode* b = pt.get<const BinaryNode>();
      if (b->size() < 8) {
         xsink->raiseException(err, "the input vector must be at least 8 bytes long (%d bytes passed)", b->size());
         return -1;
      }
      iv = (unsigned char*)b->getPtr();
      return 0;
   }

public:
   unsigned char *key[3];
   int keylen[3];

   DLLLOCAL CryptoHelper() : iv(0), output(0), output_len(0), qe(QCS_DEFAULT) {
   }

   DLLLOCAL ~CryptoHelper() {
      if (output)
         free(output);
   }

   DLLLOCAL BinaryNode *getBinary() {
      BinaryNode *b = new BinaryNode(output, output_len);
      output = 0;
      return b;
   }

   DLLLOCAL QoreStringNode *getString(const QoreEncoding *enc = QCS_DEFAULT) {
      // create the string
      QoreStringNode *str = new QoreStringNode((char *)output, output_len, output_len, enc);
      // add trailing '\0'
      str->terminate(output_len);

      output = 0;
      return str;
   }

   DLLLOCAL int setSingleKey(const char *err, const QoreValueList *params, ExceptionSink *xsink, bool has_encoding = false) {
      if (setIV(err, get_param_value(params, 2), xsink))
         return -1;

      setInput(get_param_value(params, 0));
      setKey(params, 1);
      if (has_encoding)
         setEncoding(params, 2);
      return 0;
   }

   DLLLOCAL int doCipher(const EVP_CIPHER *type, const char *cipher, int do_crypt, ExceptionSink *xsink) {
      const char *err = (do_crypt ? "ENCRYPT-ERROR" : "DECRYPT-ERROR");

      QoreEvpCipherCtxHelper ctx;
      EVP_CIPHER_CTX_init(*ctx);
      EVP_CipherInit_ex(*ctx, type, 0, 0, 0, do_crypt);
      if (key[0]) {
         if (keylen[0] > EVP_MAX_KEY_LENGTH)
            keylen[0] = EVP_MAX_KEY_LENGTH;

         if (!EVP_CIPHER_CTX_set_key_length(*ctx, keylen[0]) || !EVP_CipherInit_ex(*ctx, 0, 0, key[0], iv, -1)) {
            xsink->raiseException(err, "error setting %s key length=%d", cipher, keylen[0]);
            return -1;
         }
      }

      // we allocate 1 byte more than we need in case we return as a string so we can terminate it
      output = (unsigned char *)malloc(sizeof(char) * (input_len + (EVP_MAX_BLOCK_LENGTH * 2)));

      if (!EVP_CipherUpdate(*ctx, output, &output_len, input, input_len)) {
         xsink->raiseException(err, "error %scrypting %s block", do_crypt ? "en" : "de", cipher);
         return -1;
      }

      int tmplen;
      // Buffer passed to EVP_EncryptFinal() must be after data just encrypted to avoid overwriting it.
      if (!EVP_CipherFinal_ex(*ctx, output + output_len, &tmplen)) {
         xsink->raiseException(err, "error %scrypting final %s block", do_crypt ? "en" : "de", cipher);
         return -1;
      }

      output_len += tmplen;
      //printd(5, "cipher_intern() %s: in=%p (%d) out=%p (%d)\n", cipher, buf, len, cbuf, *size);
      return 0;
   }

   DLLLOCAL int checkKeyLen(const char *err, int n, int len, ExceptionSink *xsink) {
      if (keylen[n] < len) {
         xsink->raiseException(err, "key length is not %d bytes long (%d bytes)", len, keylen[n]);
         return -1;
      }
      keylen[n] = len;
      return 0;
   }

/*
   DLLLOCAL int setDESKey(int n, ExceptionSink *xsink) {
      // force odd parity
      //DES_set_odd_parity((DES_cblock *)key[n]);
      // populate the schedule structure
      //DES_set_key_unchecked((DES_cblock *)key[n], schedule);
      //key[n] = (unsigned char *)schedule;
      return 0;
   }
*/
};

AbstractQoreNode* missing_openssl_feature(const char *f, ExceptionSink *xsink) {
   return xsink->raiseException("MISSING-FEATURE-ERROR", "the openssl library version that qore was compiled with did not support the %s algorithm; for maximum portability, check Option::HAVE_%s before calling this function", f, f);
}

static int check_mac(const ReferenceNode* mac, int64 mac_size, ExceptionSink* xsink) {
   if (!mac_size)
      return 0;

   if (!mac) {
      xsink->raiseException("MAC-ERROR", "mac_size argument passed but no mac argument passed");
      return -1;
   }

   if (mac_size < 0) {
      xsink->raiseException("MAC-ERROR", "mac_size argument is negative (" QLLD ")", mac_size);
      return -1;
   }

   if (mac_size > Q_TAG_MAX) {
      xsink->raiseException("MAC-ERROR", "mac_size argument is too large (" QLLD "); maximum value is %d", mac_size, Q_TAG_MAX);
      return -1;
   }

   return 0;
}

static QoreValue apply_transformation(SimpleRefHolder<Transform>& t, SimpleRefHolder<BinaryNode>& rv, const char* datastr, size_t data_len, ExceptionSink* xsink) {
   // preallocate space for the decrypted data
   rv->preallocate(data_len + (EVP_MAX_BLOCK_LENGTH * 2));
   size_t rv_maxsize = rv->size();
   size_t rvsize = 0;
   size_t offset = 0;

   // decrypt data
   while (offset != data_len) {
      // get block size
      size_t bs = data_len - offset;

      // ensure we always have buffer space for the encryption algorithm
      {
         size_t new_max = rvsize + bs + (EVP_MAX_BLOCK_LENGTH * 2);

         // reallocate target buffer if necessary
         if (new_max > rv_maxsize) {
            rv_maxsize = new_max;
            rv->preallocate(rv_maxsize);
         }
      }

      size_t cbs = rv_maxsize - rvsize;
      std::pair<int64, int64> p = t->apply(datastr + offset, bs, (char*)rv->getPtr() + rvsize, cbs, xsink);
      if (*xsink)
         return QoreValue();
      offset += p.first;
      rvsize += p.second;
   }

   // close the stream
   while (true) {
      // ensure we always have buffer space for the encryption algorithm
      {
         size_t new_max = rvsize + (EVP_MAX_BLOCK_LENGTH * 2);

         // reallocate target buffer if necessary
         if (new_max > rv_maxsize) {
            rv_maxsize = new_max;
            rv->preallocate(rv_maxsize);
         }
      }

      size_t cbs = rv_maxsize - rvsize;
      std::pair<int64, int64> p = t->apply(nullptr, 0, (char*)rv->getPtr() + rvsize, cbs, xsink);
      if (*xsink)
         return QoreValue();
      if (!p.second)
         break;
      rvsize += p.second;
   }

   rv->setSize(rvsize);
   return rv.release();
}
# 515 "ql_crypto.qpp"
# 549 "ql_crypto.qpp"
# 563 "ql_crypto.qpp"
# 597 "ql_crypto.qpp"
# 631 "ql_crypto.qpp"
# 668 "ql_crypto.qpp"
# 704 "ql_crypto.qpp"
# 740 "ql_crypto.qpp"
# 779 "ql_crypto.qpp"
# 815 "ql_crypto.qpp"
# 851 "ql_crypto.qpp"
# 890 "ql_crypto.qpp"
# 926 "ql_crypto.qpp"
# 962 "ql_crypto.qpp"
# 1001 "ql_crypto.qpp"
# 1037 "ql_crypto.qpp"
# 1073 "ql_crypto.qpp"
# 1112 "ql_crypto.qpp"
# 1148 "ql_crypto.qpp"
# 1184 "ql_crypto.qpp"
# 1223 "ql_crypto.qpp"
# 1257 "ql_crypto.qpp"
# 1291 "ql_crypto.qpp"
# 1328 "ql_crypto.qpp"
# 1362 "ql_crypto.qpp"
# 1396 "ql_crypto.qpp"
# 1433 "ql_crypto.qpp"
# 1474 "ql_crypto.qpp"
# 1515 "ql_crypto.qpp"
// binary blowfish_decrypt_cbc(binary data, data key, data iv = Qore::DefaultIV) {}
static QoreValue f_blowfish_decrypt_cbc_VyVDVD(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 621 "ql_crypto.qpp"
   CryptoHelper ch;

   if (ch.setSingleKey("BLOWFISH-DECRYPT-PARAM-ERROR", args, xsink)
       || ch.doCipher(EVP_bf_cbc(), "blowfish", QCRYPTO_DECRYPT, xsink))
      return QoreValue();

   return ch.getBinary();
}

// string blowfish_decrypt_cbc_to_string(binary data, data key, data iv = Qore::DefaultIV, *string encoding) {}
static QoreValue f_blowfish_decrypt_cbc_to_string_VyVDVDNs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 658 "ql_crypto.qpp"
   CryptoHelper ch;

   if (ch.setSingleKey("BLOWFISH-DECRYPT-PARAM-ERROR", args, xsink, true)
       || ch.doCipher(EVP_bf_cbc(), "blowfish", QCRYPTO_DECRYPT, xsink))
      return QoreValue();

   return ch.getString();
}

// binary blowfish_encrypt_cbc(data data, data key, data iv = Qore::DefaultIV) {}
static QoreValue f_blowfish_encrypt_cbc_VDVDVD(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 587 "ql_crypto.qpp"
   CryptoHelper ch;

   if (ch.setSingleKey("BLOWFISH-ENCRYPT-PARAM-ERROR", args, xsink)
       || ch.doCipher(EVP_bf_cbc(), "blowfish", QCRYPTO_ENCRYPT, xsink))
      return QoreValue();

   return ch.getBinary();
}

// binary cast5_decrypt_cbc(binary data, data key, data iv = Qore::DefaultIV) {}
static QoreValue f_cast5_decrypt_cbc_VyVDVD(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1386 "ql_crypto.qpp"
   CryptoHelper ch;

   if (ch.setSingleKey("CAST5-DECRYPT-PARAM-ERROR", args, xsink)
       || ch.doCipher(EVP_cast5_cbc(), "CAST5", QCRYPTO_DECRYPT, xsink))
      return QoreValue();

   return ch.getBinary();
}

// string cast5_decrypt_cbc_to_string(binary data, data key, data iv = Qore::DefaultIV, *string encoding) {}
static QoreValue f_cast5_decrypt_cbc_to_string_VyVDVDNs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1423 "ql_crypto.qpp"
   CryptoHelper ch;

   if (ch.setSingleKey("CAST5-DECRYPT-PARAM-ERROR", args, xsink, true)
       || ch.doCipher(EVP_cast5_cbc(), "CAST5", QCRYPTO_DECRYPT, xsink))
      return QoreValue();

   return ch.getString();
}

// binary cast5_encrypt_cbc(data data, data key, data iv = Qore::DefaultIV) {}
static QoreValue f_cast5_encrypt_cbc_VDVDVD(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1352 "ql_crypto.qpp"
   CryptoHelper ch;

   if (ch.setSingleKey("CAST5-ENCRYPT-PARAM-ERROR", args, xsink)
       || ch.doCipher(EVP_cast5_cbc(), "CAST5", QCRYPTO_ENCRYPT, xsink))
      return QoreValue();

   return ch.getBinary();
}

// binary des_decrypt_cbc(binary data, data key, data iv = Qore::DefaultIV) {}
static QoreValue f_des_decrypt_cbc_VyVDVD(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 729 "ql_crypto.qpp"
   CryptoHelper ch;

   if (ch.setSingleKey("DES-DECRYPT-PARAM-ERROR", args, xsink)
       || ch.checkKeyLen("DES-KEY-ERROR", 0, 8, xsink)
       || ch.doCipher(EVP_des_cbc(), "DES", QCRYPTO_DECRYPT, xsink))
            return QoreValue();

   return ch.getBinary();
}

// string des_decrypt_cbc_to_string(binary data, data key, data iv = Qore::DefaultIV, *string encoding) {}
static QoreValue f_des_decrypt_cbc_to_string_VyVDVDNs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 768 "ql_crypto.qpp"
   CryptoHelper ch;

   if (ch.setSingleKey("DES-DECRYPT-PARAM-ERROR", args, xsink, true)
       || ch.checkKeyLen("DES-KEY-ERROR", 0, 8, xsink)
       || ch.doCipher(EVP_des_cbc(), "DES", QCRYPTO_DECRYPT, xsink))
            return QoreValue();

   return ch.getString();
}

// binary des_ede3_decrypt_cbc(data data, data key, data iv = Qore::DefaultIV) {}
static QoreValue f_des_ede3_decrypt_cbc_VDVDVD(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 951 "ql_crypto.qpp"
   CryptoHelper ch;

   if (ch.setSingleKey("DES-DECRYPT-PARAM-ERROR", args, xsink)
       || ch.checkKeyLen("DES-KEY-ERROR", 0, 24, xsink)
       || ch.doCipher(EVP_des_ede3_cbc(), "DES", QCRYPTO_DECRYPT, xsink))
      return QoreValue();

   return ch.getBinary();
}

// string des_ede3_decrypt_cbc_to_string(binary data, data key, data iv = Qore::DefaultIV, *string encoding) {}
static QoreValue f_des_ede3_decrypt_cbc_to_string_VyVDVDNs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 990 "ql_crypto.qpp"
   CryptoHelper ch;

   if (ch.setSingleKey("DES-DECRYPT-PARAM-ERROR", args, xsink, true)
       || ch.checkKeyLen("DES-KEY-ERROR", 0, 24, xsink)
       || ch.doCipher(EVP_des_ede3_cbc(), "DES", QCRYPTO_DECRYPT, xsink))
      return QoreValue();

   return ch.getString();
}

// binary des_ede3_encrypt_cbc(data data, data key, data iv = Qore::DefaultIV) {}
static QoreValue f_des_ede3_encrypt_cbc_VDVDVD(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 915 "ql_crypto.qpp"
   CryptoHelper ch;

   if (ch.setSingleKey("DES-ENCRYPT-PARAM-ERROR", args, xsink)
       || ch.checkKeyLen("DES-KEY-ERROR", 0, 24, xsink)
       || ch.doCipher(EVP_des_ede3_cbc(), "DES", QCRYPTO_ENCRYPT, xsink))
      return QoreValue();

   return ch.getBinary();
}

// binary des_ede_decrypt_cbc(binary data, data key, data iv = Qore::DefaultIV) {}
static QoreValue f_des_ede_decrypt_cbc_VyVDVD(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 840 "ql_crypto.qpp"
   CryptoHelper ch;

   if (ch.setSingleKey("DES-DECRYPT-PARAM-ERROR", args, xsink)
       || ch.checkKeyLen("DES-KEY-ERROR", 0, 16, xsink)
       || ch.doCipher(EVP_des_ede_cbc(), "DES", QCRYPTO_DECRYPT, xsink))
      return QoreValue();

   return ch.getBinary();
}

// string des_ede_decrypt_cbc_to_string(binary data, data key, data iv = Qore::DefaultIV, *string encoding) {}
static QoreValue f_des_ede_decrypt_cbc_to_string_VyVDVDNs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 879 "ql_crypto.qpp"
   CryptoHelper ch;

   if (ch.setSingleKey("DES-DECRYPT-PARAM-ERROR", args, xsink, true)
       || ch.checkKeyLen("DES-KEY-ERROR", 0, 16, xsink)
       || ch.doCipher(EVP_des_ede_cbc(), "DES", QCRYPTO_DECRYPT, xsink))
      return QoreValue();

   return ch.getString();
}

// binary des_ede_encrypt_cbc(data data, data key, data iv = Qore::DefaultIV) {}
static QoreValue f_des_ede_encrypt_cbc_VDVDVD(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 804 "ql_crypto.qpp"
   CryptoHelper ch;

   if (ch.setSingleKey("DES-ENCRYPT-PARAM-ERROR", args, xsink)
       || ch.checkKeyLen("DES-KEY-ERROR", 0, 16, xsink)
       || ch.doCipher(EVP_des_ede_cbc(), "DES", QCRYPTO_ENCRYPT, xsink))
      return QoreValue();

   return ch.getBinary();
}

// binary des_encrypt_cbc(data data, data key, data iv = Qore::DefaultIV) {}
static QoreValue f_des_encrypt_cbc_VDVDVD(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 693 "ql_crypto.qpp"
   CryptoHelper ch;

   if (ch.setSingleKey("DES-ENCRYPT-PARAM-ERROR", args, xsink)
       || ch.checkKeyLen("DES-KEY-ERROR", 0, 8, xsink)
       || ch.doCipher(EVP_des_cbc(), "DES", QCRYPTO_ENCRYPT, xsink))
            return QoreValue();

   return ch.getBinary();
}

// binary des_random_key() {}
static QoreValue f_des_random_key(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 557 "ql_crypto.qpp"
   DES_cblock *db = (DES_cblock *)malloc(sizeof(DES_cblock));
   DES_random_key(db);
   return new BinaryNode(db, sizeof(DES_cblock));
}

// binary desx_decrypt_cbc(binary data, data key, data iv = Qore::DefaultIV) {}
static QoreValue f_desx_decrypt_cbc_VyVDVD(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1062 "ql_crypto.qpp"
   CryptoHelper ch;

   if (ch.setSingleKey("DESX-DECRYPT-PARAM-ERROR", args, xsink)
       || ch.checkKeyLen("DESX-KEY-ERROR", 0, 24, xsink)
       || ch.doCipher(EVP_desx_cbc(), "DESX", QCRYPTO_DECRYPT, xsink))
      return QoreValue();

   return ch.getBinary();
}

// string desx_decrypt_cbc_to_string(binary data, data key, data iv = Qore::DefaultIV, *string encoding) {}
static QoreValue f_desx_decrypt_cbc_to_string_VyVDVDNs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1101 "ql_crypto.qpp"
   CryptoHelper ch;

   if (ch.setSingleKey("DESX-DECRYPT-PARAM-ERROR", args, xsink, true)
       || ch.checkKeyLen("DESX-KEY-ERROR", 0, 24, xsink)
       || ch.doCipher(EVP_desx_cbc(), "DESX", QCRYPTO_DECRYPT, xsink))
      return QoreValue();

   return ch.getString();
}

// binary desx_encrypt_cbc(data data, data key, data iv = Qore::DefaultIV) {}
static QoreValue f_desx_encrypt_cbc_VDVDVD(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1026 "ql_crypto.qpp"
   CryptoHelper ch;

   if (ch.setSingleKey("DESX-ENCRYPT-PARAM-ERROR", args, xsink)
       || ch.checkKeyLen("DESX-KEY-ERROR", 0, 24, xsink)
       || ch.doCipher(EVP_desx_cbc(), "DESX", QCRYPTO_ENCRYPT, xsink))
      return QoreValue();

   return ch.getBinary();
}

// binary rc2_decrypt_cbc(binary data, data key, data iv = Qore::DefaultIV) {}
static QoreValue f_rc2_decrypt_cbc_VyVDVD(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1281 "ql_crypto.qpp"
   CryptoHelper ch;

   if (ch.setSingleKey("RC2-DECRYPT-PARAM-ERROR", args, xsink)
       || ch.doCipher(EVP_rc2_cbc(), "rc2", QCRYPTO_DECRYPT, xsink))
      return QoreValue();

   return ch.getBinary();
}

// string rc2_decrypt_cbc_to_string(binary data, data key, data iv = Qore::DefaultIV, *string encoding) {}
static QoreValue f_rc2_decrypt_cbc_to_string_VyVDVDNs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1318 "ql_crypto.qpp"
   CryptoHelper ch;

   if (ch.setSingleKey("RC2-DECRYPT-PARAM-ERROR", args, xsink, true)
       || ch.doCipher(EVP_rc2_cbc(), "rc2", QCRYPTO_DECRYPT, xsink))
      return QoreValue();

   return ch.getString();
}

// binary rc2_encrypt_cbc(data data, data key, data iv = Qore::DefaultIV) {}
static QoreValue f_rc2_encrypt_cbc_VDVDVD(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1247 "ql_crypto.qpp"
   CryptoHelper ch;

   if (ch.setSingleKey("RC2-ENCRYPT-PARAM-ERROR", args, xsink)
       || ch.doCipher(EVP_rc2_cbc(), "rc2", QCRYPTO_ENCRYPT, xsink))
      return QoreValue();

   return ch.getBinary();
}

// binary rc4_decrypt(binary data, data key, data iv = Qore::DefaultIV) {}
static QoreValue f_rc4_decrypt_VyVDVD(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1174 "ql_crypto.qpp"
   CryptoHelper ch;

   if (ch.setSingleKey("RC4-DECRYPT-PARAM-ERROR", args, xsink)
       || ch.doCipher(EVP_rc4(), "rc4", QCRYPTO_DECRYPT, xsink))
      return QoreValue();

   return ch.getBinary();
}

// string rc4_decrypt_to_string(binary data, data key, data iv = Qore::DefaultIV, *string encoding) {}
static QoreValue f_rc4_decrypt_to_string_VyVDVDNs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1213 "ql_crypto.qpp"
   CryptoHelper ch;

   if (ch.setSingleKey("RC4-DECRYPT-PARAM-ERROR", args, xsink, true)
       || ch.doCipher(EVP_rc4(), "rc4", QCRYPTO_DECRYPT, xsink))
      return QoreValue();

   return ch.getString();
}

// binary rc4_encrypt(data data, data key, data iv = Qore::DefaultIV) {}
static QoreValue f_rc4_encrypt_VDVDVD(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1138 "ql_crypto.qpp"
   CryptoHelper ch;

   if (ch.setSingleKey("RC4-ENCRYPT-PARAM-ERROR", args, xsink)
       || ch.doCipher(EVP_rc4(), "rc4", QCRYPTO_ENCRYPT, xsink))
      return QoreValue();

   return ch.getBinary();
}

// binary rc5_decrypt_cbc(binary data, data key, data iv = Qore::DefaultIV) {}
static QoreValue f_rc5_decrypt_cbc_VyVDVD(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1501 "ql_crypto.qpp"
#ifndef OPENSSL_NO_RC5
   CryptoHelper ch;

   if (ch.setSingleKey("RC5-DECRYPT-PARAM-ERROR", args, xsink)
       || ch.doCipher(EVP_rc5_32_12_16_cbc(), "rc5", QCRYPTO_DECRYPT, xsink))
      return QoreValue();

   return ch.getBinary();
#else
   return missing_openssl_feature("RC5", xsink);
#endif
    return QoreValue();
}

// string rc5_decrypt_cbc_to_string(binary data, data key, data iv = Qore::DefaultIV, *string encoding) {}
static QoreValue f_rc5_decrypt_cbc_to_string_VyVDVDNs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1545 "ql_crypto.qpp"
#ifndef OPENSSL_NO_RC5
   CryptoHelper ch;

   if (ch.setSingleKey("RC5-DECRYPT-PARAM-ERROR", args, xsink, true)
       || ch.doCipher(EVP_rc5_32_12_16_cbc(), "rc5", QCRYPTO_DECRYPT, xsink))
      return QoreValue();

   return ch.getString();
#else
   return missing_openssl_feature("RC5", xsink);
#endif
    return QoreValue();
}

// binary rc5_encrypt_cbc(data data, data key, data iv = Qore::DefaultIV) {}
static QoreValue f_rc5_encrypt_cbc_VDVDVD(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1460 "ql_crypto.qpp"
#ifndef OPENSSL_NO_RC5
   CryptoHelper ch;

   if (ch.setSingleKey("RC5-ENCRYPT-PARAM-ERROR", args, xsink)
       || ch.doCipher(EVP_rc5_32_12_16_cbc(), "rc5", QCRYPTO_ENCRYPT, xsink))
      return QoreValue();

   return ch.getBinary();
#else
   return missing_openssl_feature("RC5", xsink);
#endif
    return QoreValue();
}

# 1560 "ql_crypto.qpp"
# 1580 "ql_crypto.qpp"
# 1610 "ql_crypto.qpp"
# 1633 "ql_crypto.qpp"
# 1660 "ql_crypto.qpp"
# 1689 "ql_crypto.qpp"
# 1714 "ql_crypto.qpp"
# 1746 "ql_crypto.qpp"
# 1778 "ql_crypto.qpp"
# 1810 "ql_crypto.qpp"
# 1842 "ql_crypto.qpp"
# 1871 "ql_crypto.qpp"
# 1900 "ql_crypto.qpp"
# 1930 "ql_crypto.qpp"
# 1953 "ql_crypto.qpp"
# 1983 "ql_crypto.qpp"
# 2006 "ql_crypto.qpp"
# 2029 "ql_crypto.qpp"
# 2058 "ql_crypto.qpp"
# 2081 "ql_crypto.qpp"
# 2111 "ql_crypto.qpp"
# 2141 "ql_crypto.qpp"
# 2171 "ql_crypto.qpp"
# 2201 "ql_crypto.qpp"
# 2230 "ql_crypto.qpp"
# 2259 "ql_crypto.qpp"
# 2289 "ql_crypto.qpp"
// string DSS(data data) {}
static QoreValue f_DSS_VD(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1859 "ql_crypto.qpp"
#ifndef HAVE_OPENSSL_INIT_CRYPTO
   DigestHelper dh(args);
   if (dh.doDigest(DSS_ERR, EVP_dss(), xsink))
      return QoreValue();

   return dh.getString();
#else
    missing_openssl_feature("DSS", xsink);
#endif
    return QoreValue();
}

// string DSS1(data data) {}
static QoreValue f_DSS1_VD(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1888 "ql_crypto.qpp"
#ifndef HAVE_OPENSSL_INIT_CRYPTO
   DigestHelper dh(args);
   if (dh.doDigest(DSS1_ERR, EVP_dss1(), xsink))
      return QoreValue();

   return dh.getString();
#else
    missing_openssl_feature("DSS", xsink);
#endif
    return QoreValue();
}

// binary DSS1_bin(data data) {}
static QoreValue f_DSS1_bin_VD(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2247 "ql_crypto.qpp"
#ifndef HAVE_OPENSSL_INIT_CRYPTO
   DigestHelper dh(args);
   if (dh.doDigest(DSS1_ERR, EVP_dss1(), xsink))
      return QoreValue();

   return dh.getBinary();
#else
    missing_openssl_feature("DSS", xsink);
#endif
    return QoreValue();
}

// binary DSS_bin(data data) {}
static QoreValue f_DSS_bin_VD(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2218 "ql_crypto.qpp"
#ifndef HAVE_OPENSSL_INIT_CRYPTO
   DigestHelper dh(args);
   if (dh.doDigest(DSS_ERR, EVP_dss(), xsink))
      return QoreValue();

   return dh.getBinary();
#else
    missing_openssl_feature("DSS", xsink);
#endif
    return QoreValue();
}

// string MD2(data data) {}
static QoreValue f_MD2_VD(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1598 "ql_crypto.qpp"
#if !defined(OPENSSL_NO_MD2) && !defined(NO_MD2)
   DigestHelper dh(args);
   if (dh.doDigest(MD2_ERR, EVP_md2(), xsink))
      return QoreValue();

   return dh.getString();
#else
   return missing_openssl_feature("MD2", xsink);
#endif
    return QoreValue();
}

// binary MD2_bin(data data) {}
static QoreValue f_MD2_bin_VD(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1971 "ql_crypto.qpp"
#if !defined(OPENSSL_NO_MD2) && !defined(NO_MD2)
   DigestHelper dh(args);
   if (dh.doDigest(MD2_ERR, EVP_md2(), xsink))
      return QoreValue();

   return dh.getBinary();
#else
   return missing_openssl_feature("MD2", xsink);
#endif
    return QoreValue();
}

// string MD4(data data) {}
static QoreValue f_MD4_VD(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1625 "ql_crypto.qpp"
   DigestHelper dh(args);
   if (dh.doDigest(MD4_ERR, EVP_md4(), xsink))
      return QoreValue();

   return dh.getString();
}

// binary MD4_bin(data data) {}
static QoreValue f_MD4_bin_VD(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1998 "ql_crypto.qpp"
   DigestHelper dh(args);
   if (dh.doDigest(MD4_ERR, EVP_md4(), xsink))
      return QoreValue();

   return dh.getBinary();
}

// string MD5(data data) {}
static QoreValue f_MD5_VD(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1652 "ql_crypto.qpp"
   DigestHelper dh(args);
   if (dh.doDigest(MD5_ERR, EVP_md5(), xsink))
      return QoreValue();

   return dh.getString();
}

// binary MD5_bin(data data) {}
static QoreValue f_MD5_bin_VD(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2021 "ql_crypto.qpp"
   DigestHelper dh(args);
   if (dh.doDigest(MD5_ERR, EVP_md5(), xsink))
      return QoreValue();

   return dh.getBinary();
}

// string MDC2(data data) {}
static QoreValue f_MDC2_VD(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1918 "ql_crypto.qpp"
#ifndef OPENSSL_NO_MDC2
   DigestHelper dh(args);
   if (dh.doDigest(MDC2_ERR, EVP_mdc2(), xsink))
      return QoreValue();

   return dh.getString();
#else
   return missing_openssl_feature("MDC2", xsink);
#endif
    return QoreValue();
}

// binary MDC2_bin(data data) {}
static QoreValue f_MDC2_bin_VD(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2277 "ql_crypto.qpp"
#ifndef OPENSSL_NO_MDC2
   DigestHelper dh(args);
   if (dh.doDigest(MDC2_ERR, EVP_mdc2(), xsink))
      return QoreValue();

   return dh.getBinary();
#else
   return missing_openssl_feature("MDC2", xsink);
#endif
    return QoreValue();
}

// string RIPEMD160(data data) {}
static QoreValue f_RIPEMD160_VD(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1945 "ql_crypto.qpp"
   DigestHelper dh(args);
   if (dh.doDigest(RIPEMD160_ERR, EVP_ripemd160(), xsink))
      return QoreValue();

   return dh.getString();
}

// binary RIPEMD160_binary(data data) {}
static QoreValue f_RIPEMD160_binary_VD(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2304 "ql_crypto.qpp"
   DigestHelper dh(args);
   if (dh.doDigest(RIPEMD160_ERR, EVP_ripemd160(), xsink))
      return QoreValue();

   return dh.getBinary();
}

// string SHA(data data) {}
static QoreValue f_SHA_VD(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1676 "ql_crypto.qpp"
#ifdef HAVE_OPENSSL_SHA
   DigestHelper dh(args);
   if (dh.doDigest(SHA_ERR, EVP_sha(), xsink))
      return QoreValue();

   return dh.getString();
#else
   missing_openssl_feature("SHA", xsink);
   return 0;
#endif
    return QoreValue();
}

// string SHA1(data data) {}
static QoreValue f_SHA1_VD(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1706 "ql_crypto.qpp"
   DigestHelper dh(args);
   if (dh.doDigest(SHA1_ERR, EVP_sha1(), xsink))
      return QoreValue();

   return dh.getString();
}

// binary SHA1_bin(data data) {}
static QoreValue f_SHA1_bin_VD(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2073 "ql_crypto.qpp"
   DigestHelper dh(args);
   if (dh.doDigest(SHA1_ERR, EVP_sha1(), xsink))
      return QoreValue();

   return dh.getBinary();
}

// string SHA224(data data) {}
static QoreValue f_SHA224_VD(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1734 "ql_crypto.qpp"
#if !defined(OPENSSL_NO_SHA256) && defined(HAVE_OPENSSL_SHA512)
   DigestHelper dh(args);
   if (dh.doDigest(SHA224_ERR, EVP_sha224(), xsink))
      return QoreValue();

   return dh.getString();
#else
   return missing_openssl_feature("SHA224", xsink);
#endif
    return QoreValue();
}

// binary SHA224_bin(data data) {}
static QoreValue f_SHA224_bin_VD(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2099 "ql_crypto.qpp"
#if !defined(OPENSSL_NO_SHA256) && defined(HAVE_OPENSSL_SHA512)
   DigestHelper dh(args);
   if (dh.doDigest(SHA224_ERR, EVP_sha224(), xsink))
      return QoreValue();

   return dh.getBinary();
#else
   return missing_openssl_feature("SHA224", xsink);
#endif
    return QoreValue();
}

// string SHA256(data data) {}
static QoreValue f_SHA256_VD(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1766 "ql_crypto.qpp"
#if !defined(OPENSSL_NO_SHA256) && defined(HAVE_OPENSSL_SHA512)
   DigestHelper dh(args);
   if (dh.doDigest(SHA256_ERR, EVP_sha256(), xsink))
      return QoreValue();

   return dh.getString();
#else
   return missing_openssl_feature("SHA256", xsink);
#endif
    return QoreValue();
}

// binary SHA256_bin(data data) {}
static QoreValue f_SHA256_bin_VD(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2129 "ql_crypto.qpp"
#if !defined(OPENSSL_NO_SHA256) && defined(HAVE_OPENSSL_SHA512)
   DigestHelper dh(args);
   if (dh.doDigest(SHA256_ERR, EVP_sha256(), xsink))
      return QoreValue();

   return dh.getBinary();
#else
   return missing_openssl_feature("SHA256", xsink);
#endif
    return QoreValue();
}

// string SHA384(data data) {}
static QoreValue f_SHA384_VD(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1798 "ql_crypto.qpp"
#if !defined(OPENSSL_NO_SHA512) && defined(HAVE_OPENSSL_SHA512)
   DigestHelper dh(args);
   if (dh.doDigest(SHA384_ERR, EVP_sha384(), xsink))
      return QoreValue();

   return dh.getString();
#else
   return missing_openssl_feature("SHA384", xsink);
#endif
    return QoreValue();
}

// binary SHA384_bin(data data) {}
static QoreValue f_SHA384_bin_VD(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2159 "ql_crypto.qpp"
#if !defined(OPENSSL_NO_SHA512) && defined(HAVE_OPENSSL_SHA512)
   DigestHelper dh(args);
   if (dh.doDigest(SHA384_ERR, EVP_sha384(), xsink))
      return QoreValue();

   return dh.getBinary();
#else
   return missing_openssl_feature("SHA384", xsink);
#endif
    return QoreValue();
}

// string SHA512(data data) {}
static QoreValue f_SHA512_VD(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1830 "ql_crypto.qpp"
#if !defined(OPENSSL_NO_SHA512) && defined(HAVE_OPENSSL_SHA512)
   DigestHelper dh(args);
   if (dh.doDigest(SHA512_ERR, EVP_sha512(), xsink))
      return QoreValue();

   return dh.getString();
#else
   return missing_openssl_feature("SHA512", xsink);
#endif
    return QoreValue();
}

// binary SHA512_bin(data data) {}
static QoreValue f_SHA512_bin_VD(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2189 "ql_crypto.qpp"
#if !defined(OPENSSL_NO_SHA512) && defined(HAVE_OPENSSL_SHA512)
   DigestHelper dh(args);
   if (dh.doDigest(SHA512_ERR, EVP_sha512(), xsink))
      return QoreValue();

   return dh.getBinary();
#else
   return missing_openssl_feature("SHA512", xsink);
#endif
    return QoreValue();
}

// binary SHA_bin(data data) {}
static QoreValue f_SHA_bin_VD(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2045 "ql_crypto.qpp"
#ifdef HAVE_OPENSSL_SHA
   DigestHelper dh(args);
   if (dh.doDigest(SHA_ERR, EVP_sha(), xsink))
      return QoreValue();

   return dh.getBinary();
#else
   missing_openssl_feature("SHA", xsink);
   return 0;
#endif
    return QoreValue();
}

# 2314 "ql_crypto.qpp"
# 2369 "ql_crypto.qpp"
# 2392 "ql_crypto.qpp"
# 2417 "ql_crypto.qpp"
# 2448 "ql_crypto.qpp"
# 2473 "ql_crypto.qpp"
# 2505 "ql_crypto.qpp"
# 2536 "ql_crypto.qpp"
# 2567 "ql_crypto.qpp"
# 2598 "ql_crypto.qpp"
# 2628 "ql_crypto.qpp"
# 2658 "ql_crypto.qpp"
# 2686 "ql_crypto.qpp"
# 2709 "ql_crypto.qpp"
# 2758 "ql_crypto.qpp"
# 2807 "ql_crypto.qpp"
# 2864 "ql_crypto.qpp"
# 2921 "ql_crypto.qpp"
# 2987 "ql_crypto.qpp"
# 3017 "ql_crypto.qpp"
// string DSS1_hmac(data data, string key) {}
static QoreValue f_DSS1_hmac_VDVs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* key = HARD_QORE_VALUE_STRING(args, 1);
# 2645 "ql_crypto.qpp"
#ifndef HAVE_OPENSSL_INIT_CRYPTO
   HMACHelper dh(args);
   if (dh.doHMAC(DSS1_ERR, EVP_dss1(), key, xsink))
      return QoreValue();

   return dh.getString();
#else
   (void)key;
   missing_openssl_feature("DSS", xsink);
#endif
    return QoreValue();
}

// string DSS_hmac(data data, string key) {}
static QoreValue f_DSS_hmac_VDVs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* key = HARD_QORE_VALUE_STRING(args, 1);
# 2615 "ql_crypto.qpp"
#ifndef HAVE_OPENSSL_INIT_CRYPTO
   HMACHelper dh(args);
   if (dh.doHMAC(DSS_ERR, EVP_dss(), key, xsink))
      return QoreValue();

   return dh.getString();
#else
   (void)key;
   missing_openssl_feature("DSS", xsink);
#endif
    return QoreValue();
}

// string MD2_hmac(data data, string key) {}
static QoreValue f_MD2_hmac_VDVs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* key = HARD_QORE_VALUE_STRING(args, 1);
# 2356 "ql_crypto.qpp"
#if !defined(OPENSSL_NO_MD2) && !defined(NO_MD2)
   HMACHelper dh(args);
   if (dh.doHMAC("MD2-HMAC-ERROR", EVP_md2(), key, xsink))
      return QoreValue();

   return dh.getString();
#else
   (void)key;
   return missing_openssl_feature("MD2", xsink);
#endif
    return QoreValue();
}

// string MD4_hmac(data data, string key) {}
static QoreValue f_MD4_hmac_VDVs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* key = HARD_QORE_VALUE_STRING(args, 1);
# 2384 "ql_crypto.qpp"
   HMACHelper dh(args);
   if (dh.doHMAC("MD4-HMAC-ERROR", EVP_md4(), key, xsink))
      return QoreValue();

   return dh.getString();
}

// string MD5_hmac(data data, string key) {}
static QoreValue f_MD5_hmac_VDVs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* key = HARD_QORE_VALUE_STRING(args, 1);
# 2409 "ql_crypto.qpp"
   HMACHelper dh(args);
   if (dh.doHMAC("MD5-HMAC-ERROR", EVP_md5(), key, xsink))
      return QoreValue();

   return dh.getString();
}

// string MDC2_hmac(data data, string key) {}
static QoreValue f_MDC2_hmac_VDVs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* key = HARD_QORE_VALUE_STRING(args, 1);
# 2673 "ql_crypto.qpp"
#ifndef OPENSSL_NO_MDC2
   HMACHelper dh(args);
   if (dh.doHMAC("MDC2-HMAC-ERROR", EVP_mdc2(), key, xsink))
      return QoreValue();

   return dh.getString();
#else
   (void)key;
   return missing_openssl_feature("MDC2", xsink);
#endif
    return QoreValue();
}

// string RIPEMD160_hmac(data data, string key) {}
static QoreValue f_RIPEMD160_hmac_VDVs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* key = HARD_QORE_VALUE_STRING(args, 1);
# 2701 "ql_crypto.qpp"
   HMACHelper dh(args);
   if (dh.doHMAC("RIPEMD160-HMAC-ERROR", EVP_ripemd160(), key, xsink))
      return QoreValue();

   return dh.getString();
}

// string SHA1_hmac(data data, string key) {}
static QoreValue f_SHA1_hmac_VDVs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* key = HARD_QORE_VALUE_STRING(args, 1);
# 2465 "ql_crypto.qpp"
   HMACHelper dh(args);
   if (dh.doHMAC("SHA1-HMAC-ERROR", EVP_sha1(), key, xsink))
      return QoreValue();

   return dh.getString();
}

// string SHA224_hmac(data data, string key) {}
static QoreValue f_SHA224_hmac_VDVs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* key = HARD_QORE_VALUE_STRING(args, 1);
# 2492 "ql_crypto.qpp"
#if !defined(OPENSSL_NO_SHA256) && defined(HAVE_OPENSSL_SHA512)
   HMACHelper dh(args);
   if (dh.doHMAC("SHA224-HMAC-ERROR", EVP_sha224(), key, xsink))
      return QoreValue();

   return dh.getString();
#else
   (void)key;
   return missing_openssl_feature("SHA224", xsink);
#endif
    return QoreValue();
}

// string SHA256_hmac(data data, string key) {}
static QoreValue f_SHA256_hmac_VDVs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* key = HARD_QORE_VALUE_STRING(args, 1);
# 2523 "ql_crypto.qpp"
#if !defined(OPENSSL_NO_SHA256) && defined(HAVE_OPENSSL_SHA512)
   HMACHelper dh(args);
   if (dh.doHMAC("SHA256-HMAC-ERROR", EVP_sha256(), key, xsink))
      return QoreValue();

   return dh.getString();
#else
   (void)key;
   return missing_openssl_feature("SHA256", xsink);
#endif
    return QoreValue();
}

// string SHA384_hmac(data data, string key) {}
static QoreValue f_SHA384_hmac_VDVs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* key = HARD_QORE_VALUE_STRING(args, 1);
# 2554 "ql_crypto.qpp"
#if !defined(OPENSSL_NO_SHA512) && defined(HAVE_OPENSSL_SHA512)
   HMACHelper dh(args);
   if (dh.doHMAC("SHA384-HMAC-ERROR", EVP_sha384(), key, xsink))
      return QoreValue();

   return dh.getString();
#else
   (void)key;
   return missing_openssl_feature("SHA384", xsink);
#endif
    return QoreValue();
}

// string SHA512_hmac(data data, string key) {}
static QoreValue f_SHA512_hmac_VDVs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* key = HARD_QORE_VALUE_STRING(args, 1);
# 2585 "ql_crypto.qpp"
#if !defined(OPENSSL_NO_SHA512) && defined(HAVE_OPENSSL_SHA512)
   HMACHelper dh(args);
   if (dh.doHMAC("SHA512-HMAC-ERROR", EVP_sha512(), key, xsink))
      return QoreValue();

   return dh.getString();
#else
   (void)key;
   return missing_openssl_feature("SHA512", xsink);
#endif
    return QoreValue();
}

// string SHA_hmac(data data, string key) {}
static QoreValue f_SHA_hmac_VDVs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* key = HARD_QORE_VALUE_STRING(args, 1);
# 2434 "ql_crypto.qpp"
#ifdef HAVE_OPENSSL_SHA
   HMACHelper dh(args);
   if (dh.doHMAC("SHA-HMAC-ERROR", EVP_sha(), key, xsink))
      return QoreValue();

   return dh.getString();
#else
   (void)key;
   missing_openssl_feature("SHA", xsink);
   return 0;
#endif
    return QoreValue();
}

// binary decrypt_to_binary(string alg, data data, data key, data iv = Qore::DefaultIV, *data mac, *data aad) {}
static QoreValue f_decrypt_to_binary_VsVDVDVDNDND(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* alg = HARD_QORE_VALUE_STRING(args, 0);
    QoreValue data = get_param_value(args, 1);
    QoreValue key = get_param_value(args, 2);
    QoreValue iv = get_param_value(args, 3);
    const AbstractQoreNode* mac = get_param_value(args, 4).get<const AbstractQoreNode>();
    const AbstractQoreNode* aad = get_param_value(args, 5).get<const AbstractQoreNode>();
# 2890 "ql_crypto.qpp"
   const char* datastr;
   size_t data_len;
   q_get_data(data, datastr, data_len);

   const char* keystr;
   size_t key_len;
   q_get_data(key, keystr, key_len);

   const char* ivstr;
   size_t iv_len;
   q_get_data(iv, ivstr, iv_len);

   const char* macstr = nullptr;
   size_t mac_len = 0;
   if (mac)
      q_get_data(mac, macstr, mac_len);

   const char* aadstr = nullptr;
   size_t aad_len = 0;
   if (aad)
      q_get_data(aad, aadstr, aad_len);

   SimpleRefHolder<Transform> t(EncryptionTransforms::getCryptoTransform(alg->c_str(), false, keystr, (unsigned)key_len, ivstr, (unsigned)iv_len, macstr, (unsigned)mac_len, 0, nullptr, aadstr, (unsigned)aad_len, xsink));
   if (*xsink)
      return QoreValue();

   SimpleRefHolder<BinaryNode> rv(new BinaryNode);
   return apply_transformation(t, rv, datastr, data_len, xsink);
}

// string decrypt_to_string(string alg, data data, data key, data iv = Qore::DefaultIV, *data mac, *data aad, *string encoding) {}
static QoreValue f_decrypt_to_string_VsVDVDVDNDNDNs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* alg = HARD_QORE_VALUE_STRING(args, 0);
    QoreValue data = get_param_value(args, 1);
    QoreValue key = get_param_value(args, 2);
    QoreValue iv = get_param_value(args, 3);
    const AbstractQoreNode* mac = get_param_value(args, 4).get<const AbstractQoreNode>();
    const AbstractQoreNode* aad = get_param_value(args, 5).get<const AbstractQoreNode>();
    const QoreStringNode* encoding = get_param_value(args, 6).get<const QoreStringNode>();
# 2948 "ql_crypto.qpp"
   const char* datastr;
   size_t data_len;
   q_get_data(data, datastr, data_len);

   const char* keystr;
   size_t key_len;
   q_get_data(key, keystr, key_len);

   const char* ivstr;
   size_t iv_len;
   q_get_data(iv, ivstr, iv_len);

   const char* macstr = nullptr;
   size_t mac_len = 0;
   if (mac)
      q_get_data(mac, macstr, mac_len);

   const char* aadstr = nullptr;
   size_t aad_len = 0;
   if (aad)
      q_get_data(aad, aadstr, aad_len);

   SimpleRefHolder<Transform> t(EncryptionTransforms::getCryptoTransform(alg->c_str(), false, keystr, (unsigned)key_len, ivstr, (unsigned)iv_len, macstr, (unsigned)mac_len, 0, nullptr, aadstr, (unsigned)aad_len, xsink));
   if (*xsink)
      return QoreValue();

   SimpleRefHolder<BinaryNode> bin(new BinaryNode);
   ValueHolder vh(apply_transformation(t, bin, datastr, data_len, xsink), xsink);
   if (*xsink)
      return QoreValue();

   const QoreEncoding* enc = encoding ? QEM.findCreate(encoding) : QCS_DEFAULT;

   size_t size = vh->get<BinaryNode>()->size();
   // create string with len == allocated to ensure that it gets terminated
   return size ? new QoreStringNode((char*)vh->get<BinaryNode>()->giveBuffer(), size, size, enc) : new QoreStringNode(enc);
}

// binary encrypt(string alg, data data, data key, data iv = Qore::DefaultIV, *reference<binary> mac, *int mac_size, *data aad) {}
static QoreValue f_encrypt_VsVDVDVDC18_reference_binary_NiND(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* alg = HARD_QORE_VALUE_STRING(args, 0);
    QoreValue data = get_param_value(args, 1);
    QoreValue key = get_param_value(args, 2);
    QoreValue iv = get_param_value(args, 3);
    const ReferenceNode* mac = get_param_value(args, 4).get<const ReferenceNode>();
    int64 mac_size = HARD_QORE_VALUE_INT(args, 5);
    const AbstractQoreNode* aad = get_param_value(args, 6).get<const AbstractQoreNode>();
# 2835 "ql_crypto.qpp"
   const char* datastr;
   size_t data_len;
   q_get_data(data, datastr, data_len);

   const char* keystr;
   size_t key_len;
   q_get_data(key, keystr, key_len);

   const char* ivstr;
   size_t iv_len;
   q_get_data(iv, ivstr, iv_len);

   if (check_mac(mac, mac_size, xsink))
      return QoreValue();

   const char* aadstr = nullptr;
   size_t aad_len = 0;
   if (aad)
      q_get_data(aad, aadstr, aad_len);

   SimpleRefHolder<Transform> t(EncryptionTransforms::getCryptoTransform(alg->c_str(), true, keystr, (unsigned)key_len, ivstr, (unsigned)iv_len, nullptr, 0, (unsigned)mac_size, mac, aadstr, (unsigned)aad_len, xsink));
   if (*xsink)
      return QoreValue();

   SimpleRefHolder<BinaryNode> rv(new BinaryNode);
   return apply_transformation(t, rv, datastr, data_len, xsink);
}

// hash get_crypto_info() {}
static QoreValue f_get_crypto_info(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 3007 "ql_crypto.qpp"
   ReferenceHolder<QoreHashNode> rv(new QoreHashNode, xsink);

   for (auto& i : crypto_map) {
      rv->setKeyValue(i.first.c_str(), i.second.getInfo(), nullptr);
   }

   return rv.release();
}

// Transform get_decryptor(string alg, data key, data iv = Qore::DefaultIV, *data mac, *data aad) {}
static QoreValue f_get_decryptor_VsVDVDNDND(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* alg = HARD_QORE_VALUE_STRING(args, 0);
    QoreValue key = get_param_value(args, 1);
    QoreValue iv = get_param_value(args, 2);
    const AbstractQoreNode* mac = get_param_value(args, 3).get<const AbstractQoreNode>();
    const AbstractQoreNode* aad = get_param_value(args, 4).get<const AbstractQoreNode>();
# 2784 "ql_crypto.qpp"
   const char* keystr;
   size_t key_len;
   q_get_data(key, keystr, key_len);

   const char* ivstr;
   size_t iv_len;
   q_get_data(iv, ivstr, iv_len);

   const char* macstr = nullptr;
   size_t mac_len = 0;
   if (mac)
      q_get_data(mac, macstr, mac_len);

   const char* aadstr = nullptr;
   size_t aad_len = 0;
   if (aad)
      q_get_data(aad, aadstr, aad_len);

   SimpleRefHolder<Transform> t(EncryptionTransforms::getCryptoTransform(alg->c_str(), false, keystr, (unsigned)key_len, ivstr, (unsigned)iv_len, macstr, (unsigned)mac_len, 0, nullptr, aadstr, (unsigned)aad_len, xsink));
   return *xsink ? nullptr : new QoreObject(QC_TRANSFORM, getProgram(), t.release());
}

// Transform get_encryptor(string alg, data key, data iv = Qore::DefaultIV, *reference<binary> mac, *int mac_size, *data aad) {}
static QoreValue f_get_encryptor_VsVDVDC18_reference_binary_NiND(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* alg = HARD_QORE_VALUE_STRING(args, 0);
    QoreValue key = get_param_value(args, 1);
    QoreValue iv = get_param_value(args, 2);
    const ReferenceNode* mac = get_param_value(args, 3).get<const ReferenceNode>();
    int64 mac_size = HARD_QORE_VALUE_INT(args, 4);
    const AbstractQoreNode* aad = get_param_value(args, 5).get<const AbstractQoreNode>();
# 2737 "ql_crypto.qpp"
   const char* keystr;
   size_t key_len;
   q_get_data(key, keystr, key_len);

   const char* ivstr;
   size_t iv_len;
   q_get_data(iv, ivstr, iv_len);

   if (check_mac(mac, mac_size, xsink))
      return QoreValue();

   const char* aadstr = nullptr;
   size_t aad_len = 0;
   if (aad)
      q_get_data(aad, aadstr, aad_len);

   SimpleRefHolder<Transform> t(EncryptionTransforms::getCryptoTransform(alg->c_str(), true, keystr, (unsigned)key_len, ivstr, (unsigned)iv_len, nullptr, 0, (unsigned)mac_size, mac, aadstr, (unsigned)aad_len, xsink));
   return *xsink ? nullptr : new QoreObject(QC_TRANSFORM, getProgram(), t.release());
}

// binary get_random_bytes(int size) {}
static QoreValue f_get_random_bytes_Vi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 size = HARD_QORE_VALUE_INT(args, 0);
# 3032 "ql_crypto.qpp"
   SimpleRefHolder<BinaryNode> rv(new BinaryNode);
   if (size > 0) {
      rv->preallocate(size);
      int rc = RAND_bytes((unsigned char*)rv->getPtr(), size);
      if (rc <= 0) {
         unsigned long err = ERR_get_error();
         char buf[121];
         ERR_error_string(err, buf);
         xsink->raiseException("GET-RANDOM-BYTES-ERROR", "the RAND_bytes() returned error code %d with a request of length %d: %s", rc, (int)size, buf);
         return QoreValue();
      }
   }
   return rv.release();
}


DLLLOCAL void init_crypto_functions(QoreNamespace& ns) {

    // initialization code added by qppinit
    SimpleRefHolder<BinaryNode> DefaultIV(new BinaryNode); DefaultIV->append(def_iv, 8);
    // binary blowfish_decrypt_cbc(binary data, data key, data iv = Qore::DefaultIV) {}
    ns.addBuiltinVariant("blowfish_decrypt_cbc", (q_func_n_t)f_blowfish_decrypt_cbc_VyVDVD, QC_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 3, binaryTypeInfo, NULL, "data", dataTypeInfo, NULL, "key", dataTypeInfo, DefaultIV->refSelf(), "iv");
    // string blowfish_decrypt_cbc_to_string(binary data, data key, data iv = Qore::DefaultIV, *string encoding) {}
    ns.addBuiltinVariant("blowfish_decrypt_cbc_to_string", (q_func_n_t)f_blowfish_decrypt_cbc_to_string_VyVDVDNs, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 4, binaryTypeInfo, NULL, "data", dataTypeInfo, NULL, "key", dataTypeInfo, DefaultIV->refSelf(), "iv", stringOrNothingTypeInfo, NULL, "encoding");
    // binary blowfish_encrypt_cbc(data data, data key, data iv = Qore::DefaultIV) {}
    ns.addBuiltinVariant("blowfish_encrypt_cbc", (q_func_n_t)f_blowfish_encrypt_cbc_VDVDVD, QC_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 3, dataTypeInfo, NULL, "data", dataTypeInfo, NULL, "key", dataTypeInfo, DefaultIV->refSelf(), "iv");
    // binary cast5_decrypt_cbc(binary data, data key, data iv = Qore::DefaultIV) {}
    ns.addBuiltinVariant("cast5_decrypt_cbc", (q_func_n_t)f_cast5_decrypt_cbc_VyVDVD, QC_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 3, binaryTypeInfo, NULL, "data", dataTypeInfo, NULL, "key", dataTypeInfo, DefaultIV->refSelf(), "iv");
    // string cast5_decrypt_cbc_to_string(binary data, data key, data iv = Qore::DefaultIV, *string encoding) {}
    ns.addBuiltinVariant("cast5_decrypt_cbc_to_string", (q_func_n_t)f_cast5_decrypt_cbc_to_string_VyVDVDNs, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 4, binaryTypeInfo, NULL, "data", dataTypeInfo, NULL, "key", dataTypeInfo, DefaultIV->refSelf(), "iv", stringOrNothingTypeInfo, NULL, "encoding");
    // binary cast5_encrypt_cbc(data data, data key, data iv = Qore::DefaultIV) {}
    ns.addBuiltinVariant("cast5_encrypt_cbc", (q_func_n_t)f_cast5_encrypt_cbc_VDVDVD, QC_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 3, dataTypeInfo, NULL, "data", dataTypeInfo, NULL, "key", dataTypeInfo, DefaultIV->refSelf(), "iv");
    // binary des_decrypt_cbc(binary data, data key, data iv = Qore::DefaultIV) {}
    ns.addBuiltinVariant("des_decrypt_cbc", (q_func_n_t)f_des_decrypt_cbc_VyVDVD, QC_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 3, binaryTypeInfo, NULL, "data", dataTypeInfo, NULL, "key", dataTypeInfo, DefaultIV->refSelf(), "iv");
    // string des_decrypt_cbc_to_string(binary data, data key, data iv = Qore::DefaultIV, *string encoding) {}
    ns.addBuiltinVariant("des_decrypt_cbc_to_string", (q_func_n_t)f_des_decrypt_cbc_to_string_VyVDVDNs, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 4, binaryTypeInfo, NULL, "data", dataTypeInfo, NULL, "key", dataTypeInfo, DefaultIV->refSelf(), "iv", stringOrNothingTypeInfo, NULL, "encoding");
    // binary des_ede3_decrypt_cbc(data data, data key, data iv = Qore::DefaultIV) {}
    ns.addBuiltinVariant("des_ede3_decrypt_cbc", (q_func_n_t)f_des_ede3_decrypt_cbc_VDVDVD, QC_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 3, dataTypeInfo, NULL, "data", dataTypeInfo, NULL, "key", dataTypeInfo, DefaultIV->refSelf(), "iv");
    // string des_ede3_decrypt_cbc_to_string(binary data, data key, data iv = Qore::DefaultIV, *string encoding) {}
    ns.addBuiltinVariant("des_ede3_decrypt_cbc_to_string", (q_func_n_t)f_des_ede3_decrypt_cbc_to_string_VyVDVDNs, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 4, binaryTypeInfo, NULL, "data", dataTypeInfo, NULL, "key", dataTypeInfo, DefaultIV->refSelf(), "iv", stringOrNothingTypeInfo, NULL, "encoding");
    // binary des_ede3_encrypt_cbc(data data, data key, data iv = Qore::DefaultIV) {}
    ns.addBuiltinVariant("des_ede3_encrypt_cbc", (q_func_n_t)f_des_ede3_encrypt_cbc_VDVDVD, QC_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 3, dataTypeInfo, NULL, "data", dataTypeInfo, NULL, "key", dataTypeInfo, DefaultIV->refSelf(), "iv");
    // binary des_ede_decrypt_cbc(binary data, data key, data iv = Qore::DefaultIV) {}
    ns.addBuiltinVariant("des_ede_decrypt_cbc", (q_func_n_t)f_des_ede_decrypt_cbc_VyVDVD, QC_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 3, binaryTypeInfo, NULL, "data", dataTypeInfo, NULL, "key", dataTypeInfo, DefaultIV->refSelf(), "iv");
    // string des_ede_decrypt_cbc_to_string(binary data, data key, data iv = Qore::DefaultIV, *string encoding) {}
    ns.addBuiltinVariant("des_ede_decrypt_cbc_to_string", (q_func_n_t)f_des_ede_decrypt_cbc_to_string_VyVDVDNs, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 4, binaryTypeInfo, NULL, "data", dataTypeInfo, NULL, "key", dataTypeInfo, DefaultIV->refSelf(), "iv", stringOrNothingTypeInfo, NULL, "encoding");
    // binary des_ede_encrypt_cbc(data data, data key, data iv = Qore::DefaultIV) {}
    ns.addBuiltinVariant("des_ede_encrypt_cbc", (q_func_n_t)f_des_ede_encrypt_cbc_VDVDVD, QC_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 3, dataTypeInfo, NULL, "data", dataTypeInfo, NULL, "key", dataTypeInfo, DefaultIV->refSelf(), "iv");
    // binary des_encrypt_cbc(data data, data key, data iv = Qore::DefaultIV) {}
    ns.addBuiltinVariant("des_encrypt_cbc", (q_func_n_t)f_des_encrypt_cbc_VDVDVD, QC_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 3, dataTypeInfo, NULL, "data", dataTypeInfo, NULL, "key", dataTypeInfo, DefaultIV->refSelf(), "iv");
    // binary des_random_key() {}
    ns.addBuiltinVariant("des_random_key", (q_func_n_t)f_des_random_key, QC_CONSTANT, QDOM_DEFAULT, binaryTypeInfo);
    // binary desx_decrypt_cbc(binary data, data key, data iv = Qore::DefaultIV) {}
    ns.addBuiltinVariant("desx_decrypt_cbc", (q_func_n_t)f_desx_decrypt_cbc_VyVDVD, QC_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 3, binaryTypeInfo, NULL, "data", dataTypeInfo, NULL, "key", dataTypeInfo, DefaultIV->refSelf(), "iv");
    // string desx_decrypt_cbc_to_string(binary data, data key, data iv = Qore::DefaultIV, *string encoding) {}
    ns.addBuiltinVariant("desx_decrypt_cbc_to_string", (q_func_n_t)f_desx_decrypt_cbc_to_string_VyVDVDNs, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 4, binaryTypeInfo, NULL, "data", dataTypeInfo, NULL, "key", dataTypeInfo, DefaultIV->refSelf(), "iv", stringOrNothingTypeInfo, NULL, "encoding");
    // binary desx_encrypt_cbc(data data, data key, data iv = Qore::DefaultIV) {}
    ns.addBuiltinVariant("desx_encrypt_cbc", (q_func_n_t)f_desx_encrypt_cbc_VDVDVD, QC_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 3, dataTypeInfo, NULL, "data", dataTypeInfo, NULL, "key", dataTypeInfo, DefaultIV->refSelf(), "iv");
    // binary rc2_decrypt_cbc(binary data, data key, data iv = Qore::DefaultIV) {}
    ns.addBuiltinVariant("rc2_decrypt_cbc", (q_func_n_t)f_rc2_decrypt_cbc_VyVDVD, QC_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 3, binaryTypeInfo, NULL, "data", dataTypeInfo, NULL, "key", dataTypeInfo, DefaultIV->refSelf(), "iv");
    // string rc2_decrypt_cbc_to_string(binary data, data key, data iv = Qore::DefaultIV, *string encoding) {}
    ns.addBuiltinVariant("rc2_decrypt_cbc_to_string", (q_func_n_t)f_rc2_decrypt_cbc_to_string_VyVDVDNs, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 4, binaryTypeInfo, NULL, "data", dataTypeInfo, NULL, "key", dataTypeInfo, DefaultIV->refSelf(), "iv", stringOrNothingTypeInfo, NULL, "encoding");
    // binary rc2_encrypt_cbc(data data, data key, data iv = Qore::DefaultIV) {}
    ns.addBuiltinVariant("rc2_encrypt_cbc", (q_func_n_t)f_rc2_encrypt_cbc_VDVDVD, QC_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 3, dataTypeInfo, NULL, "data", dataTypeInfo, NULL, "key", dataTypeInfo, DefaultIV->refSelf(), "iv");
    // binary rc4_decrypt(binary data, data key, data iv = Qore::DefaultIV) {}
    ns.addBuiltinVariant("rc4_decrypt", (q_func_n_t)f_rc4_decrypt_VyVDVD, QC_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 3, binaryTypeInfo, NULL, "data", dataTypeInfo, NULL, "key", dataTypeInfo, DefaultIV->refSelf(), "iv");
    // string rc4_decrypt_to_string(binary data, data key, data iv = Qore::DefaultIV, *string encoding) {}
    ns.addBuiltinVariant("rc4_decrypt_to_string", (q_func_n_t)f_rc4_decrypt_to_string_VyVDVDNs, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 4, binaryTypeInfo, NULL, "data", dataTypeInfo, NULL, "key", dataTypeInfo, DefaultIV->refSelf(), "iv", stringOrNothingTypeInfo, NULL, "encoding");
    // binary rc4_encrypt(data data, data key, data iv = Qore::DefaultIV) {}
    ns.addBuiltinVariant("rc4_encrypt", (q_func_n_t)f_rc4_encrypt_VDVDVD, QC_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 3, dataTypeInfo, NULL, "data", dataTypeInfo, NULL, "key", dataTypeInfo, DefaultIV->refSelf(), "iv");
    // binary rc5_decrypt_cbc(binary data, data key, data iv = Qore::DefaultIV) {}
    ns.addBuiltinVariant("rc5_decrypt_cbc", (q_func_n_t)f_rc5_decrypt_cbc_VyVDVD, QC_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 3, binaryTypeInfo, NULL, "data", dataTypeInfo, NULL, "key", dataTypeInfo, DefaultIV->refSelf(), "iv");
    // string rc5_decrypt_cbc_to_string(binary data, data key, data iv = Qore::DefaultIV, *string encoding) {}
    ns.addBuiltinVariant("rc5_decrypt_cbc_to_string", (q_func_n_t)f_rc5_decrypt_cbc_to_string_VyVDVDNs, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 4, binaryTypeInfo, NULL, "data", dataTypeInfo, NULL, "key", dataTypeInfo, DefaultIV->refSelf(), "iv", stringOrNothingTypeInfo, NULL, "encoding");
    // binary rc5_encrypt_cbc(data data, data key, data iv = Qore::DefaultIV) {}
    ns.addBuiltinVariant("rc5_encrypt_cbc", (q_func_n_t)f_rc5_encrypt_cbc_VDVDVD, QC_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 3, dataTypeInfo, NULL, "data", dataTypeInfo, NULL, "key", dataTypeInfo, DefaultIV->refSelf(), "iv");
    // string DSS(data data) {}
    ns.addBuiltinVariant("DSS", (q_func_n_t)f_DSS_VD, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 1, dataTypeInfo, NULL, "data");
    // string DSS1(data data) {}
    ns.addBuiltinVariant("DSS1", (q_func_n_t)f_DSS1_VD, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 1, dataTypeInfo, NULL, "data");
    // binary DSS1_bin(data data) {}
    ns.addBuiltinVariant("DSS1_bin", (q_func_n_t)f_DSS1_bin_VD, QC_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 1, dataTypeInfo, NULL, "data");
    // binary DSS_bin(data data) {}
    ns.addBuiltinVariant("DSS_bin", (q_func_n_t)f_DSS_bin_VD, QC_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 1, dataTypeInfo, NULL, "data");
    // string MD2(data data) {}
    ns.addBuiltinVariant("MD2", (q_func_n_t)f_MD2_VD, QC_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 1, dataTypeInfo, NULL, "data");
    // binary MD2_bin(data data) {}
    ns.addBuiltinVariant("MD2_bin", (q_func_n_t)f_MD2_bin_VD, QC_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 1, dataTypeInfo, NULL, "data");
    // string MD4(data data) {}
    ns.addBuiltinVariant("MD4", (q_func_n_t)f_MD4_VD, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 1, dataTypeInfo, NULL, "data");
    // binary MD4_bin(data data) {}
    ns.addBuiltinVariant("MD4_bin", (q_func_n_t)f_MD4_bin_VD, QC_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 1, dataTypeInfo, NULL, "data");
    // string MD5(data data) {}
    ns.addBuiltinVariant("MD5", (q_func_n_t)f_MD5_VD, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 1, dataTypeInfo, NULL, "data");
    // binary MD5_bin(data data) {}
    ns.addBuiltinVariant("MD5_bin", (q_func_n_t)f_MD5_bin_VD, QC_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 1, dataTypeInfo, NULL, "data");
    // string MDC2(data data) {}
    ns.addBuiltinVariant("MDC2", (q_func_n_t)f_MDC2_VD, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 1, dataTypeInfo, NULL, "data");
    // binary MDC2_bin(data data) {}
    ns.addBuiltinVariant("MDC2_bin", (q_func_n_t)f_MDC2_bin_VD, QC_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 1, dataTypeInfo, NULL, "data");
    // string RIPEMD160(data data) {}
    ns.addBuiltinVariant("RIPEMD160", (q_func_n_t)f_RIPEMD160_VD, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 1, dataTypeInfo, NULL, "data");
    // binary RIPEMD160_binary(data data) {}
    ns.addBuiltinVariant("RIPEMD160_binary", (q_func_n_t)f_RIPEMD160_binary_VD, QC_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 1, dataTypeInfo, NULL, "data");
    // string SHA(data data) {}
    ns.addBuiltinVariant("SHA", (q_func_n_t)f_SHA_VD, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 1, dataTypeInfo, NULL, "data");
    // string SHA1(data data) {}
    ns.addBuiltinVariant("SHA1", (q_func_n_t)f_SHA1_VD, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 1, dataTypeInfo, NULL, "data");
    // binary SHA1_bin(data data) {}
    ns.addBuiltinVariant("SHA1_bin", (q_func_n_t)f_SHA1_bin_VD, QC_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 1, dataTypeInfo, NULL, "data");
    // string SHA224(data data) {}
    ns.addBuiltinVariant("SHA224", (q_func_n_t)f_SHA224_VD, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 1, dataTypeInfo, NULL, "data");
    // binary SHA224_bin(data data) {}
    ns.addBuiltinVariant("SHA224_bin", (q_func_n_t)f_SHA224_bin_VD, QC_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 1, dataTypeInfo, NULL, "data");
    // string SHA256(data data) {}
    ns.addBuiltinVariant("SHA256", (q_func_n_t)f_SHA256_VD, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 1, dataTypeInfo, NULL, "data");
    // binary SHA256_bin(data data) {}
    ns.addBuiltinVariant("SHA256_bin", (q_func_n_t)f_SHA256_bin_VD, QC_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 1, dataTypeInfo, NULL, "data");
    // string SHA384(data data) {}
    ns.addBuiltinVariant("SHA384", (q_func_n_t)f_SHA384_VD, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 1, dataTypeInfo, NULL, "data");
    // binary SHA384_bin(data data) {}
    ns.addBuiltinVariant("SHA384_bin", (q_func_n_t)f_SHA384_bin_VD, QC_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 1, dataTypeInfo, NULL, "data");
    // string SHA512(data data) {}
    ns.addBuiltinVariant("SHA512", (q_func_n_t)f_SHA512_VD, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 1, dataTypeInfo, NULL, "data");
    // binary SHA512_bin(data data) {}
    ns.addBuiltinVariant("SHA512_bin", (q_func_n_t)f_SHA512_bin_VD, QC_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 1, dataTypeInfo, NULL, "data");
    // binary SHA_bin(data data) {}
    ns.addBuiltinVariant("SHA_bin", (q_func_n_t)f_SHA_bin_VD, QC_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 1, dataTypeInfo, NULL, "data");
    // string DSS1_hmac(data data, string key) {}
    ns.addBuiltinVariant("DSS1_hmac", (q_func_n_t)f_DSS1_hmac_VDVs, QC_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 2, dataTypeInfo, NULL, "data", stringTypeInfo, NULL, "key");
    // string DSS_hmac(data data, string key) {}
    ns.addBuiltinVariant("DSS_hmac", (q_func_n_t)f_DSS_hmac_VDVs, QC_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 2, dataTypeInfo, NULL, "data", stringTypeInfo, NULL, "key");
    // string MD2_hmac(data data, string key) {}
    ns.addBuiltinVariant("MD2_hmac", (q_func_n_t)f_MD2_hmac_VDVs, QC_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 2, dataTypeInfo, NULL, "data", stringTypeInfo, NULL, "key");
    // string MD4_hmac(data data, string key) {}
    ns.addBuiltinVariant("MD4_hmac", (q_func_n_t)f_MD4_hmac_VDVs, QC_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 2, dataTypeInfo, NULL, "data", stringTypeInfo, NULL, "key");
    // string MD5_hmac(data data, string key) {}
    ns.addBuiltinVariant("MD5_hmac", (q_func_n_t)f_MD5_hmac_VDVs, QC_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 2, dataTypeInfo, NULL, "data", stringTypeInfo, NULL, "key");
    // string MDC2_hmac(data data, string key) {}
    ns.addBuiltinVariant("MDC2_hmac", (q_func_n_t)f_MDC2_hmac_VDVs, QC_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 2, dataTypeInfo, NULL, "data", stringTypeInfo, NULL, "key");
    // string RIPEMD160_hmac(data data, string key) {}
    ns.addBuiltinVariant("RIPEMD160_hmac", (q_func_n_t)f_RIPEMD160_hmac_VDVs, QC_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 2, dataTypeInfo, NULL, "data", stringTypeInfo, NULL, "key");
    // string SHA1_hmac(data data, string key) {}
    ns.addBuiltinVariant("SHA1_hmac", (q_func_n_t)f_SHA1_hmac_VDVs, QC_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 2, dataTypeInfo, NULL, "data", stringTypeInfo, NULL, "key");
    // string SHA224_hmac(data data, string key) {}
    ns.addBuiltinVariant("SHA224_hmac", (q_func_n_t)f_SHA224_hmac_VDVs, QC_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 2, dataTypeInfo, NULL, "data", stringTypeInfo, NULL, "key");
    // string SHA256_hmac(data data, string key) {}
    ns.addBuiltinVariant("SHA256_hmac", (q_func_n_t)f_SHA256_hmac_VDVs, QC_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 2, dataTypeInfo, NULL, "data", stringTypeInfo, NULL, "key");
    // string SHA384_hmac(data data, string key) {}
    ns.addBuiltinVariant("SHA384_hmac", (q_func_n_t)f_SHA384_hmac_VDVs, QC_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 2, dataTypeInfo, NULL, "data", stringTypeInfo, NULL, "key");
    // string SHA512_hmac(data data, string key) {}
    ns.addBuiltinVariant("SHA512_hmac", (q_func_n_t)f_SHA512_hmac_VDVs, QC_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 2, dataTypeInfo, NULL, "data", stringTypeInfo, NULL, "key");
    // string SHA_hmac(data data, string key) {}
    ns.addBuiltinVariant("SHA_hmac", (q_func_n_t)f_SHA_hmac_VDVs, QC_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 2, dataTypeInfo, NULL, "data", stringTypeInfo, NULL, "key");
    // binary decrypt_to_binary(string alg, data data, data key, data iv = Qore::DefaultIV, *data mac, *data aad) {}
    ns.addBuiltinVariant("decrypt_to_binary", (q_func_n_t)f_decrypt_to_binary_VsVDVDVDNDND, QC_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 6, stringTypeInfo, NULL, "alg", dataTypeInfo, NULL, "data", dataTypeInfo, NULL, "key", dataTypeInfo, DefaultIV->refSelf(), "iv", dataOrNothingTypeInfo, NULL, "mac", dataOrNothingTypeInfo, NULL, "aad");
    // string decrypt_to_string(string alg, data data, data key, data iv = Qore::DefaultIV, *data mac, *data aad, *string encoding) {}
    ns.addBuiltinVariant("decrypt_to_string", (q_func_n_t)f_decrypt_to_string_VsVDVDVDNDNDNs, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 7, stringTypeInfo, NULL, "alg", dataTypeInfo, NULL, "data", dataTypeInfo, NULL, "key", dataTypeInfo, DefaultIV->refSelf(), "iv", dataOrNothingTypeInfo, NULL, "mac", dataOrNothingTypeInfo, NULL, "aad", stringOrNothingTypeInfo, NULL, "encoding");
    // binary encrypt(string alg, data data, data key, data iv = Qore::DefaultIV, *reference<binary> mac, *int mac_size, *data aad) {}
    ns.addBuiltinVariant("encrypt", (q_func_n_t)f_encrypt_VsVDVDVDC18_reference_binary_NiND, QC_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 7, stringTypeInfo, NULL, "alg", dataTypeInfo, NULL, "data", dataTypeInfo, NULL, "key", dataTypeInfo, DefaultIV->refSelf(), "iv", qore_get_complex_reference_or_nothing_type(binaryTypeInfo), NULL, "mac", bigIntOrNothingTypeInfo, NULL, "mac_size", dataOrNothingTypeInfo, NULL, "aad");
    // hash get_crypto_info() {}
    ns.addBuiltinVariant("get_crypto_info", (q_func_n_t)f_get_crypto_info, QC_CONSTANT, QDOM_DEFAULT, hashTypeInfo);
    // Transform get_decryptor(string alg, data key, data iv = Qore::DefaultIV, *data mac, *data aad) {}
    ns.addBuiltinVariant("get_decryptor", (q_func_n_t)f_get_decryptor_VsVDVDNDND, QC_RET_VALUE_ONLY, QDOM_DEFAULT, QC_TRANSFORM->getTypeInfo(), 5, stringTypeInfo, NULL, "alg", dataTypeInfo, NULL, "key", dataTypeInfo, DefaultIV->refSelf(), "iv", dataOrNothingTypeInfo, NULL, "mac", dataOrNothingTypeInfo, NULL, "aad");
    // Transform get_encryptor(string alg, data key, data iv = Qore::DefaultIV, *reference<binary> mac, *int mac_size, *data aad) {}
    ns.addBuiltinVariant("get_encryptor", (q_func_n_t)f_get_encryptor_VsVDVDC18_reference_binary_NiND, QC_RET_VALUE_ONLY, QDOM_DEFAULT, QC_TRANSFORM->getTypeInfo(), 6, stringTypeInfo, NULL, "alg", dataTypeInfo, NULL, "key", dataTypeInfo, DefaultIV->refSelf(), "iv", qore_get_complex_reference_or_nothing_type(binaryTypeInfo), NULL, "mac", bigIntOrNothingTypeInfo, NULL, "mac_size", dataOrNothingTypeInfo, NULL, "aad");
    // binary get_random_bytes(int size) {}
    ns.addBuiltinVariant("get_random_bytes", (q_func_n_t)f_get_random_bytes_Vi, QC_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 1, bigIntTypeInfo, NULL, "size");

}

DLLLOCAL void init_crypto_constants(QoreNamespace& ns) {
# 294 "ql_crypto.qpp"
# 321 "ql_crypto.qpp"
# 326 "ql_crypto.qpp"
# 331 "ql_crypto.qpp"
# 338 "ql_crypto.qpp"
# 345 "ql_crypto.qpp"
# 352 "ql_crypto.qpp"
# 359 "ql_crypto.qpp"
# 366 "ql_crypto.qpp"
# 373 "ql_crypto.qpp"
# 380 "ql_crypto.qpp"
# 387 "ql_crypto.qpp"
# 394 "ql_crypto.qpp"
# 401 "ql_crypto.qpp"
# 407 "ql_crypto.qpp"
# 413 "ql_crypto.qpp"
# 419 "ql_crypto.qpp"
# 426 "ql_crypto.qpp"
# 436 "ql_crypto.qpp"
# 446 "ql_crypto.qpp"
# 456 "ql_crypto.qpp"
# 461 "ql_crypto.qpp"
# 466 "ql_crypto.qpp"
# 471 "ql_crypto.qpp"
# 481 "ql_crypto.qpp"
    ns.addConstant("CRYPTO_ALG_AES_128", new QoreStringNode("aes128"));
    ns.addConstant("CRYPTO_ALG_AES_192", new QoreStringNode("aes192"));
    ns.addConstant("CRYPTO_ALG_AES_256", new QoreStringNode("aes256"));
    ns.addConstant("CRYPTO_ALG_BLOWFISH", new QoreStringNode("blowfish"));
    ns.addConstant("CRYPTO_ALG_BLOWFISH_CFB", new QoreStringNode("blowfish-cfb"));
    ns.addConstant("CRYPTO_ALG_BLOWFISH_OFB", new QoreStringNode("blowfish-ofb"));
    ns.addConstant("CRYPTO_ALG_CAST5", new QoreStringNode("cast5"));
    ns.addConstant("CRYPTO_ALG_CAST5_CFB", new QoreStringNode("cast5-cfb"));
    ns.addConstant("CRYPTO_ALG_CAST5_OFB", new QoreStringNode("cast5-ofb"));
    ns.addConstant("CRYPTO_ALG_DES", new QoreStringNode("des"));
    ns.addConstant("CRYPTO_ALG_DESX", new QoreStringNode("desx"));
    ns.addConstant("CRYPTO_ALG_DES_CFB", new QoreStringNode("des-cfb"));
    ns.addConstant("CRYPTO_ALG_DES_EDE", new QoreStringNode("desede"));
    ns.addConstant("CRYPTO_ALG_DES_EDE3", new QoreStringNode("desede3"));
    ns.addConstant("CRYPTO_ALG_DES_EDE3_CFB", new QoreStringNode("desede3-cfb"));
    ns.addConstant("CRYPTO_ALG_DES_EDE3_OFB", new QoreStringNode("desede3-ofb"));
    ns.addConstant("CRYPTO_ALG_DES_EDE_CFB", new QoreStringNode("desede-cfb"));
    ns.addConstant("CRYPTO_ALG_DES_EDE_OFB", new QoreStringNode("desede-ofb"));
    ns.addConstant("CRYPTO_ALG_DES_OFB", new QoreStringNode("des-ofb"));
    ns.addConstant("CRYPTO_ALG_RC2", new QoreStringNode("rc2"));
    ns.addConstant("CRYPTO_ALG_RC2_CFB", new QoreStringNode("rc2-cfb"));
    ns.addConstant("CRYPTO_ALG_RC2_OFB", new QoreStringNode("rc2-ofb"));
    ns.addConstant("CRYPTO_ALG_RC4", new QoreStringNode("rc4"));
    ns.addConstant("CRYPTO_ALG_RC5", new QoreStringNode("rc5"));
    ns.addConstant("CRYPTO_ALG_RC5_CFB", new QoreStringNode("rc5-cfb"));
    ns.addConstant("CRYPTO_ALG_RC5_OFB", new QoreStringNode("rc5-ofb"));
    BinaryNode* bin_DefaultIV = new BinaryNode;
    bin_DefaultIV->append("\x00\x00\x00\x00\x00\x00\x00\x00", 8);

    ns.addConstant("DefaultIV", bin_DefaultIV);
# 515 "ql_crypto.qpp"
# 549 "ql_crypto.qpp"
# 563 "ql_crypto.qpp"
# 597 "ql_crypto.qpp"
# 631 "ql_crypto.qpp"
# 668 "ql_crypto.qpp"
# 704 "ql_crypto.qpp"
# 740 "ql_crypto.qpp"
# 779 "ql_crypto.qpp"
# 815 "ql_crypto.qpp"
# 851 "ql_crypto.qpp"
# 890 "ql_crypto.qpp"
# 926 "ql_crypto.qpp"
# 962 "ql_crypto.qpp"
# 1001 "ql_crypto.qpp"
# 1037 "ql_crypto.qpp"
# 1073 "ql_crypto.qpp"
# 1112 "ql_crypto.qpp"
# 1148 "ql_crypto.qpp"
# 1184 "ql_crypto.qpp"
# 1223 "ql_crypto.qpp"
# 1257 "ql_crypto.qpp"
# 1291 "ql_crypto.qpp"
# 1328 "ql_crypto.qpp"
# 1362 "ql_crypto.qpp"
# 1396 "ql_crypto.qpp"
# 1433 "ql_crypto.qpp"
# 1474 "ql_crypto.qpp"
# 1515 "ql_crypto.qpp"
# 1560 "ql_crypto.qpp"
# 1580 "ql_crypto.qpp"
# 1610 "ql_crypto.qpp"
# 1633 "ql_crypto.qpp"
# 1660 "ql_crypto.qpp"
# 1689 "ql_crypto.qpp"
# 1714 "ql_crypto.qpp"
# 1746 "ql_crypto.qpp"
# 1778 "ql_crypto.qpp"
# 1810 "ql_crypto.qpp"
# 1842 "ql_crypto.qpp"
# 1871 "ql_crypto.qpp"
# 1900 "ql_crypto.qpp"
# 1930 "ql_crypto.qpp"
# 1953 "ql_crypto.qpp"
# 1983 "ql_crypto.qpp"
# 2006 "ql_crypto.qpp"
# 2029 "ql_crypto.qpp"
# 2058 "ql_crypto.qpp"
# 2081 "ql_crypto.qpp"
# 2111 "ql_crypto.qpp"
# 2141 "ql_crypto.qpp"
# 2171 "ql_crypto.qpp"
# 2201 "ql_crypto.qpp"
# 2230 "ql_crypto.qpp"
# 2259 "ql_crypto.qpp"
# 2289 "ql_crypto.qpp"
# 2314 "ql_crypto.qpp"
# 2369 "ql_crypto.qpp"
# 2392 "ql_crypto.qpp"
# 2417 "ql_crypto.qpp"
# 2448 "ql_crypto.qpp"
# 2473 "ql_crypto.qpp"
# 2505 "ql_crypto.qpp"
# 2536 "ql_crypto.qpp"
# 2567 "ql_crypto.qpp"
# 2598 "ql_crypto.qpp"
# 2628 "ql_crypto.qpp"
# 2658 "ql_crypto.qpp"
# 2686 "ql_crypto.qpp"
# 2709 "ql_crypto.qpp"
# 2758 "ql_crypto.qpp"
# 2807 "ql_crypto.qpp"
# 2864 "ql_crypto.qpp"
# 2921 "ql_crypto.qpp"
# 2987 "ql_crypto.qpp"
# 3017 "ql_crypto.qpp"

}
