/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  ql_file.qpp

  Qore Programming Language

  Copyright (C) 2003 - 2017 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/ql_file.h"
#include "qore/intern/QoreDir.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>

#ifdef _Q_WINDOWS
#include <io.h>
#include <direct.h>
#else
#include <sys/param.h>
#include <limits.h>
#include <stdlib.h>
#endif

#ifdef HAVE_GLOB_H
#include <glob.h>
#else
#include "qore/intern/glob.h"
#endif

static bool check_stat(unsigned code, const QoreString& path) {
   return (q_get_mode(path) & S_IFMT) == code;
}

#ifdef HAVE_LSTAT
static bool check_lstat(unsigned code, const QoreString& path) {
   struct stat sbuf;
   int rc;

   if ((rc = lstat(path.getBuffer(), &sbuf)))
      return false;

   return (sbuf.st_mode & S_IFMT) == code;
}
#endif

static bool is_writable_intern(const QoreStringNode* path) {
#if defined HAVE_PWD_H
   struct stat sbuf;
   int rc;

   if ((rc = stat(path->getBuffer(), &sbuf)))
      return false;

   if (S_ISDIR(sbuf.st_mode)) { // If path is a directory.
      QoreStringMaker tfn("%s/xxx_qore_tmp_test_xxx", path->getBuffer());
      rc = open(tfn.getBuffer(), O_WRONLY | O_CREAT, S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH);
      if (rc != -1) {
         close(rc);
         unlink(tfn.getBuffer());
         return true;
      }
      return false;
   }

   rc = open(path->getBuffer(), O_WRONLY);
   if (rc != -1) {
      close(rc);
      return true;
   }
   return false;

#elif defined(HAVE_ACCESS) && defined(_Q_WINDOWS)
   // only use access(2) on Windows
   return access(path->getBuffer(), W_OK) == 0;
#elif defined _Q_WINDOWS
   int ftype = q_get_mode(*path) & S_IFMT;
   if (ftype & S_ISDIR) {
      QoreStringMaker tfn("%s\\xxx_qore_tmp_test_xxx", path->getBuffer());
      HANDLE h = CreateFile(tfn.getBuffer(), GENERIC_WRITE, 0, 0, CREATE_NEW, FILE_ATTRIBUTE_NORMAL|FILE_FLAG_DELETE_ON_CLOSE, 0);
      if (h == INVALID_HANDLE_VALUE)
         return false;
      CloseHandle(h);
      return true;
   }
   else {
      // try to open the file with write permissions
      int rc = open(path->getBuffer(), O_WRONLY);
      if (rc == -1)
         return false;
      close(rc);
      return true;
   }
#error no is_writable() implementation for this platform
#endif
}
TypedHashDecl* init_hashdecl_StatInfo(QoreNamespace& ns) {
    TypedHashDecl* hd = new TypedHashDecl("StatInfo");
    hd->addMember("atime", dateTypeInfo, QoreValue());
    hd->addMember("blksize", bigIntTypeInfo, QoreValue());
    hd->addMember("blocks", bigIntTypeInfo, QoreValue());
    hd->addMember("ctime", dateTypeInfo, QoreValue());
    hd->addMember("dev", bigIntTypeInfo, QoreValue());
    hd->addMember("gid", bigIntTypeInfo, QoreValue());
    hd->addMember("inode", bigIntTypeInfo, QoreValue());
    hd->addMember("mode", bigIntTypeInfo, QoreValue());
    hd->addMember("mtime", dateTypeInfo, QoreValue());
    hd->addMember("nlink", bigIntTypeInfo, QoreValue());
    hd->addMember("perm", stringTypeInfo, QoreValue());
    hd->addMember("rdev", bigIntTypeInfo, QoreValue());
    hd->addMember("size", bigIntTypeInfo, QoreValue());
    hd->addMember("type", stringTypeInfo, QoreValue());
    hd->addMember("uid", bigIntTypeInfo, QoreValue());
    ns.addSystemHashDecl(hd);
    return hd;
}

TypedHashDecl* init_hashdecl_FilesystemInfo(QoreNamespace& ns) {
    TypedHashDecl* hd = new TypedHashDecl("FilesystemInfo");
    hd->addMember("bavail", bigIntTypeInfo, QoreValue());
    hd->addMember("bfree", bigIntTypeInfo, QoreValue());
    hd->addMember("blocks", bigIntTypeInfo, QoreValue());
    hd->addMember("bsize", bigIntTypeInfo, QoreValue());
    hd->addMember("favail", bigIntTypeInfo, QoreValue());
    hd->addMember("ffree", bigIntTypeInfo, QoreValue());
    hd->addMember("files", bigIntTypeInfo, QoreValue());
    hd->addMember("frsize", bigIntTypeInfo, QoreValue());
    hd->addMember("fsid", bigIntTypeInfo, QoreValue());
    hd->addMember("namemax", bigIntTypeInfo, QoreValue());
    ns.addSystemHashDecl(hd);
    return hd;
}

# 189 "ql_file.qpp"
# 228 "ql_file.qpp"
# 243 "ql_file.qpp"
# 265 "ql_file.qpp"
# 280 "ql_file.qpp"
# 295 "ql_file.qpp"
# 310 "ql_file.qpp"
# 335 "ql_file.qpp"
# 357 "ql_file.qpp"
# 372 "ql_file.qpp"
# 389 "ql_file.qpp"
# 406 "ql_file.qpp"
# 438 "ql_file.qpp"
# 468 "ql_file.qpp"
# 495 "ql_file.qpp"
# 519 "ql_file.qpp"
# 525 "ql_file.qpp"
# 554 "ql_file.qpp"
# 560 "ql_file.qpp"
# 584 "ql_file.qpp"
# 590 "ql_file.qpp"
# 618 "ql_file.qpp"
# 624 "ql_file.qpp"
# 652 "ql_file.qpp"
# 658 "ql_file.qpp"
# 675 "ql_file.qpp"
# 681 "ql_file.qpp"
# 696 "ql_file.qpp"
# 702 "ql_file.qpp"
# 732 "ql_file.qpp"
# 749 "ql_file.qpp"
# 765 "ql_file.qpp"
# 783 "ql_file.qpp"
# 798 "ql_file.qpp"
# 815 "ql_file.qpp"
# 838 "ql_file.qpp"
# 864 "ql_file.qpp"
# 890 "ql_file.qpp"
# 919 "ql_file.qpp"
# 957 "ql_file.qpp"
# 977 "ql_file.qpp"
# 997 "ql_file.qpp"
# 1017 "ql_file.qpp"
// bool absolute_path(string path) {}
static QoreValue f_absolute_path_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 973 "ql_file.qpp"
   return q_absolute_path(path->getBuffer());
}

// bool absolute_path_unix(string path) {}
static QoreValue f_absolute_path_unix_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 993 "ql_file.qpp"
   return q_absolute_path_unix(path->getBuffer());
}

// bool absolute_path_windows(string path) {}
static QoreValue f_absolute_path_windows_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 1013 "ql_file.qpp"
   return q_absolute_path_windows(path->getBuffer());
}

// int chdir(string path) {}
static QoreValue f_chdir_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 779 "ql_file.qpp"
   return chdir(path->getBuffer());
}

// int chmod(string path, softint mode) {}
static QoreValue f_chmod_Vsvi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
    int64 mode = HARD_QORE_VALUE_INT(args, 1);
# 761 "ql_file.qpp"
   return chmod(path->getBuffer(), (int)mode);
}

// int chown(string path, softint owner = -1, softint group = -1) {}
static QoreValue f_chown_Vsvivi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
    int64 owner = HARD_QORE_VALUE_INT(args, 1);
    int64 group = HARD_QORE_VALUE_INT(args, 2);
# 856 "ql_file.qpp"
#ifdef HAVE_CHOWN
   return chown(path->getBuffer(), (uid_t)owner, (uid_t)group);
#else
   return missing_function_error("chown", "UNIX_FILEMGT", xsink);
#endif
    return QoreValue();
}

// *string getcwd() {}
static QoreValue f_getcwd(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 793 "ql_file.qpp"
   SimpleRefHolder<QoreStringNode> rv(new QoreStringNode);
   return q_getcwd(**rv) ? 0 : rv.release();
}

// string getcwd2() {}
static QoreValue f_getcwd2(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 807 "ql_file.qpp"
   SimpleRefHolder<QoreStringNode> rv(new QoreStringNode);
   if (q_getcwd(**rv))
      return xsink->raiseErrnoException("GETCWD2-ERROR", errno, "getcwd() failed");

   return rv.release();
}

// *list glob(string glob_str) {}
static QoreValue f_glob_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* glob_str = HARD_QORE_VALUE_STRING(args, 0);
# 637 "ql_file.qpp"
   glob_t globbuf;
   if (glob(glob_str->getBuffer(), 0, 0, &globbuf)) {
      globfree(&globbuf);
      return QoreValue();
   }

   QoreListNode *l = new QoreListNode;
   for (unsigned i = 0; i < globbuf.gl_pathc; ++i)
      l->push(new QoreStringNode(globbuf.gl_pathv[i]));

   globfree(&globbuf);
   return l;
}

// nothing glob() {}
static QoreValue f_glob(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 655 "ql_file.qpp"

    return QoreValue();
}

// *hash<StatInfo> hlstat(string path) {}
static QoreValue f_hlstat_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 606 "ql_file.qpp"
   struct stat sbuf;
#ifdef HAVE_LSTAT
   if (lstat(path->getBuffer(), &sbuf))
#else
   if (stat(path->getBuffer(), &sbuf))
#endif
      return QoreValue();

   return stat_to_hash(sbuf);
}

// nothing hlstat() {}
static QoreValue f_hlstat(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 621 "ql_file.qpp"

    return QoreValue();
}

// *hash<StatInfo> hstat(string path) {}
static QoreValue f_hstat_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 576 "ql_file.qpp"
   struct stat sbuf;
   if (stat(path->getBuffer(), &sbuf))
      return QoreValue();

   return stat_to_hash(sbuf);
}

// nothing hstat() {}
static QoreValue f_hstat(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 587 "ql_file.qpp"

    return QoreValue();
}

// bool is_bdev(string path) {}
static QoreValue f_is_bdev_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 306 "ql_file.qpp"
   return check_stat(S_IFBLK, *path);
}

// bool is_cdev(string path) {}
static QoreValue f_is_cdev_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 291 "ql_file.qpp"
   return check_stat(S_IFCHR, *path);
}

// bool is_dev(string path) {}
static QoreValue f_is_dev_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 321 "ql_file.qpp"
   struct stat sbuf;
   int rc;

#ifdef HAVE_LSTAT
   if ((rc = lstat(path->getBuffer(), &sbuf)))
#else
   if ((rc = stat(path->getBuffer(), &sbuf)))
#endif
      return false;

   return ((sbuf.st_mode & S_IFMT) == S_IFCHR) || ((sbuf.st_mode & S_IFMT) == S_IFBLK);
}

// bool is_dir(string path) {}
static QoreValue f_is_dir_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 239 "ql_file.qpp"
   return check_stat(S_IFDIR, *path);
}

// bool is_executable(string path) {}
static QoreValue f_is_executable_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 419 "ql_file.qpp"
#ifdef HAVE_PWD_H
   struct stat sbuf;
   int rc;

   if ((rc = stat(path->getBuffer(), &sbuf)))
      return false;

   if (sbuf.st_mode & S_IXOTH
       || (geteuid() == sbuf.st_uid && (sbuf.st_mode & S_IXUSR))
       || (getegid() == sbuf.st_gid && (sbuf.st_mode & S_IXGRP)))
      return true;

   return false;
#else
   return missing_function_error("is_executable", xsink);
#endif
    return QoreValue();
}

// bool is_file(string path) {}
static QoreValue f_is_file_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 224 "ql_file.qpp"
   return check_stat(S_IFREG, *path);
}

// bool is_link(string path) {}
static QoreValue f_is_link_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 348 "ql_file.qpp"
#ifdef HAVE_LSTAT
   return check_lstat(S_IFLNK, *path);
#else
   missing_function_error("is_link", "UNIX_FILEMGT", xsink);
   return false;
#endif
    return QoreValue();
}

// bool is_pipe(string path) {}
static QoreValue f_is_pipe_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 276 "ql_file.qpp"
   return check_stat(S_IFIFO, *path);
}

// bool is_readable(string path) {}
static QoreValue f_is_readable_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 368 "ql_file.qpp"
   return q_path_is_readable(path->getBuffer());
}

// bool is_socket(string path) {}
static QoreValue f_is_socket_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 256 "ql_file.qpp"
#ifdef S_IFSOCK
   return check_stat(S_IFSOCK, *path);
#else
   missing_function_error("is_socket", "UNIX_FILEMGT", xsink);
   return false;
#endif
    return QoreValue();
}

// bool is_writable(string path) {}
static QoreValue f_is_writable_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 385 "ql_file.qpp"
   return is_writable_intern(path);
}

// bool is_writeable(string path) {}
static QoreValue f_is_writeable_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 402 "ql_file.qpp"
   return is_writable_intern(path);
}

// int lchown(string path, softint uid = -1, softint gid = -1) {}
static QoreValue f_lchown_Vsvivi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
    int64 uid = HARD_QORE_VALUE_INT(args, 1);
    int64 gid = HARD_QORE_VALUE_INT(args, 2);
# 882 "ql_file.qpp"
#ifdef HAVE_LCHOWN
   return lchown(path->getBuffer(), (uid_t)uid, (uid_t)gid);
#else
   return missing_function_error("lchown", "UNIX_FILEMGT", xsink);
#endif
    return QoreValue();
}

// *list lstat(string path) {}
static QoreValue f_lstat_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 542 "ql_file.qpp"
   struct stat sbuf;
#ifdef HAVE_LSTAT
   if (lstat(path->getBuffer(), &sbuf))
#else
   if (stat(path->getBuffer(), &sbuf))
#endif
      return QoreValue();

   return stat_to_list(sbuf);
}

// nothing lstat() {}
static QoreValue f_lstat(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 557 "ql_file.qpp"

    return QoreValue();
}

// int mkdir(string path, softint mode = 0777, bool parents = False) {}
static QoreValue f_mkdir_VsviVb(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
    int64 mode = HARD_QORE_VALUE_INT(args, 1);
    bool parents = HARD_QORE_VALUE_BOOL(args, 2);
# 719 "ql_file.qpp"
    if (parents) {
        QoreDir dir(xsink);
        dir.chdir(path->getBuffer(), xsink);
        return dir.create((int) mode, xsink);
    }
#ifdef _Q_WINDOWS
   return _mkdir(path->getBuffer());
#else
   return mkdir(path->getBuffer(), mode);
#endif
    return QoreValue();
}

// int mkfifo(string path, softint mode = 0600) {}
static QoreValue f_mkfifo_Vsvi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
    int64 mode = HARD_QORE_VALUE_INT(args, 1);
# 830 "ql_file.qpp"
#ifdef HAVE_MKFIFO
   return mkfifo(path->getBuffer(), mode);
#else
   return missing_function_error("mkfifo", "UNIX_FILEMGT", xsink);
#endif
    return QoreValue();
}

// string readlink(string path) {}
static QoreValue f_readlink_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 905 "ql_file.qpp"
#ifdef HAVE_READLINK
   char buf[QORE_PATH_MAX + 1];
   qore_offset_t len = readlink(path->getBuffer(), buf, QORE_PATH_MAX);
   if (len < 0)
      return xsink->raiseErrnoException("READLINK-ERROR", errno, "readlink('%s') failed", path->getBuffer());
   assert(len <= QORE_PATH_MAX);
   buf[len] = '\0';
   return new QoreStringNode(buf);
#else
   return missing_function_error("readlink", "UNIX_FILEMGT", xsink);
#endif
    return QoreValue();
}

// string realpath(string path) {}
static QoreValue f_realpath_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 1031 "ql_file.qpp"
   SimpleRefHolder<QoreStringNode> rv(new QoreStringNode(path->getEncoding()));
   return q_realpath(*path, **rv, xsink) ? 0 : rv.release();
}

// nothing rename(string old_path, string new_path) {}
static QoreValue f_rename_VsVs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* old_path = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* new_path = HARD_QORE_VALUE_STRING(args, 1);
# 453 "ql_file.qpp"
   if (old_path->empty())
      return xsink->raiseException("RENAME-ERROR", "empty path to current file name given as first argument");

   // new file name
   if (new_path->empty())
      return xsink->raiseException("RENAME-ERROR", "empty new file path given as second argument");

   int rc = rename(old_path->getBuffer(), new_path->getBuffer());
   if (rc)
      xsink->raiseErrnoException("RENAME-ERROR", errno, "rename('%s', '%s') failed", old_path->getBuffer(), new_path->getBuffer());

   return QoreValue();
}

// int rmdir(string path) {}
static QoreValue f_rmdir_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 745 "ql_file.qpp"
   return rmdir(path->getBuffer());
}

// *list stat(string path) {}
static QoreValue f_stat_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 511 "ql_file.qpp"
   struct stat sbuf;
   if (stat(path->getBuffer(), &sbuf))
      return QoreValue();

   return stat_to_list(sbuf);
}

// nothing stat() {}
static QoreValue f_stat(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 522 "ql_file.qpp"

    return QoreValue();
}

// *hash<FilesystemInfo> statvfs(string path) {}
static QoreValue f_statvfs_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 483 "ql_file.qpp"
#ifdef Q_HAVE_STATVFS
   struct statvfs vfs;
   if (statvfs(path->getBuffer(), &vfs))
      return QoreValue();

   return statvfs_to_hash(vfs);
#else
   return missing_function_error("statvfs", xsink);
#endif
    return QoreValue();
}

// nothing symlink(string old_path, string new_path) {}
static QoreValue f_symlink_VsVs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* old_path = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* new_path = HARD_QORE_VALUE_STRING(args, 1);
# 938 "ql_file.qpp"
#ifdef HAVE_SYMLINK
   if (old_path->empty())
      return xsink->raiseException("SYMLINK-ERROR", "empty path to current file name given as first argument");

   // new file name
   if (new_path->empty())
      return xsink->raiseException("SYMLINK-ERROR", "empty new file path given as second argument");

   int rc = symlink(old_path->getBuffer(), new_path->getBuffer());
   if (rc)
      xsink->raiseErrnoException("SYMLINK-ERROR", errno, "symlink('%s', '%s') failed", old_path->getBuffer(), new_path->getBuffer());

   return QoreValue();
#else
   return missing_function_error("symlink", "UNIX_FILEMGT", xsink);
#endif
    return QoreValue();
}

// int umask(softint mask) {}
static QoreValue f_umask_vi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 mask = HARD_QORE_VALUE_INT(args, 0);
# 692 "ql_file.qpp"
   return umask((int)mask);
}

// nothing umask() {}
static QoreValue f_umask(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 699 "ql_file.qpp"

    return QoreValue();
}

// int unlink(string path) {}
static QoreValue f_unlink_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 671 "ql_file.qpp"
   return unlink(path->getBuffer());
}

// nothing unlink() {}
static QoreValue f_unlink(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 678 "ql_file.qpp"

    return QoreValue();
}


DLLLOCAL void init_file_functions(QoreNamespace& ns) {
    // bool absolute_path(string path) {}
    ns.addBuiltinVariant("absolute_path", (q_func_n_t)f_absolute_path_Vs, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo, 1, stringTypeInfo, NULL, "path");
    // bool absolute_path_unix(string path) {}
    ns.addBuiltinVariant("absolute_path_unix", (q_func_n_t)f_absolute_path_unix_Vs, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo, 1, stringTypeInfo, NULL, "path");
    // bool absolute_path_windows(string path) {}
    ns.addBuiltinVariant("absolute_path_windows", (q_func_n_t)f_absolute_path_windows_Vs, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo, 1, stringTypeInfo, NULL, "path");
    // int chdir(string path) {}
    ns.addBuiltinVariant("chdir", (q_func_n_t)f_chdir_Vs, QC_NO_FLAGS, QDOM_PROCESS|QDOM_FILESYSTEM, bigIntTypeInfo, 1, stringTypeInfo, NULL, "path");
    // int chmod(string path, softint mode) {}
    ns.addBuiltinVariant("chmod", (q_func_n_t)f_chmod_Vsvi, QC_NO_FLAGS, QDOM_FILESYSTEM, bigIntTypeInfo, 2, stringTypeInfo, NULL, "path", softBigIntTypeInfo, NULL, "mode");
    // int chown(string path, softint owner = -1, softint group = -1) {}
    ns.addBuiltinVariant("chown", (q_func_n_t)f_chown_Vsvivi, QC_NO_FLAGS, QDOM_FILESYSTEM, bigIntTypeInfo, 3, stringTypeInfo, NULL, "path", softBigIntTypeInfo, new QoreBigIntNode(-1), "owner", softBigIntTypeInfo, new QoreBigIntNode(-1), "group");
    // *string getcwd() {}
    ns.addBuiltinVariant("getcwd", (q_func_n_t)f_getcwd, QC_CONSTANT, QDOM_FILESYSTEM|QDOM_EXTERNAL_INFO, stringOrNothingTypeInfo);
    // string getcwd2() {}
    ns.addBuiltinVariant("getcwd2", (q_func_n_t)f_getcwd2, QC_RET_VALUE_ONLY, QDOM_FILESYSTEM|QDOM_EXTERNAL_INFO, stringTypeInfo);
    // *list glob(string glob_str) {}
    ns.addBuiltinVariant("glob", (q_func_n_t)f_glob_Vs, QC_NO_FLAGS, QDOM_FILESYSTEM, listOrNothingTypeInfo, 1, stringTypeInfo, NULL, "glob_str");
    // nothing glob() {}
    ns.addBuiltinVariant("glob", (q_func_n_t)f_glob, QC_RUNTIME_NOOP, QDOM_FILESYSTEM, nothingTypeInfo);
    // *hash<StatInfo> hlstat(string path) {}
    ns.addBuiltinVariant("hlstat", (q_func_n_t)f_hlstat_Vs, QC_NO_FLAGS, QDOM_FILESYSTEM, hashdeclStatInfo->getTypeInfo(true), 1, stringTypeInfo, NULL, "path");
    // nothing hlstat() {}
    ns.addBuiltinVariant("hlstat", (q_func_n_t)f_hlstat, QC_RUNTIME_NOOP, QDOM_FILESYSTEM, nothingTypeInfo);
    // *hash<StatInfo> hstat(string path) {}
    ns.addBuiltinVariant("hstat", (q_func_n_t)f_hstat_Vs, QC_NO_FLAGS, QDOM_FILESYSTEM, hashdeclStatInfo->getTypeInfo(true), 1, stringTypeInfo, NULL, "path");
    // nothing hstat() {}
    ns.addBuiltinVariant("hstat", (q_func_n_t)f_hstat, QC_RUNTIME_NOOP, QDOM_FILESYSTEM, nothingTypeInfo);
    // bool is_bdev(string path) {}
    ns.addBuiltinVariant("is_bdev", (q_func_n_t)f_is_bdev_Vs, QC_CONSTANT, QDOM_FILESYSTEM, boolTypeInfo, 1, stringTypeInfo, NULL, "path");
    // bool is_cdev(string path) {}
    ns.addBuiltinVariant("is_cdev", (q_func_n_t)f_is_cdev_Vs, QC_CONSTANT, QDOM_FILESYSTEM, boolTypeInfo, 1, stringTypeInfo, NULL, "path");
    // bool is_dev(string path) {}
    ns.addBuiltinVariant("is_dev", (q_func_n_t)f_is_dev_Vs, QC_CONSTANT, QDOM_FILESYSTEM, boolTypeInfo, 1, stringTypeInfo, NULL, "path");
    // bool is_dir(string path) {}
    ns.addBuiltinVariant("is_dir", (q_func_n_t)f_is_dir_Vs, QC_CONSTANT, QDOM_FILESYSTEM, boolTypeInfo, 1, stringTypeInfo, NULL, "path");
    // bool is_executable(string path) {}
    ns.addBuiltinVariant("is_executable", (q_func_n_t)f_is_executable_Vs, QC_CONSTANT, QDOM_FILESYSTEM, boolTypeInfo, 1, stringTypeInfo, NULL, "path");
    // bool is_file(string path) {}
    ns.addBuiltinVariant("is_file", (q_func_n_t)f_is_file_Vs, QC_CONSTANT, QDOM_FILESYSTEM, boolTypeInfo, 1, stringTypeInfo, NULL, "path");
    // bool is_link(string path) {}
    ns.addBuiltinVariant("is_link", (q_func_n_t)f_is_link_Vs, QC_CONSTANT, QDOM_FILESYSTEM, boolTypeInfo, 1, stringTypeInfo, NULL, "path");
    // bool is_pipe(string path) {}
    ns.addBuiltinVariant("is_pipe", (q_func_n_t)f_is_pipe_Vs, QC_CONSTANT, QDOM_FILESYSTEM, boolTypeInfo, 1, stringTypeInfo, NULL, "path");
    // bool is_readable(string path) {}
    ns.addBuiltinVariant("is_readable", (q_func_n_t)f_is_readable_Vs, QC_CONSTANT, QDOM_FILESYSTEM, boolTypeInfo, 1, stringTypeInfo, NULL, "path");
    // bool is_socket(string path) {}
    ns.addBuiltinVariant("is_socket", (q_func_n_t)f_is_socket_Vs, QC_CONSTANT, QDOM_FILESYSTEM, boolTypeInfo, 1, stringTypeInfo, NULL, "path");
    // bool is_writable(string path) {}
    ns.addBuiltinVariant("is_writable", (q_func_n_t)f_is_writable_Vs, QC_CONSTANT, QDOM_FILESYSTEM, boolTypeInfo, 1, stringTypeInfo, NULL, "path");
    // bool is_writeable(string path) {}
    ns.addBuiltinVariant("is_writeable", (q_func_n_t)f_is_writeable_Vs, QC_CONSTANT, QDOM_FILESYSTEM, boolTypeInfo, 1, stringTypeInfo, NULL, "path");
    // int lchown(string path, softint uid = -1, softint gid = -1) {}
    ns.addBuiltinVariant("lchown", (q_func_n_t)f_lchown_Vsvivi, QC_NO_FLAGS, QDOM_FILESYSTEM, bigIntTypeInfo, 3, stringTypeInfo, NULL, "path", softBigIntTypeInfo, new QoreBigIntNode(-1), "uid", softBigIntTypeInfo, new QoreBigIntNode(-1), "gid");
    // *list lstat(string path) {}
    ns.addBuiltinVariant("lstat", (q_func_n_t)f_lstat_Vs, QC_NO_FLAGS, QDOM_FILESYSTEM, listOrNothingTypeInfo, 1, stringTypeInfo, NULL, "path");
    // nothing lstat() {}
    ns.addBuiltinVariant("lstat", (q_func_n_t)f_lstat, QC_RUNTIME_NOOP, QDOM_FILESYSTEM, nothingTypeInfo);
    // int mkdir(string path, softint mode = 0777, bool parents = False) {}
    ns.addBuiltinVariant("mkdir", (q_func_n_t)f_mkdir_VsviVb, QC_NO_FLAGS, QDOM_FILESYSTEM, bigIntTypeInfo, 3, stringTypeInfo, NULL, "path", softBigIntTypeInfo, new QoreBigIntNode(0777), "mode", boolTypeInfo, &False, "parents");
    // int mkfifo(string path, softint mode = 0600) {}
    ns.addBuiltinVariant("mkfifo", (q_func_n_t)f_mkfifo_Vsvi, QC_NO_FLAGS, QDOM_FILESYSTEM, bigIntTypeInfo, 2, stringTypeInfo, NULL, "path", softBigIntTypeInfo, new QoreBigIntNode(0600), "mode");
    // string readlink(string path) {}
    ns.addBuiltinVariant("readlink", (q_func_n_t)f_readlink_Vs, QC_NO_FLAGS, QDOM_FILESYSTEM, stringTypeInfo, 1, stringTypeInfo, NULL, "path");
    // string realpath(string path) {}
    ns.addBuiltinVariant("realpath", (q_func_n_t)f_realpath_Vs, QC_NO_FLAGS, QDOM_FILESYSTEM, stringTypeInfo, 1, stringTypeInfo, NULL, "path");
    // nothing rename(string old_path, string new_path) {}
    ns.addBuiltinVariant("rename", (q_func_n_t)f_rename_VsVs, QC_NO_FLAGS, QDOM_FILESYSTEM, nothingTypeInfo, 2, stringTypeInfo, NULL, "old_path", stringTypeInfo, NULL, "new_path");
    // int rmdir(string path) {}
    ns.addBuiltinVariant("rmdir", (q_func_n_t)f_rmdir_Vs, QC_NO_FLAGS, QDOM_FILESYSTEM, bigIntTypeInfo, 1, stringTypeInfo, NULL, "path");
    // *list stat(string path) {}
    ns.addBuiltinVariant("stat", (q_func_n_t)f_stat_Vs, QC_NO_FLAGS, QDOM_FILESYSTEM, listOrNothingTypeInfo, 1, stringTypeInfo, NULL, "path");
    // nothing stat() {}
    ns.addBuiltinVariant("stat", (q_func_n_t)f_stat, QC_RUNTIME_NOOP, QDOM_FILESYSTEM, nothingTypeInfo);
    // *hash<FilesystemInfo> statvfs(string path) {}
    ns.addBuiltinVariant("statvfs", (q_func_n_t)f_statvfs_Vs, QC_NO_FLAGS, QDOM_FILESYSTEM, hashdeclFilesystemInfo->getTypeInfo(true), 1, stringTypeInfo, NULL, "path");
    // nothing symlink(string old_path, string new_path) {}
    ns.addBuiltinVariant("symlink", (q_func_n_t)f_symlink_VsVs, QC_NO_FLAGS, QDOM_FILESYSTEM, nothingTypeInfo, 2, stringTypeInfo, NULL, "old_path", stringTypeInfo, NULL, "new_path");
    // int umask(softint mask) {}
    ns.addBuiltinVariant("umask", (q_func_n_t)f_umask_vi, QC_NO_FLAGS, QDOM_PROCESS, bigIntTypeInfo, 1, softBigIntTypeInfo, NULL, "mask");
    // nothing umask() {}
    ns.addBuiltinVariant("umask", (q_func_n_t)f_umask, QC_RUNTIME_NOOP, QDOM_PROCESS, nothingTypeInfo);
    // int unlink(string path) {}
    ns.addBuiltinVariant("unlink", (q_func_n_t)f_unlink_Vs, QC_NO_FLAGS, QDOM_FILESYSTEM, bigIntTypeInfo, 1, stringTypeInfo, NULL, "path");
    // nothing unlink() {}
    ns.addBuiltinVariant("unlink", (q_func_n_t)f_unlink, QC_RUNTIME_NOOP, QDOM_FILESYSTEM, nothingTypeInfo);

}
