/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  ql_lib.qpp

  Qore Programming Language

  Copyright (C) 2003 - 2017 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/ql_lib.h"
#include "qore/intern/ExecArgList.h"
#include "qore/intern/QoreSignal.h"
#include <qore/minitest.hpp>

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/param.h>
#include <unistd.h>
#ifdef HAVE_GRP_H
#include <grp.h>
#endif

#ifdef HAVE_SYS_WAIT_H
#include <sys/wait.h>
#endif

#include <time.h>

#ifdef HAVE_GLOB_H
#include <glob.h>
#else
#include "qore/intern/glob.h"
#endif

#ifndef SIGHUP
#define SIGHUP 0
#endif

#define MAX_GIDS 100

extern bool threads_initialized;

#ifdef DEBUG_TESTS
// Unsorted unit tests are put here
#  include "tests/ReferenceHolder_tests.cpp"
#endif

AbstractQoreNode* missing_function_error(const char* func, ExceptionSink* xsink) {
   QoreString have(func);
   have.toupr();
   return xsink->raiseException("MISSING-FEATURE-ERROR", "this system does not implement %s(); for maximum portability use the constant Option::HAVE_%s to check if this function is implemented before calling", func, have.getBuffer());
}

AbstractQoreNode* missing_function_error(const char* func, const char* opt, ExceptionSink* xsink) {
   return xsink->raiseException("MISSING-FEATURE-ERROR", "this system does not implement %s(); for maximum portability use the constant Option::HAVE_%s to check if this function is implemented before calling", func, opt);
}

#ifdef DEBUG_X
static AbstractQoreNode* runQoreTests(const QoreListNode* args, ExceptionSink* xsink) {
   minitest::result res = minitest::execute_all_tests();
   if (res.all_tests_succeeded) {
      printf("Qore runtime: %d tests succeeded\n", res.sucessful_tests_count);
      return 0;
   }

   return xsink->raiseException("A Qore test failed", "Qore test in file %s, line %d threw an exception.",
                                res.failed_test_file, res.failed_test_line);
}

static AbstractQoreNode* runRecentQoreTests(const QoreListNode* args, ExceptionSink* xsink) {
   minitest::result res = minitest::test_last_changed_files(3); // 3 last modified files
   if (res.all_tests_succeeded) {
      printf("Qore runtime: %d recent tests succeeded\n", res.sucessful_tests_count);
      return 0;
   }

   return xsink->raiseException("A Qore test failed", "Qore test in file %s, line %d threw an exception.",
                                res.failed_test_file, res.failed_test_line);
}
namespace {
TEST() {
  // just an example of empty test
}
}
#endif
TypedHashDecl* init_hashdecl_ExceptionInfo(QoreNamespace& ns) {
    TypedHashDecl* hd = new TypedHashDecl("ExceptionInfo");
    hd->addMember("arg", autoTypeInfo, QoreValue());
    hd->addMember("callstack", qore_get_complex_list_type(hashdeclCallStackInfo->getTypeInfo(false)), QoreValue());
    hd->addMember("desc", autoTypeInfo, QoreValue());
    hd->addMember("endline", bigIntTypeInfo, QoreValue());
    hd->addMember("err", autoTypeInfo, QoreValue());
    hd->addMember("file", stringTypeInfo, QoreValue());
    hd->addMember("line", bigIntTypeInfo, QoreValue());
    hd->addMember("next", hd->getTypeInfo(false), QoreValue());
    hd->addMember("offset", bigIntTypeInfo, QoreValue());
    hd->addMember("source", stringTypeInfo, QoreValue());
    hd->addMember("type", stringTypeInfo, QoreValue());
    ns.addSystemHashDecl(hd);
    return hd;
}

# 153 "ql_lib.qpp"
# 189 "ql_lib.qpp"
# 204 "ql_lib.qpp"
# 219 "ql_lib.qpp"
# 266 "ql_lib.qpp"
# 272 "ql_lib.qpp"
# 291 "ql_lib.qpp"
# 310 "ql_lib.qpp"
# 329 "ql_lib.qpp"
# 348 "ql_lib.qpp"
# 378 "ql_lib.qpp"
# 417 "ql_lib.qpp"
# 436 "ql_lib.qpp"
# 442 "ql_lib.qpp"
# 463 "ql_lib.qpp"
# 481 "ql_lib.qpp"
# 487 "ql_lib.qpp"
# 499 "ql_lib.qpp"
# 518 "ql_lib.qpp"
# 576 "ql_lib.qpp"
# 602 "ql_lib.qpp"
# 608 "ql_lib.qpp"
# 631 "ql_lib.qpp"
# 654 "ql_lib.qpp"
# 660 "ql_lib.qpp"
# 676 "ql_lib.qpp"
# 691 "ql_lib.qpp"
# 706 "ql_lib.qpp"
# 721 "ql_lib.qpp"
# 739 "ql_lib.qpp"
# 745 "ql_lib.qpp"
# 763 "ql_lib.qpp"
# 769 "ql_lib.qpp"
# 791 "ql_lib.qpp"
# 813 "ql_lib.qpp"
# 835 "ql_lib.qpp"
# 857 "ql_lib.qpp"
# 876 "ql_lib.qpp"
# 894 "ql_lib.qpp"
# 900 "ql_lib.qpp"
# 923 "ql_lib.qpp"
# 929 "ql_lib.qpp"
# 948 "ql_lib.qpp"
# 954 "ql_lib.qpp"
# 977 "ql_lib.qpp"
# 983 "ql_lib.qpp"
# 1004 "ql_lib.qpp"
// nothing abort() {}
static QoreValue f_abort(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 199 "ql_lib.qpp"
   threads_initialized = false;
   abort();
    return QoreValue();
}

// string basename(string path) {}
static QoreValue f_basename_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 733 "ql_lib.qpp"
   char* p = q_basename(path->getBuffer());
   int len = strlen(p);
   return new QoreStringNode(p, len, len + 1, path->getEncoding());
}

// nothing basename() {}
static QoreValue f_basename(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 742 "ql_lib.qpp"

    return QoreValue();
}

// nothing close_all_fd(*softbool strd) {}
static QoreValue f_close_all_fd_nb(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    bool strd = HARD_QORE_VALUE_BOOL(args, 0);
# 1017 "ql_lib.qpp"
#if defined(HAVE_UNISTD_H) && defined(_SC_OPEN_MAX)
   int start = strd ? 0 : 2;
   // get max file descriptor
   int max = (int)sysconf(_SC_OPEN_MAX);
   printd(5, "close_all_fd() start: %d max: %d\n", start, max);

   for (;start < max; ++start)
      close(start);
#else
   missing_function_error("close_all_fd", xsink);
#endif // HAVE_UNISTD
    return QoreValue();
}

// string dirname(string path) {}
static QoreValue f_dirname_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 757 "ql_lib.qpp"
   char* p = q_dirname(path->getBuffer());
   int len = strlen(p);
   return new QoreStringNode(p, len, len + 1, path->getEncoding());
}

// nothing dirname() {}
static QoreValue f_dirname(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 766 "ql_lib.qpp"

    return QoreValue();
}

// int errno() {}
static QoreValue f_errno(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 687 "ql_lib.qpp"
   return errno;
}

// nothing exec(string command) {}
static QoreValue f_exec_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* command = HARD_QORE_VALUE_STRING(args, 0);
# 212 "ql_lib.qpp"
   ExecArgList exec_args(command->getBuffer());
   execvp(exec_args.getFile(), exec_args.getArgs());

   xsink->raiseErrnoException("EXEC-ERROR", errno, "execvp() failed in child process for target '%s'", exec_args.getFile());
    return QoreValue();
}

// nothing exit(softint rc = 0) {}
static QoreValue f_exit_vi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 rc = HARD_QORE_VALUE_INT(args, 0);
# 185 "ql_lib.qpp"
   qore_exit_process(rc);
    return QoreValue();
}

// int fork() {}
static QoreValue f_fork(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 543 "ql_lib.qpp"
#ifdef HAVE_FORK
#ifdef HAVE_SIGNAL_HANDLING
   int QORE_THREAD_LIMIT = 1 + (QSM.thread_running ? 1 : 0);
#else
#define QORE_THREAD_LIMIT 1
#endif
   if (thread_list.getNumThreads() > (unsigned)(QORE_THREAD_LIMIT))
      return xsink->raiseException("ILLEGAL-FORK", "cannot fork() when other threads are running");

#ifdef HAVE_SIGNAL_HANDLING
   // we may not fork from within a signal handler
   if (QSM.thread_running && gettid() == QSM.gettid())
      return xsink->raiseException("ILLEGAL-FORK", "cannot fork() within a signal handler");

   // stop signal handling thread and make sure it can't be restarted until fork is done
   QSM.preFork();
#endif

   //printd(5, "stopped signal thread, about to fork pid %d\n", getpid()); fflush(stdout);
   int pid = fork();

#ifdef HAVE_SIGNAL_HANDLING
   // release signal handler lock
   QSM.postFork(!pid, xsink);
#endif

   return pid;
#else
   return missing_function_error("fork", xsink);
#endif
    return QoreValue();
}

// list getaddrinfo(*string node, *softstring service, softint family = AF_UNSPEC, softint flags = 0) {}
static QoreValue f_getaddrinfo_Nsnsvivi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* node = get_param_value(args, 0).get<const QoreStringNode>();
    const QoreStringNode* service = get_param_value(args, 1).get<const QoreStringNode>();
    int64 family = HARD_QORE_VALUE_INT(args, 2);
    int64 flags = HARD_QORE_VALUE_INT(args, 3);
# 1000 "ql_lib.qpp"
   return q_getaddrinfo_to_list(xsink, node ? node->getBuffer() : 0, service ? service->getBuffer() : 0, (int)family, (int)flags);
}

// int getegid() {}
static QoreValue f_getegid(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 340 "ql_lib.qpp"
#ifdef HAVE_GETEGID
   return getgid();
#else
   return missing_function_error("getegid", "UNIX_USERMGT", xsink);
#endif
    return QoreValue();
}

// int geteuid() {}
static QoreValue f_geteuid(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 302 "ql_lib.qpp"
#ifdef HAVE_GETEUID
   return getuid();
#else
   return missing_function_error("geteuid", "UNIX_USERMGT", xsink);
#endif
    return QoreValue();
}

// int getgid() {}
static QoreValue f_getgid(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 321 "ql_lib.qpp"
#ifdef HAVE_GETGID
   return getgid();
#else
   return missing_function_error("getgid", "UNIX_USERMGT", xsink);
#endif
    return QoreValue();
}

// list getgroups() {}
static QoreValue f_getgroups(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 361 "ql_lib.qpp"
#ifdef HAVE_GETGROUPS
   gid_t l[MAX_GIDS];

   int n = getgroups(MAX_GIDS, l);
   if (n == -1)
      return xsink->raiseErrnoException("GETGROUPS-ERROR", errno, "error calling getgroups()");

   QoreListNode* rv = new QoreListNode;
   for (int i = 0; i < n; ++i)
      rv->push(new QoreBigIntNode(l[i]));
   return rv;
#else
   missing_function_error("getgroups", "UNIX_USERMGT", xsink);
#endif
    return QoreValue();
}

// *string gethostbyaddr(string addr, softint type = AF_INET) {}
static QoreValue f_gethostbyaddr_Vsvi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* addr = HARD_QORE_VALUE_STRING(args, 0);
    int64 type = HARD_QORE_VALUE_INT(args, 1);
# 919 "ql_lib.qpp"
   return q_gethostbyaddr_to_string(xsink, addr->getBuffer(), type);
}

// nothing gethostbyaddr() {}
static QoreValue f_gethostbyaddr(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 926 "ql_lib.qpp"

    return QoreValue();
}

// *hash gethostbyaddr_long(string addr, softint type = AF_INET) {}
static QoreValue f_gethostbyaddr_long_Vsvi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* addr = HARD_QORE_VALUE_STRING(args, 0);
    int64 type = HARD_QORE_VALUE_INT(args, 1);
# 973 "ql_lib.qpp"
   return q_gethostbyaddr_to_hash(xsink, addr->getBuffer(), type);
}

// nothing gethostbyaddr_long() {}
static QoreValue f_gethostbyaddr_long(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 980 "ql_lib.qpp"

    return QoreValue();
}

// *string gethostbyname(string name) {}
static QoreValue f_gethostbyname_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 890 "ql_lib.qpp"
   return q_gethostbyname_to_string(name->getBuffer());
}

// nothing gethostbyname() {}
static QoreValue f_gethostbyname(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 897 "ql_lib.qpp"

    return QoreValue();
}

// *hash gethostbyname_long(string name) {}
static QoreValue f_gethostbyname_long_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 944 "ql_lib.qpp"
   return q_gethostbyname_to_hash(name->getBuffer());
}

// nothing gethostbyname_long() {}
static QoreValue f_gethostbyname_long(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 951 "ql_lib.qpp"

    return QoreValue();
}

// string gethostname() {}
static QoreValue f_gethostname(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 668 "ql_lib.qpp"
   char buf[HOSTNAMEBUFSIZE + 1];

   if (gethostname(buf, HOSTNAMEBUFSIZE))
      return xsink->raiseErrnoException("GETHOSTNAME-ERROR", errno, "gethostname() failed");
   return new QoreStringNode(buf);
}

// int getpid() {}
static QoreValue f_getpid(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 495 "ql_lib.qpp"
   return getpid();
}

// int getppid() {}
static QoreValue f_getppid(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 510 "ql_lib.qpp"
#ifdef HAVE_GETPPID
   return getppid();
#else
   return missing_function_error("getppid", xsink);
#endif
    return QoreValue();
}

// int getuid() {}
static QoreValue f_getuid(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 283 "ql_lib.qpp"
#ifdef HAVE_GETUID
   return getuid();
#else
   return missing_function_error("getuid", "UNIX_USERMGT", xsink);
#endif
    return QoreValue();
}

// int kill(softint pid, softint sig = SIGHUP) {}
static QoreValue f_kill_vivi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 pid = HARD_QORE_VALUE_INT(args, 0);
    int64 sig = HARD_QORE_VALUE_INT(args, 1);
# 594 "ql_lib.qpp"
#ifdef HAVE_KILL
   return kill(pid, sig);
#else
   return missing_function_error("kill", xsink);
#endif
    return QoreValue();
}

// nothing kill() {}
static QoreValue f_kill(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 605 "ql_lib.qpp"

    return QoreValue();
}

// int rand() {}
static QoreValue f_rand(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 622 "ql_lib.qpp"
#ifdef HAVE_RANDOM
   // return a random 64-bit integer by calling random() twice
   return random() | (((int64)random()) << 32);
#else
   return rand() | (((int64)rand()) << 32);
#endif
    return QoreValue();
}

// int setegid(softint gid) {}
static QoreValue f_setegid_vi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 gid = HARD_QORE_VALUE_INT(args, 0);
# 849 "ql_lib.qpp"
#ifdef HAVE_SETEGID
   return setegid((int)gid);
#else
   return missing_function_error("setegid", xsink);
#endif
    return QoreValue();
}

// int seteuid(softint uid) {}
static QoreValue f_seteuid_vi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 uid = HARD_QORE_VALUE_INT(args, 0);
# 805 "ql_lib.qpp"
#ifdef HAVE_SETEUID
   return seteuid((int)uid);
#else
   return missing_function_error("seteuid", xsink);
#endif
    return QoreValue();
}

// int setgid(softint gid) {}
static QoreValue f_setgid_vi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 gid = HARD_QORE_VALUE_INT(args, 0);
# 827 "ql_lib.qpp"
#ifdef HAVE_SETGID
   return setgid((int)gid);
#else
   return missing_function_error("setgid", "UNIX_USERMGT", xsink);
#endif
    return QoreValue();
}

// nothing setgroups(softlist gids) {}
static QoreValue f_setgroups_vl(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* gids = HARD_QORE_VALUE_LIST(args, 0);
# 391 "ql_lib.qpp"
#ifdef HAVE_SETGROUPS
   if (gids->empty())
      return QoreValue();

#ifdef HAVE_LOCAL_VARIADIC_ARRAYS_X
   gid_t l[gids->size()];
#else
   gid_t* l = (gid_t*)malloc(sizeof(gid_t) * gids->size());
   if (!l) {
      xsink->outOfMemory();
      return QoreValue();
   }
   ON_BLOCK_EXIT(free, l);
#endif
   for (qore_size_t i = 0, e = gids->size(); i < e; ++i) {
      const AbstractQoreNode* n = gids->retrieve_entry(i);
      l[i] = n ? n->getAsInt() : 0;
   }
   if (setgroups(gids->size(), l))
      xsink->raiseErrnoException("SETGROUPS-ERROR", errno, "error calling setgroups()");
#else
   missing_function_error("setgroups", "UNIX_USERMGT", xsink);
#endif
    return QoreValue();
}

// int setsid() {}
static QoreValue f_setsid(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 868 "ql_lib.qpp"
#ifdef HAVE_SETSID
   return setsid();
#else
   return missing_function_error("setsid", xsink);
#endif
    return QoreValue();
}

// int setuid(softint uid) {}
static QoreValue f_setuid_vi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 uid = HARD_QORE_VALUE_INT(args, 0);
# 783 "ql_lib.qpp"
#ifdef HAVE_SETUID
   return setuid((int)uid);
#else
   return missing_function_error("setuid", "UNIX_USERMGT", xsink);
#endif
    return QoreValue();
}

// int sleep(softint seconds) {}
static QoreValue f_sleep_vi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 seconds = HARD_QORE_VALUE_INT(args, 0);
# 429 "ql_lib.qpp"
   if (!seconds)
      return 0;

   return qore_usleep(seconds * 1000000ll);
}

// nothing sleep() {}
static QoreValue f_sleep(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 439 "ql_lib.qpp"

    return QoreValue();
}

// nothing srand(softint seed) {}
static QoreValue f_srand_vi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 seed = HARD_QORE_VALUE_INT(args, 0);
# 645 "ql_lib.qpp"
#ifdef HAVE_RANDOM
   srandom((int)seed);
#else
   srand((int)seed);
#endif
   return QoreValue();
}

// nothing srand() {}
static QoreValue f_srand(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 657 "ql_lib.qpp"

    return QoreValue();
}

// string strerror(softint err) {}
static QoreValue f_strerror_vi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 err = HARD_QORE_VALUE_INT(args, 0);
# 702 "ql_lib.qpp"
   return q_strerror(err);
}

// string strerror() {}
static QoreValue f_strerror(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 717 "ql_lib.qpp"
   return q_strerror(errno);
}

// int system(string command) {}
static QoreValue f_system_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* command = HARD_QORE_VALUE_STRING(args, 0);
# 236 "ql_lib.qpp"
#if defined(HAVE_FORK) && defined(HAVE_SIGNAL_HANDLING)
   // on platforms with fork(2) and signal handling, we need to fork, enable all signals, then execvp()
   pid_t pid;
   if (!(pid = fork())) {
      // enable all signals
      sigset_t mask;
      sigemptyset(&mask);
      sigprocmask(SIG_SETMASK, &mask, 0);

      // exec pgm like system(): sh -c "command"
      execl("/bin/sh", "sh", "-c", command->getBuffer(), NULL);
      fprintf(stderr, "execvp() failed in child process for target '/bin/sh' with error code %d: %s\n", errno, strerror(errno));
      _Exit(-1);
      //qore_exit_process(-1);
   }
   if (pid == -1)
      return -1;

   // wait for child to exit
   int status;
   wait(&status);
   return WIFEXITED(status) ? WEXITSTATUS(status) : -1;
#elif defined(HAVE_SYSTEM)
   return system(command->getBuffer());
#else
   return missing_function_error("system", xsink);
#endif
    return QoreValue();
}

// nothing system() {}
static QoreValue f_system(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 269 "ql_lib.qpp"

    return QoreValue();
}

// int usleep(softint usecs) {}
static QoreValue f_usleep_vi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 usecs = HARD_QORE_VALUE_INT(args, 0);
# 456 "ql_lib.qpp"
   if (!usecs)
      return 0;

   return qore_usleep(usecs);
}

// int usleep(date d) {}
static QoreValue f_usleep_Vd(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* d = HARD_QORE_VALUE_DATE(args, 0);
# 477 "ql_lib.qpp"
   return qore_usleep(d->getRelativeMicroseconds());
}

// nothing usleep() {}
static QoreValue f_usleep(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 484 "ql_lib.qpp"

    return QoreValue();
}


DLLLOCAL void init_lib_functions(QoreNamespace& ns) {
    // nothing abort() {}
    ns.addBuiltinVariant("abort", (q_func_n_t)f_abort, QC_NO_FLAGS, QDOM_PROCESS, nothingTypeInfo);
    // string basename(string path) {}
    ns.addBuiltinVariant("basename", (q_func_n_t)f_basename_Vs, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, NULL, "path");
    // nothing basename() {}
    ns.addBuiltinVariant("basename", (q_func_n_t)f_basename, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // nothing close_all_fd(*softbool strd) {}
    ns.addBuiltinVariant("close_all_fd", (q_func_n_t)f_close_all_fd_nb, QC_NO_FLAGS, QDOM_PROCESS, nothingTypeInfo, 1, softBoolOrNothingTypeInfo, NULL, "strd");
    // string dirname(string path) {}
    ns.addBuiltinVariant("dirname", (q_func_n_t)f_dirname_Vs, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, NULL, "path");
    // nothing dirname() {}
    ns.addBuiltinVariant("dirname", (q_func_n_t)f_dirname, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // int errno() {}
    ns.addBuiltinVariant("errno", (q_func_n_t)f_errno, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);
    // nothing exec(string command) {}
    ns.addBuiltinVariant("exec", (q_func_n_t)f_exec_Vs, QC_NO_FLAGS, QDOM_EXTERNAL_PROCESS|QDOM_PROCESS, nothingTypeInfo, 1, stringTypeInfo, NULL, "command");
    // nothing exit(softint rc = 0) {}
    ns.addBuiltinVariant("exit", (q_func_n_t)f_exit_vi, QC_NO_FLAGS, QDOM_PROCESS, nothingTypeInfo, 1, softBigIntTypeInfo, zero(), "rc");
    // int fork() {}
    ns.addBuiltinVariant("fork", (q_func_n_t)f_fork, QC_NO_FLAGS, QDOM_PROCESS, bigIntTypeInfo);
    // list getaddrinfo(*string node, *softstring service, softint family = AF_UNSPEC, softint flags = 0) {}
    ns.addBuiltinVariant("getaddrinfo", (q_func_n_t)f_getaddrinfo_Nsnsvivi, QC_RET_VALUE_ONLY, QDOM_EXTERNAL_INFO, listTypeInfo, 4, stringOrNothingTypeInfo, NULL, "node", softStringOrNothingTypeInfo, NULL, "service", softBigIntTypeInfo, new QoreBigIntNode(AF_UNSPEC), "family", softBigIntTypeInfo, zero(), "flags");
    // int getegid() {}
    ns.addBuiltinVariant("getegid", (q_func_n_t)f_getegid, QC_NO_FLAGS, QDOM_EXTERNAL_INFO, bigIntTypeInfo);
    // int geteuid() {}
    ns.addBuiltinVariant("geteuid", (q_func_n_t)f_geteuid, QC_NO_FLAGS, QDOM_EXTERNAL_INFO, bigIntTypeInfo);
    // int getgid() {}
    ns.addBuiltinVariant("getgid", (q_func_n_t)f_getgid, QC_NO_FLAGS, QDOM_EXTERNAL_INFO, bigIntTypeInfo);
    // list getgroups() {}
    ns.addBuiltinVariant("getgroups", (q_func_n_t)f_getgroups, QC_NO_FLAGS, QDOM_EXTERNAL_INFO, listTypeInfo);
    // *string gethostbyaddr(string addr, softint type = AF_INET) {}
    ns.addBuiltinVariant("gethostbyaddr", (q_func_n_t)f_gethostbyaddr_Vsvi, QC_CONSTANT, QDOM_EXTERNAL_INFO, stringOrNothingTypeInfo, 2, stringTypeInfo, NULL, "addr", softBigIntTypeInfo, new QoreBigIntNode(AF_INET), "type");
    // nothing gethostbyaddr() {}
    ns.addBuiltinVariant("gethostbyaddr", (q_func_n_t)f_gethostbyaddr, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // *hash gethostbyaddr_long(string addr, softint type = AF_INET) {}
    ns.addBuiltinVariant("gethostbyaddr_long", (q_func_n_t)f_gethostbyaddr_long_Vsvi, QC_CONSTANT, QDOM_EXTERNAL_INFO, hashOrNothingTypeInfo, 2, stringTypeInfo, NULL, "addr", softBigIntTypeInfo, new QoreBigIntNode(AF_INET), "type");
    // nothing gethostbyaddr_long() {}
    ns.addBuiltinVariant("gethostbyaddr_long", (q_func_n_t)f_gethostbyaddr_long, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // *string gethostbyname(string name) {}
    ns.addBuiltinVariant("gethostbyname", (q_func_n_t)f_gethostbyname_Vs, QC_CONSTANT, QDOM_EXTERNAL_INFO, stringOrNothingTypeInfo, 1, stringTypeInfo, NULL, "name");
    // nothing gethostbyname() {}
    ns.addBuiltinVariant("gethostbyname", (q_func_n_t)f_gethostbyname, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // *hash gethostbyname_long(string name) {}
    ns.addBuiltinVariant("gethostbyname_long", (q_func_n_t)f_gethostbyname_long_Vs, QC_CONSTANT, QDOM_EXTERNAL_INFO, hashOrNothingTypeInfo, 1, stringTypeInfo, NULL, "name");
    // nothing gethostbyname_long() {}
    ns.addBuiltinVariant("gethostbyname_long", (q_func_n_t)f_gethostbyname_long, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // string gethostname() {}
    ns.addBuiltinVariant("gethostname", (q_func_n_t)f_gethostname, QC_CONSTANT, QDOM_EXTERNAL_INFO, stringTypeInfo);
    // int getpid() {}
    ns.addBuiltinVariant("getpid", (q_func_n_t)f_getpid, QC_RET_VALUE_ONLY, QDOM_EXTERNAL_INFO, bigIntTypeInfo);
    // int getppid() {}
    ns.addBuiltinVariant("getppid", (q_func_n_t)f_getppid, QC_RET_VALUE_ONLY, QDOM_EXTERNAL_INFO, bigIntTypeInfo);
    // int getuid() {}
    ns.addBuiltinVariant("getuid", (q_func_n_t)f_getuid, QC_NO_FLAGS, QDOM_EXTERNAL_INFO, bigIntTypeInfo);
    // int kill(softint pid, softint sig = SIGHUP) {}
    ns.addBuiltinVariant("kill", (q_func_n_t)f_kill_vivi, QC_NO_FLAGS, QDOM_EXTERNAL_PROCESS, bigIntTypeInfo, 2, softBigIntTypeInfo, NULL, "pid", softBigIntTypeInfo, new QoreBigIntNode(SIGHUP), "sig");
    // nothing kill() {}
    ns.addBuiltinVariant("kill", (q_func_n_t)f_kill, QC_RUNTIME_NOOP, QDOM_EXTERNAL_PROCESS, nothingTypeInfo);
    // int rand() {}
    ns.addBuiltinVariant("rand", (q_func_n_t)f_rand, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);
    // int setegid(softint gid) {}
    ns.addBuiltinVariant("setegid", (q_func_n_t)f_setegid_vi, QC_NO_FLAGS, QDOM_PROCESS, bigIntTypeInfo, 1, softBigIntTypeInfo, NULL, "gid");
    // int seteuid(softint uid) {}
    ns.addBuiltinVariant("seteuid", (q_func_n_t)f_seteuid_vi, QC_NO_FLAGS, QDOM_PROCESS, bigIntTypeInfo, 1, softBigIntTypeInfo, NULL, "uid");
    // int setgid(softint gid) {}
    ns.addBuiltinVariant("setgid", (q_func_n_t)f_setgid_vi, QC_NO_FLAGS, QDOM_PROCESS, bigIntTypeInfo, 1, softBigIntTypeInfo, NULL, "gid");
    // nothing setgroups(softlist gids) {}
    ns.addBuiltinVariant("setgroups", (q_func_n_t)f_setgroups_vl, QC_NO_FLAGS, QDOM_PROCESS, nothingTypeInfo, 1, softListTypeInfo, NULL, "gids");
    // int setsid() {}
    ns.addBuiltinVariant("setsid", (q_func_n_t)f_setsid, QC_NO_FLAGS, QDOM_PROCESS, bigIntTypeInfo);
    // int setuid(softint uid) {}
    ns.addBuiltinVariant("setuid", (q_func_n_t)f_setuid_vi, QC_NO_FLAGS, QDOM_PROCESS, bigIntTypeInfo, 1, softBigIntTypeInfo, NULL, "uid");
    // int sleep(softint seconds) {}
    ns.addBuiltinVariant("sleep", (q_func_n_t)f_sleep_vi, QC_NO_FLAGS, QDOM_PROCESS, bigIntTypeInfo, 1, softBigIntTypeInfo, NULL, "seconds");
    // nothing sleep() {}
    ns.addBuiltinVariant("sleep", (q_func_n_t)f_sleep, QC_RUNTIME_NOOP, QDOM_PROCESS, nothingTypeInfo);
    // nothing srand(softint seed) {}
    ns.addBuiltinVariant("srand", (q_func_n_t)f_srand_vi, QC_NO_FLAGS, QDOM_PROCESS, nothingTypeInfo, 1, softBigIntTypeInfo, NULL, "seed");
    // nothing srand() {}
    ns.addBuiltinVariant("srand", (q_func_n_t)f_srand, QC_RUNTIME_NOOP, QDOM_PROCESS, nothingTypeInfo);
    // string strerror(softint err) {}
    ns.addBuiltinVariant("strerror", (q_func_n_t)f_strerror_vi, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, softBigIntTypeInfo, NULL, "err");
    // string strerror() {}
    ns.addBuiltinVariant("strerror", (q_func_n_t)f_strerror, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo);
    // int system(string command) {}
    ns.addBuiltinVariant("system", (q_func_n_t)f_system_Vs, QC_NO_FLAGS, QDOM_EXTERNAL_PROCESS, bigIntTypeInfo, 1, stringTypeInfo, NULL, "command");
    // nothing system() {}
    ns.addBuiltinVariant("system", (q_func_n_t)f_system, QC_RUNTIME_NOOP, QDOM_EXTERNAL_PROCESS, nothingTypeInfo);
    // int usleep(softint usecs) {}
    ns.addBuiltinVariant("usleep", (q_func_n_t)f_usleep_vi, QC_NO_FLAGS, QDOM_PROCESS, bigIntTypeInfo, 1, softBigIntTypeInfo, NULL, "usecs");
    // int usleep(date d) {}
    ns.addBuiltinVariant("usleep", (q_func_n_t)f_usleep_Vd, QC_NO_FLAGS, QDOM_PROCESS, bigIntTypeInfo, 1, dateTypeInfo, NULL, "d");
    // nothing usleep() {}
    ns.addBuiltinVariant("usleep", (q_func_n_t)f_usleep, QC_RUNTIME_NOOP, QDOM_PROCESS, nothingTypeInfo);

}
