/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  ql_misc.qpp

  Qore Programming Language

  Copyright (C) 2003 - 2017 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/ql_misc.h"
#include "qore/intern/QoreSignal.h"
#include "qore/intern/QoreNamespaceIntern.h"
#include "qore/intern/QC_Program.h"
#include "qore/intern/ModuleInfo.h"
#include "qore/intern/qore_program_private.h"
#include "qore/intern/QoreHashNodeIntern.h"

#include <string.h>
#include <time.h>
#include <errno.h>

static const QoreFunction* get_builtin_func(const QoreStringNode* str, ExceptionSink* xsink) {
   const qore_ns_private* ns;
   const QoreFunction* f = qore_root_ns_private::runtimeFindFunction(*(getRootNS()), str->getBuffer(), ns);
   if (f)
      return f;

   xsink->raiseException("NO-FUNCTION", "cannot find any builtin function '%s()'", str->getBuffer());
   return 0;
}

static const char* tlist[] = { "OPTION", "ALGORITHM", "FUNCTION" };

// signal constants - if they are not defined define them here
#ifndef SIGABRT
#define SIGABRT 0
#endif
#ifndef SIGALRM
#define SIGALRM 0
#endif
#ifndef SIGBUS
#define SIGBUS 0
#endif
#ifndef SIGCANCEL
#define SIGCANCEL 0
#endif
#ifndef SIGCHLD
#define SIGCHLD 0
#endif
#ifndef SIGCLD
#define SIGCLD 0
#endif
#ifndef SIGCONT
#define SIGCONT 0
#endif
#ifndef SIGEMT
#define SIGEMT 0
#endif
#ifndef SIGFPE
#define SIGFPE 0
#endif
#ifndef SIGFREEZE
#define SIGFREEZE 0
#endif
#ifndef SIGHUP
#define SIGHUP 0
#endif
#ifndef SIGILL
#define SIGILL 0
#endif
#ifndef SIGINFO
#define SIGINFO 0
#endif
#ifndef SIGINT
#define SIGINT 0
#endif
#ifndef SIGIO
#define SIGIO 0
#endif
#ifndef SIGIOT
#define SIGIOT 0
#endif
#ifndef SIGJVM1
#define SIGJVM1 0
#endif
#ifndef SIGJVM2
#define SIGJVM2 0
#endif
#ifndef SIGKILL
#define SIGKILL 0
#endif
#ifndef SIGLOST
#define SIGLOST 0
#endif
#ifndef SIGLWP
#define SIGLWP 0
#endif
#ifndef SIGPIPE
#define SIGPIPE 0
#endif
#ifndef SIGPOLL
#define SIGPOLL 0
#endif
#ifndef SIGPROF
#define SIGPROF 0
#endif
#ifndef SIGPWR
#define SIGPWR 0
#endif
#ifndef SIGQUIT
#define SIGQUIT 0
#endif
#ifndef SIGSEGV
#define SIGSEGV 0
#endif
#ifndef SIGSTKFLT
#define SIGSTKFLT 0
#endif
#ifndef SIGSTOP
#define SIGSTOP 0
#endif
#ifndef SIGSYS
#define SIGSYS 0
#endif
#ifndef SIGTERM
#define SIGTERM 0
#endif
#ifndef SIGTHAW
#define SIGTHAW 0
#endif
#ifndef SIGTRAP
#define SIGTRAP 0
#endif
#ifndef SIGTSTP
#define SIGTSTP 0
#endif
#ifndef SIGTTIN
#define SIGTTIN 0
#endif
#ifndef SIGTTOU
#define SIGTTOU 0
#endif
#ifndef SIGURG
#define SIGURG 0
#endif
#ifndef SIGUSR1
#define SIGUSR1 0
#endif
#ifndef SIGUSR2
#define SIGUSR2 0
#endif
#ifndef SIGVTALRM
#define SIGVTALRM 0
#endif
#ifndef SIGWAITING
#define SIGWAITING 0
#endif
#ifndef SIGWINCH
#define SIGWINCH 0
#endif
#ifndef SIGXCPU
#define SIGXCPU 0
#endif
#ifndef SIGXFSZ
#define SIGXFSZ 0
#endif
#ifndef SIGXRES
#define SIGXRES 0
#endif
# 478 "ql_misc.qpp"
# 511 "ql_misc.qpp"
// nothing remove_signal_handler(softint signal) {}
static QoreValue f_remove_signal_handler_vi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 signal = HARD_QORE_VALUE_INT(args, 0);
# 527 "ql_misc.qpp"
#ifdef HAVE_SIGNAL_HANDLING
   if (!signal || signal > QORE_SIGNAL_MAX)
      return xsink->raiseException("REMOVE-SIGNAL-HANDLER-ERROR", "%d is not a valid signal", signal);
   QSM.removeHandler(signal, xsink);
#else
   return xsink->raiseException("MISSING-FEATURE-ERROR", "this platform does not support signal handling, therefore the set_signal_handler() and remove_signal_handler() functions are not available in Qore; for maximum portability, use the constant Option::HAVE_SIGNAL_HANDLING to check if this function is implemented before calling");
#endif
    return QoreValue();
}

// nothing set_signal_handler(softint signal, code f) {}
static QoreValue f_set_signal_handler_viVc(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 signal = HARD_QORE_VALUE_INT(args, 0);
    const ResolvedCallReferenceNode* f = get_param_value(args, 1).get<const ResolvedCallReferenceNode>();
# 500 "ql_misc.qpp"
#ifdef HAVE_SIGNAL_HANDLING
   if (!signal || signal > QORE_SIGNAL_MAX)
      return xsink->raiseException("SET-SIGNAL-HANDLER-ERROR", "%d is not a valid signal", signal);

   QSM.setHandler(signal, f, xsink);
#else
   return xsink->raiseException("MISSING-FEATURE-ERROR", "this platform does not support signal handling, therefore the set_signal_handler() and remove_signal_handler() functions are not available in Qore; for maximum portability, use the constant Option::HAVE_SIGNAL_HANDLING to check if this function is implemented before calling");
#endif
    return QoreValue();
}

# 538 "ql_misc.qpp"
# 568 "ql_misc.qpp"
# 589 "ql_misc.qpp"
# 610 "ql_misc.qpp"
# 626 "ql_misc.qpp"
# 658 "ql_misc.qpp"
# 685 "ql_misc.qpp"
# 701 "ql_misc.qpp"
# 712 "ql_misc.qpp"
# 725 "ql_misc.qpp"
# 731 "ql_misc.qpp"
# 747 "ql_misc.qpp"
# 760 "ql_misc.qpp"
# 776 "ql_misc.qpp"
# 782 "ql_misc.qpp"
# 803 "ql_misc.qpp"
# 822 "ql_misc.qpp"
# 828 "ql_misc.qpp"
# 847 "ql_misc.qpp"
# 853 "ql_misc.qpp"
# 865 "ql_misc.qpp"
# 905 "ql_misc.qpp"
# 911 "ql_misc.qpp"
# 922 "ql_misc.qpp"
# 928 "ql_misc.qpp"
# 946 "ql_misc.qpp"
# 977 "ql_misc.qpp"
# 1003 "ql_misc.qpp"
# 1009 "ql_misc.qpp"
# 1043 "ql_misc.qpp"
# 1049 "ql_misc.qpp"
# 1071 "ql_misc.qpp"
# 1091 "ql_misc.qpp"
# 1097 "ql_misc.qpp"
# 1122 "ql_misc.qpp"
# 1145 "ql_misc.qpp"
# 1164 "ql_misc.qpp"
# 1170 "ql_misc.qpp"
# 1194 "ql_misc.qpp"
# 1217 "ql_misc.qpp"
# 1223 "ql_misc.qpp"
# 1251 "ql_misc.qpp"
# 1273 "ql_misc.qpp"
# 1300 "ql_misc.qpp"
# 1322 "ql_misc.qpp"
# 1347 "ql_misc.qpp"
# 1358 "ql_misc.qpp"
# 1374 "ql_misc.qpp"
# 1398 "ql_misc.qpp"
# 1404 "ql_misc.qpp"
# 1429 "ql_misc.qpp"
# 1446 "ql_misc.qpp"
# 1471 "ql_misc.qpp"
# 1496 "ql_misc.qpp"
# 1518 "ql_misc.qpp"
# 1524 "ql_misc.qpp"
# 1546 "ql_misc.qpp"
# 1571 "ql_misc.qpp"
# 1596 "ql_misc.qpp"
# 1623 "ql_misc.qpp"
# 1650 "ql_misc.qpp"
# 1672 "ql_misc.qpp"
# 1694 "ql_misc.qpp"
# 1700 "ql_misc.qpp"
# 1725 "ql_misc.qpp"
# 1750 "ql_misc.qpp"
# 1775 "ql_misc.qpp"
# 1800 "ql_misc.qpp"
# 1827 "ql_misc.qpp"
# 1852 "ql_misc.qpp"
# 1877 "ql_misc.qpp"
# 1902 "ql_misc.qpp"
# 1913 "ql_misc.qpp"
# 1934 "ql_misc.qpp"
# 1957 "ql_misc.qpp"
# 1976 "ql_misc.qpp"
# 1999 "ql_misc.qpp"
# 2005 "ql_misc.qpp"
# 2023 "ql_misc.qpp"
# 2041 "ql_misc.qpp"
# 2047 "ql_misc.qpp"
# 2070 "ql_misc.qpp"
# 2093 "ql_misc.qpp"
# 2110 "ql_misc.qpp"
# 2116 "ql_misc.qpp"
# 2138 "ql_misc.qpp"
# 2182 "ql_misc.qpp"
# 2188 "ql_misc.qpp"
# 2209 "ql_misc.qpp"
# 2215 "ql_misc.qpp"
# 2239 "ql_misc.qpp"
# 2245 "ql_misc.qpp"
# 2264 "ql_misc.qpp"
# 2288 "ql_misc.qpp"
# 2311 "ql_misc.qpp"
# 2317 "ql_misc.qpp"
# 2341 "ql_misc.qpp"
# 2374 "ql_misc.qpp"
# 2409 "ql_misc.qpp"
# 2421 "ql_misc.qpp"
# 2433 "ql_misc.qpp"
# 2445 "ql_misc.qpp"
# 2473 "ql_misc.qpp"
# 2504 "ql_misc.qpp"
# 2549 "ql_misc.qpp"
# 2563 "ql_misc.qpp"
# 2579 "ql_misc.qpp"
# 2613 "ql_misc.qpp"
# 2633 "ql_misc.qpp"
# 2656 "ql_misc.qpp"
# 2672 "ql_misc.qpp"
// string backquote(string cmd, *reference<int> rc) {}
static QoreValue f_backquote_VsC15_reference_int_(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* cmd = HARD_QORE_VALUE_STRING(args, 0);
    const ReferenceNode* rc = get_param_value(args, 1).get<const ReferenceNode>();
# 1030 "ql_misc.qpp"
   int prc;
   SimpleRefHolder<QoreStringNode> str(backquoteEval(cmd->getBuffer(), prc, xsink));
   if (!*xsink && rc) {
      QoreTypeSafeReferenceHelper rh(rc, xsink);
      if (!rh)
         return QoreValue();

      rh.assign((int64)prc);
   }
   return str.release();
}

// nothing backquote() {}
static QoreValue f_backquote(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1046 "ql_misc.qpp"

    return QoreValue();
}

// auto call_builtin_function(string name, ...) {}
static QoreValue f_call_builtin_function_VsVV(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 645 "ql_misc.qpp"
   const QoreFunction* f = get_builtin_func(name, xsink);
   if (!f)
      return QoreValue();

   ReferenceHolder<QoreListNode> vargs(xsink);
   // if there are arguments to pass, create argument list by copying current list
   if (num_args(args) > 1)
      vargs = args->getOldList(1);

   return f->evalDynamic(*vargs, xsink).takeNode();
}

// auto call_builtin_function_args(string name, *softlist<auto> vargs) {}
static QoreValue f_call_builtin_function_args_VsC15_softlist_auto_(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
    const QoreListNode* vargs = get_param_value(args, 1).get<const QoreListNode>();
# 677 "ql_misc.qpp"
   const QoreFunction* f = get_builtin_func(name, xsink);
   if (!f)
      return QoreValue();

   return f->evalDynamic(vargs, xsink).takeNode();
}

// auto call_function(string name, ...) {}
static QoreValue f_call_function_VsVV(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 559 "ql_misc.qpp"
   ReferenceHolder<QoreListNode> vargs(xsink);
   // if there are arguments to pass, create argument list by copying current list
   if (num_args(args) > 1)
      vargs = args->getOldList(1);

   return getProgram()->callFunction(name->getBuffer(), *vargs, xsink);
}

// auto call_function(code f, ...) {}
static QoreValue f_call_function_VcVV(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const ResolvedCallReferenceNode* f = get_param_value(args, 0).get<const ResolvedCallReferenceNode>();
# 580 "ql_misc.qpp"
   ReferenceHolder<QoreListNode> vargs(xsink);
   // if there are arguments to pass, create argument list by copying current list
   if (num_args(args) > 1)
      vargs = args->getOldList(1);

   return f->execValue(*vargs, xsink).takeNode();
}

// auto call_function_args(string name, *softlist<auto> vargs) {}
static QoreValue f_call_function_args_VsC15_softlist_auto_(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
    const QoreListNode* vargs = get_param_value(args, 1).get<const QoreListNode>();
# 606 "ql_misc.qpp"
   return getProgram()->callFunction(name->getBuffer(), vargs, xsink);
}

// auto call_function_args(code f, *softlist<auto> vargs) {}
static QoreValue f_call_function_args_VcC15_softlist_auto_(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const ResolvedCallReferenceNode* f = get_param_value(args, 0).get<const ResolvedCallReferenceNode>();
    const QoreListNode* vargs = get_param_value(args, 1).get<const QoreListNode>();
# 622 "ql_misc.qpp"
   return f->execValue(vargs, xsink).takeNode();
}

// string decode_uri_request(string uri) {}
static QoreValue f_decode_uri_request_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* uri = HARD_QORE_VALUE_STRING(args, 0);
# 2368 "ql_misc.qpp"
   QoreStringNodeHolder str(new QoreStringNode(QCS_UTF8));
   str->concatDecodeUriRequest(*uri, xsink);
   return *xsink ? QoreValue() : str.release();
}

// string decode_url(string url) {}
static QoreValue f_decode_url_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* url = HARD_QORE_VALUE_STRING(args, 0);
# 2305 "ql_misc.qpp"
   QoreStringNodeHolder str(new QoreStringNode(QCS_UTF8));
   str->concatDecodeUrl(*url, xsink);
   return *xsink ? QoreValue() : str.release();
}

// nothing decode_url() {}
static QoreValue f_decode_url(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2314 "ql_misc.qpp"

    return QoreValue();
}

// string encode_uri_request(string url) {}
static QoreValue f_encode_uri_request_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* url = HARD_QORE_VALUE_STRING(args, 0);
# 2403 "ql_misc.qpp"
   QoreStringNodeHolder str(new QoreStringNode(url->getEncoding()));
   str->concatEncodeUriRequest(xsink, url);
   return *xsink ? QoreValue() : str.release();
}

// string encode_url(string url, softbool encode_all = False) {}
static QoreValue f_encode_url_Vsvb(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* url = HARD_QORE_VALUE_STRING(args, 0);
    bool encode_all = HARD_QORE_VALUE_BOOL(args, 1);
# 2335 "ql_misc.qpp"
   QoreStringNodeHolder str(new QoreStringNode(url->getEncoding()));
   str->concatEncodeUrl(xsink, url, encode_all);
   return *xsink ? QoreValue() : str.release();
}

// bool exists(...) {}
static QoreValue f_exists_VV(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 695 "ql_misc.qpp"
   // to emulate the exists operator, we must return True if more than one argument is passed
   // as this will appear to be a list to the exists operator, which is different from NOTHING
   return (num_args(args) <= 1) ? !get_param_value(args, 0).isNothing() : true;
}

// bool existsFunction(string name) {}
static QoreValue f_existsFunction_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 708 "ql_misc.qpp"
   return getProgram()->existsFunction(name->getBuffer());
}

// bool existsFunction(code c) {}
static QoreValue f_existsFunction_Vc(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 721 "ql_misc.qpp"
   return true;
}

// nothing existsFunction() {}
static QoreValue f_existsFunction(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 728 "ql_misc.qpp"

    return QoreValue();
}

// bool exists_function(string name) {}
static QoreValue f_exists_function_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 743 "ql_misc.qpp"
   return getProgram()->existsFunction(name->getBuffer());
}

// bool exists_function(code c) {}
static QoreValue f_exists_function_Vc(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 756 "ql_misc.qpp"
   return true;
}

// *string functionType(string name) {}
static QoreValue f_functionType_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 767 "ql_misc.qpp"
   const qore_ns_private* ns;
   const QoreFunction* f = qore_root_ns_private::runtimeFindFunction(*(getRootNS()), name->getBuffer(), ns);
   if (!f)
      return QoreValue();

   return new QoreStringNode(f->hasBuiltin() ? "builtin" : "user");
}

// nothing functionType() {}
static QoreValue f_functionType(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 779 "ql_misc.qpp"

    return QoreValue();
}

// *string function_type(string name) {}
static QoreValue f_function_type_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 794 "ql_misc.qpp"
   const qore_ns_private* ns;
   const QoreFunction* f = qore_root_ns_private::runtimeFindFunction(*(getRootNS()), name->getBuffer(), ns);
   if (!f)
      return QoreValue();

   return new QoreStringNode(f->hasBuiltin() ? "builtin" : "user");
}

// *int getByte(string str, softint offset = 0) {}
static QoreValue f_getByte_Vsvi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 offset = HARD_QORE_VALUE_INT(args, 1);
# 1508 "ql_misc.qpp"
   unsigned char* ptr = (unsigned char* )str->getBuffer();
   int size = str->strlen();

   if (offset >= size || offset < 0)
      return QoreValue();

   return ptr[offset];
}

// nothing getByte() {}
static QoreValue f_getByte(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1521 "ql_misc.qpp"

    return QoreValue();
}

// *int getByte(binary b, softint offset = 0) {}
static QoreValue f_getByte_Vyvi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* b = HARD_QORE_VALUE_BINARY(args, 0);
    int64 offset = HARD_QORE_VALUE_INT(args, 1);
# 1536 "ql_misc.qpp"
   unsigned char* ptr = (unsigned char* )b->getPtr();
   int size = b->size();

   if (offset >= size || offset < 0)
      return QoreValue();

   return ptr[offset];
}

// string getClassName(object obj) {}
static QoreValue f_getClassName_Vo(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreObject* obj = HARD_QORE_VALUE_OBJECT(args, 0);
# 918 "ql_misc.qpp"
   return new QoreStringNode(obj->getClass()->getName());
}

// nothing getClassName() {}
static QoreValue f_getClassName(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 925 "ql_misc.qpp"

    return QoreValue();
}

// list getFeatureList() {}
static QoreValue f_getFeatureList(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1354 "ql_misc.qpp"
   return getProgram()->getFeatureList();
}

// hash<string,hash> getModuleHash() {}
static QoreValue f_getModuleHash(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1318 "ql_misc.qpp"
   return MM.getModuleHash();
}

// list<hash> getModuleList() {}
static QoreValue f_getModuleList(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1269 "ql_misc.qpp"
   return MM.getModuleList();
}

// *int getWord32(string str, softint offset = 0) {}
static QoreValue f_getWord32_Vsvi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 offset = HARD_QORE_VALUE_INT(args, 1);
# 1662 "ql_misc.qpp"
   unsigned char* ptr = (unsigned char* )str->getBuffer();
   int size = str->strlen();

   if (offset >= (size - 3) || offset < 0)
      return QoreValue();

   return ntohl(*((unsigned int *)&ptr[offset]));
}

// *int getWord32(binary b, softint offset = 0) {}
static QoreValue f_getWord32_Vyvi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* b = HARD_QORE_VALUE_BINARY(args, 0);
    int64 offset = HARD_QORE_VALUE_INT(args, 1);
# 1684 "ql_misc.qpp"
   unsigned char* ptr = (unsigned char* )b->getPtr();
   int size = b->size();

   if (offset >= (size - 3) || offset < 0)
      return QoreValue();

   return (int64)ntohl(*((unsigned int *)&ptr[offset]));
}

// nothing getWord32() {}
static QoreValue f_getWord32(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1697 "ql_misc.qpp"

    return QoreValue();
}

// *int get_byte(string str, softint offset = 0) {}
static QoreValue f_get_byte_Vsvi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 offset = HARD_QORE_VALUE_INT(args, 1);
# 1461 "ql_misc.qpp"
   unsigned char* ptr = (unsigned char* )str->getBuffer();
   int size = str->strlen();

   if (offset >= size || offset < 0)
      return QoreValue();

   return ptr[offset];
}

// *int get_byte(binary b, softint offset = 0) {}
static QoreValue f_get_byte_Vyvi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* b = HARD_QORE_VALUE_BINARY(args, 0);
    int64 offset = HARD_QORE_VALUE_INT(args, 1);
# 1486 "ql_misc.qpp"
   unsigned char* ptr = (unsigned char* )b->getPtr();
   int size = b->size();

   if (offset >= size || offset < 0)
      return QoreValue();

   return ptr[offset];
}

// string get_class_name(object obj) {}
static QoreValue f_get_class_name_Vo(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreObject* obj = HARD_QORE_VALUE_OBJECT(args, 0);
# 942 "ql_misc.qpp"
   return new QoreStringNode(obj->getClass()->getName());
}

// string get_default_encoding() {}
static QoreValue f_get_default_encoding(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 861 "ql_misc.qpp"
   return new QoreStringNode(QCS_DEFAULT->getCode());
}

// string get_ex_pos(hash ex) {}
static QoreValue f_get_ex_pos_Vh(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreHashNode* ex = HARD_QORE_VALUE_HASH(args, 0);
# 2596 "ql_misc.qpp"
   QoreValue n = ex->getValueKeyValue("file");
   QoreStringNode* str = new QoreStringNode(n.getType() == NT_STRING ? n.get<const QoreStringNode>()->c_str() : "<unknown>");
   int ln = (int)ex->getValueKeyValue("line").getAsBigInt();
   str->sprintf(":%d", ln);
   n = ex->getValueKeyValue("source");
   if (n.getType() == NT_STRING) {
      const QoreStringNode* sstr = n.get<const QoreStringNode>();
      if (!sstr->empty()) {
         int offset = (int)ex->getValueKeyValue("offset").getAsBigInt();
         str->sprintf(" (source \"%s\":%d)", sstr->c_str(), ln + offset);
      }
   }

   return str;
}

// list get_feature_list() {}
static QoreValue f_get_feature_list(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1370 "ql_misc.qpp"
   return getProgram()->getFeatureList();
}

// hash get_global_vars() {}
static QoreValue f_get_global_vars(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2668 "ql_misc.qpp"
   return getProgram()->getGlobalVars();
}

// hash get_local_vars(int frame) {}
static QoreValue f_get_local_vars_Vi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 frame = HARD_QORE_VALUE_INT(args, 0);
# 2629 "ql_misc.qpp"
    return thread_get_local_vars(frame, xsink);
}

// hash<string,hash> get_module_hash() {}
static QoreValue f_get_module_hash(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1343 "ql_misc.qpp"
   return MM.getModuleHash();
}

// list<hash> get_module_list() {}
static QoreValue f_get_module_list(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1296 "ql_misc.qpp"
   return MM.getModuleList();
}

// int get_parse_options() {}
static QoreValue f_get_parse_options(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2559 "ql_misc.qpp"
   return runtime_get_parse_options();
}

// hash get_qore_library_info() {}
static QoreValue f_get_qore_library_info(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2524 "ql_misc.qpp"
   QoreHashNode* h = new QoreHashNode;

   h->setKeyValue("PlatformOS", new QoreStringNode(qore_target_os), xsink);
   h->setKeyValue("PlatformCPU", new QoreStringNode(qore_target_arch), xsink);
   h->setKeyValue("VersionString", new QoreStringNode(qore_version_string), xsink);
   h->setKeyValue("VersionMajor", new QoreBigIntNode(qore_version_major), xsink);
   h->setKeyValue("VersionMinor", new QoreBigIntNode(qore_version_minor), xsink);
   h->setKeyValue("VersionSub", new QoreBigIntNode(qore_version_sub), xsink);
   h->setKeyValue("Build", new QoreBigIntNode(qore_build_number), xsink);

   h->setKeyValue("UserModuleVerDir", new QoreStringNode(qore_user_module_ver_dir), xsink);
   h->setKeyValue("UserModuleDir", new QoreStringNode(qore_user_module_dir), xsink);
   h->setKeyValue("BinaryModuleVerDir", new QoreStringNode(qore_module_ver_dir), xsink);
   h->setKeyValue("BinaryModuleDir", new QoreStringNode(qore_module_dir), xsink);
   h->setKeyValue("ModulePath", new QoreStringNodeMaker("%s:%s:%s:%s", qore_user_module_ver_dir, qore_user_module_dir, qore_module_ver_dir, qore_module_dir), xsink);

   h->setKeyValue("BuildHost", new QoreStringNode(qore_build_host), xsink);
   h->setKeyValue("Compiler", new QoreStringNode(qore_cplusplus_compiler), xsink);
   h->setKeyValue("CFLAGS", new QoreStringNode(qore_cflags), xsink);
   h->setKeyValue("LDFLAGS", new QoreStringNode(qore_ldflags), xsink);

   return h;
}

// hash get_qore_option_hash() {}
static QoreValue f_get_qore_option_hash(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2489 "ql_misc.qpp"
   QoreHashNode* rv = new QoreHashNode;

   for (unsigned j = 0; j < qore_option_list_size; ++j) {
      QoreHashNode* h = new QoreHashNode;
      h->setKeyValue("option", new QoreStringNode(qore_option_list[j].option), xsink);
      h->setKeyValue("constant", new QoreStringNode(qore_option_list[j].constant), xsink);
      h->setKeyValue("type", new QoreStringNode(tlist[qore_option_list[j].type]), xsink);
      h->setKeyValue("value", get_bool_node(qore_option_list[j].value), xsink);

      rv->setKeyValue(qore_option_list[j].constant, h, 0);
   }
   return rv;
}

// list get_qore_option_list() {}
static QoreValue f_get_qore_option_list(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2459 "ql_misc.qpp"
   QoreListNode* l = new QoreListNode;

   for (unsigned j = 0; j < qore_option_list_size; ++j) {
      QoreHashNode* h = new QoreHashNode;
      h->setKeyValue("option", new QoreStringNode(qore_option_list[j].option), xsink);
      h->setKeyValue("constant", new QoreStringNode(qore_option_list[j].constant), xsink);
      h->setKeyValue("type", new QoreStringNode(tlist[qore_option_list[j].type]), xsink);
      h->setKeyValue("value", get_bool_node(qore_option_list[j].value), xsink);
      l->push(h);
   }
   return l;
}

// *string get_script_dir() {}
static QoreValue f_get_script_dir(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2429 "ql_misc.qpp"
   return getProgram()->getScriptDir();
}

// *string get_script_name() {}
static QoreValue f_get_script_name(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2441 "ql_misc.qpp"
   return getProgram()->getScriptName();
}

// *string get_script_path() {}
static QoreValue f_get_script_path(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2417 "ql_misc.qpp"
   return getProgram()->getScriptPath();
}

// *int get_word_16(string str, softint offset = 0) {}
static QoreValue f_get_word_16_Vsvi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 offset = HARD_QORE_VALUE_INT(args, 1);
# 1561 "ql_misc.qpp"
   unsigned char* ptr = (unsigned char* )str->getBuffer();
   int size = str->strlen();

   if (offset >= (size - 1) || offset < 0)
      return QoreValue();

   return ntohs(*((unsigned short*)&ptr[offset]));
}

// *int get_word_16(binary b, softint offset = 0) {}
static QoreValue f_get_word_16_Vyvi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* b = HARD_QORE_VALUE_BINARY(args, 0);
    int64 offset = HARD_QORE_VALUE_INT(args, 1);
# 1586 "ql_misc.qpp"
   unsigned char* ptr = (unsigned char* )b->getPtr();
   int size = b->size();

   if (offset >= (size - 1) || offset < 0)
      return QoreValue();

   return ntohs(*((unsigned short*)&ptr[offset]));
}

// *int get_word_16_lsb(string str, softint offset = 0) {}
static QoreValue f_get_word_16_lsb_Vsvi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 offset = HARD_QORE_VALUE_INT(args, 1);
# 1765 "ql_misc.qpp"
   unsigned char* ptr = (unsigned char* )str->getBuffer();
   int size = str->strlen();

   if (offset >= (size - 1) || offset < 0)
      return QoreValue();

   return (int64)LSBi2(*((unsigned short *)&ptr[offset]));
}

// *int get_word_16_lsb(binary b, softint offset = 0) {}
static QoreValue f_get_word_16_lsb_Vyvi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* b = HARD_QORE_VALUE_BINARY(args, 0);
    int64 offset = HARD_QORE_VALUE_INT(args, 1);
# 1790 "ql_misc.qpp"
   unsigned char* ptr = (unsigned char* )b->getPtr();
   int size = b->size();

   if (offset >= (size - 1) || offset < 0)
      return QoreValue();

   return (int64)LSBi2(*((unsigned short *)&ptr[offset]));
}

// *int get_word_32(string str, softint offset = 0) {}
static QoreValue f_get_word_32_Vsvi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 offset = HARD_QORE_VALUE_INT(args, 1);
# 1613 "ql_misc.qpp"
   unsigned char* ptr = (unsigned char* )str->getBuffer();
   int size = str->strlen();

   if (offset >= (size - 3) || offset < 0)
      return QoreValue();

   return ntohl(*((unsigned int*)&ptr[offset]));
}

// *int get_word_32(binary b, softint offset = 0) {}
static QoreValue f_get_word_32_Vyvi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* b = HARD_QORE_VALUE_BINARY(args, 0);
    int64 offset = HARD_QORE_VALUE_INT(args, 1);
# 1640 "ql_misc.qpp"
   unsigned char* ptr = (unsigned char* )b->getPtr();
   int size = b->size();

   if (offset >= (size - 3) || offset < 0)
      return QoreValue();

   return ntohl(*((unsigned int*)&ptr[offset]));
}

// *int get_word_32_lsb(string str, softint offset = 0) {}
static QoreValue f_get_word_32_lsb_Vsvi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 offset = HARD_QORE_VALUE_INT(args, 1);
# 1817 "ql_misc.qpp"
   unsigned char* ptr = (unsigned char* )str->getBuffer();
   int size = str->strlen();

   if (offset >= (size - 3) || offset < 0)
      return QoreValue();

   return (int64)LSBi4(*((unsigned int *)&ptr[offset]));
}

// *int get_word_32_lsb(binary b, softint offset = 0) {}
static QoreValue f_get_word_32_lsb_Vyvi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* b = HARD_QORE_VALUE_BINARY(args, 0);
    int64 offset = HARD_QORE_VALUE_INT(args, 1);
# 1842 "ql_misc.qpp"
   unsigned char* ptr = (unsigned char* )b->getPtr();
   int size = b->size();

   if (offset >= (size - 3) || offset < 0)
      return QoreValue();

   return (int64)LSBi4(*((unsigned int *)&ptr[offset]));
}

// *int get_word_64(string str, softint offset = 0) {}
static QoreValue f_get_word_64_Vsvi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 offset = HARD_QORE_VALUE_INT(args, 1);
# 1715 "ql_misc.qpp"
   unsigned char* ptr = (unsigned char* )str->getBuffer();
   int size = str->strlen();

   if (offset >= (size - 7) || offset < 0)
      return QoreValue();

   return (int64)MSBi8(*((int64 *)&ptr[offset]));
}

// *int get_word_64(binary b, softint offset = 0) {}
static QoreValue f_get_word_64_Vyvi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* b = HARD_QORE_VALUE_BINARY(args, 0);
    int64 offset = HARD_QORE_VALUE_INT(args, 1);
# 1740 "ql_misc.qpp"
   unsigned char* ptr = (unsigned char* )b->getPtr();
   int size = b->size();

   if (offset >= (size - 7) || offset < 0)
      return QoreValue();

   return (int64)MSBi8(*((int64 *)&ptr[offset]));
}

// *int get_word_64_lsb(string str, softint offset = 0) {}
static QoreValue f_get_word_64_lsb_Vsvi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 offset = HARD_QORE_VALUE_INT(args, 1);
# 1867 "ql_misc.qpp"
   unsigned char* ptr = (unsigned char* )str->getBuffer();
   int size = str->strlen();

   if (offset >= (size - 7) || offset < 0)
      return QoreValue();

   return (int64)LSBi8(*((int64 *)&ptr[offset]));
}

// *int get_word_64_lsb(binary b, softint offset = 0) {}
static QoreValue f_get_word_64_lsb_Vyvi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* b = HARD_QORE_VALUE_BINARY(args, 0);
    int64 offset = HARD_QORE_VALUE_INT(args, 1);
# 1892 "ql_misc.qpp"
   unsigned char* ptr = (unsigned char* )b->getPtr();
   int size = b->size();

   if (offset >= (size - 7) || offset < 0)
      return QoreValue();

   return (int64)LSBi8(*((int64 *)&ptr[offset]));
}

// bool has_key(hash h, string key) {}
static QoreValue f_has_key_VhVs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreHashNode* h = HARD_QORE_VALUE_HASH(args, 0);
    const QoreStringNode* key = HARD_QORE_VALUE_STRING(args, 1);
# 1420 "ql_misc.qpp"
   TempEncodingHelper tmp(key, QCS_DEFAULT, xsink);
   if (*xsink)
      return QoreValue();
   bool exists;
   h->getValueKeyValueExistence(tmp->c_str(), exists);
   return exists;
}

// bool has_key(object obj, string key) {}
static QoreValue f_has_key_VoVs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreObject* obj = HARD_QORE_VALUE_OBJECT(args, 0);
    const QoreStringNode* key = HARD_QORE_VALUE_STRING(args, 1);
# 1440 "ql_misc.qpp"
   AutoVLock vl(xsink);
   AbstractQoreNode* *ptr = obj->getExistingValuePtr(key, &vl, xsink);
   return (bool)ptr;
}

// list hash_values(hash h) {}
static QoreValue f_hash_values_Vh(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreHashNode* h = HARD_QORE_VALUE_HASH(args, 0);
# 1388 "ql_misc.qpp"
   QoreListNode* l = new QoreListNode;

   ConstHashIterator hi(h);
   while (hi.next())
      l->push(hi.getReferencedValue());

   return l;
}

// nothing hash_values() {}
static QoreValue f_hash_values(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1401 "ql_misc.qpp"

    return QoreValue();
}

// int hextoint(string str) {}
static QoreValue f_hextoint_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
# 2150 "ql_misc.qpp"
   if (!str->strlen())
      return 0;

   int64 rc = 0;
   int64 pow = 0;
   const char* buf = str->getBuffer();
   qore_size_t len = str->size();
   if (*buf == '0' && *(buf + 1) == 'x') {
      buf += 2;
      len -= 2;
   }
   else if (*buf == 'x') {
      buf++;
      --len;
   }
   for (const char* p = buf + len - 1; p >= buf; p--) {
      int n = get_nibble(*p, xsink);
      if (xsink->isException())
	 return 0;
      if (!pow) {
	 rc = n;
	 pow = 16;
      }
      else {
	 rc += n * pow;
	 pow *= 16;
      }
   }
   return rc;
}

// nothing hextoint() {}
static QoreValue f_hextoint(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2185 "ql_misc.qpp"

    return QoreValue();
}

// string html_decode(string str) {}
static QoreValue f_html_decode_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
# 842 "ql_misc.qpp"
   QoreStringNodeHolder rv(new QoreStringNode(str->getEncoding()));
   return rv->concatDecode(xsink, *str, CD_HTML) ? QoreValue() : rv.release();
}

// nothing html_decode() {}
static QoreValue f_html_decode(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 850 "ql_misc.qpp"

    return QoreValue();
}

// string html_encode(string str) {}
static QoreValue f_html_encode_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
# 817 "ql_misc.qpp"
   QoreStringNodeHolder rv(new QoreStringNode(str->getEncoding()));
   return rv->concatEncode(xsink, *str, CE_HTML) ? QoreValue() : rv.release();
}

// nothing html_encode() {}
static QoreValue f_html_encode(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 825 "ql_misc.qpp"

    return QoreValue();
}

// nothing load_module(string name) {}
static QoreValue f_load_module_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 2235 "ql_misc.qpp"
   MM.runTimeLoadModule(name->getBuffer(), xsink);
    return QoreValue();
}

// nothing load_module() {}
static QoreValue f_load_module(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2242 "ql_misc.qpp"

    return QoreValue();
}

// nothing load_user_module_with_program(string name, Qore::Program pgm) {}
static QoreValue f_load_user_module_with_program_VsC7Program(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
    TAKE_HARD_QORE_VALUE_OBJ_DATA(pgm, QoreProgram, args, 1, CID_PROGRAM, "load_user_module_with_program()", "Qore::Program", xsink);
    if (*xsink)
        return 0;
# 2260 "ql_misc.qpp"
   QMM.runTimeLoadModule(*xsink, name->getBuffer(), getProgram(), pgm);
    return QoreValue();
}

// string makeBase64String(string str, softint maxlinelen = -1) {}
static QoreValue f_makeBase64String_Vsvi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 maxlinelen = HARD_QORE_VALUE_INT(args, 1);
# 1065 "ql_misc.qpp"
   QoreStringNode* rv = new QoreStringNode;
   rv->concatBase64(str, maxlinelen);
   return rv;
}

// string makeBase64String(binary bin, softint maxlinelen = -1) {}
static QoreValue f_makeBase64String_Vyvi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* bin = HARD_QORE_VALUE_BINARY(args, 0);
    int64 maxlinelen = HARD_QORE_VALUE_INT(args, 1);
# 1087 "ql_misc.qpp"
   return new QoreStringNode(bin, maxlinelen);
}

// nothing makeBase64String() {}
static QoreValue f_makeBase64String(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1094 "ql_misc.qpp"

    return QoreValue();
}

// string makeHexString(string str) {}
static QoreValue f_makeHexString_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
# 2017 "ql_misc.qpp"
   QoreStringNode* rv = new QoreStringNode;
   rv->concatHex(str);
   return rv;
}

// string makeHexString(binary bin) {}
static QoreValue f_makeHexString_Vy(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* bin = HARD_QORE_VALUE_BINARY(args, 0);
# 2035 "ql_misc.qpp"
   QoreStringNode* str = new QoreStringNode;
   str->concatHex(bin);
   return str;
}

// nothing makeHexString() {}
static QoreValue f_makeHexString(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2044 "ql_misc.qpp"

    return QoreValue();
}

// string make_base64_string(string str, softint maxlinelen = -1) {}
static QoreValue f_make_base64_string_Vsvi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 maxlinelen = HARD_QORE_VALUE_INT(args, 1);
# 1116 "ql_misc.qpp"
   QoreStringNode* rv = new QoreStringNode;
   rv->concatBase64(str, maxlinelen);
   return rv;
}

// string make_base64_string(binary bin, softint maxlinelen = -1) {}
static QoreValue f_make_base64_string_Vyvi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* bin = HARD_QORE_VALUE_BINARY(args, 0);
    int64 maxlinelen = HARD_QORE_VALUE_INT(args, 1);
# 1141 "ql_misc.qpp"
   return new QoreStringNode(bin, maxlinelen);
}

// string make_hex_string(string str) {}
static QoreValue f_make_hex_string_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
# 2064 "ql_misc.qpp"
   QoreStringNode* rv = new QoreStringNode;
   rv->concatHex(str);
   return rv;
}

// string make_hex_string(binary bin) {}
static QoreValue f_make_hex_string_Vy(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* bin = HARD_QORE_VALUE_BINARY(args, 0);
# 2087 "ql_misc.qpp"
   QoreStringNode* str = new QoreStringNode;
   str->concatHex(bin);
   return str;
}

// *hash parse(string code, string label, *softint warning_mask, *string source, *softint offset, softbool format_label = True) {}
static QoreValue f_parse_VsVsniNsnivb(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* code = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* label = HARD_QORE_VALUE_STRING(args, 1);
    int64 warning_mask = HARD_QORE_VALUE_INT(args, 2);
    const QoreStringNode* source = get_param_value(args, 3).get<const QoreStringNode>();
    int64 offset = HARD_QORE_VALUE_INT(args, 4);
    bool format_label = HARD_QORE_VALUE_BOOL(args, 5);
# 889 "ql_misc.qpp"
   QoreStringMaker nlabel(label->getEncoding(), format_label ? "<run-time-loaded: %s>" : "%s", label->getBuffer());

   if (warning_mask) {
      ExceptionSink wsink;
      getProgram()->parse(code, &nlabel, xsink, &wsink, warning_mask, source, offset);
      if (!wsink.isException())
         return QoreValue();

      QoreException* e = wsink.catchException();
      return e->makeExceptionObjectAndDelete(xsink);
   }

   getProgram()->parse(code, &nlabel, xsink, 0, 0, source, offset);
    return QoreValue();
}

// nothing parse() {}
static QoreValue f_parse(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 908 "ql_misc.qpp"

    return QoreValue();
}

// binary parseBase64String(string str) {}
static QoreValue f_parseBase64String_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
# 1160 "ql_misc.qpp"
   return str->parseBase64(xsink);
}

// nothing parseBase64String() {}
static QoreValue f_parseBase64String(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1167 "ql_misc.qpp"

    return QoreValue();
}

// string parseBase64StringToString(string str, *string encoding) {}
static QoreValue f_parseBase64StringToString_VsNs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* encoding = get_param_value(args, 1).get<const QoreStringNode>();
# 1212 "ql_misc.qpp"
   const QoreEncoding *qe = encoding ? QEM.findCreate(encoding) : QCS_DEFAULT;
   return str->parseBase64ToString(qe, xsink);
}

// nothing parseBase64StringToString() {}
static QoreValue f_parseBase64StringToString(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1220 "ql_misc.qpp"

    return QoreValue();
}

// binary parseHexString(string hexstr) {}
static QoreValue f_parseHexString_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* hexstr = HARD_QORE_VALUE_STRING(args, 0);
# 2106 "ql_misc.qpp"
   return hexstr->parseHex(xsink);
}

// nothing parseHexString() {}
static QoreValue f_parseHexString(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2113 "ql_misc.qpp"

    return QoreValue();
}

// *hash parseURL(string url, bool keep_brackets = False) {}
static QoreValue f_parseURL_VsVb(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* url = HARD_QORE_VALUE_STRING(args, 0);
    bool keep_brackets = HARD_QORE_VALUE_BOOL(args, 1);
# 998 "ql_misc.qpp"
   QoreURL qurl(url, keep_brackets);
   return qurl.isValid() ? qurl.getHash() : QoreValue();
}

// nothing parseURL() {}
static QoreValue f_parseURL(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1006 "ql_misc.qpp"

    return QoreValue();
}

// binary parse_base64_string(string str) {}
static QoreValue f_parse_base64_string_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
# 1190 "ql_misc.qpp"
   return str->parseBase64(xsink);
}

// string parse_base64_string_to_string(string str, *string encoding) {}
static QoreValue f_parse_base64_string_to_string_VsNs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* encoding = get_param_value(args, 1).get<const QoreStringNode>();
# 1246 "ql_misc.qpp"
   const QoreEncoding *qe = encoding ? QEM.findCreate(encoding) : QCS_DEFAULT;
   return str->parseBase64ToString(qe, xsink);
}

// binary parse_hex_string(string hexstr) {}
static QoreValue f_parse_hex_string_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* hexstr = HARD_QORE_VALUE_STRING(args, 0);
# 2134 "ql_misc.qpp"
   return hexstr->parseHex(xsink);
}

// hash parse_url(string url, bool keep_brackets = False) {}
static QoreValue f_parse_url_VsVb(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* url = HARD_QORE_VALUE_STRING(args, 0);
    bool keep_brackets = HARD_QORE_VALUE_BOOL(args, 1);
# 969 "ql_misc.qpp"
   QoreURL qurl(url, keep_brackets, xsink);
   if (*xsink)
      return QoreValue();
   assert(qurl.isValid());
   return qurl.getHash();
}

// nothing reload_module(string name) {}
static QoreValue f_reload_module_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 2284 "ql_misc.qpp"
   QMM.runTimeLoadModule(*xsink, name->getBuffer(), 0, 0, QMLO_RELOAD);
    return QoreValue();
}

// nothing set_global_var_value(string name, auto value) {}
static QoreValue f_set_global_var_value_VsVA(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
    QoreValue value = get_param_value(args, 1);
# 2689 "ql_misc.qpp"
   getProgram()->setGlobalVarValue(name->c_str(), value.refSelf(), xsink);
    return QoreValue();
}

// nothing set_local_var_value(int frame, string var, auto value) {}
static QoreValue f_set_local_var_value_ViVsVA(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 frame = HARD_QORE_VALUE_INT(args, 0);
    const QoreStringNode* var = HARD_QORE_VALUE_STRING(args, 1);
    QoreValue value = get_param_value(args, 2);
# 2651 "ql_misc.qpp"
   if (thread_set_local_var_value(frame, var->c_str(), value, xsink) == 1 && thread_set_closure_var_value(frame, var->c_str(), value, xsink) == 1 && !xsink->isEvent())
      xsink->raiseException("UNKNOWN-VARIABLE", "cannot find local variable '%s' in stack frame %d", var->c_str(), (int)frame);
    return QoreValue();
}

// nothing set_return_value(auto val) {}
static QoreValue f_set_return_value_VA(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue val = get_param_value(args, 0);
# 2575 "ql_misc.qpp"
   qore_program_private::setReturnValue(*getProgram(), val.getReferencedValue(), xsink);
    return QoreValue();
}

// string splice(string str) {}
static QoreValue f_splice_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1909 "ql_misc.qpp"
   return new QoreStringNode;
}

// string splice(string str, softint start) {}
static QoreValue f_splice_Vsvi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 start = HARD_QORE_VALUE_INT(args, 1);
# 1928 "ql_misc.qpp"
   QoreStringNodeHolder pstr(str->copy());
   pstr->splice(start, xsink);
   return *xsink ? QoreValue() : pstr.release();
}

// string splice(string str, softint start, softint len, *string nstr) {}
static QoreValue f_splice_VsviviNs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 start = HARD_QORE_VALUE_INT(args, 1);
    int64 len = HARD_QORE_VALUE_INT(args, 2);
    const QoreStringNode* nstr = get_param_value(args, 3).get<const QoreStringNode>();
# 1951 "ql_misc.qpp"
   QoreStringNodeHolder pstr(str->copy());
   pstr->splice(start, len, nstr, xsink);
   return *xsink ? QoreValue() : pstr.release();
}

// list splice(list l, softint start) {}
static QoreValue f_splice_Vlvi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
    int64 start = HARD_QORE_VALUE_INT(args, 1);
# 1970 "ql_misc.qpp"
   ReferenceHolder<QoreListNode> lst(l->copy(), xsink);
   lst->splice(start, xsink);
   return *xsink ? QoreValue() : lst.release();
}

// list splice(list l, softint start, softint len, *softlist nlist) {}
static QoreValue f_splice_Vlvivinl(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
    int64 start = HARD_QORE_VALUE_INT(args, 1);
    int64 len = HARD_QORE_VALUE_INT(args, 2);
    const QoreListNode* nlist = get_param_value(args, 3).get<const QoreListNode>();
# 1993 "ql_misc.qpp"
   ReferenceHolder<QoreListNode> lst(l->copy(), xsink);
   lst->splice(start, len, nlist, xsink);
   return *xsink ? QoreValue() : lst.release();
}

// nothing splice() {}
static QoreValue f_splice(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2002 "ql_misc.qpp"

    return QoreValue();
}

// int strtoint(string num, softint base = 10) {}
static QoreValue f_strtoint_Vsvi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* num = HARD_QORE_VALUE_STRING(args, 0);
    int64 base = HARD_QORE_VALUE_INT(args, 1);
# 2201 "ql_misc.qpp"
   errno = 0;
   int64 rv = strtoll(num->getBuffer(), 0, base);
   if (errno == EINVAL || errno == ERANGE)
      xsink->raiseException("STRTOINT-ERROR", "could not parse '%s' to an integer", num->getBuffer());
   return rv;
}

// nothing strtoint() {}
static QoreValue f_strtoint(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2212 "ql_misc.qpp"

    return QoreValue();
}


DLLLOCAL void init_misc_functions(QoreNamespace& ns) {
    // nothing remove_signal_handler(softint signal) {}
    ns.addBuiltinVariant("remove_signal_handler", (q_func_n_t)f_remove_signal_handler_vi, QC_NO_FLAGS, QDOM_PROCESS, nothingTypeInfo, 1, softBigIntTypeInfo, NULL, "signal");
    // nothing set_signal_handler(softint signal, code f) {}
    ns.addBuiltinVariant("set_signal_handler", (q_func_n_t)f_set_signal_handler_viVc, QC_NO_FLAGS, QDOM_PROCESS, nothingTypeInfo, 2, softBigIntTypeInfo, NULL, "signal", codeTypeInfo, NULL, "f");
    // string backquote(string cmd, *reference<int> rc) {}
    ns.addBuiltinVariant("backquote", (q_func_n_t)f_backquote_VsC15_reference_int_, QC_NO_FLAGS, QDOM_EXTERNAL_PROCESS, stringTypeInfo, 2, stringTypeInfo, NULL, "cmd", qore_get_complex_reference_or_nothing_type(bigIntTypeInfo), NULL, "rc");
    // nothing backquote() {}
    ns.addBuiltinVariant("backquote", (q_func_n_t)f_backquote, QC_RUNTIME_NOOP, QDOM_EXTERNAL_PROCESS, nothingTypeInfo);
    // auto call_builtin_function(string name, ...) {}
    ns.addBuiltinVariant("call_builtin_function", (q_func_n_t)f_call_builtin_function_VsVV, QC_USES_EXTRA_ARGS, QDOM_DEFAULT, autoTypeInfo, 1, stringTypeInfo, NULL, "name");
    // auto call_builtin_function_args(string name, *softlist<auto> vargs) {}
    ns.addBuiltinVariant("call_builtin_function_args", (q_func_n_t)f_call_builtin_function_args_VsC15_softlist_auto_, QC_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 2, stringTypeInfo, NULL, "name", softAutoListOrNothingTypeInfo, NULL, "vargs");
    // auto call_function(string name, ...) {}
    ns.addBuiltinVariant("call_function", (q_func_n_t)f_call_function_VsVV, QC_USES_EXTRA_ARGS, QDOM_DEFAULT, autoTypeInfo, 1, stringTypeInfo, NULL, "name");
    // auto call_function(code f, ...) {}
    ns.addBuiltinVariant("call_function", (q_func_n_t)f_call_function_VcVV, QC_USES_EXTRA_ARGS, QDOM_DEFAULT, autoTypeInfo, 1, codeTypeInfo, NULL, "f");
    // auto call_function_args(string name, *softlist<auto> vargs) {}
    ns.addBuiltinVariant("call_function_args", (q_func_n_t)f_call_function_args_VsC15_softlist_auto_, QC_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 2, stringTypeInfo, NULL, "name", softAutoListOrNothingTypeInfo, NULL, "vargs");
    // auto call_function_args(code f, *softlist<auto> vargs) {}
    ns.addBuiltinVariant("call_function_args", (q_func_n_t)f_call_function_args_VcC15_softlist_auto_, QC_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 2, codeTypeInfo, NULL, "f", softAutoListOrNothingTypeInfo, NULL, "vargs");
    // string decode_uri_request(string uri) {}
    ns.addBuiltinVariant("decode_uri_request", (q_func_n_t)f_decode_uri_request_Vs, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, NULL, "uri");
    // string decode_url(string url) {}
    ns.addBuiltinVariant("decode_url", (q_func_n_t)f_decode_url_Vs, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, NULL, "url");
    // nothing decode_url() {}
    ns.addBuiltinVariant("decode_url", (q_func_n_t)f_decode_url, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // string encode_uri_request(string url) {}
    ns.addBuiltinVariant("encode_uri_request", (q_func_n_t)f_encode_uri_request_Vs, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, NULL, "url");
    // string encode_url(string url, softbool encode_all = False) {}
    ns.addBuiltinVariant("encode_url", (q_func_n_t)f_encode_url_Vsvb, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 2, stringTypeInfo, NULL, "url", softBoolTypeInfo, &False, "encode_all");
    // bool exists(...) {}
    ns.addBuiltinVariant("exists", (q_func_n_t)f_exists_VV, QC_CONSTANT|QC_USES_EXTRA_ARGS, QDOM_DEFAULT, boolTypeInfo);
    // bool existsFunction(string name) {}
    ns.addBuiltinVariant("existsFunction", (q_func_n_t)f_existsFunction_Vs, QC_CONSTANT|QC_DEPRECATED, QDOM_DEFAULT, boolTypeInfo, 1, stringTypeInfo, NULL, "name");
    // bool existsFunction(code c) {}
    ns.addBuiltinVariant("existsFunction", (q_func_n_t)f_existsFunction_Vc, QC_NOOP|QC_DEPRECATED, QDOM_DEFAULT, boolTypeInfo, 1, codeTypeInfo, NULL, "c");
    // nothing existsFunction() {}
    ns.addBuiltinVariant("existsFunction", (q_func_n_t)f_existsFunction, QC_RUNTIME_NOOP|QC_DEPRECATED, QDOM_DEFAULT, nothingTypeInfo);
    // bool exists_function(string name) {}
    ns.addBuiltinVariant("exists_function", (q_func_n_t)f_exists_function_Vs, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo, 1, stringTypeInfo, NULL, "name");
    // bool exists_function(code c) {}
    ns.addBuiltinVariant("exists_function", (q_func_n_t)f_exists_function_Vc, QC_NOOP, QDOM_DEFAULT, boolTypeInfo, 1, codeTypeInfo, NULL, "c");
    // *string functionType(string name) {}
    ns.addBuiltinVariant("functionType", (q_func_n_t)f_functionType_Vs, QC_CONSTANT|QC_DEPRECATED, QDOM_DEFAULT, stringOrNothingTypeInfo, 1, stringTypeInfo, NULL, "name");
    // nothing functionType() {}
    ns.addBuiltinVariant("functionType", (q_func_n_t)f_functionType, QC_RUNTIME_NOOP|QC_DEPRECATED, QDOM_DEFAULT, nothingTypeInfo);
    // *string function_type(string name) {}
    ns.addBuiltinVariant("function_type", (q_func_n_t)f_function_type_Vs, QC_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo, 1, stringTypeInfo, NULL, "name");
    // *int getByte(string str, softint offset = 0) {}
    ns.addBuiltinVariant("getByte", (q_func_n_t)f_getByte_Vsvi, QC_CONSTANT|QC_DEPRECATED, QDOM_DEFAULT, bigIntOrNothingTypeInfo, 2, stringTypeInfo, NULL, "str", softBigIntTypeInfo, zero(), "offset");
    // nothing getByte() {}
    ns.addBuiltinVariant("getByte", (q_func_n_t)f_getByte, QC_RUNTIME_NOOP|QC_DEPRECATED, QDOM_DEFAULT, nothingTypeInfo);
    // *int getByte(binary b, softint offset = 0) {}
    ns.addBuiltinVariant("getByte", (q_func_n_t)f_getByte_Vyvi, QC_CONSTANT|QC_DEPRECATED, QDOM_DEFAULT, bigIntOrNothingTypeInfo, 2, binaryTypeInfo, NULL, "b", softBigIntTypeInfo, zero(), "offset");
    // string getClassName(object obj) {}
    ns.addBuiltinVariant("getClassName", (q_func_n_t)f_getClassName_Vo, QC_CONSTANT|QC_DEPRECATED, QDOM_DEFAULT, stringTypeInfo, 1, objectTypeInfo, NULL, "obj");
    // nothing getClassName() {}
    ns.addBuiltinVariant("getClassName", (q_func_n_t)f_getClassName, QC_RUNTIME_NOOP|QC_DEPRECATED, QDOM_DEFAULT, nothingTypeInfo);
    // list getFeatureList() {}
    ns.addBuiltinVariant("getFeatureList", (q_func_n_t)f_getFeatureList, QC_DEPRECATED, QDOM_DEFAULT, listTypeInfo);
    // hash<string,hash> getModuleHash() {}
    ns.addBuiltinVariant("getModuleHash", (q_func_n_t)f_getModuleHash, QC_DEPRECATED, QDOM_DEFAULT, qore_get_complex_hash_type(hashTypeInfo));
    // list<hash> getModuleList() {}
    ns.addBuiltinVariant("getModuleList", (q_func_n_t)f_getModuleList, QC_DEPRECATED, QDOM_DEFAULT, qore_get_complex_list_type(hashTypeInfo));
    // *int getWord32(string str, softint offset = 0) {}
    ns.addBuiltinVariant("getWord32", (q_func_n_t)f_getWord32_Vsvi, QC_CONSTANT|QC_DEPRECATED, QDOM_DEFAULT, bigIntOrNothingTypeInfo, 2, stringTypeInfo, NULL, "str", softBigIntTypeInfo, zero(), "offset");
    // *int getWord32(binary b, softint offset = 0) {}
    ns.addBuiltinVariant("getWord32", (q_func_n_t)f_getWord32_Vyvi, QC_CONSTANT|QC_DEPRECATED, QDOM_DEFAULT, bigIntOrNothingTypeInfo, 2, binaryTypeInfo, NULL, "b", softBigIntTypeInfo, zero(), "offset");
    // nothing getWord32() {}
    ns.addBuiltinVariant("getWord32", (q_func_n_t)f_getWord32, QC_RUNTIME_NOOP|QC_DEPRECATED, QDOM_DEFAULT, nothingTypeInfo);
    // *int get_byte(string str, softint offset = 0) {}
    ns.addBuiltinVariant("get_byte", (q_func_n_t)f_get_byte_Vsvi, QC_CONSTANT, QDOM_DEFAULT, bigIntOrNothingTypeInfo, 2, stringTypeInfo, NULL, "str", softBigIntTypeInfo, zero(), "offset");
    // *int get_byte(binary b, softint offset = 0) {}
    ns.addBuiltinVariant("get_byte", (q_func_n_t)f_get_byte_Vyvi, QC_CONSTANT, QDOM_DEFAULT, bigIntOrNothingTypeInfo, 2, binaryTypeInfo, NULL, "b", softBigIntTypeInfo, zero(), "offset");
    // string get_class_name(object obj) {}
    ns.addBuiltinVariant("get_class_name", (q_func_n_t)f_get_class_name_Vo, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, objectTypeInfo, NULL, "obj");
    // string get_default_encoding() {}
    ns.addBuiltinVariant("get_default_encoding", (q_func_n_t)f_get_default_encoding, QC_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo);
    // string get_ex_pos(hash ex) {}
    ns.addBuiltinVariant("get_ex_pos", (q_func_n_t)f_get_ex_pos_Vh, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, hashTypeInfo, NULL, "ex");
    // list get_feature_list() {}
    ns.addBuiltinVariant("get_feature_list", (q_func_n_t)f_get_feature_list, QC_NO_FLAGS, QDOM_DEFAULT, listTypeInfo);
    // hash get_global_vars() {}
    ns.addBuiltinVariant("get_global_vars", (q_func_n_t)f_get_global_vars, QC_CONSTANT, QDOM_DEFAULT, hashTypeInfo);
    // hash get_local_vars(int frame) {}
    ns.addBuiltinVariant("get_local_vars", (q_func_n_t)f_get_local_vars_Vi, QC_RET_VALUE_ONLY, QDOM_DEBUGGER, hashTypeInfo, 1, bigIntTypeInfo, NULL, "frame");
    // hash<string,hash> get_module_hash() {}
    ns.addBuiltinVariant("get_module_hash", (q_func_n_t)f_get_module_hash, QC_NO_FLAGS, QDOM_DEFAULT, qore_get_complex_hash_type(hashTypeInfo));
    // list<hash> get_module_list() {}
    ns.addBuiltinVariant("get_module_list", (q_func_n_t)f_get_module_list, QC_NO_FLAGS, QDOM_DEFAULT, qore_get_complex_list_type(hashTypeInfo));
    // int get_parse_options() {}
    ns.addBuiltinVariant("get_parse_options", (q_func_n_t)f_get_parse_options, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);
    // hash get_qore_library_info() {}
    ns.addBuiltinVariant("get_qore_library_info", (q_func_n_t)f_get_qore_library_info, QC_CONSTANT, QDOM_DEFAULT, hashTypeInfo);
    // hash get_qore_option_hash() {}
    ns.addBuiltinVariant("get_qore_option_hash", (q_func_n_t)f_get_qore_option_hash, QC_CONSTANT, QDOM_DEFAULT, hashTypeInfo);
    // list get_qore_option_list() {}
    ns.addBuiltinVariant("get_qore_option_list", (q_func_n_t)f_get_qore_option_list, QC_CONSTANT, QDOM_DEFAULT, listTypeInfo);
    // *string get_script_dir() {}
    ns.addBuiltinVariant("get_script_dir", (q_func_n_t)f_get_script_dir, QC_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo);
    // *string get_script_name() {}
    ns.addBuiltinVariant("get_script_name", (q_func_n_t)f_get_script_name, QC_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo);
    // *string get_script_path() {}
    ns.addBuiltinVariant("get_script_path", (q_func_n_t)f_get_script_path, QC_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo);
    // *int get_word_16(string str, softint offset = 0) {}
    ns.addBuiltinVariant("get_word_16", (q_func_n_t)f_get_word_16_Vsvi, QC_CONSTANT, QDOM_DEFAULT, bigIntOrNothingTypeInfo, 2, stringTypeInfo, NULL, "str", softBigIntTypeInfo, zero(), "offset");
    // *int get_word_16(binary b, softint offset = 0) {}
    ns.addBuiltinVariant("get_word_16", (q_func_n_t)f_get_word_16_Vyvi, QC_CONSTANT, QDOM_DEFAULT, bigIntOrNothingTypeInfo, 2, binaryTypeInfo, NULL, "b", softBigIntTypeInfo, zero(), "offset");
    // *int get_word_16_lsb(string str, softint offset = 0) {}
    ns.addBuiltinVariant("get_word_16_lsb", (q_func_n_t)f_get_word_16_lsb_Vsvi, QC_CONSTANT, QDOM_DEFAULT, bigIntOrNothingTypeInfo, 2, stringTypeInfo, NULL, "str", softBigIntTypeInfo, zero(), "offset");
    // *int get_word_16_lsb(binary b, softint offset = 0) {}
    ns.addBuiltinVariant("get_word_16_lsb", (q_func_n_t)f_get_word_16_lsb_Vyvi, QC_CONSTANT, QDOM_DEFAULT, bigIntOrNothingTypeInfo, 2, binaryTypeInfo, NULL, "b", softBigIntTypeInfo, zero(), "offset");
    // *int get_word_32(string str, softint offset = 0) {}
    ns.addBuiltinVariant("get_word_32", (q_func_n_t)f_get_word_32_Vsvi, QC_CONSTANT, QDOM_DEFAULT, bigIntOrNothingTypeInfo, 2, stringTypeInfo, NULL, "str", softBigIntTypeInfo, zero(), "offset");
    // *int get_word_32(binary b, softint offset = 0) {}
    ns.addBuiltinVariant("get_word_32", (q_func_n_t)f_get_word_32_Vyvi, QC_CONSTANT, QDOM_DEFAULT, bigIntOrNothingTypeInfo, 2, binaryTypeInfo, NULL, "b", softBigIntTypeInfo, zero(), "offset");
    // *int get_word_32_lsb(string str, softint offset = 0) {}
    ns.addBuiltinVariant("get_word_32_lsb", (q_func_n_t)f_get_word_32_lsb_Vsvi, QC_CONSTANT, QDOM_DEFAULT, bigIntOrNothingTypeInfo, 2, stringTypeInfo, NULL, "str", softBigIntTypeInfo, zero(), "offset");
    // *int get_word_32_lsb(binary b, softint offset = 0) {}
    ns.addBuiltinVariant("get_word_32_lsb", (q_func_n_t)f_get_word_32_lsb_Vyvi, QC_CONSTANT, QDOM_DEFAULT, bigIntOrNothingTypeInfo, 2, binaryTypeInfo, NULL, "b", softBigIntTypeInfo, zero(), "offset");
    // *int get_word_64(string str, softint offset = 0) {}
    ns.addBuiltinVariant("get_word_64", (q_func_n_t)f_get_word_64_Vsvi, QC_CONSTANT, QDOM_DEFAULT, bigIntOrNothingTypeInfo, 2, stringTypeInfo, NULL, "str", softBigIntTypeInfo, zero(), "offset");
    // *int get_word_64(binary b, softint offset = 0) {}
    ns.addBuiltinVariant("get_word_64", (q_func_n_t)f_get_word_64_Vyvi, QC_CONSTANT, QDOM_DEFAULT, bigIntOrNothingTypeInfo, 2, binaryTypeInfo, NULL, "b", softBigIntTypeInfo, zero(), "offset");
    // *int get_word_64_lsb(string str, softint offset = 0) {}
    ns.addBuiltinVariant("get_word_64_lsb", (q_func_n_t)f_get_word_64_lsb_Vsvi, QC_CONSTANT, QDOM_DEFAULT, bigIntOrNothingTypeInfo, 2, stringTypeInfo, NULL, "str", softBigIntTypeInfo, zero(), "offset");
    // *int get_word_64_lsb(binary b, softint offset = 0) {}
    ns.addBuiltinVariant("get_word_64_lsb", (q_func_n_t)f_get_word_64_lsb_Vyvi, QC_CONSTANT, QDOM_DEFAULT, bigIntOrNothingTypeInfo, 2, binaryTypeInfo, NULL, "b", softBigIntTypeInfo, zero(), "offset");
    // bool has_key(hash h, string key) {}
    ns.addBuiltinVariant("has_key", (q_func_n_t)f_has_key_VhVs, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo, 2, hashTypeInfo, NULL, "h", stringTypeInfo, NULL, "key");
    // bool has_key(object obj, string key) {}
    ns.addBuiltinVariant("has_key", (q_func_n_t)f_has_key_VoVs, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo, 2, objectTypeInfo, NULL, "obj", stringTypeInfo, NULL, "key");
    // list hash_values(hash h) {}
    ns.addBuiltinVariant("hash_values", (q_func_n_t)f_hash_values_Vh, QC_CONSTANT, QDOM_DEFAULT, listTypeInfo, 1, hashTypeInfo, NULL, "h");
    // nothing hash_values() {}
    ns.addBuiltinVariant("hash_values", (q_func_n_t)f_hash_values, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // int hextoint(string str) {}
    ns.addBuiltinVariant("hextoint", (q_func_n_t)f_hextoint_Vs, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, stringTypeInfo, NULL, "str");
    // nothing hextoint() {}
    ns.addBuiltinVariant("hextoint", (q_func_n_t)f_hextoint, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // string html_decode(string str) {}
    ns.addBuiltinVariant("html_decode", (q_func_n_t)f_html_decode_Vs, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, NULL, "str");
    // nothing html_decode() {}
    ns.addBuiltinVariant("html_decode", (q_func_n_t)f_html_decode, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // string html_encode(string str) {}
    ns.addBuiltinVariant("html_encode", (q_func_n_t)f_html_encode_Vs, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, NULL, "str");
    // nothing html_encode() {}
    ns.addBuiltinVariant("html_encode", (q_func_n_t)f_html_encode, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // nothing load_module(string name) {}
    ns.addBuiltinVariant("load_module", (q_func_n_t)f_load_module_Vs, QC_NO_FLAGS, QDOM_MODULES, nothingTypeInfo, 1, stringTypeInfo, NULL, "name");
    // nothing load_module() {}
    ns.addBuiltinVariant("load_module", (q_func_n_t)f_load_module, QC_RUNTIME_NOOP, QDOM_MODULES, nothingTypeInfo);
    // nothing load_user_module_with_program(string name, Qore::Program pgm) {}
    ns.addBuiltinVariant("load_user_module_with_program", (q_func_n_t)f_load_user_module_with_program_VsC7Program, QC_NO_FLAGS, QDOM_MODULES, nothingTypeInfo, 2, stringTypeInfo, NULL, "name", QC_PROGRAM->getTypeInfo(), NULL, "pgm");
    // string makeBase64String(string str, softint maxlinelen = -1) {}
    ns.addBuiltinVariant("makeBase64String", (q_func_n_t)f_makeBase64String_Vsvi, QC_CONSTANT|QC_DEPRECATED, QDOM_DEFAULT, stringTypeInfo, 2, stringTypeInfo, NULL, "str", softBigIntTypeInfo, new QoreBigIntNode(-1), "maxlinelen");
    // string makeBase64String(binary bin, softint maxlinelen = -1) {}
    ns.addBuiltinVariant("makeBase64String", (q_func_n_t)f_makeBase64String_Vyvi, QC_CONSTANT|QC_DEPRECATED, QDOM_DEFAULT, stringTypeInfo, 2, binaryTypeInfo, NULL, "bin", softBigIntTypeInfo, new QoreBigIntNode(-1), "maxlinelen");
    // nothing makeBase64String() {}
    ns.addBuiltinVariant("makeBase64String", (q_func_n_t)f_makeBase64String, QC_RUNTIME_NOOP|QC_DEPRECATED, QDOM_DEFAULT, nothingTypeInfo);
    // string makeHexString(string str) {}
    ns.addBuiltinVariant("makeHexString", (q_func_n_t)f_makeHexString_Vs, QC_CONSTANT|QC_DEPRECATED, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, NULL, "str");
    // string makeHexString(binary bin) {}
    ns.addBuiltinVariant("makeHexString", (q_func_n_t)f_makeHexString_Vy, QC_CONSTANT|QC_DEPRECATED, QDOM_DEFAULT, stringTypeInfo, 1, binaryTypeInfo, NULL, "bin");
    // nothing makeHexString() {}
    ns.addBuiltinVariant("makeHexString", (q_func_n_t)f_makeHexString, QC_RUNTIME_NOOP|QC_DEPRECATED, QDOM_DEFAULT, nothingTypeInfo);
    // string make_base64_string(string str, softint maxlinelen = -1) {}
    ns.addBuiltinVariant("make_base64_string", (q_func_n_t)f_make_base64_string_Vsvi, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 2, stringTypeInfo, NULL, "str", softBigIntTypeInfo, new QoreBigIntNode(-1), "maxlinelen");
    // string make_base64_string(binary bin, softint maxlinelen = -1) {}
    ns.addBuiltinVariant("make_base64_string", (q_func_n_t)f_make_base64_string_Vyvi, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 2, binaryTypeInfo, NULL, "bin", softBigIntTypeInfo, new QoreBigIntNode(-1), "maxlinelen");
    // string make_hex_string(string str) {}
    ns.addBuiltinVariant("make_hex_string", (q_func_n_t)f_make_hex_string_Vs, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, NULL, "str");
    // string make_hex_string(binary bin) {}
    ns.addBuiltinVariant("make_hex_string", (q_func_n_t)f_make_hex_string_Vy, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, binaryTypeInfo, NULL, "bin");
    // *hash parse(string code, string label, *softint warning_mask, *string source, *softint offset, softbool format_label = True) {}
    ns.addBuiltinVariant("parse", (q_func_n_t)f_parse_VsVsniNsnivb, QC_NO_FLAGS, QDOM_EMBEDDED_LOGIC|QDOM_IN_MODULE, hashOrNothingTypeInfo, 6, stringTypeInfo, NULL, "code", stringTypeInfo, NULL, "label", softBigIntOrNothingTypeInfo, NULL, "warning_mask", stringOrNothingTypeInfo, NULL, "source", softBigIntOrNothingTypeInfo, NULL, "offset", softBoolTypeInfo, &True, "format_label");
    // nothing parse() {}
    ns.addBuiltinVariant("parse", (q_func_n_t)f_parse, QC_RUNTIME_NOOP, QDOM_EMBEDDED_LOGIC|QDOM_IN_MODULE, nothingTypeInfo);
    // binary parseBase64String(string str) {}
    ns.addBuiltinVariant("parseBase64String", (q_func_n_t)f_parseBase64String_Vs, QC_RET_VALUE_ONLY|QC_DEPRECATED, QDOM_DEFAULT, binaryTypeInfo, 1, stringTypeInfo, NULL, "str");
    // nothing parseBase64String() {}
    ns.addBuiltinVariant("parseBase64String", (q_func_n_t)f_parseBase64String, QC_RUNTIME_NOOP|QC_DEPRECATED, QDOM_DEFAULT, nothingTypeInfo);
    // string parseBase64StringToString(string str, *string encoding) {}
    ns.addBuiltinVariant("parseBase64StringToString", (q_func_n_t)f_parseBase64StringToString_VsNs, QC_RET_VALUE_ONLY|QC_DEPRECATED, QDOM_DEFAULT, stringTypeInfo, 2, stringTypeInfo, NULL, "str", stringOrNothingTypeInfo, NULL, "encoding");
    // nothing parseBase64StringToString() {}
    ns.addBuiltinVariant("parseBase64StringToString", (q_func_n_t)f_parseBase64StringToString, QC_RUNTIME_NOOP|QC_DEPRECATED, QDOM_DEFAULT, nothingTypeInfo);
    // binary parseHexString(string hexstr) {}
    ns.addBuiltinVariant("parseHexString", (q_func_n_t)f_parseHexString_Vs, QC_RET_VALUE_ONLY|QC_DEPRECATED, QDOM_DEFAULT, binaryTypeInfo, 1, stringTypeInfo, NULL, "hexstr");
    // nothing parseHexString() {}
    ns.addBuiltinVariant("parseHexString", (q_func_n_t)f_parseHexString, QC_RUNTIME_NOOP|QC_DEPRECATED, QDOM_DEFAULT, nothingTypeInfo);
    // *hash parseURL(string url, bool keep_brackets = False) {}
    ns.addBuiltinVariant("parseURL", (q_func_n_t)f_parseURL_VsVb, QC_CONSTANT, QDOM_DEFAULT, hashOrNothingTypeInfo, 2, stringTypeInfo, NULL, "url", boolTypeInfo, &False, "keep_brackets");
    // nothing parseURL() {}
    ns.addBuiltinVariant("parseURL", (q_func_n_t)f_parseURL, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // binary parse_base64_string(string str) {}
    ns.addBuiltinVariant("parse_base64_string", (q_func_n_t)f_parse_base64_string_Vs, QC_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 1, stringTypeInfo, NULL, "str");
    // string parse_base64_string_to_string(string str, *string encoding) {}
    ns.addBuiltinVariant("parse_base64_string_to_string", (q_func_n_t)f_parse_base64_string_to_string_VsNs, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 2, stringTypeInfo, NULL, "str", stringOrNothingTypeInfo, NULL, "encoding");
    // binary parse_hex_string(string hexstr) {}
    ns.addBuiltinVariant("parse_hex_string", (q_func_n_t)f_parse_hex_string_Vs, QC_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 1, stringTypeInfo, NULL, "hexstr");
    // hash parse_url(string url, bool keep_brackets = False) {}
    ns.addBuiltinVariant("parse_url", (q_func_n_t)f_parse_url_VsVb, QC_NO_FLAGS, QDOM_DEFAULT, hashTypeInfo, 2, stringTypeInfo, NULL, "url", boolTypeInfo, &False, "keep_brackets");
    // nothing reload_module(string name) {}
    ns.addBuiltinVariant("reload_module", (q_func_n_t)f_reload_module_Vs, QC_NO_FLAGS, QDOM_MODULES|QDOM_INJECTION, nothingTypeInfo, 1, stringTypeInfo, NULL, "name");
    // nothing set_global_var_value(string name, auto value) {}
    ns.addBuiltinVariant("set_global_var_value", (q_func_n_t)f_set_global_var_value_VsVA, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, NULL, "name", autoTypeInfo, NULL, "value");
    // nothing set_local_var_value(int frame, string var, auto value) {}
    ns.addBuiltinVariant("set_local_var_value", (q_func_n_t)f_set_local_var_value_ViVsVA, QC_NO_FLAGS, QDOM_DEBUGGER, nothingTypeInfo, 3, bigIntTypeInfo, NULL, "frame", stringTypeInfo, NULL, "var", autoTypeInfo, NULL, "value");
    // nothing set_return_value(auto val) {}
    ns.addBuiltinVariant("set_return_value", (q_func_n_t)f_set_return_value_VA, QC_NO_FLAGS, QDOM_PROCESS, nothingTypeInfo, 1, autoTypeInfo, NULL, "val");
    // string splice(string str) {}
    ns.addBuiltinVariant("splice", (q_func_n_t)f_splice_Vs, QC_NOOP, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, NULL, "str");
    // string splice(string str, softint start) {}
    ns.addBuiltinVariant("splice", (q_func_n_t)f_splice_Vsvi, QC_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 2, stringTypeInfo, NULL, "str", softBigIntTypeInfo, NULL, "start");
    // string splice(string str, softint start, softint len, *string nstr) {}
    ns.addBuiltinVariant("splice", (q_func_n_t)f_splice_VsviviNs, QC_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 4, stringTypeInfo, NULL, "str", softBigIntTypeInfo, NULL, "start", softBigIntTypeInfo, NULL, "len", stringOrNothingTypeInfo, NULL, "nstr");
    // list splice(list l, softint start) {}
    ns.addBuiltinVariant("splice", (q_func_n_t)f_splice_Vlvi, QC_NO_FLAGS, QDOM_DEFAULT, listTypeInfo, 2, listTypeInfo, NULL, "l", softBigIntTypeInfo, NULL, "start");
    // list splice(list l, softint start, softint len, *softlist nlist) {}
    ns.addBuiltinVariant("splice", (q_func_n_t)f_splice_Vlvivinl, QC_NO_FLAGS, QDOM_DEFAULT, listTypeInfo, 4, listTypeInfo, NULL, "l", softBigIntTypeInfo, NULL, "start", softBigIntTypeInfo, NULL, "len", softListOrNothingTypeInfo, NULL, "nlist");
    // nothing splice() {}
    ns.addBuiltinVariant("splice", (q_func_n_t)f_splice, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // int strtoint(string num, softint base = 10) {}
    ns.addBuiltinVariant("strtoint", (q_func_n_t)f_strtoint_Vsvi, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 2, stringTypeInfo, NULL, "num", softBigIntTypeInfo, new QoreBigIntNode(10), "base");
    // nothing strtoint() {}
    ns.addBuiltinVariant("strtoint", (q_func_n_t)f_strtoint, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);

}

DLLLOCAL void init_misc_constants(QoreNamespace& ns) {
    ns.addConstant("CE_ALL", new QoreBigIntNode(CE_ALL));
    ns.addConstant("CE_HTML", new QoreBigIntNode(CE_HTML));
    ns.addConstant("CE_NONASCII", new QoreBigIntNode(CE_NONASCII));
    ns.addConstant("CE_XHTML", new QoreBigIntNode(CE_XHTML));
    ns.addConstant("CE_XML", new QoreBigIntNode(CE_XML));
    ns.addConstant("CD_ALL", new QoreBigIntNode(CD_ALL));
    ns.addConstant("CD_HTML", new QoreBigIntNode(CD_HTML));
    ns.addConstant("CD_NUM_REF", new QoreBigIntNode(CD_NUM_REF));
    ns.addConstant("CD_XHTML", new QoreBigIntNode(CD_XHTML));
    ns.addConstant("CD_XML", new QoreBigIntNode(CD_XML));
# 285 "ql_misc.qpp"
# 380 "ql_misc.qpp"
# 429 "ql_misc.qpp"
    QoreHashNode* hash_NameToSignal = new QoreHashNode;
    hash_NameToSignal->setKeyValue("SIGABRT", new QoreBigIntNode(SIGABRT), 0);
    hash_NameToSignal->setKeyValue("SIGALRM", new QoreBigIntNode(SIGALRM), 0);
    hash_NameToSignal->setKeyValue("SIGBUS", new QoreBigIntNode(SIGBUS), 0);
    hash_NameToSignal->setKeyValue("SIGCANCEL", new QoreBigIntNode(SIGCANCEL), 0);
    hash_NameToSignal->setKeyValue("SIGCHLD", new QoreBigIntNode(SIGCHLD), 0);
    hash_NameToSignal->setKeyValue("SIGCLD", new QoreBigIntNode(SIGCLD), 0);
    hash_NameToSignal->setKeyValue("SIGCONT", new QoreBigIntNode(SIGCONT), 0);
    hash_NameToSignal->setKeyValue("SIGEMT", new QoreBigIntNode(SIGEMT), 0);
    hash_NameToSignal->setKeyValue("SIGFPE", new QoreBigIntNode(SIGFPE), 0);
    hash_NameToSignal->setKeyValue("SIGFREEZE", new QoreBigIntNode(SIGFREEZE), 0);
    hash_NameToSignal->setKeyValue("SIGHUP", new QoreBigIntNode(SIGHUP), 0);
    hash_NameToSignal->setKeyValue("SIGILL", new QoreBigIntNode(SIGILL), 0);
    hash_NameToSignal->setKeyValue("SIGINFO", new QoreBigIntNode(SIGINFO), 0);
    hash_NameToSignal->setKeyValue("SIGINT", new QoreBigIntNode(SIGINT), 0);
    hash_NameToSignal->setKeyValue("SIGIO", new QoreBigIntNode(SIGIO), 0);
    hash_NameToSignal->setKeyValue("SIGIOT", new QoreBigIntNode(SIGIOT), 0);
    hash_NameToSignal->setKeyValue("SIGJVM1", new QoreBigIntNode(SIGJVM1), 0);
    hash_NameToSignal->setKeyValue("SIGJVM2", new QoreBigIntNode(SIGJVM2), 0);
    hash_NameToSignal->setKeyValue("SIGKILL", new QoreBigIntNode(SIGKILL), 0);
    hash_NameToSignal->setKeyValue("SIGLOST", new QoreBigIntNode(SIGLOST), 0);
    hash_NameToSignal->setKeyValue("SIGLWP", new QoreBigIntNode(SIGLWP), 0);
    hash_NameToSignal->setKeyValue("SIGPIPE", new QoreBigIntNode(SIGPIPE), 0);
    hash_NameToSignal->setKeyValue("SIGPOLL", new QoreBigIntNode(SIGPOLL), 0);
    hash_NameToSignal->setKeyValue("SIGPROF", new QoreBigIntNode(SIGPROF), 0);
    hash_NameToSignal->setKeyValue("SIGPWR", new QoreBigIntNode(SIGPWR), 0);
    hash_NameToSignal->setKeyValue("SIGQUIT", new QoreBigIntNode(SIGQUIT), 0);
    hash_NameToSignal->setKeyValue("SIGSEGV", new QoreBigIntNode(SIGSEGV), 0);
    hash_NameToSignal->setKeyValue("SIGSTKFLT", new QoreBigIntNode(SIGSTKFLT), 0);
    hash_NameToSignal->setKeyValue("SIGSTOP", new QoreBigIntNode(SIGSTOP), 0);
    hash_NameToSignal->setKeyValue("SIGSYS", new QoreBigIntNode(SIGSYS), 0);
    hash_NameToSignal->setKeyValue("SIGTERM", new QoreBigIntNode(SIGTERM), 0);
    hash_NameToSignal->setKeyValue("SIGTHAW", new QoreBigIntNode(SIGTHAW), 0);
    hash_NameToSignal->setKeyValue("SIGTRAP", new QoreBigIntNode(SIGTRAP), 0);
    hash_NameToSignal->setKeyValue("SIGTSTP", new QoreBigIntNode(SIGTSTP), 0);
    hash_NameToSignal->setKeyValue("SIGTTIN", new QoreBigIntNode(SIGTTIN), 0);
    hash_NameToSignal->setKeyValue("SIGTTOU", new QoreBigIntNode(SIGTTOU), 0);
    hash_NameToSignal->setKeyValue("SIGURG", new QoreBigIntNode(SIGURG), 0);
    hash_NameToSignal->setKeyValue("SIGUSR1", new QoreBigIntNode(SIGUSR1), 0);
    hash_NameToSignal->setKeyValue("SIGUSR2", new QoreBigIntNode(SIGUSR2), 0);
    hash_NameToSignal->setKeyValue("SIGVTALRM", new QoreBigIntNode(SIGVTALRM), 0);
    hash_NameToSignal->setKeyValue("SIGWAITING", new QoreBigIntNode(SIGWAITING), 0);
    hash_NameToSignal->setKeyValue("SIGWINCH", new QoreBigIntNode(SIGWINCH), 0);
    hash_NameToSignal->setKeyValue("SIGXCPU", new QoreBigIntNode(SIGXCPU), 0);
    hash_NameToSignal->setKeyValue("SIGXFSZ", new QoreBigIntNode(SIGXFSZ), 0);
    hash_NameToSignal->setKeyValue("SIGXRES", new QoreBigIntNode(SIGXRES), 0);

    ns.addConstant("NameToSignal", hash_NameToSignal);
    ns.addConstant("SIGABRT", new QoreBigIntNode(SIGABRT));
    ns.addConstant("SIGALRM", new QoreBigIntNode(SIGALRM));
    ns.addConstant("SIGBUS", new QoreBigIntNode(SIGBUS));
    ns.addConstant("SIGCANCEL", new QoreBigIntNode(SIGCANCEL));
    ns.addConstant("SIGCHLD", new QoreBigIntNode(SIGCHLD));
    ns.addConstant("SIGCLD", new QoreBigIntNode(SIGCLD));
    ns.addConstant("SIGCONT", new QoreBigIntNode(SIGCONT));
    ns.addConstant("SIGEMT", new QoreBigIntNode(SIGEMT));
    ns.addConstant("SIGFPE", new QoreBigIntNode(SIGFPE));
    ns.addConstant("SIGFREEZE", new QoreBigIntNode(SIGFREEZE));
    ns.addConstant("SIGHUP", new QoreBigIntNode(SIGHUP));
    ns.addConstant("SIGILL", new QoreBigIntNode(SIGILL));
    ns.addConstant("SIGINFO", new QoreBigIntNode(SIGINFO));
    ns.addConstant("SIGINT", new QoreBigIntNode(SIGINT));
    ns.addConstant("SIGIO", new QoreBigIntNode(SIGIO));
    ns.addConstant("SIGIOT", new QoreBigIntNode(SIGIOT));
    ns.addConstant("SIGJVM1", new QoreBigIntNode(SIGJVM1));
    ns.addConstant("SIGJVM2", new QoreBigIntNode(SIGJVM2));
    ns.addConstant("SIGKILL", new QoreBigIntNode(SIGKILL));
    ns.addConstant("SIGLOST", new QoreBigIntNode(SIGLOST));
    ns.addConstant("SIGLWP", new QoreBigIntNode(SIGLWP));
    ns.addConstant("SIGPIPE", new QoreBigIntNode(SIGPIPE));
    ns.addConstant("SIGPOLL", new QoreBigIntNode(SIGPOLL));
    ns.addConstant("SIGPROF", new QoreBigIntNode(SIGPROF));
    ns.addConstant("SIGPWR", new QoreBigIntNode(SIGPWR));
    ns.addConstant("SIGQUIT", new QoreBigIntNode(SIGQUIT));
    ns.addConstant("SIGSEGV", new QoreBigIntNode(SIGSEGV));
    ns.addConstant("SIGSTKFLT", new QoreBigIntNode(SIGSTKFLT));
    ns.addConstant("SIGSTOP", new QoreBigIntNode(SIGSTOP));
    ns.addConstant("SIGSYS", new QoreBigIntNode(SIGSYS));
    ns.addConstant("SIGTERM", new QoreBigIntNode(SIGTERM));
    ns.addConstant("SIGTHAW", new QoreBigIntNode(SIGTHAW));
    ns.addConstant("SIGTRAP", new QoreBigIntNode(SIGTRAP));
    ns.addConstant("SIGTSTP", new QoreBigIntNode(SIGTSTP));
    ns.addConstant("SIGTTIN", new QoreBigIntNode(SIGTTIN));
    ns.addConstant("SIGTTOU", new QoreBigIntNode(SIGTTOU));
    ns.addConstant("SIGURG", new QoreBigIntNode(SIGURG));
    ns.addConstant("SIGUSR1", new QoreBigIntNode(SIGUSR1));
    ns.addConstant("SIGUSR2", new QoreBigIntNode(SIGUSR2));
    ns.addConstant("SIGVTALRM", new QoreBigIntNode(SIGVTALRM));
    ns.addConstant("SIGWAITING", new QoreBigIntNode(SIGWAITING));
    ns.addConstant("SIGWINCH", new QoreBigIntNode(SIGWINCH));
    ns.addConstant("SIGXCPU", new QoreBigIntNode(SIGXCPU));
    ns.addConstant("SIGXFSZ", new QoreBigIntNode(SIGXFSZ));
    ns.addConstant("SIGXRES", new QoreBigIntNode(SIGXRES));
    QoreHashNode* hash_SignalToName = new QoreHashNode;
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGABRT), new QoreStringNode("SIGABRT"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGALRM), new QoreStringNode("SIGALRM"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGBUS), new QoreStringNode("SIGBUS"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGCANCEL), new QoreStringNode("SIGCANCEL"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGCHLD), new QoreStringNode("SIGCHLD"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGCONT), new QoreStringNode("SIGCONT"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGEMT), new QoreStringNode("SIGEMT"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGFPE), new QoreStringNode("SIGFPE"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGFREEZE), new QoreStringNode("SIGFREEZE"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGHUP), new QoreStringNode("SIGHUP"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGILL), new QoreStringNode("SIGILL"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGINFO), new QoreStringNode("SIGINFO"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGINT), new QoreStringNode("SIGINT"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGIO), new QoreStringNode("SIGIO"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGIOT), new QoreStringNode("SIGIOT"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGJVM1), new QoreStringNode("SIGJVM1"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGJVM2), new QoreStringNode("SIGJVM2"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGKILL), new QoreStringNode("SIGKILL"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGLOST), new QoreStringNode("SIGLOST"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGLWP), new QoreStringNode("SIGLWP"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGPIPE), new QoreStringNode("SIGPIPE"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGPOLL), new QoreStringNode("SIGPOLL"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGPROF), new QoreStringNode("SIGPROF"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGPWR), new QoreStringNode("SIGPWR"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGQUIT), new QoreStringNode("SIGQUIT"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGSEGV), new QoreStringNode("SIGSEGV"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGSTKSZ), new QoreStringNode("SIGSTKSZ"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGSTOP), new QoreStringNode("SIGSTOP"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGSYS), new QoreStringNode("SIGSYS"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGTERM), new QoreStringNode("SIGTERM"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGTHAW), new QoreStringNode("SIGTHAW"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGTRAP), new QoreStringNode("SIGTRAP"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGTSTP), new QoreStringNode("SIGTSTP"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGTTIN), new QoreStringNode("SIGTTIN"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGTTOU), new QoreStringNode("SIGTTOU"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGURG), new QoreStringNode("SIGURG"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGUSR1), new QoreStringNode("SIGUSR1"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGUSR2), new QoreStringNode("SIGUSR2"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGVTALRM), new QoreStringNode("SIGVTALRM"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGWAITING), new QoreStringNode("SIGWAITING"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGWINCH), new QoreStringNode("SIGWINCH"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGXCPU), new QoreStringNode("SIGXCPU"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGXFSZ), new QoreStringNode("SIGXFSZ"), 0);
    hash_SignalToName->setKeyValue(MAKE_STRING_FROM_SYMBOL(SIGXRES), new QoreStringNode("SIGXRES"), 0);

    ns.addConstant("SignalToName", hash_SignalToName);
# 478 "ql_misc.qpp"
# 511 "ql_misc.qpp"
# 538 "ql_misc.qpp"
# 568 "ql_misc.qpp"
# 589 "ql_misc.qpp"
# 610 "ql_misc.qpp"
# 626 "ql_misc.qpp"
# 658 "ql_misc.qpp"
# 685 "ql_misc.qpp"
# 701 "ql_misc.qpp"
# 712 "ql_misc.qpp"
# 725 "ql_misc.qpp"
# 731 "ql_misc.qpp"
# 747 "ql_misc.qpp"
# 760 "ql_misc.qpp"
# 776 "ql_misc.qpp"
# 782 "ql_misc.qpp"
# 803 "ql_misc.qpp"
# 822 "ql_misc.qpp"
# 828 "ql_misc.qpp"
# 847 "ql_misc.qpp"
# 853 "ql_misc.qpp"
# 865 "ql_misc.qpp"
# 905 "ql_misc.qpp"
# 911 "ql_misc.qpp"
# 922 "ql_misc.qpp"
# 928 "ql_misc.qpp"
# 946 "ql_misc.qpp"
# 977 "ql_misc.qpp"
# 1003 "ql_misc.qpp"
# 1009 "ql_misc.qpp"
# 1043 "ql_misc.qpp"
# 1049 "ql_misc.qpp"
# 1071 "ql_misc.qpp"
# 1091 "ql_misc.qpp"
# 1097 "ql_misc.qpp"
# 1122 "ql_misc.qpp"
# 1145 "ql_misc.qpp"
# 1164 "ql_misc.qpp"
# 1170 "ql_misc.qpp"
# 1194 "ql_misc.qpp"
# 1217 "ql_misc.qpp"
# 1223 "ql_misc.qpp"
# 1251 "ql_misc.qpp"
# 1273 "ql_misc.qpp"
# 1300 "ql_misc.qpp"
# 1322 "ql_misc.qpp"
# 1347 "ql_misc.qpp"
# 1358 "ql_misc.qpp"
# 1374 "ql_misc.qpp"
# 1398 "ql_misc.qpp"
# 1404 "ql_misc.qpp"
# 1429 "ql_misc.qpp"
# 1446 "ql_misc.qpp"
# 1471 "ql_misc.qpp"
# 1496 "ql_misc.qpp"
# 1518 "ql_misc.qpp"
# 1524 "ql_misc.qpp"
# 1546 "ql_misc.qpp"
# 1571 "ql_misc.qpp"
# 1596 "ql_misc.qpp"
# 1623 "ql_misc.qpp"
# 1650 "ql_misc.qpp"
# 1672 "ql_misc.qpp"
# 1694 "ql_misc.qpp"
# 1700 "ql_misc.qpp"
# 1725 "ql_misc.qpp"
# 1750 "ql_misc.qpp"
# 1775 "ql_misc.qpp"
# 1800 "ql_misc.qpp"
# 1827 "ql_misc.qpp"
# 1852 "ql_misc.qpp"
# 1877 "ql_misc.qpp"
# 1902 "ql_misc.qpp"
# 1913 "ql_misc.qpp"
# 1934 "ql_misc.qpp"
# 1957 "ql_misc.qpp"
# 1976 "ql_misc.qpp"
# 1999 "ql_misc.qpp"
# 2005 "ql_misc.qpp"
# 2023 "ql_misc.qpp"
# 2041 "ql_misc.qpp"
# 2047 "ql_misc.qpp"
# 2070 "ql_misc.qpp"
# 2093 "ql_misc.qpp"
# 2110 "ql_misc.qpp"
# 2116 "ql_misc.qpp"
# 2138 "ql_misc.qpp"
# 2182 "ql_misc.qpp"
# 2188 "ql_misc.qpp"
# 2209 "ql_misc.qpp"
# 2215 "ql_misc.qpp"
# 2239 "ql_misc.qpp"
# 2245 "ql_misc.qpp"
# 2264 "ql_misc.qpp"
# 2288 "ql_misc.qpp"
# 2311 "ql_misc.qpp"
# 2317 "ql_misc.qpp"
# 2341 "ql_misc.qpp"
# 2374 "ql_misc.qpp"
# 2409 "ql_misc.qpp"
# 2421 "ql_misc.qpp"
# 2433 "ql_misc.qpp"
# 2445 "ql_misc.qpp"
# 2473 "ql_misc.qpp"
# 2504 "ql_misc.qpp"
# 2549 "ql_misc.qpp"
# 2563 "ql_misc.qpp"
# 2579 "ql_misc.qpp"
# 2613 "ql_misc.qpp"
# 2633 "ql_misc.qpp"
# 2656 "ql_misc.qpp"
# 2672 "ql_misc.qpp"

}
