/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  ql_object.qpp

  Qore Programming Language

  Copyright (C) 2003 - 2017 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/ql_object.h"
#include "qore/intern/qore_program_private.h"
#include "qore/intern/QoreClassIntern.h"

static QoreObject* create_object_intern(const QoreStringNode* class_name, unsigned arg_offset, const QoreValueList* args, ExceptionSink* xsink) {
   TempEncodingHelper tmp(class_name, QCS_DEFAULT, xsink);
   if (!tmp) {
      assert(*xsink);
      return 0;
   }

   const QoreClass* qc = qore_program_private::runtimeFindClass(*getProgram(), tmp->getBuffer(), xsink);
   if (!qc) {
      if (!*xsink)
         xsink->raiseException("CREATE-OBJECT-ERROR", "cannot resolve class '%s' for instantiation", tmp->getBuffer());
      return 0;
   }

   if (getProgram()->getParseOptions64() & qc->getDomain())
      xsink->raiseException("CREATE-OBJECT-ERROR", "current Program sandboxing restrictions do not allow access to the '%s' class", qc->getName());

   if (qore_class_private::runtimeCheckInstantiateClass(*qc, xsink))
      return 0;

   const QoreValueList* real_args;
   ReferenceHolder<QoreValueList> tmp_args(xsink);
   if (arg_offset) {
      tmp_args = args->size() > arg_offset ? args->copyListFrom(arg_offset) : 0;
      real_args = *tmp_args;
   }
   else
      real_args = args;

   return qc->execConstructor(real_args, xsink);
}
# 70 "ql_object.qpp"
# 75 "ql_object.qpp"
# 87 "ql_object.qpp"
# 93 "ql_object.qpp"
# 110 "ql_object.qpp"
# 144 "ql_object.qpp"
# 150 "ql_object.qpp"
# 184 "ql_object.qpp"
# 205 "ql_object.qpp"
# 211 "ql_object.qpp"
# 237 "ql_object.qpp"
# 258 "ql_object.qpp"
# 276 "ql_object.qpp"
# 300 "ql_object.qpp"
// auto callObjectMethod(object obj, string method, ...) {}
static QoreValue f_callObjectMethod_VoVsVV(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreObject* obj = HARD_QORE_VALUE_OBJECT(args, 0);
    const QoreStringNode* method = HARD_QORE_VALUE_STRING(args, 1);
# 132 "ql_object.qpp"
   ReferenceHolder<QoreListNode> call_args(xsink);

   // if there are arguments to pass
   if (num_args(args) > 2) {
      // create argument list by copying current list
      call_args = args->getOldList(2);
   }

   return obj->evalMethod(method, *call_args, xsink);
}

// nothing callObjectMethod() {}
static QoreValue f_callObjectMethod(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 147 "ql_object.qpp"

    return QoreValue();
}

// auto callObjectMethodArgs(object obj, string method, *softlist<auto> varg) {}
static QoreValue f_callObjectMethodArgs_VoVsC15_softlist_auto_(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreObject* obj = HARD_QORE_VALUE_OBJECT(args, 0);
    const QoreStringNode* method = HARD_QORE_VALUE_STRING(args, 1);
    const QoreListNode* varg = get_param_value(args, 2).get<const QoreListNode>();
# 201 "ql_object.qpp"
   return obj->evalMethod(method, varg, xsink);
}

// nothing callObjectMethodArgs() {}
static QoreValue f_callObjectMethodArgs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 208 "ql_object.qpp"

    return QoreValue();
}

// auto call_object_method(object obj, string method, ...) {}
static QoreValue f_call_object_method_VoVsVV(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreObject* obj = HARD_QORE_VALUE_OBJECT(args, 0);
    const QoreStringNode* method = HARD_QORE_VALUE_STRING(args, 1);
# 172 "ql_object.qpp"
   ReferenceHolder<QoreListNode> call_args(xsink);

   // if there are arguments to pass
   if (num_args(args) > 2) {
      // create argument list by copying current list
      call_args = args->getOldList(2);
   }

   return obj->evalMethod(method, *call_args, xsink);
}

// auto call_object_method_args(object obj, string method, *softlist<auto> varg) {}
static QoreValue f_call_object_method_args_VoVsC15_softlist_auto_(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreObject* obj = HARD_QORE_VALUE_OBJECT(args, 0);
    const QoreStringNode* method = HARD_QORE_VALUE_STRING(args, 1);
    const QoreListNode* varg = get_param_value(args, 2).get<const QoreListNode>();
# 233 "ql_object.qpp"
   return obj->evalMethod(method, varg, xsink);
}

// auto call_pseudo(any val, string meth, ...) {}
static QoreValue f_call_pseudo_VaVsVV(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue val = get_param_value(args, 0);
    const QoreStringNode* meth = HARD_QORE_VALUE_STRING(args, 1);
# 251 "ql_object.qpp"
   ReferenceHolder<QoreListNode> nargs(xsink);
   if (args->size() > 2)
      nargs = args->getOldList(2);
   return pseudo_classes_eval(val, meth->getBuffer(), *nargs, xsink).takeNode();
}

// auto call_pseudo_args(any val, string meth, *softlist<auto> argv) {}
static QoreValue f_call_pseudo_args_VaVsC15_softlist_auto_(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue val = get_param_value(args, 0);
    const QoreStringNode* meth = HARD_QORE_VALUE_STRING(args, 1);
    const QoreListNode* argv = get_param_value(args, 2).get<const QoreListNode>();
# 272 "ql_object.qpp"
   return pseudo_classes_eval(val, meth->getBuffer(), argv, xsink).takeNode();
}

// object create_object(string class_name, ...) {}
static QoreValue f_create_object_VsVV(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* class_name = HARD_QORE_VALUE_STRING(args, 0);
# 296 "ql_object.qpp"
   return create_object_intern(class_name, 1, args, xsink);
}

// object create_object_args(string class_name, *softlist<auto> argv) {}
static QoreValue f_create_object_args_VsC15_softlist_auto_(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* class_name = HARD_QORE_VALUE_STRING(args, 0);
    const QoreListNode* argv = get_param_value(args, 1).get<const QoreListNode>();
# 320 "ql_object.qpp"
   ReferenceHolder<QoreValueList> arg_list(xsink);
   if (argv)
      arg_list = new QoreValueList(argv);
   return create_object_intern(class_name, 0, *arg_list, xsink);
}

// list getMethodList(object obj) {}
static QoreValue f_getMethodList_Vo(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreObject* obj = HARD_QORE_VALUE_OBJECT(args, 0);
# 83 "ql_object.qpp"
   return obj->getClass()->getMethodList();
}

// nothing getMethodList() {}
static QoreValue f_getMethodList(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 90 "ql_object.qpp"

    return QoreValue();
}

// list get_method_list(object obj) {}
static QoreValue f_get_method_list_Vo(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreObject* obj = HARD_QORE_VALUE_OBJECT(args, 0);
# 106 "ql_object.qpp"
   return obj->getClass()->getMethodList();
}


DLLLOCAL void init_object_functions(QoreNamespace& ns) {
    // auto callObjectMethod(object obj, string method, ...) {}
    ns.addBuiltinVariant("callObjectMethod", (q_func_n_t)f_callObjectMethod_VoVsVV, QC_DEPRECATED|QC_USES_EXTRA_ARGS, QDOM_DEFAULT, autoTypeInfo, 2, objectTypeInfo, NULL, "obj", stringTypeInfo, NULL, "method");
    // nothing callObjectMethod() {}
    ns.addBuiltinVariant("callObjectMethod", (q_func_n_t)f_callObjectMethod, QC_RUNTIME_NOOP|QC_DEPRECATED, QDOM_DEFAULT, nothingTypeInfo);
    // auto callObjectMethodArgs(object obj, string method, *softlist<auto> varg) {}
    ns.addBuiltinVariant("callObjectMethodArgs", (q_func_n_t)f_callObjectMethodArgs_VoVsC15_softlist_auto_, QC_DEPRECATED, QDOM_DEFAULT, autoTypeInfo, 3, objectTypeInfo, NULL, "obj", stringTypeInfo, NULL, "method", softAutoListOrNothingTypeInfo, NULL, "varg");
    // nothing callObjectMethodArgs() {}
    ns.addBuiltinVariant("callObjectMethodArgs", (q_func_n_t)f_callObjectMethodArgs, QC_RUNTIME_NOOP|QC_DEPRECATED, QDOM_DEFAULT, nothingTypeInfo);
    // auto call_object_method(object obj, string method, ...) {}
    ns.addBuiltinVariant("call_object_method", (q_func_n_t)f_call_object_method_VoVsVV, QC_USES_EXTRA_ARGS, QDOM_DEFAULT, autoTypeInfo, 2, objectTypeInfo, NULL, "obj", stringTypeInfo, NULL, "method");
    // auto call_object_method_args(object obj, string method, *softlist<auto> varg) {}
    ns.addBuiltinVariant("call_object_method_args", (q_func_n_t)f_call_object_method_args_VoVsC15_softlist_auto_, QC_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 3, objectTypeInfo, NULL, "obj", stringTypeInfo, NULL, "method", softAutoListOrNothingTypeInfo, NULL, "varg");
    // auto call_pseudo(any val, string meth, ...) {}
    ns.addBuiltinVariant("call_pseudo", (q_func_n_t)f_call_pseudo_VaVsVV, QC_USES_EXTRA_ARGS, QDOM_DEFAULT, autoTypeInfo, 2, anyTypeInfo, NULL, "val", stringTypeInfo, NULL, "meth");
    // auto call_pseudo_args(any val, string meth, *softlist<auto> argv) {}
    ns.addBuiltinVariant("call_pseudo_args", (q_func_n_t)f_call_pseudo_args_VaVsC15_softlist_auto_, QC_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 3, anyTypeInfo, NULL, "val", stringTypeInfo, NULL, "meth", softAutoListOrNothingTypeInfo, NULL, "argv");
    // object create_object(string class_name, ...) {}
    ns.addBuiltinVariant("create_object", (q_func_n_t)f_create_object_VsVV, QC_USES_EXTRA_ARGS, QDOM_DEFAULT, objectTypeInfo, 1, stringTypeInfo, NULL, "class_name");
    // object create_object_args(string class_name, *softlist<auto> argv) {}
    ns.addBuiltinVariant("create_object_args", (q_func_n_t)f_create_object_args_VsC15_softlist_auto_, QC_NO_FLAGS, QDOM_DEFAULT, objectTypeInfo, 2, stringTypeInfo, NULL, "class_name", softAutoListOrNothingTypeInfo, NULL, "argv");
    // list getMethodList(object obj) {}
    ns.addBuiltinVariant("getMethodList", (q_func_n_t)f_getMethodList_Vo, QC_CONSTANT|QC_DEPRECATED, QDOM_DEFAULT, listTypeInfo, 1, objectTypeInfo, NULL, "obj");
    // nothing getMethodList() {}
    ns.addBuiltinVariant("getMethodList", (q_func_n_t)f_getMethodList, QC_RUNTIME_NOOP|QC_DEPRECATED, QDOM_DEFAULT, nothingTypeInfo);
    // list get_method_list(object obj) {}
    ns.addBuiltinVariant("get_method_list", (q_func_n_t)f_get_method_list_Vo, QC_CONSTANT, QDOM_DEFAULT, listTypeInfo, 1, objectTypeInfo, NULL, "obj");

}
