/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  ql_string.qpp

  Qore Programming Language

  Copyright (C) 2003 - 2017 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/ql_string.h"
#include "qore/intern/qore_number_private.h"

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <float.h>

// for use in floating-point number formatting
// from: http://stackoverflow.com/questions/16839658/printf-width-specifier-to-maintain-precision-of-floating-point-value
#ifdef DBL_DECIMAL_DIG
  #define OP_DBL_Digs (DBL_DECIMAL_DIG)
#else
  #ifdef DECIMAL_DIG
    #define OP_DBL_Digs (DECIMAL_DIG)
  #else
    #define OP_DBL_Digs (DBL_DIG + 3)
  #endif
#endif

static int fix_num(QoreString& tmp, const QoreString& fmt, ExceptionSink* xsink) {
   assert(xsink);
   qore_offset_t off = fmt.getByteOffset(1, xsink);
   //printd(5, "fix_num tmp: '%s' fmt: '%s' off: %lld\n", tmp.getBuffer(), fmt.getBuffer(), off);
   if (*xsink)
      return -1;
   if (off < 0)
      return 0;

   // replace all thousands separators with nothing
   QoreString t(fmt.getBuffer(), off, fmt.getEncoding());
   tmp.replaceAll(t.getBuffer(), "");

   if (fmt.size() <= (qore_size_t)off)
      return 0;

   t.clear();
   if (t.concat(fmt, 1, 1, xsink))
      return -1;

   if (t.size() == 1 && t[0] == '.')
      return 0;

   off = tmp.bindex(t.getBuffer(), 0);
   if (off < 0)
      return 0;

   tmp.replace(off, t.size(), ".");
   return 0;
}

// finds the last occurrence of needle in haystack at or before position pos
// pos must be a non-negative valid byte offset in haystack
/*
static int rindex_simple_intern(const char* haystack, int hlen, const char* needle, int nlen, int pos) {
   // if the offset does not allow for the needle string to be present, then adjust
   if ((pos + nlen) > hlen) {
      pos = hlen - nlen;
      if (pos < 0)
         return -1;
   }

   while (pos != -1) {
      if (!strncmp(haystack + pos, needle, nlen))
         return pos;
      pos--;
   }
   return -1;
}
*/

static const char* memstr(const char* str, const char* pattern, qore_size_t pl, qore_size_t len) {
   while (true) {
      const char* p = (const char* )memchr(str, pattern[0], len);
      if (!p)
         return 0;

      //printd(5, "memstr() pattern: %s str: %p p: %p len: %d pl: %d remaining: %d (%c %c %c)\n", pattern, str, p, len, pl, len-(p-str), p[1], p[2], p[3]);

      // if there is not enough string left for the pattern, then return
      if ((len - (p - str)) < pl)
         return 0;

      bool found = true;
      for (qore_size_t i = 1; i < pl; ++i) {
         if (pattern[i] != p[i]) {
            len -= (p - str + 1);
            str = p + 1;
            found = false;
            break;
         }
      }
      if (!found)
         continue;

      // found
      //printd(5, "memstr() got it! p: %p\n", p);
      return p;
   }
   return 0;
}

static void split_add_element(QoreListNode* l, const char* str, unsigned len, const QoreEncoding *enc) {
   if (enc)
      l->push(new QoreStringNode(str, len, enc));
   else {
      BinaryNode* b = new BinaryNode;
      b->append(str, len);
      l->push(b);
   }
}

QoreListNode* split_intern(const char* pattern, qore_size_t pl, const char* str, qore_size_t sl, const QoreEncoding *enc, bool with_separator) {
   QoreListNode* l = new QoreListNode(stringTypeInfo);
   const char* ostr = str;
   while (const char* p = memstr(str, pattern, pl, sl - (str - ostr))) {
      split_add_element(l, str, p - str + (with_separator ? pl : 0), enc);
      str = p + pl;
   }
   // add last field if there is data remaining
   if (sl - (str - ostr))
      split_add_element(l, str, sl - (str - ostr), enc);

   return l;
}

QoreListNode* split_with_quote(const QoreString* sep, const QoreString* str, const QoreString* quote, bool trim_unquoted, ExceptionSink* xsink) {
   // convert pattern encoding to string if necessary
   TempEncodingHelper pat(sep, str->getEncoding(), xsink);
   if (*xsink)
      return 0;

   // convert quote to string if necessary
   TempEncodingHelper tquote(quote, str->getEncoding(), xsink);
   if (*xsink)
      return 0;

   //printd(5, "split_with_quote() sep: %s str: %s quote: %s trim_unquoted: %d\n", pat->getBuffer(), str->getBuffer(), tquote->getBuffer(), trim_unquoted);

   if (!tquote->strlen() || tquote->strlen() > sep->strlen())
      return split_intern(pat->getBuffer(), pat->strlen(), str->getBuffer(), str->strlen(), str->getEncoding());

   ReferenceHolder<QoreListNode> l(new QoreListNode(stringTypeInfo), xsink);
   const char* ostr = str->getBuffer();
   qore_size_t sl = str->strlen();
   const char* tpattern = pat->getBuffer();
   qore_size_t pl = pat->strlen();

   const char* ststr = ostr;

   // remaining byte length
   qore_size_t len = sl;

   while (len > 0) {
      // see if the field begins with the quote string
      // and if the remaining string length is at least big enough for two quote strings
      if ((tquote->strlen() * 2) <= len
          && !memcmp(tquote->getBuffer(), ststr, tquote->strlen())) {
         // advance pointer past quote
         ststr += tquote->strlen();
         // find next quote character, ignore escaped quotes
         const char* tstr = ststr;
         const char* p;
         while (true) {
            p = memstr(tstr, tquote->getBuffer(), tquote->strlen(), len);
            if (!p) {
               xsink->raiseException("SPLIT-ERROR", "cannot find closing quote '%s' in field " QSD, tquote->getBuffer(), l->size() + 1);
               return nullptr;
            }
            if (p == tstr)
               break;
            if (*(p - 1) != '\\')
               break;
            tstr = p + 1;
         }
         // optimistically add the field to the list
         l->push(new QoreStringNode(ststr, p - ststr, str->getEncoding()));

         ststr = p + tquote->strlen();
         // see if we are at the end of the string
         len = sl + (ostr - ststr);

         if (!len)
            break;

         // or a separator string comes next
         if (len < pl || memcmp(tpattern, ststr, pl)) {
            xsink->raiseException("SPLIT-ERROR", "separator pattern '%s' does not follow end quote in field " QSD, tpattern, l->size());
            return nullptr;
         }
         ststr += pl;
         len -= pl;
         continue;
      }

      const char* p = memstr(ststr, tpattern, pl, sl - (ststr - ostr));
      if (!p) {
         QoreStringNode* se = new QoreStringNode(ststr, sl - (ststr - ostr), str->getEncoding());
         if (trim_unquoted && se->trim(xsink))
            return nullptr;
         l->push(se);
         break;
      }

      QoreStringNode* se = new QoreStringNode(ststr, p - ststr, str->getEncoding());
      if (trim_unquoted && se->trim(xsink))
         return nullptr;
      l->push(se);
      len -= (p - ststr);
      ststr = p + pl;
   }

   return l.release();
}

QoreStringNode* join_intern(const QoreStringNode* p0, const QoreValueList* l, int offset, ExceptionSink* xsink) {
   assert(xsink);
   SimpleRefHolder<QoreStringNode> str(new QoreStringNode(p0->getEncoding()));

   for (unsigned i = offset; i < l->size(); i++) {
      QoreValue p = l->retrieveEntry(i);
      if (!p.isNothing()) {
         QoreStringValueHelper t(p, p0->getEncoding(), xsink);
         if (*xsink)
            return nullptr;

         str->concat(*t);
      }
      if (i < (l->size() - 1))
         str->concat(p0);
   }

   return str.release();
}

QoreStringNode* join_intern(const QoreStringNode* p0, const QoreListNode* l, int offset, ExceptionSink* xsink) {
   assert(xsink);
   SimpleRefHolder<QoreStringNode> str(new QoreStringNode(p0->getEncoding()));

   for (unsigned i = offset; i < l->size(); i++) {
      const AbstractQoreNode* p = l->retrieve_entry(i);
      if (p) {
         QoreStringValueHelper t(p, p0->getEncoding(), xsink);
         if (*xsink)
            return 0;

         str->concat(*t);
      }
      if (i < (l->size() - 1))
         str->concat(p0);
   }

   return str.release();
}

QoreStringNode* format_float_intern(const QoreString& fmt, double num, ExceptionSink* xsink) {
   if (fmt.empty())
      return q_fix_decimal(new QoreStringNodeMaker("%.9g", num));
   SimpleRefHolder<QoreStringNode> str(q_fix_decimal(new QoreStringNodeMaker("%.*f", OP_DBL_Digs + 6, num)));
   if (qore_number_private::formatNumberString(**str, fmt, xsink))
      return 0;
   return str.release();
}

QoreStringNode* format_float_intern(int prec, const QoreString& dsep, const QoreString& tsep, double num, ExceptionSink* xsink) {
   SimpleRefHolder<QoreStringNode> str(q_fix_decimal(new QoreStringNodeMaker("%.*f", OP_DBL_Digs + 6, num)));
   if (qore_number_private::formatNumberString(**str, prec, dsep, tsep, xsink))
      return 0;
   return str.release();
}
# 330 "ql_string.qpp"
# 406 "ql_string.qpp"
# 431 "ql_string.qpp"
# 447 "ql_string.qpp"
# 453 "ql_string.qpp"
# 460 "ql_string.qpp"
# 482 "ql_string.qpp"
# 488 "ql_string.qpp"
# 495 "ql_string.qpp"
# 525 "ql_string.qpp"
# 531 "ql_string.qpp"
# 561 "ql_string.qpp"
# 567 "ql_string.qpp"
# 592 "ql_string.qpp"
# 618 "ql_string.qpp"
# 642 "ql_string.qpp"
# 667 "ql_string.qpp"
# 673 "ql_string.qpp"
# 704 "ql_string.qpp"
# 711 "ql_string.qpp"
# 740 "ql_string.qpp"
# 747 "ql_string.qpp"
# 778 "ql_string.qpp"
# 785 "ql_string.qpp"
# 814 "ql_string.qpp"
# 821 "ql_string.qpp"
# 842 "ql_string.qpp"
# 849 "ql_string.qpp"
# 869 "ql_string.qpp"
# 876 "ql_string.qpp"
# 882 "ql_string.qpp"
# 908 "ql_string.qpp"
# 934 "ql_string.qpp"
# 952 "ql_string.qpp"
# 959 "ql_string.qpp"
# 976 "ql_string.qpp"
# 982 "ql_string.qpp"
# 1000 "ql_string.qpp"
# 1006 "ql_string.qpp"
# 1023 "ql_string.qpp"
# 1029 "ql_string.qpp"
# 1059 "ql_string.qpp"
# 1065 "ql_string.qpp"
# 1120 "ql_string.qpp"
# 1126 "ql_string.qpp"
# 1158 "ql_string.qpp"
# 1164 "ql_string.qpp"
# 1248 "ql_string.qpp"
# 1254 "ql_string.qpp"
# 1272 "ql_string.qpp"
# 1292 "ql_string.qpp"
# 1298 "ql_string.qpp"
# 1318 "ql_string.qpp"
# 1343 "ql_string.qpp"
# 1349 "ql_string.qpp"
# 1367 "ql_string.qpp"
# 1394 "ql_string.qpp"
# 1400 "ql_string.qpp"
# 1419 "ql_string.qpp"
# 1446 "ql_string.qpp"
# 1467 "ql_string.qpp"
# 1496 "ql_string.qpp"
# 1542 "ql_string.qpp"
# 1579 "ql_string.qpp"
# 1600 "ql_string.qpp"
# 1606 "ql_string.qpp"
# 1624 "ql_string.qpp"
# 1643 "ql_string.qpp"
# 1665 "ql_string.qpp"
# 1672 "ql_string.qpp"
# 1694 "ql_string.qpp"
# 1701 "ql_string.qpp"
# 1725 "ql_string.qpp"
# 1747 "ql_string.qpp"
# 1754 "ql_string.qpp"
# 1778 "ql_string.qpp"
# 1785 "ql_string.qpp"
# 1809 "ql_string.qpp"
# 1816 "ql_string.qpp"
# 1842 "ql_string.qpp"
# 1866 "ql_string.qpp"
# 1873 "ql_string.qpp"
# 1884 "ql_string.qpp"
# 1900 "ql_string.qpp"
# 1914 "ql_string.qpp"
# 1943 "ql_string.qpp"
# 1972 "ql_string.qpp"
// int bindex(softstring str, softstring substr, softint pos = 0) {}
static QoreValue f_bindex_vsvsvi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* substr = HARD_QORE_VALUE_STRING(args, 1);
    int64 pos = HARD_QORE_VALUE_INT(args, 2);
# 736 "ql_string.qpp"
   return str->bindex(*substr, (qore_offset_t)pos);
}

// int bindex() {}
static QoreValue f_bindex(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 743 "ql_string.qpp"
   return -1;
}

// int brindex(softstring str, softstring substr, softint pos = -1) {}
static QoreValue f_brindex_vsvsvi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* substr = HARD_QORE_VALUE_STRING(args, 1);
    int64 pos = HARD_QORE_VALUE_INT(args, 2);
# 810 "ql_string.qpp"
   return str->brindex(*substr, (qore_offset_t)pos);
}

// int brindex() {}
static QoreValue f_brindex(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 817 "ql_string.qpp"
   return -1;
}

// string chomp(string str) {}
static QoreValue f_chomp_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
# 1312 "ql_string.qpp"
   QoreStringNode* rv = str->copy();
   rv->chomp();
   return rv;
}

// *string chomp(reference<string> str) {}
static QoreValue f_chomp_C17reference_string_(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const ReferenceNode* str = HARD_QORE_VALUE_REF(args, 0);
# 1330 "ql_string.qpp"
   QoreTypeSafeReferenceHelper ref(str, xsink);
   if (!ref || ref.getType() != NT_STRING)
      return QoreValue();

   QoreStringNode* rv = reinterpret_cast<QoreStringNode* >(ref.getUnique(xsink));
   if (*xsink)
      return QoreValue();

   rv->chomp();
   return rv->refSelf();
}

// nothing chomp() {}
static QoreValue f_chomp(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1346 "ql_string.qpp"

    return QoreValue();
}

// string chr(softint val, *string encoding) {}
static QoreValue f_chr_viNs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 val = HARD_QORE_VALUE_INT(args, 0);
    const QoreStringNode* encoding = get_param_value(args, 1).get<const QoreStringNode>();
# 862 "ql_string.qpp"
   const QoreEncoding *qe = encoding ? QEM.findCreate(encoding) : QCS_DEFAULT;
   QoreStringNode* rv = new QoreStringNode(qe);
   rv->concat((char)val);
   return rv;
}

// string chr(any arg) {}
static QoreValue f_chr_Va(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 872 "ql_string.qpp"
   return new QoreStringNode('\0');
}

// nothing chr() {}
static QoreValue f_chr(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 879 "ql_string.qpp"

    return QoreValue();
}

// string convert_encoding(string str, string encoding) {}
static QoreValue f_convert_encoding_VsVs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* encoding = HARD_QORE_VALUE_STRING(args, 1);
# 996 "ql_string.qpp"
   return str->convertEncoding(QEM.findCreate(encoding), xsink);
}

// nothing convert_encoding() {}
static QoreValue f_convert_encoding(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1003 "ql_string.qpp"

    return QoreValue();
}

// string f_printf(string fmt, ...) {}
static QoreValue f_f_printf_VsVV(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1772 "ql_string.qpp"
   AbstractQoreNode* node;
   print_node(stdout, node = q_sprintf(args, 1, 0, xsink));
   return node;
}

// string f_printf() {}
static QoreValue f_f_printf(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1781 "ql_string.qpp"
   return null_string();
}

// string f_sprintf(string fmt, ...) {}
static QoreValue f_f_sprintf_VsVV(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1661 "ql_string.qpp"
   return q_sprintf(args, 1, 0, xsink);
}

// string f_sprintf() {}
static QoreValue f_f_sprintf(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1668 "ql_string.qpp"
   return null_string();
}

// string f_vprintf(string fmt, any varg) {}
static QoreValue f_f_vprintf_VsVa(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1836 "ql_string.qpp"
   AbstractQoreNode* node;
   print_node(stdout, node = q_vsprintf(args, 1, 0, xsink));
   return node;
}

// string f_vsprintf(string fmt, any varg) {}
static QoreValue f_f_vsprintf_VsVa(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1721 "ql_string.qpp"
   return q_vsprintf(args, 1, 0, xsink);
}

// nothing flush() {}
static QoreValue f_flush(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1880 "ql_string.qpp"
   fflush(stdout);
    return QoreValue();
}

// string force_encoding(string str, string encoding) {}
static QoreValue f_force_encoding_VsVs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* encoding = HARD_QORE_VALUE_STRING(args, 1);
# 1019 "ql_string.qpp"
   return new QoreStringNode(str->getBuffer(), str->strlen(), QEM.findCreate(encoding));
}

// nothing force_encoding() {}
static QoreValue f_force_encoding(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1026 "ql_string.qpp"

    return QoreValue();
}

// string format_number(string fmt, softfloat num) {}
static QoreValue f_format_number_Vsvf(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* fmt = HARD_QORE_VALUE_STRING(args, 0);
    double num = HARD_QORE_VALUE_FLOAT(args, 1);
# 1596 "ql_string.qpp"
   return format_float_intern(*fmt, num, xsink);
}

// nothing format_number() {}
static QoreValue f_format_number(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1603 "ql_string.qpp"

    return QoreValue();
}

// string get_encoding(string str) {}
static QoreValue f_get_encoding_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
# 972 "ql_string.qpp"
   return new QoreStringNode(str->getEncoding()->getCode());
}

// nothing get_encoding() {}
static QoreValue f_get_encoding(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 979 "ql_string.qpp"

    return QoreValue();
}

// int index(softstring str, softstring substr, softint pos = 0) {}
static QoreValue f_index_vsvsvi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* substr = HARD_QORE_VALUE_STRING(args, 1);
    int64 pos = HARD_QORE_VALUE_INT(args, 2);
# 700 "ql_string.qpp"
   return str->index(*substr, pos, xsink);
}

// int index() {}
static QoreValue f_index(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 707 "ql_string.qpp"
   return -1;
}

// string join(string str, ...) {}
static QoreValue f_join_VsVV(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
# 1268 "ql_string.qpp"
   return join_intern(str, args, 1, xsink);
}

// string join(string str, list l) {}
static QoreValue f_join_VsVl(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 1);
# 1288 "ql_string.qpp"
   return join_intern(str, l, 0, xsink);
}

// nothing join() {}
static QoreValue f_join(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1295 "ql_string.qpp"

    return QoreValue();
}

// int length(softstring str) {}
static QoreValue f_length_vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
# 427 "ql_string.qpp"
   return str->length();
}

// int length(binary bin) {}
static QoreValue f_length_Vy(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* bin = HARD_QORE_VALUE_BINARY(args, 0);
# 443 "ql_string.qpp"
   return bin->size();
}

// nothing length() {}
static QoreValue f_length(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 450 "ql_string.qpp"

    return QoreValue();
}

// int length(any arg) {}
static QoreValue f_length_Va(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 456 "ql_string.qpp"
   return 0;
}

// string ltrim(string str, *string chars) {}
static QoreValue f_ltrim_VsNs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* chars = get_param_value(args, 1).get<const QoreStringNode>();
# 1414 "ql_string.qpp"
   SimpleRefHolder<QoreStringNode> rv(str->copy());
   return rv->trimLeading(xsink, chars) ? QoreValue() : rv.release();
}

// *string ltrim(reference<string> str, *string chars) {}
static QoreValue f_ltrim_C17reference_string_Ns(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const ReferenceNode* str = HARD_QORE_VALUE_REF(args, 0);
    const QoreStringNode* chars = get_param_value(args, 1).get<const QoreStringNode>();
# 1434 "ql_string.qpp"
   QoreTypeSafeReferenceHelper ref(str, xsink);
   if (!ref || ref.getType() != NT_STRING)
      return QoreValue();

   QoreStringNode* rv = reinterpret_cast<QoreStringNode*>(ref.getUnique(xsink));
   if (*xsink)
      return QoreValue();

   return rv->trimLeading(xsink, chars) ? QoreValue() : rv->refSelf();
}

// int ord(softstring str, softint offset = 0) {}
static QoreValue f_ord_vsvi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 offset = HARD_QORE_VALUE_INT(args, 1);
# 835 "ql_string.qpp"
   if (offset < 0 || (qore_size_t)offset >= str->strlen())
      return -1;

   return (unsigned char)str->getBuffer()[offset];
}

// int ord() {}
static QoreValue f_ord(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 845 "ql_string.qpp"
   return 0;
}

// bool parse_boolean(string str) {}
static QoreValue f_parse_boolean_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
# 1896 "ql_string.qpp"
   return q_parse_bool(str->getBuffer());
}

// bool parse_boolean(any val) {}
static QoreValue f_parse_boolean_Va(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue val = get_param_value(args, 0);
# 1910 "ql_string.qpp"
   return val.getAsBool();
}

// float parse_float(string str, string fmt) {}
static QoreValue f_parse_float_VsVs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* fmt = HARD_QORE_VALUE_STRING(args, 1);
# 1960 "ql_string.qpp"
   TempEncodingHelper tfmt(fmt, str->getEncoding(), xsink);
   if (*xsink)
      return QoreValue();

   QoreString tmp(*str);
   if (fix_num(tmp, **tfmt, xsink))
      return QoreValue();

   return q_strtod(tmp.getBuffer());
}

// int parse_int(string str, string fmt) {}
static QoreValue f_parse_int_VsVs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* fmt = HARD_QORE_VALUE_STRING(args, 1);
# 1989 "ql_string.qpp"
   TempEncodingHelper tfmt(fmt, str->getEncoding(), xsink);
   if (*xsink)
      return QoreValue();

   QoreString tmp(*str);
   if (fix_num(tmp, **tfmt, xsink))
      return QoreValue();

   return strtoll(tmp.getBuffer(), 0, 10);
}

// number parse_number(string str, string fmt) {}
static QoreValue f_parse_number_VsVs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* fmt = HARD_QORE_VALUE_STRING(args, 1);
# 1931 "ql_string.qpp"
   TempEncodingHelper tfmt(fmt, str->getEncoding(), xsink);
   if (*xsink)
      return QoreValue();

   QoreString tmp(*str);
   if (fix_num(tmp, **tfmt, xsink))
      return QoreValue();

   return new QoreNumberNode(tmp.getBuffer());
}

// nothing print(...) {}
static QoreValue f_print_VV(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1635 "ql_string.qpp"
   if (!args)
      return QoreValue();

   for (unsigned i = 0; i < args->size(); i++)
      print_node(stdout, get_param_value(args, i));
    return QoreValue();
}

// string printf(string fmt, ...) {}
static QoreValue f_printf_VsVV(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1803 "ql_string.qpp"
   AbstractQoreNode* node;
   print_node(stdout, node = q_sprintf(args, 0, 0, xsink));
   return node;
}

// string printf() {}
static QoreValue f_printf(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1812 "ql_string.qpp"
   return null_string();
}

// bool regex(string str, string regex, int options = 0) {}
static QoreValue f_regex_VsVsVi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* regex = HARD_QORE_VALUE_STRING(args, 1);
    int64 options = HARD_QORE_VALUE_INT(args, 2);
# 1051 "ql_string.qpp"
   QoreRegex qr(*regex, options, xsink);
   if (*xsink)
      return QoreValue();

   return qr.exec(str, xsink);
}

// nothing regex() {}
static QoreValue f_regex(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1062 "ql_string.qpp"

    return QoreValue();
}

// *list<string> regex_extract(string str, string regex, int options = 0) {}
static QoreValue f_regex_extract_VsVsVi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* regex = HARD_QORE_VALUE_STRING(args, 1);
    int64 options = HARD_QORE_VALUE_INT(args, 2);
# 1150 "ql_string.qpp"
   QoreRegex qr(*regex, options, xsink);
   if (*xsink)
      return QoreValue();

   return qr.extractSubstrings(str, xsink);
}

// nothing regex_extract() {}
static QoreValue f_regex_extract(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1161 "ql_string.qpp"

    return QoreValue();
}

// string regex_subst(string str, string regex, string subst, int options = 0) {}
static QoreValue f_regex_subst_VsVsVsVi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* regex = HARD_QORE_VALUE_STRING(args, 1);
    const QoreStringNode* subst = HARD_QORE_VALUE_STRING(args, 2);
    int64 options = HARD_QORE_VALUE_INT(args, 3);
# 1102 "ql_string.qpp"
   bool global;
   if (options & QRE_GLOBAL) {
      global = true;
      options &= 0xffffffff;
   }
   else
      global = false;

   QoreRegexSubst qrs(regex, (int)options, xsink);
   if (*xsink)
      return QoreValue();

   if (global)
      qrs.setGlobal();
   return qrs.exec(str, subst, xsink);
}

// nothing regex_subst() {}
static QoreValue f_regex_subst(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1123 "ql_string.qpp"

    return QoreValue();
}

// string replace(string str, string source, string target, int start = 0, int end = -1) {}
static QoreValue f_replace_VsVsVsViVi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* source = HARD_QORE_VALUE_STRING(args, 1);
    const QoreStringNode* target = HARD_QORE_VALUE_STRING(args, 2);
    int64 start = HARD_QORE_VALUE_INT(args, 3);
    int64 end = HARD_QORE_VALUE_INT(args, 4);
# 1185 "ql_string.qpp"
    const QoreEncoding* ccs = str->getEncoding();

    TempEncodingHelper t1(source, ccs, xsink);
    if (*xsink)
        return QoreValue();

    if (t1->empty() || !end || (end > 0 && end < start))
        return str->refSelf();

    TempEncodingHelper t2(target, ccs, xsink);
    if (*xsink)
        return QoreValue();

    QoreStringNode* nstr = new QoreStringNode(ccs);

    const char* cstr = str->c_str();
    size_t plen = t1->size();

    // only execute the replace logic if there is a pattern
    if (plen) {
        const char* pattern = t1->c_str();

        if (start) {
            const char* cn = cstr + str->size();
            qore_size_t i = ccs->getByteLen(cstr, cn, start, xsink);
            cstr = cn;
            if (*xsink || (i == str->size()))
                return QoreValue();

            cstr += i;
        }

        if (end > 0) {
            const char* cn = cstr + str->size();
            qore_size_t i = ccs->getByteLen(cstr, cn, end, xsink);
            cstr = cn;
            if (*xsink || i == str->size())
                return QoreValue();

            cstr += i;
        }

        while (const char* p = (const char*)q_memmem(cstr, str->size() - (cstr - str->c_str()), pattern, plen)) {
            //printd(5, "str: %p p: %p '%s' '%s'->'%s'\n", cstr, p, cstr, pattern, t1->c_str());
            if ((end > 0) && (p - str->c_str() + plen) > (size_t)end)
                break;

            if (p != cstr)
                nstr->concat(cstr, p - cstr);
            nstr->concat(*t2);

            cstr = p + plen;
        }
    }

    // add last field
    if (*cstr)
        nstr->concat(cstr);

    return nstr;
}

// nothing replace() {}
static QoreValue f_replace(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1251 "ql_string.qpp"

    return QoreValue();
}

// string reverse(softstring str) {}
static QoreValue f_reverse_vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
# 1620 "ql_string.qpp"
   return str->reverse();
}

// int rindex(softstring str, softstring substr, softint pos = -1) {}
static QoreValue f_rindex_vsvsvi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* substr = HARD_QORE_VALUE_STRING(args, 1);
    int64 pos = HARD_QORE_VALUE_INT(args, 2);
# 774 "ql_string.qpp"
   return str->rindex(*substr, (qore_offset_t)pos, xsink);
}

// int rindex() {}
static QoreValue f_rindex(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 781 "ql_string.qpp"
   return -1;
}

// string rtrim(string str, *string chars) {}
static QoreValue f_rtrim_VsNs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* chars = get_param_value(args, 1).get<const QoreStringNode>();
# 1462 "ql_string.qpp"
   SimpleRefHolder<QoreStringNode> rv(str->copy());
   return rv->trimTrailing(xsink, chars) ? QoreValue() : rv.release();
}

// *string rtrim(reference<string> str, *string chars) {}
static QoreValue f_rtrim_C17reference_string_Ns(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const ReferenceNode* str = HARD_QORE_VALUE_REF(args, 0);
    const QoreStringNode* chars = get_param_value(args, 1).get<const QoreStringNode>();
# 1484 "ql_string.qpp"
   QoreTypeSafeReferenceHelper ref(str, xsink);
   if (!ref || ref.getType() != NT_STRING)
      return QoreValue();

   QoreStringNode* rv = reinterpret_cast<QoreStringNode*>(ref.getUnique(xsink));
   if (*xsink)
      return QoreValue();

   return rv->trimTrailing(xsink, chars) ? QoreValue() : rv->refSelf();
}

// list<string> split(string sep, string str, bool with_separator = False) {}
static QoreValue f_split_VsVsVb(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* sep = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 1);
    bool with_separator = HARD_QORE_VALUE_BOOL(args, 2);
# 899 "ql_string.qpp"
   // convert pattern encoding to string if necessary
   TempEncodingHelper temp(sep, str->getEncoding(), xsink);
   if (*xsink)
      return QoreValue();

   return split_intern(temp->getBuffer(), temp->strlen(), str->getBuffer(), str->strlen(), str->getEncoding(), with_separator);
}

// list<string> split(string sep, string str, string quote, bool trim_unquoted = False) {}
static QoreValue f_split_VsVsVsVb(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* sep = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 1);
    const QoreStringNode* quote = HARD_QORE_VALUE_STRING(args, 2);
    bool trim_unquoted = HARD_QORE_VALUE_BOOL(args, 3);
# 930 "ql_string.qpp"
   return split_with_quote(sep, str, quote, trim_unquoted, xsink);
}

// list<string> split(binary sep, binary data) {}
static QoreValue f_split_VyVy(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* sep = HARD_QORE_VALUE_BINARY(args, 0);
    const BinaryNode* data = HARD_QORE_VALUE_BINARY(args, 1);
# 948 "ql_string.qpp"
   return split_intern((const char*)sep->getPtr(), sep->size(), (const char*)data->getPtr(), data->size(), 0);
}

// list<string> split() {}
static QoreValue f_split(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 955 "ql_string.qpp"
   return new QoreListNode(stringTypeInfo);
}

// string sprintf(string fmt, ...) {}
static QoreValue f_sprintf_VsVV(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1690 "ql_string.qpp"
   return q_sprintf(args, 0, 0, xsink);
}

// string sprintf() {}
static QoreValue f_sprintf(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1697 "ql_string.qpp"
   return null_string();
}

// int strlen(softstring str) {}
static QoreValue f_strlen_vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
# 478 "ql_string.qpp"
   return str->strlen();
}

// nothing strlen() {}
static QoreValue f_strlen(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 485 "ql_string.qpp"

    return QoreValue();
}

// int strlen(any arg) {}
static QoreValue f_strlen_Va(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 491 "ql_string.qpp"
   return 0;
}

// string strmul(softstring str, softint smul, *softint offset) {}
static QoreValue f_strmul_vsvini(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 smul = HARD_QORE_VALUE_INT(args, 1);
    int64 offset = HARD_QORE_VALUE_INT(args, 2);
# 1555 "ql_string.qpp"
   if (offset < 0)
      return xsink->raiseException("STRMUL-ERROR", "Offset argument has to be >= 0; value: " QLLD, offset);

   if (smul < 0)
      return xsink->raiseException("STRMUL-ERROR", "Multiple argument has to be >= 0; value: " QLLD, smul);

   QoreStringNodeHolder ret(new QoreStringNode);
   while (smul > 0) {
      ret->concat(str);
      --smul;
   }

   if (offset) {
      if (offset > (int64)ret->length())
         return xsink->raiseException("STRMUL-ERROR", "Offset (" QLLD ") is greater than string length (" QSD ")", offset, ret->length());
      ret->splice(ret->length() - offset, xsink);
      if (*xsink)
         return QoreValue();
   }

   return ret.release();
}

// string substr(softstring str, softint start) {}
static QoreValue f_substr_vsvi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 start = HARD_QORE_VALUE_INT(args, 1);
# 585 "ql_string.qpp"
   QoreStringNode* rv = str->substr(start, xsink);
   if (!rv)
      rv = new QoreStringNode(str->getEncoding());
   return rv;
}

// string substr(softstring str, softint start, softint len) {}
static QoreValue f_substr_vsvivi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 start = HARD_QORE_VALUE_INT(args, 1);
    int64 len = HARD_QORE_VALUE_INT(args, 2);
# 611 "ql_string.qpp"
   QoreStringNode* rv = str->substr(start, len, xsink);
   if (!rv)
      rv = new QoreStringNode(str->getEncoding());
   return rv;
}

// binary substr(binary b, softint start) {}
static QoreValue f_substr_Vyvi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* b = HARD_QORE_VALUE_BINARY(args, 0);
    int64 start = HARD_QORE_VALUE_INT(args, 1);
# 636 "ql_string.qpp"
   BinaryNode* b1 = new BinaryNode;
   b->substr(*b1, start);
   return b1;
}

// binary substr(binary b, softint start, softint len) {}
static QoreValue f_substr_Vyvivi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* b = HARD_QORE_VALUE_BINARY(args, 0);
    int64 start = HARD_QORE_VALUE_INT(args, 1);
    int64 len = HARD_QORE_VALUE_INT(args, 2);
# 661 "ql_string.qpp"
   BinaryNode* b1 = new BinaryNode;
   b->substr(*b1, start, len);
   return b1;
}

// nothing substr() {}
static QoreValue f_substr(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 670 "ql_string.qpp"

    return QoreValue();
}

// string tolower(string str) {}
static QoreValue f_tolower_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
# 517 "ql_string.qpp"
   SimpleRefHolder<QoreStringNode> rv(new QoreStringNode(str->getEncoding()));
   if (do_tolower(*(*rv), *str, xsink))
      return QoreValue();

   return rv.release();
}

// nothing tolower() {}
static QoreValue f_tolower(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 528 "ql_string.qpp"

    return QoreValue();
}

// string toupper(string str) {}
static QoreValue f_toupper_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
# 553 "ql_string.qpp"
   SimpleRefHolder<QoreStringNode> rv(new QoreStringNode(str->getEncoding()));
   if (do_toupper(*(*rv), *str, xsink))
      return QoreValue();

   return rv.release();
}

// nothing toupper() {}
static QoreValue f_toupper(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 564 "ql_string.qpp"

    return QoreValue();
}

// string trim(string str, *string chars) {}
static QoreValue f_trim_VsNs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* chars = get_param_value(args, 1).get<const QoreStringNode>();
# 1362 "ql_string.qpp"
   SimpleRefHolder<QoreStringNode> rv(str->copy());
   return rv->trim(xsink, chars) ? QoreValue() : rv.release();
}

// *string trim(reference<string> str, *string chars) {}
static QoreValue f_trim_C17reference_string_Ns(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const ReferenceNode* str = HARD_QORE_VALUE_REF(args, 0);
    const QoreStringNode* chars = get_param_value(args, 1).get<const QoreStringNode>();
# 1382 "ql_string.qpp"
   QoreTypeSafeReferenceHelper ref(str, xsink);
   if (!ref || ref.getType() != NT_STRING)
      return QoreValue();

   QoreStringNode* rv = reinterpret_cast<QoreStringNode*>(ref.getUnique(xsink));
   if (*xsink)
      return QoreValue();

   return rv->trim(xsink, chars) ? QoreValue() : rv->refSelf();
}

// nothing trim() {}
static QoreValue f_trim(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1397 "ql_string.qpp"

    return QoreValue();
}

// string trunc_str(softstring str, softint len, *string encoding) {}
static QoreValue f_trunc_str_vsviNs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    int64 len = HARD_QORE_VALUE_INT(args, 1);
    const QoreStringNode* encoding = get_param_value(args, 2).get<const QoreStringNode>();
# 1511 "ql_string.qpp"
   const QoreEncoding *enc = encoding ? QEM.findCreate(encoding) : str->getEncoding();
   if (len <= 0)
      return new QoreStringNode(enc);

   TempEncodingHelper tmp(str, enc, xsink);
   if (!tmp)
      return QoreValue();

   if (tmp->strlen() < (qore_size_t)len) {
      len = tmp->strlen();
      return new QoreStringNode(tmp.giveBuffer(), len, len + 1, enc);
   }

   if (!enc->isMultiByte())
      return new QoreStringNode(tmp->getBuffer(), len, enc);

   // find position of last character fitting in len bytes
   const char* p = tmp->getBuffer();
   int64 sl = 0;
   while (true) {
      qore_offset_t size = enc->getCharLen(p, len - sl);
      if (size <= 0 || ((sl + size) > len))
         break;
      sl += size;
      p += size;
   }

   return new QoreStringNode(tmp->getBuffer(), sl, enc);
}

// string vprintf(string fmt, ...) {}
static QoreValue f_vprintf_VsVV(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1860 "ql_string.qpp"
   AbstractQoreNode* node;
   print_node(stdout, node = q_vsprintf(args, 0, 0, xsink));
   return node;
}

// string vprintf() {}
static QoreValue f_vprintf(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1869 "ql_string.qpp"
   return null_string();
}

// string vsprintf(string fmt, any varg) {}
static QoreValue f_vsprintf_VsVa(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1743 "ql_string.qpp"
   return q_vsprintf(args, 0, 0, xsink);
}

// string vsprintf() {}
static QoreValue f_vsprintf(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1750 "ql_string.qpp"
   return null_string();
}


DLLLOCAL void init_string_functions(QoreNamespace& ns) {
    // int bindex(softstring str, softstring substr, softint pos = 0) {}
    ns.addBuiltinVariant("bindex", (q_func_n_t)f_bindex_vsvsvi, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 3, softStringTypeInfo, NULL, "str", softStringTypeInfo, NULL, "substr", softBigIntTypeInfo, zero(), "pos");
    // int bindex() {}
    ns.addBuiltinVariant("bindex", (q_func_n_t)f_bindex, QC_RUNTIME_NOOP, QDOM_DEFAULT, bigIntTypeInfo);
    // int brindex(softstring str, softstring substr, softint pos = -1) {}
    ns.addBuiltinVariant("brindex", (q_func_n_t)f_brindex_vsvsvi, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 3, softStringTypeInfo, NULL, "str", softStringTypeInfo, NULL, "substr", softBigIntTypeInfo, new QoreBigIntNode(-1), "pos");
    // int brindex() {}
    ns.addBuiltinVariant("brindex", (q_func_n_t)f_brindex, QC_RUNTIME_NOOP, QDOM_DEFAULT, bigIntTypeInfo);
    // string chomp(string str) {}
    ns.addBuiltinVariant("chomp", (q_func_n_t)f_chomp_Vs, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, NULL, "str");
    // *string chomp(reference<string> str) {}
    ns.addBuiltinVariant("chomp", (q_func_n_t)f_chomp_C17reference_string_, QC_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo, 1, qore_get_complex_reference_type(stringTypeInfo), NULL, "str");
    // nothing chomp() {}
    ns.addBuiltinVariant("chomp", (q_func_n_t)f_chomp, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // string chr(softint val, *string encoding) {}
    ns.addBuiltinVariant("chr", (q_func_n_t)f_chr_viNs, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 2, softBigIntTypeInfo, NULL, "val", stringOrNothingTypeInfo, NULL, "encoding");
    // string chr(any arg) {}
    ns.addBuiltinVariant("chr", (q_func_n_t)f_chr_Va, QC_NOOP, QDOM_DEFAULT, stringTypeInfo, 1, anyTypeInfo, NULL, "arg");
    // nothing chr() {}
    ns.addBuiltinVariant("chr", (q_func_n_t)f_chr, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // string convert_encoding(string str, string encoding) {}
    ns.addBuiltinVariant("convert_encoding", (q_func_n_t)f_convert_encoding_VsVs, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 2, stringTypeInfo, NULL, "str", stringTypeInfo, NULL, "encoding");
    // nothing convert_encoding() {}
    ns.addBuiltinVariant("convert_encoding", (q_func_n_t)f_convert_encoding, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // string f_printf(string fmt, ...) {}
    ns.addBuiltinVariant("f_printf", (q_func_n_t)f_f_printf_VsVV, QC_USES_EXTRA_ARGS, QDOM_TERMINAL_IO, stringTypeInfo, 1, stringTypeInfo, NULL, "fmt");
    // string f_printf() {}
    ns.addBuiltinVariant("f_printf", (q_func_n_t)f_f_printf, QC_NOOP, QDOM_DEFAULT, stringTypeInfo);
    // string f_sprintf(string fmt, ...) {}
    ns.addBuiltinVariant("f_sprintf", (q_func_n_t)f_f_sprintf_VsVV, QC_CONSTANT|QC_USES_EXTRA_ARGS, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, NULL, "fmt");
    // string f_sprintf() {}
    ns.addBuiltinVariant("f_sprintf", (q_func_n_t)f_f_sprintf, QC_NOOP, QDOM_DEFAULT, stringTypeInfo);
    // string f_vprintf(string fmt, any varg) {}
    ns.addBuiltinVariant("f_vprintf", (q_func_n_t)f_f_vprintf_VsVa, QC_NO_FLAGS, QDOM_TERMINAL_IO, stringTypeInfo, 2, stringTypeInfo, NULL, "fmt", anyTypeInfo, NULL, "varg");
    // string f_vsprintf(string fmt, any varg) {}
    ns.addBuiltinVariant("f_vsprintf", (q_func_n_t)f_f_vsprintf_VsVa, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 2, stringTypeInfo, NULL, "fmt", anyTypeInfo, NULL, "varg");
    // nothing flush() {}
    ns.addBuiltinVariant("flush", (q_func_n_t)f_flush, QC_NO_FLAGS, QDOM_TERMINAL_IO, nothingTypeInfo);
    // string force_encoding(string str, string encoding) {}
    ns.addBuiltinVariant("force_encoding", (q_func_n_t)f_force_encoding_VsVs, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 2, stringTypeInfo, NULL, "str", stringTypeInfo, NULL, "encoding");
    // nothing force_encoding() {}
    ns.addBuiltinVariant("force_encoding", (q_func_n_t)f_force_encoding, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // string format_number(string fmt, softfloat num) {}
    ns.addBuiltinVariant("format_number", (q_func_n_t)f_format_number_Vsvf, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 2, stringTypeInfo, NULL, "fmt", softFloatTypeInfo, NULL, "num");
    // nothing format_number() {}
    ns.addBuiltinVariant("format_number", (q_func_n_t)f_format_number, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // string get_encoding(string str) {}
    ns.addBuiltinVariant("get_encoding", (q_func_n_t)f_get_encoding_Vs, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, NULL, "str");
    // nothing get_encoding() {}
    ns.addBuiltinVariant("get_encoding", (q_func_n_t)f_get_encoding, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // int index(softstring str, softstring substr, softint pos = 0) {}
    ns.addBuiltinVariant("index", (q_func_n_t)f_index_vsvsvi, QC_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo, 3, softStringTypeInfo, NULL, "str", softStringTypeInfo, NULL, "substr", softBigIntTypeInfo, zero(), "pos");
    // int index() {}
    ns.addBuiltinVariant("index", (q_func_n_t)f_index, QC_RUNTIME_NOOP, QDOM_DEFAULT, bigIntTypeInfo);
    // string join(string str, ...) {}
    ns.addBuiltinVariant("join", (q_func_n_t)f_join_VsVV, QC_RET_VALUE_ONLY|QC_USES_EXTRA_ARGS, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, NULL, "str");
    // string join(string str, list l) {}
    ns.addBuiltinVariant("join", (q_func_n_t)f_join_VsVl, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 2, stringTypeInfo, NULL, "str", listTypeInfo, NULL, "l");
    // nothing join() {}
    ns.addBuiltinVariant("join", (q_func_n_t)f_join, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // int length(softstring str) {}
    ns.addBuiltinVariant("length", (q_func_n_t)f_length_vs, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, softStringTypeInfo, NULL, "str");
    // int length(binary bin) {}
    ns.addBuiltinVariant("length", (q_func_n_t)f_length_Vy, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, binaryTypeInfo, NULL, "bin");
    // nothing length() {}
    ns.addBuiltinVariant("length", (q_func_n_t)f_length, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // int length(any arg) {}
    ns.addBuiltinVariant("length", (q_func_n_t)f_length_Va, QC_NOOP, QDOM_DEFAULT, bigIntTypeInfo, 1, anyTypeInfo, NULL, "arg");
    // string ltrim(string str, *string chars) {}
    ns.addBuiltinVariant("ltrim", (q_func_n_t)f_ltrim_VsNs, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 2, stringTypeInfo, NULL, "str", stringOrNothingTypeInfo, NULL, "chars");
    // *string ltrim(reference<string> str, *string chars) {}
    ns.addBuiltinVariant("ltrim", (q_func_n_t)f_ltrim_C17reference_string_Ns, QC_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo, 2, qore_get_complex_reference_type(stringTypeInfo), NULL, "str", stringOrNothingTypeInfo, NULL, "chars");
    // int ord(softstring str, softint offset = 0) {}
    ns.addBuiltinVariant("ord", (q_func_n_t)f_ord_vsvi, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 2, softStringTypeInfo, NULL, "str", softBigIntTypeInfo, zero(), "offset");
    // int ord() {}
    ns.addBuiltinVariant("ord", (q_func_n_t)f_ord, QC_RUNTIME_NOOP, QDOM_DEFAULT, bigIntTypeInfo);
    // bool parse_boolean(string str) {}
    ns.addBuiltinVariant("parse_boolean", (q_func_n_t)f_parse_boolean_Vs, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo, 1, stringTypeInfo, NULL, "str");
    // bool parse_boolean(any val) {}
    ns.addBuiltinVariant("parse_boolean", (q_func_n_t)f_parse_boolean_Va, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo, 1, anyTypeInfo, NULL, "val");
    // float parse_float(string str, string fmt) {}
    ns.addBuiltinVariant("parse_float", (q_func_n_t)f_parse_float_VsVs, QC_RET_VALUE_ONLY, QDOM_DEFAULT, floatTypeInfo, 2, stringTypeInfo, NULL, "str", stringTypeInfo, NULL, "fmt");
    // int parse_int(string str, string fmt) {}
    ns.addBuiltinVariant("parse_int", (q_func_n_t)f_parse_int_VsVs, QC_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo, 2, stringTypeInfo, NULL, "str", stringTypeInfo, NULL, "fmt");
    // number parse_number(string str, string fmt) {}
    ns.addBuiltinVariant("parse_number", (q_func_n_t)f_parse_number_VsVs, QC_RET_VALUE_ONLY, QDOM_DEFAULT, numberTypeInfo, 2, stringTypeInfo, NULL, "str", stringTypeInfo, NULL, "fmt");
    // nothing print(...) {}
    ns.addBuiltinVariant("print", (q_func_n_t)f_print_VV, QC_USES_EXTRA_ARGS, QDOM_TERMINAL_IO, nothingTypeInfo);
    // string printf(string fmt, ...) {}
    ns.addBuiltinVariant("printf", (q_func_n_t)f_printf_VsVV, QC_USES_EXTRA_ARGS, QDOM_TERMINAL_IO, stringTypeInfo, 1, stringTypeInfo, NULL, "fmt");
    // string printf() {}
    ns.addBuiltinVariant("printf", (q_func_n_t)f_printf, QC_NOOP, QDOM_TERMINAL_IO, stringTypeInfo);
    // bool regex(string str, string regex, int options = 0) {}
    ns.addBuiltinVariant("regex", (q_func_n_t)f_regex_VsVsVi, QC_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo, 3, stringTypeInfo, NULL, "str", stringTypeInfo, NULL, "regex", bigIntTypeInfo, zero(), "options");
    // nothing regex() {}
    ns.addBuiltinVariant("regex", (q_func_n_t)f_regex, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // *list<string> regex_extract(string str, string regex, int options = 0) {}
    ns.addBuiltinVariant("regex_extract", (q_func_n_t)f_regex_extract_VsVsVi, QC_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_or_nothing_type(stringTypeInfo), 3, stringTypeInfo, NULL, "str", stringTypeInfo, NULL, "regex", bigIntTypeInfo, zero(), "options");
    // nothing regex_extract() {}
    ns.addBuiltinVariant("regex_extract", (q_func_n_t)f_regex_extract, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // string regex_subst(string str, string regex, string subst, int options = 0) {}
    ns.addBuiltinVariant("regex_subst", (q_func_n_t)f_regex_subst_VsVsVsVi, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 4, stringTypeInfo, NULL, "str", stringTypeInfo, NULL, "regex", stringTypeInfo, NULL, "subst", bigIntTypeInfo, zero(), "options");
    // nothing regex_subst() {}
    ns.addBuiltinVariant("regex_subst", (q_func_n_t)f_regex_subst, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // string replace(string str, string source, string target, int start = 0, int end = -1) {}
    ns.addBuiltinVariant("replace", (q_func_n_t)f_replace_VsVsVsViVi, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 5, stringTypeInfo, NULL, "str", stringTypeInfo, NULL, "source", stringTypeInfo, NULL, "target", bigIntTypeInfo, zero(), "start", bigIntTypeInfo, new QoreBigIntNode(-1), "end");
    // nothing replace() {}
    ns.addBuiltinVariant("replace", (q_func_n_t)f_replace, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // string reverse(softstring str) {}
    ns.addBuiltinVariant("reverse", (q_func_n_t)f_reverse_vs, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, softStringTypeInfo, NULL, "str");
    // int rindex(softstring str, softstring substr, softint pos = -1) {}
    ns.addBuiltinVariant("rindex", (q_func_n_t)f_rindex_vsvsvi, QC_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo, 3, softStringTypeInfo, NULL, "str", softStringTypeInfo, NULL, "substr", softBigIntTypeInfo, new QoreBigIntNode(-1), "pos");
    // int rindex() {}
    ns.addBuiltinVariant("rindex", (q_func_n_t)f_rindex, QC_RUNTIME_NOOP, QDOM_DEFAULT, bigIntTypeInfo);
    // string rtrim(string str, *string chars) {}
    ns.addBuiltinVariant("rtrim", (q_func_n_t)f_rtrim_VsNs, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 2, stringTypeInfo, NULL, "str", stringOrNothingTypeInfo, NULL, "chars");
    // *string rtrim(reference<string> str, *string chars) {}
    ns.addBuiltinVariant("rtrim", (q_func_n_t)f_rtrim_C17reference_string_Ns, QC_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo, 2, qore_get_complex_reference_type(stringTypeInfo), NULL, "str", stringOrNothingTypeInfo, NULL, "chars");
    // list<string> split(string sep, string str, bool with_separator = False) {}
    ns.addBuiltinVariant("split", (q_func_n_t)f_split_VsVsVb, QC_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(stringTypeInfo), 3, stringTypeInfo, NULL, "sep", stringTypeInfo, NULL, "str", boolTypeInfo, &False, "with_separator");
    // list<string> split(string sep, string str, string quote, bool trim_unquoted = False) {}
    ns.addBuiltinVariant("split", (q_func_n_t)f_split_VsVsVsVb, QC_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(stringTypeInfo), 4, stringTypeInfo, NULL, "sep", stringTypeInfo, NULL, "str", stringTypeInfo, NULL, "quote", boolTypeInfo, &False, "trim_unquoted");
    // list<string> split(binary sep, binary data) {}
    ns.addBuiltinVariant("split", (q_func_n_t)f_split_VyVy, QC_CONSTANT, QDOM_DEFAULT, qore_get_complex_list_type(stringTypeInfo), 2, binaryTypeInfo, NULL, "sep", binaryTypeInfo, NULL, "data");
    // list<string> split() {}
    ns.addBuiltinVariant("split", (q_func_n_t)f_split, QC_RUNTIME_NOOP, QDOM_DEFAULT, qore_get_complex_list_type(stringTypeInfo));
    // string sprintf(string fmt, ...) {}
    ns.addBuiltinVariant("sprintf", (q_func_n_t)f_sprintf_VsVV, QC_CONSTANT|QC_USES_EXTRA_ARGS, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, NULL, "fmt");
    // string sprintf() {}
    ns.addBuiltinVariant("sprintf", (q_func_n_t)f_sprintf, QC_NOOP, QDOM_DEFAULT, stringTypeInfo);
    // int strlen(softstring str) {}
    ns.addBuiltinVariant("strlen", (q_func_n_t)f_strlen_vs, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, softStringTypeInfo, NULL, "str");
    // nothing strlen() {}
    ns.addBuiltinVariant("strlen", (q_func_n_t)f_strlen, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // int strlen(any arg) {}
    ns.addBuiltinVariant("strlen", (q_func_n_t)f_strlen_Va, QC_NOOP, QDOM_DEFAULT, bigIntTypeInfo, 1, anyTypeInfo, NULL, "arg");
    // string strmul(softstring str, softint smul, *softint offset) {}
    ns.addBuiltinVariant("strmul", (q_func_n_t)f_strmul_vsvini, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 3, softStringTypeInfo, NULL, "str", softBigIntTypeInfo, NULL, "smul", softBigIntOrNothingTypeInfo, NULL, "offset");
    // string substr(softstring str, softint start) {}
    ns.addBuiltinVariant("substr", (q_func_n_t)f_substr_vsvi, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 2, softStringTypeInfo, NULL, "str", softBigIntTypeInfo, NULL, "start");
    // string substr(softstring str, softint start, softint len) {}
    ns.addBuiltinVariant("substr", (q_func_n_t)f_substr_vsvivi, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 3, softStringTypeInfo, NULL, "str", softBigIntTypeInfo, NULL, "start", softBigIntTypeInfo, NULL, "len");
    // binary substr(binary b, softint start) {}
    ns.addBuiltinVariant("substr", (q_func_n_t)f_substr_Vyvi, QC_CONSTANT, QDOM_DEFAULT, binaryTypeInfo, 2, binaryTypeInfo, NULL, "b", softBigIntTypeInfo, NULL, "start");
    // binary substr(binary b, softint start, softint len) {}
    ns.addBuiltinVariant("substr", (q_func_n_t)f_substr_Vyvivi, QC_CONSTANT, QDOM_DEFAULT, binaryTypeInfo, 3, binaryTypeInfo, NULL, "b", softBigIntTypeInfo, NULL, "start", softBigIntTypeInfo, NULL, "len");
    // nothing substr() {}
    ns.addBuiltinVariant("substr", (q_func_n_t)f_substr, QC_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // string tolower(string str) {}
    ns.addBuiltinVariant("tolower", (q_func_n_t)f_tolower_Vs, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, NULL, "str");
    // nothing tolower() {}
    ns.addBuiltinVariant("tolower", (q_func_n_t)f_tolower, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // string toupper(string str) {}
    ns.addBuiltinVariant("toupper", (q_func_n_t)f_toupper_Vs, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, NULL, "str");
    // nothing toupper() {}
    ns.addBuiltinVariant("toupper", (q_func_n_t)f_toupper, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // string trim(string str, *string chars) {}
    ns.addBuiltinVariant("trim", (q_func_n_t)f_trim_VsNs, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 2, stringTypeInfo, NULL, "str", stringOrNothingTypeInfo, NULL, "chars");
    // *string trim(reference<string> str, *string chars) {}
    ns.addBuiltinVariant("trim", (q_func_n_t)f_trim_C17reference_string_Ns, QC_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo, 2, qore_get_complex_reference_type(stringTypeInfo), NULL, "str", stringOrNothingTypeInfo, NULL, "chars");
    // nothing trim() {}
    ns.addBuiltinVariant("trim", (q_func_n_t)f_trim, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // string trunc_str(softstring str, softint len, *string encoding) {}
    ns.addBuiltinVariant("trunc_str", (q_func_n_t)f_trunc_str_vsviNs, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 3, softStringTypeInfo, NULL, "str", softBigIntTypeInfo, NULL, "len", stringOrNothingTypeInfo, NULL, "encoding");
    // string vprintf(string fmt, ...) {}
    ns.addBuiltinVariant("vprintf", (q_func_n_t)f_vprintf_VsVV, QC_USES_EXTRA_ARGS, QDOM_TERMINAL_IO, stringTypeInfo, 1, stringTypeInfo, NULL, "fmt");
    // string vprintf() {}
    ns.addBuiltinVariant("vprintf", (q_func_n_t)f_vprintf, QC_NOOP, QDOM_DEFAULT, stringTypeInfo);
    // string vsprintf(string fmt, any varg) {}
    ns.addBuiltinVariant("vsprintf", (q_func_n_t)f_vsprintf_VsVa, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 2, stringTypeInfo, NULL, "fmt", anyTypeInfo, NULL, "varg");
    // string vsprintf() {}
    ns.addBuiltinVariant("vsprintf", (q_func_n_t)f_vsprintf, QC_NOOP, QDOM_DEFAULT, stringTypeInfo);

}

DLLLOCAL void init_string_constants(QoreNamespace& ns) {
# 309 "ql_string.qpp"
# 317 "ql_string.qpp"
# 320 "ql_string.qpp"
# 323 "ql_string.qpp"
    ns.addConstant("RE_Caseless", new QoreBigIntNode(PCRE_CASELESS));
    ns.addConstant("RE_DotAll", new QoreBigIntNode(PCRE_DOTALL));
    ns.addConstant("RE_Extended", new QoreBigIntNode(PCRE_EXTENDED));
    ns.addConstant("RE_Global", new QoreBigIntNode(QRE_GLOBAL));
    ns.addConstant("RE_MultiLine", new QoreBigIntNode(PCRE_MULTILINE));
# 330 "ql_string.qpp"
# 406 "ql_string.qpp"
# 431 "ql_string.qpp"
# 447 "ql_string.qpp"
# 453 "ql_string.qpp"
# 460 "ql_string.qpp"
# 482 "ql_string.qpp"
# 488 "ql_string.qpp"
# 495 "ql_string.qpp"
# 525 "ql_string.qpp"
# 531 "ql_string.qpp"
# 561 "ql_string.qpp"
# 567 "ql_string.qpp"
# 592 "ql_string.qpp"
# 618 "ql_string.qpp"
# 642 "ql_string.qpp"
# 667 "ql_string.qpp"
# 673 "ql_string.qpp"
# 704 "ql_string.qpp"
# 711 "ql_string.qpp"
# 740 "ql_string.qpp"
# 747 "ql_string.qpp"
# 778 "ql_string.qpp"
# 785 "ql_string.qpp"
# 814 "ql_string.qpp"
# 821 "ql_string.qpp"
# 842 "ql_string.qpp"
# 849 "ql_string.qpp"
# 869 "ql_string.qpp"
# 876 "ql_string.qpp"
# 882 "ql_string.qpp"
# 908 "ql_string.qpp"
# 934 "ql_string.qpp"
# 952 "ql_string.qpp"
# 959 "ql_string.qpp"
# 976 "ql_string.qpp"
# 982 "ql_string.qpp"
# 1000 "ql_string.qpp"
# 1006 "ql_string.qpp"
# 1023 "ql_string.qpp"
# 1029 "ql_string.qpp"
# 1059 "ql_string.qpp"
# 1065 "ql_string.qpp"
# 1120 "ql_string.qpp"
# 1126 "ql_string.qpp"
# 1158 "ql_string.qpp"
# 1164 "ql_string.qpp"
# 1248 "ql_string.qpp"
# 1254 "ql_string.qpp"
# 1272 "ql_string.qpp"
# 1292 "ql_string.qpp"
# 1298 "ql_string.qpp"
# 1318 "ql_string.qpp"
# 1343 "ql_string.qpp"
# 1349 "ql_string.qpp"
# 1367 "ql_string.qpp"
# 1394 "ql_string.qpp"
# 1400 "ql_string.qpp"
# 1419 "ql_string.qpp"
# 1446 "ql_string.qpp"
# 1467 "ql_string.qpp"
# 1496 "ql_string.qpp"
# 1542 "ql_string.qpp"
# 1579 "ql_string.qpp"
# 1600 "ql_string.qpp"
# 1606 "ql_string.qpp"
# 1624 "ql_string.qpp"
# 1643 "ql_string.qpp"
# 1665 "ql_string.qpp"
# 1672 "ql_string.qpp"
# 1694 "ql_string.qpp"
# 1701 "ql_string.qpp"
# 1725 "ql_string.qpp"
# 1747 "ql_string.qpp"
# 1754 "ql_string.qpp"
# 1778 "ql_string.qpp"
# 1785 "ql_string.qpp"
# 1809 "ql_string.qpp"
# 1816 "ql_string.qpp"
# 1842 "ql_string.qpp"
# 1866 "ql_string.qpp"
# 1873 "ql_string.qpp"
# 1884 "ql_string.qpp"
# 1900 "ql_string.qpp"
# 1914 "ql_string.qpp"
# 1943 "ql_string.qpp"
# 1972 "ql_string.qpp"

}
