/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  ql_thread.qpp

  POSIX thread library for Qore

  Qore Programming Language

  Copyright (C) 2003 - 2017 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/ql_thread.h"
#include "qore/intern/QC_TimeZone.h"
#include "qore/intern/qore_program_private.h"
#include "qore/intern/QC_AbstractThreadResource.h"
#include "qore/intern/QoreHashNodeIntern.h"
#include "qore/intern/QC_TimeZone.h"

#include <pthread.h>
#include <thread>

extern QoreClass* QC_TIMEZONE;

static void delete_thread_data_intern(const QoreValueList* args, ExceptionSink* xsink) {
   assert(xsink);
   // get thread data hash
   QoreHashNode *data = getProgram()->getThreadData();

   // iterate through arguments and delete each key
   for (unsigned i = 0; i < args->size(); i++) {
      QoreValue p = get_param_value(args, i);
      if (!p.isNothing()) {
         QoreStringValueHelper t(p, QCS_DEFAULT, xsink);
         if (*xsink)
            break;

         data->deleteKey(*t, xsink);
         if (*xsink)
            break;
      }
   }
}

static QoreHashNode *remove_thread_data_intern(const QoreValueList* args, ExceptionSink* xsink) {
   assert(xsink);
   // get thread data hash
   QoreHashNode *data = getProgram()->getThreadData();

   QoreHashNode* rv = new QoreHashNode;

   // iterate through arguments and remove each key
   for (unsigned i = 0; i < args->size(); i++) {
      QoreValue p = get_param_value(args, i);
      if (!p.isNothing()) {
         QoreStringValueHelper t(p, QCS_DEFAULT, xsink);
         if (*xsink)
            break;

         rv->setKeyValue(*t, data->takeKeyValue((*t)->getBuffer()), xsink);
         assert(!*xsink);
      }
   }

   return rv;
}

static void delete_thread_data_intern(const QoreListNode* args, ExceptionSink* xsink) {
   assert(xsink);
   // get thread data hash
   QoreHashNode *data = getProgram()->getThreadData();

   // iterate through arguments and delete each key
   for (unsigned i = 0; i < args->size(); i++) {
      const AbstractQoreNode* p = get_param(args, i);
      if (!is_nothing(p)) {
         QoreStringValueHelper t(p, QCS_DEFAULT, xsink);
         if (*xsink)
            break;

         data->deleteKey(*t, xsink);
         if (*xsink)
            break;
      }
   }
}

static QoreHashNode *remove_thread_data_intern(const QoreListNode* args, ExceptionSink* xsink) {
   assert(xsink);
   // get thread data hash
   QoreHashNode *data = getProgram()->getThreadData();

   QoreHashNode* rv = new QoreHashNode;

   // iterate through arguments and remove each key
   for (unsigned i = 0; i < args->size(); i++) {
      const AbstractQoreNode* p = get_param(args, i);
      if (!is_nothing(p)) {
         QoreStringValueHelper t(p, QCS_DEFAULT, xsink);
         if (*xsink)
            break;

         rv->setKeyValue(*t, data->takeKeyValue((*t)->getBuffer()), xsink);
         assert(!*xsink);
      }
   }

   return rv;
}
TypedHashDecl* init_hashdecl_CallStackInfo(QoreNamespace& ns) {
    TypedHashDecl* hd = new TypedHashDecl("CallStackInfo");
    hd->addMember("endline", bigIntTypeInfo, QoreValue());
    hd->addMember("file", stringTypeInfo, QoreValue());
    hd->addMember("function", stringTypeInfo, QoreValue());
    hd->addMember("line", bigIntTypeInfo, QoreValue());
    hd->addMember("offset", bigIntTypeInfo, QoreValue());
    hd->addMember("source", stringTypeInfo, QoreValue());
    hd->addMember("type", stringTypeInfo, QoreValue());
    hd->addMember("typecode", bigIntTypeInfo, QoreValue());
    ns.addSystemHashDecl(hd);
    return hd;
}

# 163 "ql_thread.qpp"
# 182 "ql_thread.qpp"
# 197 "ql_thread.qpp"
# 213 "ql_thread.qpp"
# 228 "ql_thread.qpp"
# 244 "ql_thread.qpp"
# 250 "ql_thread.qpp"
# 268 "ql_thread.qpp"
# 285 "ql_thread.qpp"
# 302 "ql_thread.qpp"
# 319 "ql_thread.qpp"
# 332 "ql_thread.qpp"
# 351 "ql_thread.qpp"
# 357 "ql_thread.qpp"
# 373 "ql_thread.qpp"
# 390 "ql_thread.qpp"
# 421 "ql_thread.qpp"
# 449 "ql_thread.qpp"
# 471 "ql_thread.qpp"
# 504 "ql_thread.qpp"
# 542 "ql_thread.qpp"
# 577 "ql_thread.qpp"
# 599 "ql_thread.qpp"
# 624 "ql_thread.qpp"
# 644 "ql_thread.qpp"
# 668 "ql_thread.qpp"
# 691 "ql_thread.qpp"
# 716 "ql_thread.qpp"
# 739 "ql_thread.qpp"
# 763 "ql_thread.qpp"
// nothing delete_all_thread_data() {}
static QoreValue f_delete_all_thread_data(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 328 "ql_thread.qpp"
   qore_program_private::clearThreadData(*getProgram(), xsink);
    return QoreValue();
}

// nothing delete_thread_data(...) {}
static QoreValue f_delete_thread_data_VV(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 263 "ql_thread.qpp"
   if (num_args(args))
      delete_thread_data_intern(args, xsink);
    return QoreValue();
}

// nothing delete_thread_data(list l) {}
static QoreValue f_delete_thread_data_Vl(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
# 281 "ql_thread.qpp"
   delete_thread_data_intern(l, xsink);
    return QoreValue();
}

// hash<string,list<hash<CallStackInfo>>> getAllThreadCallStacks() {}
static QoreValue f_getAllThreadCallStacks(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 382 "ql_thread.qpp"
#ifdef QORE_RUNTIME_THREAD_STACK_TRACE
   return getAllCallStacks();
#else
   return xsink->raiseException("MISSING-FEATURE-ERROR", "this version of the Qore library was built without support for runtime thread stack tracing; check Qore::Option::HAVE_RUNTIME_THREAD_STACK_TRACE before calling");
#endif
    return QoreValue();
}

// hash<string,list<hash<CallStackInfo>>> get_all_thread_call_stacks() {}
static QoreValue f_get_all_thread_call_stacks(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 413 "ql_thread.qpp"
#ifdef QORE_RUNTIME_THREAD_STACK_TRACE
   return getAllCallStacks();
#else
   return xsink->raiseException("MISSING-FEATURE-ERROR", "this version of the Qore library was built without support for runtime thread stack tracing; check Qore::Option::HAVE_RUNTIME_THREAD_STACK_TRACE before calling");
#endif
    return QoreValue();
}

// hash get_all_thread_data() {}
static QoreValue f_get_all_thread_data(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 368 "ql_thread.qpp"
   const QoreHashNode* h = getProgram()->getThreadData();
   return h->copy();
}

// list<hash<CallStackInfo>> get_thread_call_stack() {}
static QoreValue f_get_thread_call_stack(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 441 "ql_thread.qpp"
#ifdef QORE_RUNTIME_THREAD_STACK_TRACE
   return getCallStack()->getCallStack();
#else
   return xsink->raiseException("MISSING-FEATURE-ERROR", "this version of the Qore library was built without support for runtime thread stack tracing; check Qore::Option::HAVE_RUNTIME_THREAD_STACK_TRACE before calling");
#endif
    return QoreValue();
}

// auto get_thread_data(string key) {}
static QoreValue f_get_thread_data_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* key = HARD_QORE_VALUE_STRING(args, 0);
# 343 "ql_thread.qpp"
   TempEncodingHelper tmp(key, QCS_DEFAULT, xsink);
   if (!tmp)
      return QoreValue();
   const QoreHashNode* h = getProgram()->getThreadData();
   return qore_hash_private::get(*h)->getReferencedKeyValueIntern(tmp->c_str());
}

// nothing get_thread_data() {}
static QoreValue f_get_thread_data(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 354 "ql_thread.qpp"

    return QoreValue();
}

// *TimeZone get_thread_tz() {}
static QoreValue f_get_thread_tz(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 660 "ql_thread.qpp"
   bool set;
   const AbstractQoreZoneInfo* tz = get_thread_tz(set);
   if (!set)
      return QoreValue();
   return new QoreObject(QC_TIMEZONE, 0, new TimeZoneData(tz));
}

// int gettid() {}
static QoreValue f_gettid(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 178 "ql_thread.qpp"
   return gettid();
}

// nothing mark_thread_resources() {}
static QoreValue f_mark_thread_resources(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 573 "ql_thread.qpp"
   mark_thread_resources();
    return QoreValue();
}

// int num_threads() {}
static QoreValue f_num_threads(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 193 "ql_thread.qpp"
   return thread_list.getNumThreads();
}

// *hash remove_thread_data(...) {}
static QoreValue f_remove_thread_data_VV(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 298 "ql_thread.qpp"
   return num_args(args) ? remove_thread_data_intern(args, xsink) : 0;
}

// hash remove_thread_data(list l) {}
static QoreValue f_remove_thread_data_Vl(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
# 315 "ql_thread.qpp"
   return remove_thread_data_intern(l, xsink);
}

// bool remove_thread_resource(AbstractThreadResource resource) {}
static QoreValue f_remove_thread_resource_C22AbstractThreadResource(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(resource, AbstractQoreThreadResource, args, 0, CID_ABSTRACTTHREADRESOURCE, "remove_thread_resource()", "AbstractThreadResource", xsink);
    if (*xsink)
        return 0;
# 711 "ql_thread.qpp"
   ReferenceHolder<AbstractQoreThreadResource> holder(resource, xsink);
   return remove_thread_resource(resource) ? false : true;
}

// bool remove_thread_resource(code resource) {}
static QoreValue f_remove_thread_resource_Vc(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const ResolvedCallReferenceNode* resource = get_param_value(args, 0).get<const ResolvedCallReferenceNode>();
# 759 "ql_thread.qpp"
   return remove_thread_resource(resource, xsink) ? false : true;
}

// nothing save_thread_data(hash h) {}
static QoreValue f_save_thread_data_Vh(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreHashNode* h = HARD_QORE_VALUE_HASH(args, 0);
# 223 "ql_thread.qpp"
   QoreHashNode* data = getProgram()->getThreadData();
   data->merge(h, xsink);
    return QoreValue();
}

// nothing save_thread_data(string key, auto value) {}
static QoreValue f_save_thread_data_VsVA(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* key = HARD_QORE_VALUE_STRING(args, 0);
    QoreValue value = get_param_value(args, 1);
# 239 "ql_thread.qpp"
   QoreHashNode* data = getProgram()->getThreadData();
   data->setKeyValue(key, value.getReferencedValue(), xsink);
    return QoreValue();
}

// nothing save_thread_data() {}
static QoreValue f_save_thread_data(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 247 "ql_thread.qpp"

    return QoreValue();
}

// bool set_thread_init(*code init) {}
static QoreValue f_set_thread_init_Nc(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const ResolvedCallReferenceNode* init = get_param_value(args, 0).get<const ResolvedCallReferenceNode>();
# 595 "ql_thread.qpp"
   return qore_program_private::setThreadInit(*getProgram(), init, xsink);
}

// nothing set_thread_resource(AbstractThreadResource resource) {}
static QoreValue f_set_thread_resource_C22AbstractThreadResource(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(resource, AbstractQoreThreadResource, args, 0, CID_ABSTRACTTHREADRESOURCE, "set_thread_resource()", "AbstractThreadResource", xsink);
    if (*xsink)
        return 0;
# 686 "ql_thread.qpp"
   ReferenceHolder<AbstractQoreThreadResource> holder(resource, xsink);
   set_thread_resource(resource);
    return QoreValue();
}

// nothing set_thread_resource(code resource, auto arg) {}
static QoreValue f_set_thread_resource_VcVA(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const ResolvedCallReferenceNode* resource = get_param_value(args, 0).get<const ResolvedCallReferenceNode>();
    QoreValue arg = get_param_value(args, 1);
# 735 "ql_thread.qpp"
   set_thread_resource(resource, arg);
    return QoreValue();
}

// nothing set_thread_tz(TimeZone zone) {}
static QoreValue f_set_thread_tz_C8TimeZone(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(zone, TimeZoneData, args, 0, CID_TIMEZONE, "set_thread_tz()", "TimeZone", xsink);
    if (*xsink)
        return 0;
# 619 "ql_thread.qpp"
   ReferenceHolder<TimeZoneData> holder(zone, xsink);
   set_thread_tz(zone->get());
    return QoreValue();
}

// nothing set_thread_tz() {}
static QoreValue f_set_thread_tz(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 640 "ql_thread.qpp"
   clear_thread_tz();
    return QoreValue();
}

// list<int> thread_list() {}
static QoreValue f_thread_list(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 209 "ql_thread.qpp"
   return get_thread_list();
}

// nothing thread_yield() {}
static QoreValue f_thread_yield(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 771 "ql_thread.qpp"
#ifdef _Q_WINDOWS
    // cnanot find a working mingw c++ compiler with support for std::this_thread
    SwitchToThread();
#else
    std::this_thread::yield();
#endif
    return QoreValue();
}

// nothing throwThreadResourceExceptions() {}
static QoreValue f_throwThreadResourceExceptions(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 467 "ql_thread.qpp"
   purge_thread_resources(xsink);
    return QoreValue();
}

// nothing throw_thread_resource_exceptions() {}
static QoreValue f_throw_thread_resource_exceptions(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 500 "ql_thread.qpp"
   purge_thread_resources(xsink);
    return QoreValue();
}

// bool throw_thread_resource_exceptions_to_mark() {}
static QoreValue f_throw_thread_resource_exceptions_to_mark(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 538 "ql_thread.qpp"
   return (bool)purge_thread_resources_to_mark(xsink);
}


DLLLOCAL void init_thread_functions(QoreNamespace& ns) {
    // nothing delete_all_thread_data() {}
    ns.addBuiltinVariant("delete_all_thread_data", (q_func_n_t)f_delete_all_thread_data, QC_NO_FLAGS, QDOM_THREAD_CONTROL, nothingTypeInfo);
    // nothing delete_thread_data(...) {}
    ns.addBuiltinVariant("delete_thread_data", (q_func_n_t)f_delete_thread_data_VV, QC_USES_EXTRA_ARGS, QDOM_THREAD_CONTROL, nothingTypeInfo);
    // nothing delete_thread_data(list l) {}
    ns.addBuiltinVariant("delete_thread_data", (q_func_n_t)f_delete_thread_data_Vl, QC_NO_FLAGS, QDOM_THREAD_CONTROL, nothingTypeInfo, 1, listTypeInfo, NULL, "l");
    // hash<string,list<hash<CallStackInfo>>> getAllThreadCallStacks() {}
    ns.addBuiltinVariant("getAllThreadCallStacks", (q_func_n_t)f_getAllThreadCallStacks, QC_DEPRECATED, QDOM_THREAD_CONTROL|QDOM_THREAD_INFO, qore_get_complex_hash_type(qore_get_complex_list_type(hashdeclCallStackInfo->getTypeInfo(false))));
    // hash<string,list<hash<CallStackInfo>>> get_all_thread_call_stacks() {}
    ns.addBuiltinVariant("get_all_thread_call_stacks", (q_func_n_t)f_get_all_thread_call_stacks, QC_NO_FLAGS, QDOM_THREAD_CONTROL|QDOM_THREAD_INFO, qore_get_complex_hash_type(qore_get_complex_list_type(hashdeclCallStackInfo->getTypeInfo(false))));
    // hash get_all_thread_data() {}
    ns.addBuiltinVariant("get_all_thread_data", (q_func_n_t)f_get_all_thread_data, QC_RET_VALUE_ONLY, QDOM_THREAD_CONTROL|QDOM_THREAD_INFO, hashTypeInfo);
    // list<hash<CallStackInfo>> get_thread_call_stack() {}
    ns.addBuiltinVariant("get_thread_call_stack", (q_func_n_t)f_get_thread_call_stack, QC_NO_FLAGS, QDOM_THREAD_CONTROL|QDOM_THREAD_INFO, qore_get_complex_list_type(hashdeclCallStackInfo->getTypeInfo(false)));
    // auto get_thread_data(string key) {}
    ns.addBuiltinVariant("get_thread_data", (q_func_n_t)f_get_thread_data_Vs, QC_RET_VALUE_ONLY, QDOM_THREAD_CONTROL|QDOM_THREAD_INFO, autoTypeInfo, 1, stringTypeInfo, NULL, "key");
    // nothing get_thread_data() {}
    ns.addBuiltinVariant("get_thread_data", (q_func_n_t)f_get_thread_data, QC_RUNTIME_NOOP, QDOM_THREAD_CONTROL|QDOM_THREAD_INFO, nothingTypeInfo);
    // *TimeZone get_thread_tz() {}
    ns.addBuiltinVariant("get_thread_tz", (q_func_n_t)f_get_thread_tz, QC_NO_FLAGS, QDOM_LOCALE_CONTROL, QC_TIMEZONE->getOrNothingTypeInfo());
    // int gettid() {}
    ns.addBuiltinVariant("gettid", (q_func_n_t)f_gettid, QC_RET_VALUE_ONLY, QDOM_THREAD_INFO, bigIntTypeInfo);
    // nothing mark_thread_resources() {}
    ns.addBuiltinVariant("mark_thread_resources", (q_func_n_t)f_mark_thread_resources, QC_NO_FLAGS, QDOM_THREAD_CONTROL, nothingTypeInfo);
    // int num_threads() {}
    ns.addBuiltinVariant("num_threads", (q_func_n_t)f_num_threads, QC_RET_VALUE_ONLY, QDOM_THREAD_INFO, bigIntTypeInfo);
    // *hash remove_thread_data(...) {}
    ns.addBuiltinVariant("remove_thread_data", (q_func_n_t)f_remove_thread_data_VV, QC_USES_EXTRA_ARGS, QDOM_THREAD_CONTROL, hashOrNothingTypeInfo);
    // hash remove_thread_data(list l) {}
    ns.addBuiltinVariant("remove_thread_data", (q_func_n_t)f_remove_thread_data_Vl, QC_NO_FLAGS, QDOM_THREAD_CONTROL, hashTypeInfo, 1, listTypeInfo, NULL, "l");
    // bool remove_thread_resource(AbstractThreadResource resource) {}
    ns.addBuiltinVariant("remove_thread_resource", (q_func_n_t)f_remove_thread_resource_C22AbstractThreadResource, QC_NO_FLAGS, QDOM_THREAD_CONTROL, boolTypeInfo, 1, QC_ABSTRACTTHREADRESOURCE->getTypeInfo(), NULL, "resource");
    // bool remove_thread_resource(code resource) {}
    ns.addBuiltinVariant("remove_thread_resource", (q_func_n_t)f_remove_thread_resource_Vc, QC_NO_FLAGS, QDOM_THREAD_CONTROL, boolTypeInfo, 1, codeTypeInfo, NULL, "resource");
    // nothing save_thread_data(hash h) {}
    ns.addBuiltinVariant("save_thread_data", (q_func_n_t)f_save_thread_data_Vh, QC_NO_FLAGS, QDOM_THREAD_CONTROL, nothingTypeInfo, 1, hashTypeInfo, NULL, "h");
    // nothing save_thread_data(string key, auto value) {}
    ns.addBuiltinVariant("save_thread_data", (q_func_n_t)f_save_thread_data_VsVA, QC_NO_FLAGS, QDOM_THREAD_CONTROL, nothingTypeInfo, 2, stringTypeInfo, NULL, "key", autoTypeInfo, NULL, "value");
    // nothing save_thread_data() {}
    ns.addBuiltinVariant("save_thread_data", (q_func_n_t)f_save_thread_data, QC_RUNTIME_NOOP, QDOM_THREAD_CONTROL, nothingTypeInfo);
    // bool set_thread_init(*code init) {}
    ns.addBuiltinVariant("set_thread_init", (q_func_n_t)f_set_thread_init_Nc, QC_NO_FLAGS, QDOM_THREAD_CONTROL, boolTypeInfo, 1, codeOrNothingTypeInfo, NULL, "init");
    // nothing set_thread_resource(AbstractThreadResource resource) {}
    ns.addBuiltinVariant("set_thread_resource", (q_func_n_t)f_set_thread_resource_C22AbstractThreadResource, QC_NO_FLAGS, QDOM_THREAD_CONTROL, nothingTypeInfo, 1, QC_ABSTRACTTHREADRESOURCE->getTypeInfo(), NULL, "resource");
    // nothing set_thread_resource(code resource, auto arg) {}
    ns.addBuiltinVariant("set_thread_resource", (q_func_n_t)f_set_thread_resource_VcVA, QC_NO_FLAGS, QDOM_THREAD_CONTROL, nothingTypeInfo, 2, codeTypeInfo, NULL, "resource", autoTypeInfo, NULL, "arg");
    // nothing set_thread_tz(TimeZone zone) {}
    ns.addBuiltinVariant("set_thread_tz", (q_func_n_t)f_set_thread_tz_C8TimeZone, QC_NO_FLAGS, QDOM_LOCALE_CONTROL, nothingTypeInfo, 1, QC_TIMEZONE->getTypeInfo(), NULL, "zone");
    // nothing set_thread_tz() {}
    ns.addBuiltinVariant("set_thread_tz", (q_func_n_t)f_set_thread_tz, QC_NO_FLAGS, QDOM_LOCALE_CONTROL, nothingTypeInfo);
    // list<int> thread_list() {}
    ns.addBuiltinVariant("thread_list", (q_func_n_t)f_thread_list, QC_RET_VALUE_ONLY, QDOM_THREAD_INFO, qore_get_complex_list_type(bigIntTypeInfo));
    // nothing thread_yield() {}
    ns.addBuiltinVariant("thread_yield", (q_func_n_t)f_thread_yield, QC_NO_FLAGS, QDOM_PROCESS, nothingTypeInfo);
    // nothing throwThreadResourceExceptions() {}
    ns.addBuiltinVariant("throwThreadResourceExceptions", (q_func_n_t)f_throwThreadResourceExceptions, QC_DEPRECATED, QDOM_THREAD_CONTROL, nothingTypeInfo);
    // nothing throw_thread_resource_exceptions() {}
    ns.addBuiltinVariant("throw_thread_resource_exceptions", (q_func_n_t)f_throw_thread_resource_exceptions, QC_NO_FLAGS, QDOM_THREAD_CONTROL, nothingTypeInfo);
    // bool throw_thread_resource_exceptions_to_mark() {}
    ns.addBuiltinVariant("throw_thread_resource_exceptions_to_mark", (q_func_n_t)f_throw_thread_resource_exceptions_to_mark, QC_NO_FLAGS, QDOM_THREAD_CONTROL, boolTypeInfo);

}
