/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  ql_time.qpp

  Qore Programming Language

  Copyright (C) 2003 - 2017 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/ql_time.h"
#include "qore/intern/QC_TimeZone.h"
#include "qore/intern/qore_date_private.h"
#include "qore/intern/QoreHashNodeIntern.h"

#include <stdio.h>
#include <time.h>
#include <sys/time.h>

static int qd_get_digit(unsigned& rv, const char*& p, unsigned factor) {
   if (*p < '0' || *p > '9')
      return -1;

   rv += (*p - '0') * factor;
   ++p;
   return 0;
}

// returns 0 if invalid data is encountered
static unsigned parse_int_2(const char *p) {
   unsigned rv = 0;
   if (qd_get_digit(rv, p, 10))
      return 0;
   if (qd_get_digit(rv, p, 1))
      return 0;
   return rv;
}

// returns 0 if invalid data is encountered
static unsigned parse_int_3(const char *p) {
   unsigned rv = 0;
   if (qd_get_digit(rv, p, 100))
      return 0;
   if (qd_get_digit(rv, p, 10))
      return 0;
   if (qd_get_digit(rv, p, 1))
      return 0;
   return rv;
}

// returns 0 if invalid data is encountered
static unsigned parse_int_4(const char *p) {
   unsigned rv = 0;
   if (qd_get_digit(rv, p, 1000))
      return 0;
   if (qd_get_digit(rv, p, 100))
      return 0;
   if (qd_get_digit(rv, p, 10))
      return 0;
   if (qd_get_digit(rv, p, 1))
      return 0;
   return rv;
}

// returns 0 if invalid data is encountered
static unsigned parse_int_6(const char *p) {
   unsigned rv = 0;
   if (qd_get_digit(rv, p, 100000))
      return 0;
   if (qd_get_digit(rv, p, 10000))
      return 0;
   if (qd_get_digit(rv, p, 1000))
      return 0;
   if (qd_get_digit(rv, p, 100))
      return 0;
   if (qd_get_digit(rv, p, 10))
      return 0;
   if (qd_get_digit(rv, p, 1))
      return 0;
   return rv;
}

QoreHashNode* date_info(const DateTime& d) {
   qore_tm info;
   d.getInfo(info);

   QoreHashNode* h = new QoreHashNode(hashdeclDateTimeInfo, nullptr);
   qore_hash_private* ph = qore_hash_private::get(*h);
   ph->setKeyValueIntern("relative", get_bool_node(d.isRelative()));
   ph->setKeyValueIntern("year", new QoreBigIntNode(info.year));
   ph->setKeyValueIntern("month", new QoreBigIntNode(info.month));
   ph->setKeyValueIntern("day", new QoreBigIntNode(info.day));
   ph->setKeyValueIntern("hour", new QoreBigIntNode(info.hour));
   ph->setKeyValueIntern("minute", new QoreBigIntNode(info.minute));
   ph->setKeyValueIntern("second", new QoreBigIntNode(info.second));
   ph->setKeyValueIntern("microsecond", new QoreBigIntNode(info.us));

   if (d.isAbsolute()) {
      ph->setKeyValueIntern("dow", new QoreBigIntNode(d.getDayOfWeek()));
      ph->setKeyValueIntern("doy", new QoreBigIntNode(d.getDayNumber()));
      ph->setKeyValueIntern("utc_secs_east", new QoreBigIntNode(info.utc_secs_east));
      ph->setKeyValueIntern("dst", get_bool_node(info.dst));
      ph->setKeyValueIntern("zone_name", new QoreStringNode(info.zone_name));
      ph->setKeyValueIntern("zone", new QoreObject(QC_TIMEZONE, 0, new TimeZoneData(info.zone)));
   }

   return h;
}

DateTimeNode* make_date_with_mask(const AbstractQoreZoneInfo* tz, const QoreString& dtstr, const QoreString& mask, ExceptionSink* xsink) {
   qore_tm dt;
   dt.clear();
   dt.month = 1;
   dt.day = 1;

   const char* d = dtstr.getBuffer();
   const char* de = d + dtstr.strlen();

   const char* s = mask.getBuffer();
   const char* se = s + mask.strlen();

   while (*s) {
      // break if we run out of data in either the mask or the date string
      if (s >= se)
         break;
      switch (*s) {
         case 'Y':
            if (s[1] != 'Y') {
               xsink->raiseException("INVALID-DATE", "'Y' has to be used as 'YY' or 'YYYY'");
               return 0;
               break;
            }
            if (d >= de) {
               xsink->raiseException("INVALID-DATE", "no more input to process for year mask");
               return 0;
            }
            if (s[2] == 'Y' && s[3] == 'Y') {
               dt.year = parse_int_4(d);
               s += 3;
               d += 3;
            }
            else {
               ++s;
               // obtain the current century
               {
                  DateTime tmpdt(q_epoch());
                  dt.year = parse_int_2(d) + (tmpdt.getYear() / 100 * 100);
               }
               ++d;
            }
            break;

         case 'M':
            if (d >= de) {
               xsink->raiseException("INVALID-DATE", "no more input to process for month mask");
               return 0;
            }
            if (s[1] == 'M') {
               dt.month = parse_int_2(d);
               s++;
               d++;
               break;
            }
            // 'M' is not supported because there is no clear way how to get eg. 1 or 11
            if ((s[1] == 'o' || s[1] == 'O') && (s[2] == 'n' || s[2] == 'N')) {
               QoreString str(d, 3);
               dt.month = str.strlen() == 3 ? qore_date_info::getMonthIxFromAbbr(str.getBuffer()) : -1;
               if (dt.month < 0 || dt.month > 11) {
                  xsink->raiseException("INVALID-DATE", "Invalid 'Mon' string: '%s'", !str.empty() ? str.getBuffer() : "<none>");
                  return 0;
               }

               ++dt.month;
               s += 2;
               d += 2;
               break;
            }
            break;

         case 'm':
            if (d >= de) {
               xsink->raiseException("INVALID-DATE", "no more input to process for minute mask");
               return 0;
            }
            if (s[1] == 'm') {
               dt.minute = parse_int_2(d);
               s++;
               d++;
               break;
            }
            if (s[1] == 's') {
               dt.us = parse_int_3(d) * 1000;
               ++s;
               d += 2;
               break;
            }
            break;

         case 'D':
         case 'd':
            if (d >= de) {
               xsink->raiseException("INVALID-DATE", "no more input to process for day mask");
               return 0;
            }
            if (s[1] == 'D' || s[1] == 'd') {
               dt.day = parse_int_2(d);
               s++;
               d++;
            }
            break;

         case 'H':
         case 'h':
            if (d >= de) {
               xsink->raiseException("INVALID-DATE", "no more input to process for hour mask");
               return 0;
            }
            if (s[1] == 'H' || s[1] == 'h') {
               dt.hour = parse_int_2(d);
               s++;
               d++;
            }
            break;

         case 's':
            if (s[1] == 's' && s[2] == 's') {
               if (d >= de) {
                  xsink->raiseException("INVALID-DATE", "no more input to process for sub-second mask");
                  return 0;
               }
               if (s[3] == 's' && s[4] == 's' && s[5] == 's') {
                  dt.us = parse_int_6(d);
                  s += 5;
                  d += 5;
               }
               else {
                  dt.us = parse_int_3(d) * 1000;
                  s += 2;
                  d += 2;
               }
            }
            break;

         case 'S':
            if (d >= de) {
               xsink->raiseException("INVALID-DATE", "no more input to process for second mask");
               return 0;
            }
            if (s[1] == 'S') {
               dt.second = parse_int_2(d);
               s++;
               d++;
            }
            break;

         case 'u':
            if (s[1] == 's') {
               if (d >= de) {
                  xsink->raiseException("INVALID-DATE", "no more input to process for microsecond mask");
                  return 0;
               }
               dt.us = parse_int_6(d);
               ++s;
               d += 5;
               break;
            }
      }
      s++;
      d++;
   }

   if (*xsink)
      return 0;

   SimpleRefHolder<DateTimeNode> rv(DateTimeNode::makeAbsolute(tz, dt.year, dt.month, dt.day, dt.hour, dt.minute, dt.second, dt.us, xsink));
   return *xsink ? 0 : rv.release();
}
TypedHashDecl* init_hashdecl_DateTimeInfo(QoreNamespace& ns) {
    TypedHashDecl* hd = new TypedHashDecl("DateTimeInfo");
    hd->addMember("day", bigIntTypeInfo, QoreValue());
    hd->addMember("dow", bigIntTypeInfo, QoreValue());
    hd->addMember("doy", bigIntTypeInfo, QoreValue());
    hd->addMember("dst", boolTypeInfo, QoreValue());
    hd->addMember("hour", bigIntTypeInfo, QoreValue());
    hd->addMember("microsecond", bigIntTypeInfo, QoreValue());
    hd->addMember("minute", bigIntTypeInfo, QoreValue());
    hd->addMember("month", bigIntTypeInfo, QoreValue());
    hd->addMember("relative", boolTypeInfo, QoreValue());
    hd->addMember("second", bigIntTypeInfo, QoreValue());
    hd->addMember("utc_secs_east", bigIntTypeInfo, QoreValue());
    hd->addMember("year", bigIntTypeInfo, QoreValue());
    hd->addMember("zone", QC_TIMEZONE->getTypeInfo(), QoreValue());
    hd->addMember("zone_name", stringTypeInfo, QoreValue());
    ns.addSystemHashDecl(hd);
    return hd;
}

TypedHashDecl* init_hashdecl_IsoWeekInfo(QoreNamespace& ns) {
    TypedHashDecl* hd = new TypedHashDecl("IsoWeekInfo");
    hd->addMember("day", bigIntTypeInfo, QoreValue());
    hd->addMember("week", bigIntTypeInfo, QoreValue());
    hd->addMember("year", bigIntTypeInfo, QoreValue());
    ns.addSystemHashDecl(hd);
    return hd;
}

# 348 "ql_time.qpp"
# 410 "ql_time.qpp"
# 433 "ql_time.qpp"
# 456 "ql_time.qpp"
# 473 "ql_time.qpp"
# 494 "ql_time.qpp"
# 500 "ql_time.qpp"
# 520 "ql_time.qpp"
# 537 "ql_time.qpp"
# 555 "ql_time.qpp"
# 572 "ql_time.qpp"
# 589 "ql_time.qpp"
# 607 "ql_time.qpp"
# 624 "ql_time.qpp"
# 641 "ql_time.qpp"
# 647 "ql_time.qpp"
# 666 "ql_time.qpp"
# 672 "ql_time.qpp"
# 691 "ql_time.qpp"
# 697 "ql_time.qpp"
# 721 "ql_time.qpp"
# 727 "ql_time.qpp"
# 751 "ql_time.qpp"
# 757 "ql_time.qpp"
# 781 "ql_time.qpp"
# 787 "ql_time.qpp"
# 811 "ql_time.qpp"
# 817 "ql_time.qpp"
# 841 "ql_time.qpp"
# 847 "ql_time.qpp"
# 871 "ql_time.qpp"
# 877 "ql_time.qpp"
# 908 "ql_time.qpp"
# 914 "ql_time.qpp"
# 945 "ql_time.qpp"
# 962 "ql_time.qpp"
# 968 "ql_time.qpp"
# 985 "ql_time.qpp"
# 991 "ql_time.qpp"
# 1008 "ql_time.qpp"
# 1014 "ql_time.qpp"
# 1031 "ql_time.qpp"
# 1037 "ql_time.qpp"
# 1054 "ql_time.qpp"
# 1060 "ql_time.qpp"
# 1079 "ql_time.qpp"
# 1085 "ql_time.qpp"
# 1104 "ql_time.qpp"
# 1110 "ql_time.qpp"
# 1129 "ql_time.qpp"
# 1148 "ql_time.qpp"
# 1154 "ql_time.qpp"
# 1166 "ql_time.qpp"
# 1172 "ql_time.qpp"
# 1191 "ql_time.qpp"
# 1205 "ql_time.qpp"
# 1211 "ql_time.qpp"
# 1232 "ql_time.qpp"
# 1247 "ql_time.qpp"
# 1253 "ql_time.qpp"
# 1275 "ql_time.qpp"
# 1299 "ql_time.qpp"
# 1305 "ql_time.qpp"
# 1336 "ql_time.qpp"
# 1359 "ql_time.qpp"
# 1365 "ql_time.qpp"
# 1390 "ql_time.qpp"
# 1408 "ql_time.qpp"
# 1431 "ql_time.qpp"
# 1448 "ql_time.qpp"
# 1465 "ql_time.qpp"
# 1482 "ql_time.qpp"
# 1498 "ql_time.qpp"
# 1504 "ql_time.qpp"
# 1520 "ql_time.qpp"
# 1546 "ql_time.qpp"
# 1571 "ql_time.qpp"
# 1595 "ql_time.qpp"
# 1622 "ql_time.qpp"
# 1639 "ql_time.qpp"
# 1656 "ql_time.qpp"
# 1673 "ql_time.qpp"
# 1680 "ql_time.qpp"
# 1697 "ql_time.qpp"
# 1704 "ql_time.qpp"
# 1715 "ql_time.qpp"
# 1743 "ql_time.qpp"
# 1760 "ql_time.qpp"
# 1777 "ql_time.qpp"
# 1786 "ql_time.qpp"
# 1795 "ql_time.qpp"
// int clock_getmicros() {}
static QoreValue f_clock_getmicros(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1478 "ql_time.qpp"
   return q_clock_getmicros();
}

// int clock_getmillis() {}
static QoreValue f_clock_getmillis(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1444 "ql_time.qpp"
   return q_clock_getmillis();
}

// int clock_getnanos() {}
static QoreValue f_clock_getnanos(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1461 "ql_time.qpp"
   return q_clock_getnanos();
}

// date date(date dt) {}
static QoreValue f_date_Vd(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1711 "ql_time.qpp"
  return dt->refSelf();
}

// date date(string dtstr) {}
static QoreValue f_date_Vs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* dtstr = HARD_QORE_VALUE_STRING(args, 0);
# 1738 "ql_time.qpp"
   DateTimeNodeValueHelper date(dtstr, xsink);
   return *xsink ? 0 : date.getReferencedValue();
}

// date date(softfloat f) {}
static QoreValue f_date_vf(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    double f = HARD_QORE_VALUE_FLOAT(args, 0);
# 1756 "ql_time.qpp"
   return DateTimeNode::makeRelativeFromSeconds((int64)f, (int)((f - (double)((int)f)) * 1000000));
}

// date date(int i) {}
static QoreValue f_date_Vi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 i = HARD_QORE_VALUE_INT(args, 0);
# 1773 "ql_time.qpp"
   return DateTimeNode::makeRelativeFromSeconds(i);
}

// date date() {}
static QoreValue f_date(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1782 "ql_time.qpp"
   return zero_date();
}

// date date(null null) {}
static QoreValue f_date_Vu(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1791 "ql_time.qpp"
   return zero_date();
}

// date date(string dtstr, string mask) {}
static QoreValue f_date_VsVs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* dtstr = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* mask = HARD_QORE_VALUE_STRING(args, 1);
# 1811 "ql_time.qpp"
   return make_date_with_mask(currentTZ(), *dtstr, *mask, xsink);
}

// hash<DateTimeInfo> date_info(date dt) {}
static QoreValue f_date_info_Vd(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1635 "ql_time.qpp"
   return date_info(*dt);
}

// hash<DateTimeInfo> date_info() {}
static QoreValue f_date_info(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1648 "ql_time.qpp"
   int us;
   int64 seconds = q_epoch_us(us);
   DateTime d;
   d.setDate(currentTZ(), seconds, us);
   return date_info(d);
}

// date date_ms(softint ms) {}
static QoreValue f_date_ms_vi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 ms = HARD_QORE_VALUE_INT(args, 0);
# 1494 "ql_time.qpp"
   return new DateTimeNode(ms / 1000, (int)(ms % 1000));
}

// nothing date_ms() {}
static QoreValue f_date_ms(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1501 "ql_time.qpp"

    return QoreValue();
}

// date date_us(softint us) {}
static QoreValue f_date_us_vi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 us = HARD_QORE_VALUE_INT(args, 0);
# 1516 "ql_time.qpp"
   return DateTimeNode::makeAbsoluteLocal(currentTZ(), us / 1000000, (int)(us % 1000000));
}

// date days(softint days) {}
static QoreValue f_days_vi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 days = HARD_QORE_VALUE_INT(args, 0);
# 777 "ql_time.qpp"
   return new DateTimeNode(0, 0, (int)days, 0, 0, 0, 0, true);
}

// nothing days() {}
static QoreValue f_days(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 784 "ql_time.qpp"

    return QoreValue();
}

// string format_date(string format, date dt) {}
static QoreValue f_format_date_VsVd(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* format = HARD_QORE_VALUE_STRING(args, 0);
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 1);
# 488 "ql_time.qpp"
   QoreStringNode *rv = new QoreStringNode;
   dt->format(*rv, format->getBuffer());
   return rv;
}

// nothing format_date() {}
static QoreValue f_format_date(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 497 "ql_time.qpp"

    return QoreValue();
}

// date getDateFromISOWeek(softint year, softint week, softint day = 1) {}
static QoreValue f_getDateFromISOWeek_vivivi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 year = HARD_QORE_VALUE_INT(args, 0);
    int64 week = HARD_QORE_VALUE_INT(args, 1);
    int64 day = HARD_QORE_VALUE_INT(args, 2);
# 1404 "ql_time.qpp"
   return DateTimeNode::getDateFromISOWeek(year, week, day, xsink);
}

// int getDayNumber(date dt) {}
static QoreValue f_getDayNumber_Vd(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1162 "ql_time.qpp"
   return dt->getDayNumber();
}

// nothing getDayNumber() {}
static QoreValue f_getDayNumber(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1169 "ql_time.qpp"

    return QoreValue();
}

// int getDayOfWeek(date dt) {}
static QoreValue f_getDayOfWeek_Vd(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1201 "ql_time.qpp"
   return dt->getDayOfWeek();
}

// nothing getDayOfWeek() {}
static QoreValue f_getDayOfWeek(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1208 "ql_time.qpp"

    return QoreValue();
}

// int getISODayOfWeek(date dt) {}
static QoreValue f_getISODayOfWeek_Vd(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1242 "ql_time.qpp"
   int d = dt->getDayOfWeek();
   return (!d ? 7 : d);
}

// nothing getISODayOfWeek() {}
static QoreValue f_getISODayOfWeek(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1250 "ql_time.qpp"

    return QoreValue();
}

// hash<IsoWeekInfo> getISOWeekHash(date dt) {}
static QoreValue f_getISOWeekHash_Vd(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1286 "ql_time.qpp"
   int year, week, day;
   dt->getISOWeek(year, week, day);

   QoreHashNode* h = new QoreHashNode(hashdeclIsoWeekInfo, xsink);
   qore_hash_private* ph = qore_hash_private::get(*h);
   ph->setKeyValueIntern("year", new QoreBigIntNode(year));
   ph->setKeyValueIntern("week", new QoreBigIntNode(week));
   ph->setKeyValueIntern("day", new QoreBigIntNode(day));

   return h;
}

// nothing getISOWeekHash() {}
static QoreValue f_getISOWeekHash(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1302 "ql_time.qpp"

    return QoreValue();
}

// string getISOWeekString(date dt) {}
static QoreValue f_getISOWeekString_Vd(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1351 "ql_time.qpp"
   int year, week, day;
   dt->getISOWeek(year, week, day);
   QoreStringNode *str = new QoreStringNode();
   str->sprintf("%04d-W%02d-%d", year, week, day);
   return str;
}

// nothing getISOWeekString() {}
static QoreValue f_getISOWeekString(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1362 "ql_time.qpp"

    return QoreValue();
}

// date get_date_from_iso_week(softint year, softint week, softint day = 1) {}
static QoreValue f_get_date_from_iso_week_vivivi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 year = HARD_QORE_VALUE_INT(args, 0);
    int64 week = HARD_QORE_VALUE_INT(args, 1);
    int64 day = HARD_QORE_VALUE_INT(args, 2);
# 1427 "ql_time.qpp"
   return DateTimeNode::getDateFromISOWeek(year, week, day, xsink);
}

// int get_day_number(date dt) {}
static QoreValue f_get_day_number_Vd(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1187 "ql_time.qpp"
   return dt->getDayNumber();
}

// int get_day_of_week(date dt) {}
static QoreValue f_get_day_of_week_Vd(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1228 "ql_time.qpp"
   return dt->getDayOfWeek();
}

// int get_days(date dt) {}
static QoreValue f_get_days_Vd(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1004 "ql_time.qpp"
   return dt->getDay();
}

// nothing get_days() {}
static QoreValue f_get_days(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1011 "ql_time.qpp"

    return QoreValue();
}

// int get_duration_microseconds(date dt) {}
static QoreValue f_get_duration_microseconds_Vd(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1591 "ql_time.qpp"
   return dt->getRelativeMicroseconds();
}

// int get_duration_milliseconds(date dt) {}
static QoreValue f_get_duration_milliseconds_Vd(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1567 "ql_time.qpp"
   return dt->getRelativeMilliseconds();
}

// int get_duration_seconds(date dt) {}
static QoreValue f_get_duration_seconds_Vd(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1542 "ql_time.qpp"
   return dt->getRelativeSeconds();
}

// float get_duration_seconds_f(date dt) {}
static QoreValue f_get_duration_seconds_f_Vd(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1618 "ql_time.qpp"
   return dt->getRelativeSecondsDouble();
}

// int get_epoch_seconds(date dt) {}
static QoreValue f_get_epoch_seconds_Vd(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 662 "ql_time.qpp"
   return dt->getEpochSecondsUTC();
}

// nothing get_epoch_seconds() {}
static QoreValue f_get_epoch_seconds(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 669 "ql_time.qpp"

    return QoreValue();
}

// int get_hours(date dt) {}
static QoreValue f_get_hours_Vd(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1027 "ql_time.qpp"
   return dt->getHour();
}

// nothing get_hours() {}
static QoreValue f_get_hours(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1034 "ql_time.qpp"

    return QoreValue();
}

// int get_iso_day_of_week(date dt) {}
static QoreValue f_get_iso_day_of_week_Vd(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1270 "ql_time.qpp"
   int d = dt->getDayOfWeek();
   return (!d ? 7 : d);
}

// hash<IsoWeekInfo> get_iso_week_hash(date dt) {}
static QoreValue f_get_iso_week_hash_Vd(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1323 "ql_time.qpp"
   int year, week, day;
   dt->getISOWeek(year, week, day);

   QoreHashNode* h = new QoreHashNode(hashdeclIsoWeekInfo, xsink);
   qore_hash_private* ph = qore_hash_private::get(*h);
   ph->setKeyValueIntern("year", new QoreBigIntNode(year));
   ph->setKeyValueIntern("week", new QoreBigIntNode(week));
   ph->setKeyValueIntern("day", new QoreBigIntNode(day));

   return h;
}

// string get_iso_week_string(date dt) {}
static QoreValue f_get_iso_week_string_Vd(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1382 "ql_time.qpp"
   int year, week, day;
   dt->getISOWeek(year, week, day);
   QoreStringNode *str = new QoreStringNode();
   str->sprintf("%04d-W%02d-%d", year, week, day);
   return str;
}

// int get_microseconds(date dt) {}
static QoreValue f_get_microseconds_Vd(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1125 "ql_time.qpp"
   return dt->getMicrosecond();
}

// date get_midnight(date dt) {}
static QoreValue f_get_midnight_Vd(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1142 "ql_time.qpp"
   DateTimeNode *rv = new DateTimeNode(*dt);
   rv->setTime(0, 0, 0);
   return rv;
}

// nothing get_midnight() {}
static QoreValue f_get_midnight(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1151 "ql_time.qpp"

    return QoreValue();
}

// int get_milliseconds(date dt) {}
static QoreValue f_get_milliseconds_Vd(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1100 "ql_time.qpp"
   return dt->getMillisecond();
}

// nothing get_milliseconds() {}
static QoreValue f_get_milliseconds(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1107 "ql_time.qpp"

    return QoreValue();
}

// int get_minutes(date dt) {}
static QoreValue f_get_minutes_Vd(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1050 "ql_time.qpp"
   return dt->getMinute();
}

// nothing get_minutes() {}
static QoreValue f_get_minutes(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1057 "ql_time.qpp"

    return QoreValue();
}

// int get_months(date dt) {}
static QoreValue f_get_months_Vd(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 981 "ql_time.qpp"
   return dt->getMonth();
}

// nothing get_months() {}
static QoreValue f_get_months(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 988 "ql_time.qpp"

    return QoreValue();
}

// int get_seconds(date dt) {}
static QoreValue f_get_seconds_Vd(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1075 "ql_time.qpp"
   return dt->getSecond();
}

// nothing get_seconds() {}
static QoreValue f_get_seconds(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1082 "ql_time.qpp"

    return QoreValue();
}

// int get_years(date dt) {}
static QoreValue f_get_years_Vd(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 958 "ql_time.qpp"
   return dt->getYear();
}

// nothing get_years() {}
static QoreValue f_get_years(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 965 "ql_time.qpp"

    return QoreValue();
}

// date gmtime() {}
static QoreValue f_gmtime(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 585 "ql_time.qpp"
   return DateTimeNode::makeAbsolute(NULL, (int64)time(0), 0);
}

// date gmtime(softint secs, softint us = 0) {}
static QoreValue f_gmtime_vivi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 secs = HARD_QORE_VALUE_INT(args, 0);
    int64 us = HARD_QORE_VALUE_INT(args, 1);
# 603 "ql_time.qpp"
   return DateTimeNode::makeAbsolute(NULL, secs, (int)us);
}

// date gmtime(date dt) {}
static QoreValue f_gmtime_Vd(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 620 "ql_time.qpp"
   return DateTimeNode::makeAbsolute(NULL, dt->getEpochSecondsUTC(), dt->getMicrosecond());
}

// date hours(softint hours) {}
static QoreValue f_hours_vi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 hours = HARD_QORE_VALUE_INT(args, 0);
# 807 "ql_time.qpp"
   return new DateTimeNode(0, 0, 0, (int)hours, 0, 0, 0, true);
}

// nothing hours() {}
static QoreValue f_hours(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 814 "ql_time.qpp"

    return QoreValue();
}

// bool is_date_absolute(date dt) {}
static QoreValue f_is_date_absolute_Vd(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1693 "ql_time.qpp"
   return dt->isAbsolute();
}

// bool is_date_absolute() {}
static QoreValue f_is_date_absolute(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1700 "ql_time.qpp"
   return false;
}

// bool is_date_relative(date dt) {}
static QoreValue f_is_date_relative_Vd(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 1669 "ql_time.qpp"
   return dt->isRelative();
}

// bool is_date_relative() {}
static QoreValue f_is_date_relative(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1676 "ql_time.qpp"
   return false;
}

// date localtime() {}
static QoreValue f_localtime(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 533 "ql_time.qpp"
   return DateTimeNode::makeAbsolute(currentTZ(), (int64)time(0), 0);
}

// date localtime(softint secs, softint us = 0) {}
static QoreValue f_localtime_vivi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 secs = HARD_QORE_VALUE_INT(args, 0);
    int64 us = HARD_QORE_VALUE_INT(args, 1);
# 551 "ql_time.qpp"
   return DateTimeNode::makeAbsolute(currentTZ(), secs, (int)us);
}

// date localtime(date dt) {}
static QoreValue f_localtime_Vd(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 568 "ql_time.qpp"
   return DateTimeNode::makeAbsolute(currentTZ(), dt->getEpochSecondsUTC(), dt->getMicrosecond());
}

// date microseconds(softint us) {}
static QoreValue f_microseconds_vi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 us = HARD_QORE_VALUE_INT(args, 0);
# 934 "ql_time.qpp"
   int64 secs = us / 10000000;
   us -= (secs * 10000000);
   int64 minutes = secs / 60;
   secs -= (minutes * 60);
   int64 hours = minutes / 60;
   minutes -= (hours * 60);

   return DateTimeNode::makeRelative(0, 0, 0, hours, minutes, secs, (int)us);
}

// date milliseconds(softint ms) {}
static QoreValue f_milliseconds_vi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 ms = HARD_QORE_VALUE_INT(args, 0);
# 897 "ql_time.qpp"
   int64 secs = ms / 1000;
   ms -= (secs * 1000);
   int64 minutes = secs / 60;
   secs -= (minutes * 60);
   int64 hours = minutes / 60;
   minutes -= (hours * 60);

   return new DateTimeNode(0, 0, 0, (int)hours, (int)minutes, (int)secs, (int)ms, true);
}

// nothing milliseconds() {}
static QoreValue f_milliseconds(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 911 "ql_time.qpp"

    return QoreValue();
}

// date minutes(softint minutes) {}
static QoreValue f_minutes_vi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 minutes = HARD_QORE_VALUE_INT(args, 0);
# 837 "ql_time.qpp"
   return new DateTimeNode(0, 0, 0, 0, (int)minutes, 0, 0, true);
}

// nothing minutes() {}
static QoreValue f_minutes(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 844 "ql_time.qpp"

    return QoreValue();
}

// int mktime(date dt) {}
static QoreValue f_mktime_Vd(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 687 "ql_time.qpp"
   return dt->getEpochSecondsUTC();
}

// nothing mktime() {}
static QoreValue f_mktime(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 694 "ql_time.qpp"

    return QoreValue();
}

// date months(softint months) {}
static QoreValue f_months_vi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 months = HARD_QORE_VALUE_INT(args, 0);
# 747 "ql_time.qpp"
   return new DateTimeNode(0, (int)months, 0, 0, 0, 0, 0, true);
}

// nothing months() {}
static QoreValue f_months(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 754 "ql_time.qpp"

    return QoreValue();
}

// date now() {}
static QoreValue f_now(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 516 "ql_time.qpp"
   return DateTimeNode::makeAbsolute(currentTZ(), (int64)time(0), 0);
}

// date now_ms() {}
static QoreValue f_now_ms(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 426 "ql_time.qpp"
   int us;
   int64 seconds = q_epoch_us(us);

   return DateTimeNode::makeAbsolute(currentTZ(), seconds, (us / 1000) * 1000);
}

// date now_us() {}
static QoreValue f_now_us(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 449 "ql_time.qpp"
   int us;
   int64 seconds = q_epoch_us(us);

   return DateTimeNode::makeAbsolute(currentTZ(), seconds, us);
}

// date now_utc() {}
static QoreValue f_now_utc(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 467 "ql_time.qpp"
   int us;
   int64 seconds = q_epoch_us(us);
   return DateTimeNode::makeAbsolute(NULL, seconds, us);
}

// date seconds(softint seconds) {}
static QoreValue f_seconds_vi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 seconds = HARD_QORE_VALUE_INT(args, 0);
# 867 "ql_time.qpp"
   return new DateTimeNode(0, 0, 0, 0, 0, (int)seconds, 0, true);
}

// nothing seconds() {}
static QoreValue f_seconds(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 874 "ql_time.qpp"

    return QoreValue();
}

// int timegm(date dt) {}
static QoreValue f_timegm_Vd(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const DateTimeNode* dt = HARD_QORE_VALUE_DATE(args, 0);
# 637 "ql_time.qpp"
   return dt->getEpochSeconds();
}

// nothing timegm() {}
static QoreValue f_timegm(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 644 "ql_time.qpp"

    return QoreValue();
}

// date years(softint years) {}
static QoreValue f_years_vi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 years = HARD_QORE_VALUE_INT(args, 0);
# 717 "ql_time.qpp"
   return new DateTimeNode((int)years, 0, 0, 0, 0, 0, 0, true);
}

// nothing years() {}
static QoreValue f_years(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 724 "ql_time.qpp"

    return QoreValue();
}


DLLLOCAL void init_time_functions(QoreNamespace& ns) {
    // int clock_getmicros() {}
    ns.addBuiltinVariant("clock_getmicros", (q_func_n_t)f_clock_getmicros, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);
    // int clock_getmillis() {}
    ns.addBuiltinVariant("clock_getmillis", (q_func_n_t)f_clock_getmillis, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);
    // int clock_getnanos() {}
    ns.addBuiltinVariant("clock_getnanos", (q_func_n_t)f_clock_getnanos, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);
    // date date(date dt) {}
    ns.addBuiltinVariant("date", (q_func_n_t)f_date_Vd, QC_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 1, dateTypeInfo, NULL, "dt");
    // date date(string dtstr) {}
    ns.addBuiltinVariant("date", (q_func_n_t)f_date_Vs, QC_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 1, stringTypeInfo, NULL, "dtstr");
    // date date(softfloat f) {}
    ns.addBuiltinVariant("date", (q_func_n_t)f_date_vf, QC_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 1, softFloatTypeInfo, NULL, "f");
    // date date(int i) {}
    ns.addBuiltinVariant("date", (q_func_n_t)f_date_Vi, QC_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 1, bigIntTypeInfo, NULL, "i");
    // date date() {}
    ns.addBuiltinVariant("date", (q_func_n_t)f_date, QC_CONSTANT, QDOM_DEFAULT, dateTypeInfo);
    // date date(null null) {}
    ns.addBuiltinVariant("date", (q_func_n_t)f_date_Vu, QC_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 1, nullTypeInfo, NULL, "null");
    // date date(string dtstr, string mask) {}
    ns.addBuiltinVariant("date", (q_func_n_t)f_date_VsVs, QC_RET_VALUE_ONLY, QDOM_DEFAULT, dateTypeInfo, 2, stringTypeInfo, NULL, "dtstr", stringTypeInfo, NULL, "mask");
    // hash<DateTimeInfo> date_info(date dt) {}
    ns.addBuiltinVariant("date_info", (q_func_n_t)f_date_info_Vd, QC_CONSTANT, QDOM_DEFAULT, hashdeclDateTimeInfo->getTypeInfo(false), 1, dateTypeInfo, NULL, "dt");
    // hash<DateTimeInfo> date_info() {}
    ns.addBuiltinVariant("date_info", (q_func_n_t)f_date_info, QC_CONSTANT, QDOM_DEFAULT, hashdeclDateTimeInfo->getTypeInfo(false));
    // date date_ms(softint ms) {}
    ns.addBuiltinVariant("date_ms", (q_func_n_t)f_date_ms_vi, QC_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 1, softBigIntTypeInfo, NULL, "ms");
    // nothing date_ms() {}
    ns.addBuiltinVariant("date_ms", (q_func_n_t)f_date_ms, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // date date_us(softint us) {}
    ns.addBuiltinVariant("date_us", (q_func_n_t)f_date_us_vi, QC_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 1, softBigIntTypeInfo, NULL, "us");
    // date days(softint days) {}
    ns.addBuiltinVariant("days", (q_func_n_t)f_days_vi, QC_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 1, softBigIntTypeInfo, NULL, "days");
    // nothing days() {}
    ns.addBuiltinVariant("days", (q_func_n_t)f_days, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // string format_date(string format, date dt) {}
    ns.addBuiltinVariant("format_date", (q_func_n_t)f_format_date_VsVd, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 2, stringTypeInfo, NULL, "format", dateTypeInfo, NULL, "dt");
    // nothing format_date() {}
    ns.addBuiltinVariant("format_date", (q_func_n_t)f_format_date, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // date getDateFromISOWeek(softint year, softint week, softint day = 1) {}
    ns.addBuiltinVariant("getDateFromISOWeek", (q_func_n_t)f_getDateFromISOWeek_vivivi, QC_RET_VALUE_ONLY|QC_DEPRECATED, QDOM_DEFAULT, dateTypeInfo, 3, softBigIntTypeInfo, NULL, "year", softBigIntTypeInfo, NULL, "week", softBigIntTypeInfo, new QoreBigIntNode(1), "day");
    // int getDayNumber(date dt) {}
    ns.addBuiltinVariant("getDayNumber", (q_func_n_t)f_getDayNumber_Vd, QC_CONSTANT|QC_DEPRECATED, QDOM_DEFAULT, bigIntTypeInfo, 1, dateTypeInfo, NULL, "dt");
    // nothing getDayNumber() {}
    ns.addBuiltinVariant("getDayNumber", (q_func_n_t)f_getDayNumber, QC_RUNTIME_NOOP|QC_DEPRECATED, QDOM_DEFAULT, nothingTypeInfo);
    // int getDayOfWeek(date dt) {}
    ns.addBuiltinVariant("getDayOfWeek", (q_func_n_t)f_getDayOfWeek_Vd, QC_CONSTANT|QC_DEPRECATED, QDOM_DEFAULT, bigIntTypeInfo, 1, dateTypeInfo, NULL, "dt");
    // nothing getDayOfWeek() {}
    ns.addBuiltinVariant("getDayOfWeek", (q_func_n_t)f_getDayOfWeek, QC_RUNTIME_NOOP|QC_DEPRECATED, QDOM_DEFAULT, nothingTypeInfo);
    // int getISODayOfWeek(date dt) {}
    ns.addBuiltinVariant("getISODayOfWeek", (q_func_n_t)f_getISODayOfWeek_Vd, QC_CONSTANT|QC_DEPRECATED, QDOM_DEFAULT, bigIntTypeInfo, 1, dateTypeInfo, NULL, "dt");
    // nothing getISODayOfWeek() {}
    ns.addBuiltinVariant("getISODayOfWeek", (q_func_n_t)f_getISODayOfWeek, QC_RUNTIME_NOOP|QC_DEPRECATED, QDOM_DEFAULT, nothingTypeInfo);
    // hash<IsoWeekInfo> getISOWeekHash(date dt) {}
    ns.addBuiltinVariant("getISOWeekHash", (q_func_n_t)f_getISOWeekHash_Vd, QC_CONSTANT|QC_DEPRECATED, QDOM_DEFAULT, hashdeclIsoWeekInfo->getTypeInfo(false), 1, dateTypeInfo, NULL, "dt");
    // nothing getISOWeekHash() {}
    ns.addBuiltinVariant("getISOWeekHash", (q_func_n_t)f_getISOWeekHash, QC_RUNTIME_NOOP|QC_DEPRECATED, QDOM_DEFAULT, nothingTypeInfo);
    // string getISOWeekString(date dt) {}
    ns.addBuiltinVariant("getISOWeekString", (q_func_n_t)f_getISOWeekString_Vd, QC_CONSTANT|QC_DEPRECATED, QDOM_DEFAULT, stringTypeInfo, 1, dateTypeInfo, NULL, "dt");
    // nothing getISOWeekString() {}
    ns.addBuiltinVariant("getISOWeekString", (q_func_n_t)f_getISOWeekString, QC_RUNTIME_NOOP|QC_DEPRECATED, QDOM_DEFAULT, nothingTypeInfo);
    // date get_date_from_iso_week(softint year, softint week, softint day = 1) {}
    ns.addBuiltinVariant("get_date_from_iso_week", (q_func_n_t)f_get_date_from_iso_week_vivivi, QC_RET_VALUE_ONLY, QDOM_DEFAULT, dateTypeInfo, 3, softBigIntTypeInfo, NULL, "year", softBigIntTypeInfo, NULL, "week", softBigIntTypeInfo, new QoreBigIntNode(1), "day");
    // int get_day_number(date dt) {}
    ns.addBuiltinVariant("get_day_number", (q_func_n_t)f_get_day_number_Vd, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, dateTypeInfo, NULL, "dt");
    // int get_day_of_week(date dt) {}
    ns.addBuiltinVariant("get_day_of_week", (q_func_n_t)f_get_day_of_week_Vd, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, dateTypeInfo, NULL, "dt");
    // int get_days(date dt) {}
    ns.addBuiltinVariant("get_days", (q_func_n_t)f_get_days_Vd, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, dateTypeInfo, NULL, "dt");
    // nothing get_days() {}
    ns.addBuiltinVariant("get_days", (q_func_n_t)f_get_days, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // int get_duration_microseconds(date dt) {}
    ns.addBuiltinVariant("get_duration_microseconds", (q_func_n_t)f_get_duration_microseconds_Vd, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, dateTypeInfo, NULL, "dt");
    // int get_duration_milliseconds(date dt) {}
    ns.addBuiltinVariant("get_duration_milliseconds", (q_func_n_t)f_get_duration_milliseconds_Vd, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, dateTypeInfo, NULL, "dt");
    // int get_duration_seconds(date dt) {}
    ns.addBuiltinVariant("get_duration_seconds", (q_func_n_t)f_get_duration_seconds_Vd, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, dateTypeInfo, NULL, "dt");
    // float get_duration_seconds_f(date dt) {}
    ns.addBuiltinVariant("get_duration_seconds_f", (q_func_n_t)f_get_duration_seconds_f_Vd, QC_CONSTANT, QDOM_DEFAULT, floatTypeInfo, 1, dateTypeInfo, NULL, "dt");
    // int get_epoch_seconds(date dt) {}
    ns.addBuiltinVariant("get_epoch_seconds", (q_func_n_t)f_get_epoch_seconds_Vd, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, dateTypeInfo, NULL, "dt");
    // nothing get_epoch_seconds() {}
    ns.addBuiltinVariant("get_epoch_seconds", (q_func_n_t)f_get_epoch_seconds, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // int get_hours(date dt) {}
    ns.addBuiltinVariant("get_hours", (q_func_n_t)f_get_hours_Vd, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, dateTypeInfo, NULL, "dt");
    // nothing get_hours() {}
    ns.addBuiltinVariant("get_hours", (q_func_n_t)f_get_hours, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // int get_iso_day_of_week(date dt) {}
    ns.addBuiltinVariant("get_iso_day_of_week", (q_func_n_t)f_get_iso_day_of_week_Vd, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, dateTypeInfo, NULL, "dt");
    // hash<IsoWeekInfo> get_iso_week_hash(date dt) {}
    ns.addBuiltinVariant("get_iso_week_hash", (q_func_n_t)f_get_iso_week_hash_Vd, QC_CONSTANT, QDOM_DEFAULT, hashdeclIsoWeekInfo->getTypeInfo(false), 1, dateTypeInfo, NULL, "dt");
    // string get_iso_week_string(date dt) {}
    ns.addBuiltinVariant("get_iso_week_string", (q_func_n_t)f_get_iso_week_string_Vd, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, dateTypeInfo, NULL, "dt");
    // int get_microseconds(date dt) {}
    ns.addBuiltinVariant("get_microseconds", (q_func_n_t)f_get_microseconds_Vd, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, dateTypeInfo, NULL, "dt");
    // date get_midnight(date dt) {}
    ns.addBuiltinVariant("get_midnight", (q_func_n_t)f_get_midnight_Vd, QC_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 1, dateTypeInfo, NULL, "dt");
    // nothing get_midnight() {}
    ns.addBuiltinVariant("get_midnight", (q_func_n_t)f_get_midnight, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // int get_milliseconds(date dt) {}
    ns.addBuiltinVariant("get_milliseconds", (q_func_n_t)f_get_milliseconds_Vd, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, dateTypeInfo, NULL, "dt");
    // nothing get_milliseconds() {}
    ns.addBuiltinVariant("get_milliseconds", (q_func_n_t)f_get_milliseconds, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // int get_minutes(date dt) {}
    ns.addBuiltinVariant("get_minutes", (q_func_n_t)f_get_minutes_Vd, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, dateTypeInfo, NULL, "dt");
    // nothing get_minutes() {}
    ns.addBuiltinVariant("get_minutes", (q_func_n_t)f_get_minutes, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // int get_months(date dt) {}
    ns.addBuiltinVariant("get_months", (q_func_n_t)f_get_months_Vd, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, dateTypeInfo, NULL, "dt");
    // nothing get_months() {}
    ns.addBuiltinVariant("get_months", (q_func_n_t)f_get_months, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // int get_seconds(date dt) {}
    ns.addBuiltinVariant("get_seconds", (q_func_n_t)f_get_seconds_Vd, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, dateTypeInfo, NULL, "dt");
    // nothing get_seconds() {}
    ns.addBuiltinVariant("get_seconds", (q_func_n_t)f_get_seconds, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // int get_years(date dt) {}
    ns.addBuiltinVariant("get_years", (q_func_n_t)f_get_years_Vd, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, dateTypeInfo, NULL, "dt");
    // nothing get_years() {}
    ns.addBuiltinVariant("get_years", (q_func_n_t)f_get_years, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // date gmtime() {}
    ns.addBuiltinVariant("gmtime", (q_func_n_t)f_gmtime, QC_CONSTANT, QDOM_DEFAULT, dateTypeInfo);
    // date gmtime(softint secs, softint us = 0) {}
    ns.addBuiltinVariant("gmtime", (q_func_n_t)f_gmtime_vivi, QC_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 2, softBigIntTypeInfo, NULL, "secs", softBigIntTypeInfo, zero(), "us");
    // date gmtime(date dt) {}
    ns.addBuiltinVariant("gmtime", (q_func_n_t)f_gmtime_Vd, QC_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 1, dateTypeInfo, NULL, "dt");
    // date hours(softint hours) {}
    ns.addBuiltinVariant("hours", (q_func_n_t)f_hours_vi, QC_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 1, softBigIntTypeInfo, NULL, "hours");
    // nothing hours() {}
    ns.addBuiltinVariant("hours", (q_func_n_t)f_hours, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // bool is_date_absolute(date dt) {}
    ns.addBuiltinVariant("is_date_absolute", (q_func_n_t)f_is_date_absolute_Vd, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo, 1, dateTypeInfo, NULL, "dt");
    // bool is_date_absolute() {}
    ns.addBuiltinVariant("is_date_absolute", (q_func_n_t)f_is_date_absolute, QC_NOOP, QDOM_DEFAULT, boolTypeInfo);
    // bool is_date_relative(date dt) {}
    ns.addBuiltinVariant("is_date_relative", (q_func_n_t)f_is_date_relative_Vd, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo, 1, dateTypeInfo, NULL, "dt");
    // bool is_date_relative() {}
    ns.addBuiltinVariant("is_date_relative", (q_func_n_t)f_is_date_relative, QC_NOOP, QDOM_DEFAULT, boolTypeInfo);
    // date localtime() {}
    ns.addBuiltinVariant("localtime", (q_func_n_t)f_localtime, QC_CONSTANT, QDOM_DEFAULT, dateTypeInfo);
    // date localtime(softint secs, softint us = 0) {}
    ns.addBuiltinVariant("localtime", (q_func_n_t)f_localtime_vivi, QC_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 2, softBigIntTypeInfo, NULL, "secs", softBigIntTypeInfo, zero(), "us");
    // date localtime(date dt) {}
    ns.addBuiltinVariant("localtime", (q_func_n_t)f_localtime_Vd, QC_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 1, dateTypeInfo, NULL, "dt");
    // date microseconds(softint us) {}
    ns.addBuiltinVariant("microseconds", (q_func_n_t)f_microseconds_vi, QC_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 1, softBigIntTypeInfo, NULL, "us");
    // date milliseconds(softint ms) {}
    ns.addBuiltinVariant("milliseconds", (q_func_n_t)f_milliseconds_vi, QC_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 1, softBigIntTypeInfo, NULL, "ms");
    // nothing milliseconds() {}
    ns.addBuiltinVariant("milliseconds", (q_func_n_t)f_milliseconds, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // date minutes(softint minutes) {}
    ns.addBuiltinVariant("minutes", (q_func_n_t)f_minutes_vi, QC_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 1, softBigIntTypeInfo, NULL, "minutes");
    // nothing minutes() {}
    ns.addBuiltinVariant("minutes", (q_func_n_t)f_minutes, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // int mktime(date dt) {}
    ns.addBuiltinVariant("mktime", (q_func_n_t)f_mktime_Vd, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, dateTypeInfo, NULL, "dt");
    // nothing mktime() {}
    ns.addBuiltinVariant("mktime", (q_func_n_t)f_mktime, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // date months(softint months) {}
    ns.addBuiltinVariant("months", (q_func_n_t)f_months_vi, QC_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 1, softBigIntTypeInfo, NULL, "months");
    // nothing months() {}
    ns.addBuiltinVariant("months", (q_func_n_t)f_months, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // date now() {}
    ns.addBuiltinVariant("now", (q_func_n_t)f_now, QC_CONSTANT, QDOM_DEFAULT, dateTypeInfo);
    // date now_ms() {}
    ns.addBuiltinVariant("now_ms", (q_func_n_t)f_now_ms, QC_CONSTANT, QDOM_DEFAULT, dateTypeInfo);
    // date now_us() {}
    ns.addBuiltinVariant("now_us", (q_func_n_t)f_now_us, QC_CONSTANT, QDOM_DEFAULT, dateTypeInfo);
    // date now_utc() {}
    ns.addBuiltinVariant("now_utc", (q_func_n_t)f_now_utc, QC_CONSTANT, QDOM_DEFAULT, dateTypeInfo);
    // date seconds(softint seconds) {}
    ns.addBuiltinVariant("seconds", (q_func_n_t)f_seconds_vi, QC_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 1, softBigIntTypeInfo, NULL, "seconds");
    // nothing seconds() {}
    ns.addBuiltinVariant("seconds", (q_func_n_t)f_seconds, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // int timegm(date dt) {}
    ns.addBuiltinVariant("timegm", (q_func_n_t)f_timegm_Vd, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, dateTypeInfo, NULL, "dt");
    // nothing timegm() {}
    ns.addBuiltinVariant("timegm", (q_func_n_t)f_timegm, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // date years(softint years) {}
    ns.addBuiltinVariant("years", (q_func_n_t)f_years_vi, QC_CONSTANT, QDOM_DEFAULT, dateTypeInfo, 1, softBigIntTypeInfo, NULL, "years");
    // nothing years() {}
    ns.addBuiltinVariant("years", (q_func_n_t)f_years, QC_RUNTIME_NOOP, QDOM_DEFAULT, nothingTypeInfo);

}
